@@ -10,6 +10,7 @@
 #undef bzDecompress
 #undef bzDecompressInit
 #undef EAZEL_SERVICES
+#undef ENABLE_PROFILER
 #undef ENABLE_NLS
 #undef HAVE_CATGETS
 #undef HAVE_GETTEXT
@@ -18,4 +19,4 @@
 #undef HAVE_RPM
 #undef HAVE_STPCPY
 #undef USING_OAF
-#undef NAUTILUS_TIMESTAMP
+#undef EAZEL_BUILD_TIMESTAMP
