@@ -18,6 +18,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  *  Author: Andy Hertzfeld <andy@eazel.com>
+ *          Shane Butler <shane_b@bigfoot.com>
  *
  */
 
@@ -43,6 +44,7 @@
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
 #include <libnautilus-extensions/nautilus-metadata.h>
 #include <libnautilus-extensions/nautilus-string.h>
+#include <libnautilus-extensions/nautilus-label.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtksignal.h>
 #include <gnome.h>
@@ -50,6 +52,9 @@
 #include <libgnorba/gnorba.h>
 #include <limits.h>
 
+#define HARDWARE_FONT_SIZE 14
+#define HARDWARE_TITLE_FONT_SIZE 24
+
 struct _NautilusHardwareViewDetails {
         char *uri;
         NautilusView *nautilus_view;
@@ -163,7 +168,7 @@ read_proc_info(const gchar* proc_filename)
 	}
 	fclose(thisFile);
 	
-	result = strdup(string_data->str);
+	result = g_strdup(string_data->str);
 	g_string_free(string_data, TRUE);
 	g_free(path_name);
 
@@ -199,7 +204,7 @@ extract_info(gchar* data, const gchar *field_name, gint nth)
 	/* add the requested node if the field was found */
 	if (field_data == NULL)
 		return NULL;
-	field_data = strdup(field_data);
+	field_data = g_strdup(field_data);
 	g_strfreev(info_array);
 	return field_data;
 }
@@ -213,12 +218,14 @@ get_CPU_description(gint nth)
 	GString* string_data = g_string_new("");
 	char* proc_data = read_proc_info("cpuinfo");
 
-	if(extract_info(proc_data, "processor", nth) == NULL) {
+	temp_str = extract_info(proc_data, "processor", nth);
+	if(temp_str == NULL) {
 		/* can't find nth processor */
 		g_free(proc_data);
 		g_string_free(string_data, TRUE);
 		return NULL;
 	}
+	g_free (temp_str);
 	
 	temp_str = extract_info(proc_data, "model name", nth);
 	g_string_append(string_data, temp_str);
@@ -236,7 +243,7 @@ get_CPU_description(gint nth)
 	g_free(temp_str);
 		
 	g_free(proc_data);
-	result = strdup(string_data->str);
+	result = g_strdup(string_data->str);
 	g_string_free(string_data, TRUE);
 
 	return result;	
@@ -250,12 +257,11 @@ get_RAM_description (void)
 	GString* string_data = g_string_new("");
 	char* proc_data = read_proc_info("meminfo");
 	
-	temp_str = extract_info(proc_data, "MemTotal", 0);
+	temp_str = extract_info (proc_data, "MemTotal", 0);
 	/* strip kbyte suffix */
 	temp_str[strlen(temp_str) - 3] = '\0';
 
-        /* FIXME: Would 1024 give a better result? */
-        num_str = gnome_vfs_format_file_size_for_display (1000 * atoi (temp_str));
+        num_str = gnome_vfs_format_file_size_for_display (1024 * atoi (temp_str));
 	
 	g_string_append(string_data, num_str);
 	g_string_append(string_data, " RAM");
@@ -263,12 +269,71 @@ get_RAM_description (void)
 	g_free(temp_str);
 
 	g_free(proc_data);
-	result = strdup(string_data->str);
+	result = g_strdup(string_data->str);
 	g_string_free(string_data, TRUE);
 
 	return result;
 }
 
+static char*
+get_IDE_description(char *device)
+{
+        char *temp_str, *num_str, *result;
+        GString* string_data = g_string_new("");
+        char *proc_file;
+
+        /* Read model information string */
+        proc_file = g_strdup_printf("%s/model", device);
+        temp_str = read_proc_info(proc_file);
+        temp_str[strlen(temp_str) - 1] = '\0';
+        g_string_append(string_data, temp_str);
+        g_free(temp_str);
+        g_free(proc_file);
+        
+        /* Read media type */
+        proc_file = g_strdup_printf("%s/media", device);
+        temp_str = read_proc_info(proc_file);
+        g_free(proc_file);
+        
+        /* If a hard disk, get the size */
+        if(!strcmp(temp_str, "disk\n")) {
+                unsigned long capacity;
+
+                g_free(temp_str);
+
+                proc_file = g_strdup_printf("%s/capacity", device);
+                temp_str = read_proc_info(proc_file);
+                temp_str[strlen(temp_str) - 1] = '\0';
+
+                /* NOTE: this should be 
+                   capacity = strtoul (...)
+                   num_str = gnome_vfs_format_file_size_for_display (512 * capacity);
+                   
+                   (512 bytes pr sector)
+                   but 512 * 23579136 (numsectors) = 12072517632 > unsigned long,
+                   which is the type that gnome_vfs uses */
+                /* This converts & calcs the sectors into MB's */
+                capacity = 512 * (strtoul (temp_str, NULL, 10) / (1024 * 1024));
+                if (capacity > 1024) {
+                        num_str = g_strdup_printf ("%lu GB", capacity / 1024);
+                } else {
+                        num_str = g_strdup_printf ("%lu MB", capacity);
+                }
+                g_string_append(string_data, "\n");
+                g_string_append(string_data, num_str);
+                g_free(temp_str);
+                g_free(proc_file);
+        } else {
+                g_free(temp_str);
+        }
+        
+        result = strdup(string_data->str);
+        g_string_free(string_data, TRUE);
+        
+        return result;
+}
+
+
 /* shared utility to allocate a title for a form */
 
 static void
@@ -277,7 +342,6 @@ setup_form_title (NautilusHardwareView *view, const char* image_name, const char
 	GtkWidget *temp_widget;
 	char *file_name;	
 	GtkWidget *temp_container = gtk_hbox_new(FALSE, 0);
-	GdkFont *font;
 
 	gtk_box_pack_start (GTK_BOX(view->details->form), temp_container, 0, 0, 4);	
 	gtk_widget_show(temp_container);
@@ -292,25 +356,37 @@ setup_form_title (NautilusHardwareView *view, const char* image_name, const char
 		}
 	}
 	
- 	temp_widget = gtk_label_new (title_text);
-
-        font = nautilus_font_factory_get_font_from_preferences (18);
-
-	nautilus_gtk_widget_set_font (temp_widget, font);
-        gdk_font_unref (font);
+ 	temp_widget = nautilus_label_new (title_text);
+	nautilus_label_set_font_size (NAUTILUS_LABEL (temp_widget), HARDWARE_TITLE_FONT_SIZE);
 
 	gtk_box_pack_start (GTK_BOX (temp_container), temp_widget, 0, 0, 8);			 	
 	gtk_widget_show (temp_widget);
 }
 
-/* set up the widgetry for the overview page */
+/* utility to add an element to the hardware view */
+static void
+add_element_to_table (GtkWidget *table, GtkWidget *element, int element_index)
+{
+	int column_pos, row_pos;
+	
+	column_pos = element_index % 3;
+	row_pos = element_index / 3;
+  	
+	gtk_table_attach (GTK_TABLE (table),element,
+			  column_pos, column_pos + 1, row_pos ,row_pos + 1, 
+			  GTK_FILL, GTK_FILL, 12, 12);
+}
 
-static void setup_overview_form(NautilusHardwareView *view)
+/* set up the widgetry for the overview page */
+static void setup_overview_form (NautilusHardwareView *view)
 {
 	char  *file_name, *temp_text;
-	GtkWidget *temp_widget, *temp_box;
-	GtkWidget *container_box;
-	int cpunum = 0;
+	GtkWidget *temp_widget, *pixmap_widget, *temp_box;
+	GtkWidget *table;
+	int cpunum, element_index;
+	DIR *directory;
+        struct dirent* entry;
+        char *device, *proc_file, *ide_media;
 	
 	/* allocate a vbox as the container */	
 	view->details->form = gtk_vbox_new(FALSE,0);
@@ -318,18 +394,18 @@ static void setup_overview_form(NautilusHardwareView *view)
 	gtk_widget_show(view->details->form);
 
 	/* set up the title */	
+	setup_form_title (view, NULL, "Hardware Overview");
+
+	/* allocate a table to hold the elements */
+	table =  gtk_table_new (3, 3, FALSE);	
+	gtk_box_pack_start (GTK_BOX (view->details->form), table, 0, 0, 2);	
+	gtk_widget_show (GTK_WIDGET(table));
+   	element_index = 0;
 	
-	setup_form_title(view, NULL, "Hardware Overview");
-
-	/* allocate a horizontal box */
-
-	container_box = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (view->details->form), container_box, 0, 0, 2);	
-	gtk_widget_show (GTK_WIDGET(container_box));
-
+   	cpunum = 0;
 	while( (temp_text = get_CPU_description(cpunum)) != NULL ) {
 		temp_box = gtk_vbox_new(FALSE, 4);
-		gtk_box_pack_start (GTK_BOX (container_box), temp_box, 0, 0, 24);	
+		add_element_to_table (table, temp_box, element_index++);
 		gtk_widget_show (temp_box);
 
 		file_name = nautilus_pixmap_file ("cpu.png");
@@ -338,8 +414,8 @@ static void setup_overview_form(NautilusHardwareView *view)
 			gtk_widget_show(temp_widget);
 			g_free (file_name);
 		
-		//temp_text = get_CPU_description(cpunum);
-		temp_widget = gtk_label_new (temp_text);
+		temp_widget = nautilus_label_new (temp_text);
+		nautilus_label_set_font_size (NAUTILUS_LABEL (temp_widget), HARDWARE_FONT_SIZE);
 		g_free(temp_text);
 		gtk_box_pack_start(GTK_BOX(temp_box), temp_widget, 0, 0, 0 );			
 		gtk_widget_show (temp_widget);
@@ -348,9 +424,8 @@ static void setup_overview_form(NautilusHardwareView *view)
 	}
 
 	/* set up the memory info */
-	
-	temp_box = gtk_vbox_new(FALSE, 4);
-  	gtk_box_pack_start (GTK_BOX (container_box), temp_box, 0, 0, 24);	
+       	temp_box = gtk_vbox_new(FALSE, 4);
+	add_element_to_table (table, temp_box, element_index++);
 	gtk_widget_show (temp_box);
 
  	file_name = nautilus_pixmap_file ("memory_chip.gif");
@@ -359,12 +434,59 @@ static void setup_overview_form(NautilusHardwareView *view)
   	gtk_widget_show(temp_widget);
   	g_free (file_name);
 	
-	temp_text = get_RAM_description();
-	temp_widget = gtk_label_new (temp_text);
+	temp_text = get_RAM_description ();
+	temp_widget = nautilus_label_new (temp_text);
+	nautilus_label_set_font_size (NAUTILUS_LABEL (temp_widget), HARDWARE_FONT_SIZE);
 	g_free(temp_text);
 	gtk_box_pack_start(GTK_BOX(temp_box), temp_widget, 0, 0, 0 );			
  	gtk_widget_show (temp_widget);
 	
+        /* Set up ide devices : by Shane Butler <shane_b@bigfoot.com> */
+        /* Open the ide devices directory */
+        if((directory = opendir("/proc/ide/")) != NULL) {
+                while((entry = readdir(directory)) != NULL) {
+                        /* Scan though each entry for actual device dirs */
+                        if(!strncmp(entry->d_name, "hd", 2)) {
+                                temp_box = gtk_vbox_new(FALSE, 4);
+				add_element_to_table (table, temp_box, element_index++);
+                                gtk_widget_show(temp_box);
+                                
+                                device = g_strdup_printf("ide/%s", entry->d_name);
+                                
+                                proc_file = g_strdup_printf("%s/media", device);
+                                ide_media = read_proc_info(proc_file);
+                                g_free(proc_file);
+                                
+                                /* Set the icon depending on the type of device */
+                                if(!strcmp(ide_media, "disk\n")) {
+                                        file_name = nautilus_pixmap_file("i-harddisk.png");
+                                } else if(!strcmp(ide_media, "cdrom\n")) {
+                                        file_name = nautilus_pixmap_file("CD_drive.png");
+                                } else {
+                                        /* some other device ... still set an icon */
+                                        file_name = nautilus_pixmap_file("i-harddisk.png");
+                                }
+                                
+				pixmap_widget = GTK_WIDGET (gnome_pixmap_new_from_file(file_name));
+				gtk_box_pack_start (GTK_BOX(temp_box), pixmap_widget, 0, 0, 0);
+				gtk_widget_show(pixmap_widget);
+				g_free(file_name);
+				g_free(ide_media);
+                                
+				temp_text = get_IDE_description (device);
+				temp_widget = nautilus_label_new (temp_text);
+				nautilus_label_set_font_size (NAUTILUS_LABEL (temp_widget), HARDWARE_FONT_SIZE);
+				nautilus_label_set_text_justification (NAUTILUS_LABEL (temp_widget), GTK_JUSTIFY_CENTER);
+
+				g_free(temp_text);
+                                gtk_box_pack_start(GTK_BOX(temp_box), temp_widget, 0, 0, 0);
+                                gtk_widget_show(temp_widget);
+   
+                                g_free(device);
+                        }
+                }
+                closedir(directory);
+        }
 }
 
 /* set up the widgetry for the CPU page */
@@ -380,10 +502,11 @@ static void setup_CPU_form(NautilusHardwareView *view)
 	gtk_widget_show(view->details->form);
 
 	/* set up the title */	
-	setup_form_title(view, NULL, "CPU");
+	setup_form_title (view, NULL, "CPU");
 	
 	message = "This is a placeholder for the CPU page.";
-	temp_widget = gtk_label_new (message);
+	temp_widget = nautilus_label_new (message);
+	nautilus_label_set_font_size (NAUTILUS_LABEL (temp_widget), HARDWARE_FONT_SIZE);
  	gtk_label_set_line_wrap(GTK_LABEL(temp_widget), TRUE);
 	
 	gtk_box_pack_start(GTK_BOX(view->details->form), temp_widget, 0, 0, 12);			
@@ -403,16 +526,42 @@ static void setup_RAM_form(NautilusHardwareView *view)
 	gtk_widget_show(view->details->form);
 
 	/* set up the title */	
-	setup_form_title(view, NULL, "RAM");
+	setup_form_title (view, NULL, "RAM");
 	
 	message = "This is a placeholder for the RAM page.";
-	temp_widget = gtk_label_new (message);
+	temp_widget = nautilus_label_new (message);
+	nautilus_label_set_font_size (NAUTILUS_LABEL (temp_widget), HARDWARE_FONT_SIZE);
  	gtk_label_set_line_wrap(GTK_LABEL(temp_widget), TRUE);
 	
 	gtk_box_pack_start(GTK_BOX(view->details->form), temp_widget, 0, 0, 12);			
  	gtk_widget_show (temp_widget);
 }
 
+/* set up the widgetry for the IDE page */
+
+static void setup_IDE_form(NautilusHardwareView *view)
+{
+        char *message;
+        GtkWidget *temp_widget;
+        
+        /* allocate a vbox as the container */  
+        view->details->form = gtk_vbox_new(FALSE,0);
+        gtk_container_add (GTK_CONTAINER (view), view->details->form);  
+        gtk_widget_show(view->details->form);
+        
+        /* set up the title */  
+        setup_form_title (view, NULL, "IDE");
+        
+        message = "This is a placeholder for the IDE page.";
+        temp_widget = nautilus_label_new (message);
+	nautilus_label_set_font_size (NAUTILUS_LABEL (temp_widget), HARDWARE_FONT_SIZE);
+        gtk_label_set_line_wrap(GTK_LABEL(temp_widget), TRUE);
+        
+        gtk_box_pack_start(GTK_BOX(view->details->form), temp_widget, 0, 0, 12);            
+        gtk_widget_show (temp_widget);
+}
+
+
 /* utility for checking uri */
 static gboolean is_location(char *document_str, const char *place_str)
 {
@@ -440,14 +589,17 @@ nautilus_hardware_view_load_uri (NautilusHardwareView *view, const char *uri)
 	document_name = strchr(uri, ':');
 	
 	/* load the appropriate form, based on the uri and the registration state */
-	if (is_location(document_name, "overview"))
+	if (is_location(document_name, "overview")) {
 		setup_overview_form(view);
-	else if (is_location(document_name, "CPU"))
+	} else if (is_location(document_name, "CPU")) {
 		setup_CPU_form(view);
-	else if (is_location(document_name, "RAM"))
+	} else if (is_location(document_name, "RAM")) {
 		setup_RAM_form(view);
-	else
+        } else if (is_location(document_name, "IDE")) {
+                setup_IDE_form(view);
+	} else {
 		setup_overview_form(view); /* if we don't understand it, go to the overview */
+        }
 }
 
 static void
