@@ -3,10 +3,9 @@
 #include "gnome.h"
 
 #define IS_IN_SECT(context) (((SectContext *)context->data)->state == IN_SECT)
-
 ElementInfo sect_elements[] = {
 	{ ARTICLE, "article", (startElementSAXFunc) article_start_element, (endElementSAXFunc) sect_article_end_element, NULL},
-	{ BOOK, "book", NULL, NULL, NULL},
+	{ BOOK, "book", (startElementSAXFunc) book_start_element, NULL, NULL},
 	{ SECTION, "section", (startElementSAXFunc) sect_sect_start_element, (endElementSAXFunc) sect_sect_end_element, NULL},
 	{ SECT1, "sect1", (startElementSAXFunc) sect_sect_start_element, (endElementSAXFunc) sect_sect_end_element, NULL},
 	{ SECT2, "sect2", (startElementSAXFunc) sect_sect_start_element, (endElementSAXFunc) sect_sect_end_element, NULL},
@@ -18,6 +17,7 @@ ElementInfo sect_elements[] = {
 	{ BOOKINFO, "bookinfo", NULL, NULL, NULL},
 	{ ARTHEADER, "artheader", NULL, NULL, NULL}, //(startElementSAXFunc) artheader_start_element, (endElementSAXFunc) sect_artheader_end_element, NULL},
 	{ ARTICLEINFO, "articleinfo", NULL, NULL, NULL},
+	{ GLOSSARYINFO, "glossaryinfo", NULL, NULL, NULL},
 	{ AUTHORGROUP, "authorgroup", NULL, NULL, NULL},
 	{ AUTHOR, "author", (startElementSAXFunc) sect_author_start_element, NULL, NULL},
 	{ FIRSTNAME, "firstname", NULL, NULL, (charactersSAXFunc) sect_author_characters },
@@ -26,7 +26,7 @@ ElementInfo sect_elements[] = {
 	{ AFFILIATION, "affiliation", NULL, NULL, NULL},
 	{ EMAIL, "email", NULL, NULL, (charactersSAXFunc) sect_email_characters },
 	{ ORGNAME, "orgname", NULL, NULL, (charactersSAXFunc) sect_author_characters },
-	{ ADDRESS, "address", NULL, NULL, NULL},
+	{ ADDRESS, "address", NULL, NULL, (charactersSAXFunc) sect_write_characters},
 	{ COPYRIGHT, "copyright", NULL, NULL, NULL},
 	{ YEAR, "year", NULL, NULL, (charactersSAXFunc) sect_copyright_characters},
 	{ HOLDER, "holder", NULL, NULL, (charactersSAXFunc) sect_copyright_characters},
@@ -84,9 +84,35 @@ ElementInfo sect_elements[] = {
 	{ PARAMDEF, "paramdef", (startElementSAXFunc) sect_paramdef_start_element, NULL, (charactersSAXFunc) sect_write_characters},
 	{ VOID, "void", (startElementSAXFunc) sect_void_start_element, NULL, NULL },
 	{ GUISUBMENU, "guisubmenu", (startElementSAXFunc) sect_menu_start_element, (endElementSAXFunc) sect_menu_end_element, (charactersSAXFunc) sect_write_characters},
-	{ INTERFACE, "interface", (startElementSAXFunc) sect_interface_start_element, (endElementSAXFunc) sect_interface_end_element, (charactersSAXFunc) sect_write_characters},
+	{ INTERFACE, "interface", NULL, NULL, (charactersSAXFunc) sect_write_characters},
 	{ LINK, "link", (startElementSAXFunc) sect_link_start_element, (endElementSAXFunc) sect_link_end_element, (charactersSAXFunc) sect_write_characters},
 	{ MENUCHOICE, "menuchoice", NULL, NULL, NULL},
+	{ TABLE, "table", (startElementSAXFunc) sect_table_with_border_start_element, (endElementSAXFunc) sect_table_end_element, NULL},
+	{ INFORMALTABLE, "informaltable", (startElementSAXFunc) sect_informaltable_start_element, (endElementSAXFunc) sect_informaltable_end_element, NULL},
+	{ ROW, "row", (startElementSAXFunc) sect_row_start_element, (endElementSAXFunc) sect_row_end_element, NULL},
+	{ ENTRY, "entry", (startElementSAXFunc) sect_entry_start_element, (endElementSAXFunc) sect_entry_end_element, (charactersSAXFunc) sect_write_characters},
+	{ THEAD, "thead", (startElementSAXFunc) sect_thead_start_element, (endElementSAXFunc) sect_thead_end_element, NULL},
+	{ TBODY, "tbody", (startElementSAXFunc) sect_tbody_start_element, (endElementSAXFunc) sect_tbody_end_element, NULL},
+	{ ACRONYM, "acronym", NULL, NULL, (charactersSAXFunc) sect_write_characters},
+	{ MARKUP, "markup", NULL, NULL, (charactersSAXFunc) sect_write_characters},
+	{ SIMPLELIST, "simplelist", (startElementSAXFunc) sect_table_without_border_start_element, (endElementSAXFunc) sect_table_end_element, NULL},
+	{ MEMBER, "member", (startElementSAXFunc) sect_member_start_element, (endElementSAXFunc) sect_member_end_element, (charactersSAXFunc) sect_write_characters},
+	{ MOUSEBUTTON, "mousebutton", NULL, NULL, (charactersSAXFunc) sect_write_characters},
+	{ SUPERSCRIPT, "superscript", (startElementSAXFunc) sect_sup_start_element, (endElementSAXFunc) sect_sup_end_element, (charactersSAXFunc) sect_write_characters},
+	{ SYSTEMITEM, "systemitem", NULL, NULL, (charactersSAXFunc) sect_write_characters},
+	{ VARNAME, "varname", (startElementSAXFunc) sect_tt_start_element, (endElementSAXFunc) sect_tt_end_element, (charactersSAXFunc) sect_write_characters},
+	{ BLOCKQUOTE, "blockquote", (startElementSAXFunc) sect_blockquote_start_element, (endElementSAXFunc) sect_blockquote_end_element, (charactersSAXFunc) sect_write_characters},
+	{ QUOTE, "quote", (startElementSAXFunc) sect_quote_start_element, (endElementSAXFunc) sect_quote_end_element, (charactersSAXFunc) sect_write_characters},
+	{ OPTION, "option", (startElementSAXFunc) sect_tt_start_element, (endElementSAXFunc) sect_tt_end_element, (charactersSAXFunc) sect_write_characters},
+	{ ENVAR, "envar", (startElementSAXFunc) sect_tt_start_element, (endElementSAXFunc) sect_tt_end_element, (charactersSAXFunc) sect_write_characters},
+	{ COMPUTEROUTPUT, "computeroutput", (startElementSAXFunc) sect_tt_start_element, (endElementSAXFunc) sect_tt_end_element, (charactersSAXFunc) sect_write_characters},
+	{ INLINEGRAPHIC, "inlinegraphic", (startElementSAXFunc) sect_inlinegraphic_start_element, NULL, NULL},
+	{ LEGALNOTICE, "legalnotice", (startElementSAXFunc) sect_legalnotice_start_element, (endElementSAXFunc) sect_legalnotice_end_element, (charactersSAXFunc) sect_legalnotice_characters},
+	{ QUESTION, "question", (startElementSAXFunc) sect_question_start_element, (endElementSAXFunc) sect_formalpara_end_element, NULL /* (charactersSAXFunc) sect_write_characters */},
+	{ ANSWER, "answer", (startElementSAXFunc) sect_answer_start_element, (endElementSAXFunc) sect_formalpara_end_element, NULL /* charactersSAXFunc) sect_write_characters */},
+	{ CHAPTER, "chapter", (startElementSAXFunc) sect_sect_start_element, (endElementSAXFunc) sect_sect_end_element, NULL},
+	{ PREFACE, "preface", (startElementSAXFunc) sect_sect_start_element, (endElementSAXFunc) sect_sect_end_element, NULL},
+	{ APPENDIX, "appendix", (startElementSAXFunc) sect_sect_start_element, (endElementSAXFunc) sect_sect_end_element, NULL},
 	{ UNDEFINED, NULL, NULL, NULL, NULL}
 };
 
@@ -112,6 +138,7 @@ sect_print (Context *context, gchar *format, ...)
 	list = g_slist_prepend (list, GINT_TO_POINTER (FUNCDEF));
 	list = g_slist_prepend (list, GINT_TO_POINTER (PARAMDEF));
 	index = find_first_parent (context, list);
+	g_slist_free (list);
 
 	switch (index) {
 	case FUNCDEF:
@@ -161,12 +188,13 @@ sect_print (Context *context, gchar *format, ...)
 		g_free (string);
 		break;
 	default:
-		if ((*string == '<') && (*(string +1) == '\000'))
+		if ((*string == '<') && (*(string +1) == '\000')) {
 			printf ("&lt;");
-		else if ((*string == '&') && (*(string +1) == '\000'))
+		} else if ((*string == '&') && (*(string +1) == '\000')) {
 			printf ("&amp;");
-		else
+		} else {
 			printf (string);
+		}
 		g_free (string);
 	}
 }
@@ -191,6 +219,7 @@ sect_write_characters (Context *context,
 
 	if (!IS_IN_SECT (context))
 		return;
+		
 
 	temp = g_strndup (chars, len);
 	sect_print (context, temp);
@@ -215,7 +244,8 @@ sect_article_end_element (Context *context, const gchar *name)
 		}
 		g_print ("</TABLE>");
 	}
-	sect_print (context, "</BODY>\n</HEAD>\n");
+	/* FIXME: Is the below needed now that we have the footer printed out */
+/*	sect_print (context, "</BODY>\n</HEAD>\n"); */
 }
 
 void
@@ -226,17 +256,22 @@ sect_para_start_element (Context *context, const gchar *name, const xmlChar **at
 
 	if (!IS_IN_SECT (context))
 		return;
-
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (PREFACE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (CHAPTER));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT1));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT2));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT3));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT4));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT5));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (APPENDIX));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECTION));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FIGURE));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FOOTNOTE));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FORMALPARA));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (LISTITEM));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (LEGALNOTICE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (QUESTION));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (ANSWER));
 	stack_el = find_first_element (context, element_list);
 
 	g_slist_free (element_list);
@@ -244,16 +279,22 @@ sect_para_start_element (Context *context, const gchar *name, const xmlChar **at
 		return;
 
 	switch (stack_el->info->index) {
+	case PREFACE:
+	case CHAPTER:
 	case SECT1:
 	case SECT2:
 	case SECT3:
 	case SECT4:
 	case SECT5:
+	case APPENDIX:
 	case SECTION:
+	case LEGALNOTICE:
 		sect_print (context, "<P>\n");
 		break;
 	case FORMALPARA:
 	case LISTITEM:
+	case QUESTION:
+	case ANSWER:
 	default:
 		break;
 	};
@@ -268,16 +309,22 @@ sect_para_end_element (Context *context, const gchar *name)
 	if (!IS_IN_SECT (context))
 		return;
 
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (PREFACE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (CHAPTER));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT1));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT2));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT3));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT4));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT5));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (APPENDIX));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECTION));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FIGURE));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FOOTNOTE));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FORMALPARA));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (LISTITEM));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (LEGALNOTICE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (QUESTION));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (ANSWER));	
 	stack_el = find_first_element (context, element_list);
 
 	g_slist_free (element_list);
@@ -285,16 +332,22 @@ sect_para_end_element (Context *context, const gchar *name)
 		return;
 
 	switch (stack_el->info->index) {
+	case PREFACE:
+	case CHAPTER:
 	case SECT1:
 	case SECT2:
 	case SECT3:
 	case SECT4:
 	case SECT5:
+	case APPENDIX:
 	case SECTION:
+	case LEGALNOTICE:
 		sect_print (context, "</P>\n");
 		break;
 	case FORMALPARA:
 	case LISTITEM:
+	case QUESTION:
+	case ANSWER:
 	default:
 		break;
 	};
@@ -329,7 +382,10 @@ sect_sect_start_element (Context *context,
 	gchar **atrs_ptr;
 	SectContext *sect_context = (SectContext *)context->data;
 
-	g_return_if_fail (strlen (name) >= 5);
+	if (g_strcasecmp (name, "section") == 0) {
+		return;
+	}
+
 	atrs_ptr = (gchar **) atrs;
 	while (atrs_ptr && *atrs_ptr) {
 		if (!g_strcasecmp (*atrs_ptr, "id")) {
@@ -345,6 +401,36 @@ sect_sect_start_element (Context *context,
 	}
 
 	switch (name[4]) {
+	case 'a':
+		/* Preface tag */
+	        context->preface++;
+		context->chapter = 0;
+		context->sect1 = 0;
+		context->sect2 = 0;
+		context->sect3 = 0;
+		context->sect4 = 0;
+		context->sect5 = 0;
+		break;
+	case 't':
+		/* Chapter tag */
+		context->chapter++;
+		context->sect1 = 0;
+		context->sect2 = 0;
+		context->sect3 = 0;
+		context->sect4 = 0;
+		context->sect5 = 0;
+		break;
+	case 'n':
+		/* Appendix tag */
+		context->appendix++;
+		context->chapter = 0;
+		context->sect1 = 0;
+		context->sect2 = 0;
+		context->sect3 = 0;
+		context->sect4 = 0;
+		context->sect5 = 0;
+		break;
+		
 	case '1':
 		if (sect_context->state != LOOKING_FOR_SECT_TITLE) {
 			g_free (sect_context->prev);
@@ -369,8 +455,8 @@ sect_sect_start_element (Context *context,
 		break;
 	case '4':
 		context->sect4++;
-		break;
 		context->sect5 = 0;
+		break;
 	case '5':
 		context->sect4++;
 		break;
@@ -385,17 +471,33 @@ sect_sect_end_element (Context *context,
 {
 	gchar **atrs_ptr;
 
-	g_return_if_fail (strlen (name) >= 5);
-
+	if (g_strcasecmp (name, "section") == 0) {
+		return;
+	}
+	
 	switch (name[4]) {
+	case 'a':
+	        context->chapter = 0;
+		break;
+	case 't':
+		context->sect1 = 0;
+		break;
 	case '1':
 		context->sect2 = 0;
+		break;
 	case '2':
 		context->sect3 = 0;
+		break;
 	case '3':
 		context->sect4 = 0;
+		break;
 	case '4':
 		context->sect5 = 0;
+		break;
+	case 'n':
+		/* Nothing should come after the appendix tag */
+		/* context->appendix = 0; */
+		break;
 	default:
 		break;
 	}
@@ -500,6 +602,19 @@ sect_email_characters (Context *context, const gchar *chars, int len)
 	g_free (temp);
 }
 
+void
+sect_country_characters (Context *context, const char *chars, int len)
+{
+	char *temp;
+	
+	if (!IS_IN_SECT (context))
+		return;
+	
+	temp = g_strndup (chars, len);
+	sect_print (context, "<br>%s", temp);
+	g_free (temp);
+}
+
 void
 sect_copyright_characters (Context *context,
 			  const gchar *chars,
@@ -536,33 +651,47 @@ sect_title_start_element (Context *context,
 	GSList *element_list = NULL;
 	StackElement *stack_el;
 	gchar **atrs_ptr;
+	SectContext *sect_context;
 
-	if (!IS_IN_SECT (context))
+	sect_context = (SectContext *) context->data;
+	if (!IS_IN_SECT (context)) 
 		return;
 
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (PREFACE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (CHAPTER));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT1));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT2));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT3));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT4));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT5));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (APPENDIX));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECTION));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FIGURE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (TABLE));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FORMALPARA));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (IMPORTANT));
 	stack_el = find_first_element (context, element_list);
 
 	g_slist_free (element_list);
 	if (stack_el == NULL)
 		return;
 
-
 	switch (stack_el->info->index) {
+	case TABLE:
+		sect_print (context, "<P><B>Table %d. ", sect_context->table_count); 
+		break;
+	case PREFACE:
+	case CHAPTER:
 	case SECT1:
 	case SECT2:
 	case SECT3:
 	case SECT4:
 	case SECT5:
+	case APPENDIX:
 	case SECTION:
-		if (context->sect2 == 0)
+		if (context->sect1 == 0)
+			sect_print (context, "<H1>");
+		else if (context->sect2 == 0)
 			sect_print (context, "<H2>");
 		else
 			sect_print (context, "<H3>");
@@ -598,18 +727,35 @@ sect_title_end_element (Context *context,
 	if (!IS_IN_SECT (context))
 		return;
 
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (PREFACE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (CHAPTER));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT1));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT2));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT3));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT4));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT5));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (APPENDIX));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECTION));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FIGURE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (TABLE));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FORMALPARA));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (IMPORTANT));
 
 	index = find_first_parent (context, element_list);
 
 	switch (index) {
+	case TABLE:
+		sect_print (context, "</B></P>\n");
+		break;
+	case PREFACE:
+	case APPENDIX:
+	        sect_print (context, "</A></H1>\n");
+		break;
+	case CHAPTER:
+		/* FIXME: This sometimes causes the output of invalid HTML because in sect_title_characters
+		 * we could switch states (from LOOKING_FOR_SECT_TITLE to IN_SECT */
+		sect_print (context, "</A></H1>\n");
+		break;
 	case SECT1:
 	case SECTION:
 		sect_print (context, "</A></H2>\n");
@@ -644,25 +790,28 @@ sect_title_characters (Context *context,
 	if (((SectContext *)context->data)->state == LOOKING_FOR_SECT_TITLE) {
 		StackElement *stack_el;
 
+		element_list = g_slist_prepend (element_list, GINT_TO_POINTER (PREFACE));
+		element_list = g_slist_prepend (element_list, GINT_TO_POINTER (CHAPTER));
 		element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT1));
 		element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT2));
 		element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT3));
 		element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT4));
 		element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT5));
+		element_list = g_slist_prepend (element_list, GINT_TO_POINTER (APPENDIX));
 		element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECTION));
 
 		stack_el = find_first_element (context, element_list);
 
 		temp = g_strndup (chars, len);
 
-		sect_print (context, "<TITLE>%s</TITLE>\n</HEAD>\n", temp);
+		sect_print (context, "<HEAD>\n<TITLE>%s</TITLE>\n</HEAD>\n", temp);
 		sect_print (context, "<BODY BGCOLOR=\"#FFFFFF\" TEXT=\"#000000\" LINK=\"#0000FF\" VLINK=\"#840084\" ALINK=\"#0000FF\">\n");
 		if (stack_el == NULL)
-			sect_print (context, "<A href=\"help:%s\"><font size=3>Up to Table of Contents</font></A><BR>\n",
+			sect_print (context, "<A href=\"gnome-help:%s\"><font size=3>Up to Table of Contents</font></A><BR>\n",
 				 context->base_file);
 #if 0
 		else
-			sect_print (context, "<A href=\"help:%s#%s\"><font size=3>Up to %s</font></A><BR>\n",
+			sect_print (context, "<A href=\"gnome-help:%s#%s\"><font size=3>Up to %s</font></A><BR>\n",
 				 context->base_file,
 				 sect_context->topid,
 				 sect_context->top);
@@ -678,25 +827,32 @@ sect_title_characters (Context *context,
 
 	temp = g_strndup (chars, len);
 
-
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (PREFACE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (CHAPTER));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT1));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT2));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT3));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT4));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT5));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (APPENDIX));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECTION));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (ARTHEADER));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FIGURE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (TABLE));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FORMALPARA));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (IMPORTANT));
 
 	index = find_first_parent (context, element_list);
 
 	switch (index) {
+	case PREFACE:
+	case CHAPTER:
 	case SECT1:
 	case SECT2:
 	case SECT3:
 	case SECT4:
 	case SECT5:
+	case APPENDIX:
 	case SECTION:
 	case FORMALPARA:
 		sect_print (context, temp);
@@ -711,6 +867,10 @@ sect_title_characters (Context *context,
 	case FIGURE:
 		((SectContext *) context->data)->figure->title = temp;
 		break;
+	case TABLE:
+		sect_print (context, temp);
+		g_free (temp);
+		break;
 	default:
 		g_free (temp);
 		break;
@@ -748,7 +908,7 @@ sect_link_start_element (Context *context,
 	if (!IS_IN_SECT (context))
 		return;
 
-	sect_print (context, "<A HREF=\"help:%s", context->base_file);
+	sect_print (context, "<A HREF=\"gnome-help:%s", context->base_file);
 	atrs_ptr = (gchar **) atrs;
 	while (atrs_ptr && *atrs_ptr) {
 		if (g_strcasecmp (*atrs_ptr, "linkend") == 0) {
@@ -776,12 +936,25 @@ sect_xref_start_element (Context *context,
 {
 	gchar **atrs_ptr;
 	gchar *title = NULL;
+	gchar *fignum_from_figure_id = NULL;
 
 	if (!IS_IN_SECT (context))
 		return;
+	
 
-	sect_print (context, "<A HREF=\"help:%s", context->base_file);
 	atrs_ptr = (gchar **) atrs;
+	if (*atrs_ptr) {
+		atrs_ptr++;
+		fignum_from_figure_id = g_hash_table_lookup (context->figure_data, *atrs_ptr);
+		atrs_ptr--;
+	}
+
+	if (fignum_from_figure_id != NULL) {
+		sect_print (context, "Figure %s", fignum_from_figure_id);
+		return;
+	}
+	
+	sect_print (context, "<A HREF=\"gnome-help:%s", context->base_file);
 	while (atrs_ptr && *atrs_ptr) {
 		if (!g_strcasecmp (*atrs_ptr, "linkend")) {
 			atrs_ptr++;
@@ -794,10 +967,11 @@ sect_xref_start_element (Context *context,
 	if (*atrs_ptr)
 		title = g_hash_table_lookup (((SectContext *)context->data)->title_hash, *atrs_ptr);
 
-	if (title == NULL)
-		sect_print (context, "\">the section here</A>");
-	else
-		sect_print (context, "\">the section <EM>%s</EM></A>", title);
+	if (title == NULL) {
+			sect_print (context, "\">the section here</A>");
+	} else {
+			sect_print (context, "\">the section <EM>%s</EM></A>", title);
+	}
 }
 
 void
@@ -856,7 +1030,8 @@ sect_figure_end_element (Context *context,
  		if (sect_context->figure->id)
 			sect_print (context, "</A>");
 		sect_print (context, "</B><P>");
-		sect_print (context, "<IMG SRC=\"%s\" ALT=\"%s\"><P>",
+		sect_print (context, "<IMG SRC=\"file://%s%s\" ALT=\"%s\"><P>",
+			    context->base_path,
 			    sect_context->figure->img,
 			    (sect_context->figure->alt) ? sect_context->figure->alt : "IMAGE");
 	}
@@ -868,21 +1043,81 @@ sect_figure_end_element (Context *context,
 	sect_context->figure = NULL;
 }
 
+void
+sect_inlinegraphic_start_element (Context *context,
+				  const char *name,
+                                  const xmlChar **atrs)
+{
+	char **atrs_ptr;
+	char *format;
+	char *fileref;
+	char *lowcaseformat;
+	
+	if (!IS_IN_SECT (context))
+		return;
+
+	format = NULL;
+	fileref = NULL;
+	atrs_ptr = (char **) atrs;
+	while (atrs_ptr && *atrs_ptr) {
+		if (g_strcasecmp (*atrs_ptr, "format") == 0) {
+			atrs_ptr++;
+			format = *atrs_ptr;
+			atrs_ptr++;
+			continue;
+		} else if (g_strcasecmp (*atrs_ptr, "fileref") == 0) {
+			atrs_ptr++;
+			fileref = *atrs_ptr;
+			atrs_ptr++;
+			continue;
+		}
+		atrs_ptr += 2;
+	}
+
+	if (fileref == NULL) {
+		return;
+	}
+
+	/* FIXME: Should we put an 'ALT' tag here? */	
+	if (format == NULL) {
+		/* Default is PNG */
+		sect_print (context, "<IMG SRC=\"file://%s%s.png\">", context->base_path, fileref);
+	} else 	if (g_strcasecmp (format, "gif") == 0) {
+		sect_print (context, "<IMG SRC=\"file://%s%s.gif\">", context->base_path, fileref);
+	} else if ((g_strcasecmp (format, "jpg") == 0) ||
+		   (g_strcasecmp (format, "jpeg") == 0)) {
+		/* Some people decide to use .jpg, others use .jpeg */
+		lowcaseformat = g_strdup (format);
+		g_strdown (lowcaseformat);
+		sect_print (context, "<IMG SRC=\"file://%s%s.%s\">", context->base_path, fileref, lowcaseformat);
+		g_free (lowcaseformat);
+	} else {
+		/* Unknown file format */
+		sect_print (context, "<IMG SRC=\"file://%s%s.%s\">", context->base_path, fileref, format);
+	} 
+}
+
+
 void
 sect_graphic_start_element (Context *context,
-			    const gchar *name,
+			    const char *name,
 			    const xmlChar **atrs)
 {
-	gchar **atrs_ptr;
-	gchar *format = NULL;
-	gchar *fileref = NULL;
-	gchar *lowcaseformat = NULL;
-	SectContext *sect_context = (SectContext *)context->data;
+	char **atrs_ptr;
+	char *format;
+	char *fileref;
+	char *lowcaseformat;
+	SectContext *sect_context;
 
 	if (!IS_IN_SECT (context))
 		return;
 
 	atrs_ptr = (gchar **) atrs;
+	format = NULL;
+	fileref = NULL;
+	lowcaseformat = NULL;
+	sect_context = (SectContext *)context->data;
+	
 	while (atrs_ptr && *atrs_ptr) {
 		if (!g_strcasecmp (*atrs_ptr, "format")) {
 			atrs_ptr++;
@@ -906,7 +1141,10 @@ sect_graphic_start_element (Context *context,
 	if (fileref == NULL)
 		return;
 
-	if (g_strcasecmp (format, "gif") == 0) {
+	if (format == NULL) {
+		/* Default image type */
+		sect_context->figure->img = g_strdup_printf ("%s.png", fileref);
+	} else if (g_strcasecmp (format, "gif") == 0) {
 		sect_context->figure->img = g_strdup_printf ("%s.gif", fileref);
 	} else if ((g_strcasecmp (format, "jpg") == 0) ||
 		   (g_strcasecmp (format, "jpeg") == 0)) {
@@ -916,30 +1154,11 @@ sect_graphic_start_element (Context *context,
 		sect_context->figure->img = g_strdup_printf ("%s.%s", fileref, lowcaseformat);
 		g_free (lowcaseformat);
 	} else {
-		sect_context->figure->img = g_strdup_printf ("%s.png", fileref);
+		/* Unknown image type */
+		sect_context->figure->img = g_strdup_printf ("%s.%s", fileref, format);
 	} 
 }
-
-void
-sect_interface_start_element (Context *context,
-				   const char *name,
-				   const xmlChar **atrs)
-{
-	if (!IS_IN_SECT (context))
-		return;
-
-	sect_print (context, "<SPAN CLASS=\"INTERFACE\">");
-}
-
-void 
-sect_interface_end_element (Context *context,
-				 const char *name)
-{
-	if (!IS_IN_SECT (context))
-		return;
-
-	sect_print (context, "</SPAN>");
-}
+	
 
 void
 sect_em_start_element (Context *context,
@@ -994,14 +1213,6 @@ sect_menu_start_element (Context *context,
 	if (!IS_IN_SECT (context))
 		return;
 	
-/*	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT1));
-	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT2));
-	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT3));
-	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT4));
-	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT5));
-	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECTION));
-	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FIGURE));
-	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FORMALPARA)); */
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (MENUCHOICE));
 
 	index = find_first_parent (context, element_list);
@@ -1061,15 +1272,6 @@ sect_b_end_element (Context *context,
 	sect_print (context, "</B>");
 }
 
-void sect_b_arrow_end_element (Context *context,
-                               const gchar *name)
-{
-	        if (!IS_IN_SECT (context))
-			return;
-		
-		sect_print (context, "-&gt;</B>");
-}
-
 void
 sect_tti_start_element (Context *context,
 		       const gchar *name,
@@ -1183,7 +1385,7 @@ sect_variablelist_start_element (Context *context,
 	if (!IS_IN_SECT (context))
 		return;
 
-	sect_print (context, "<OL>");
+	sect_print (context, "<UL>");
 }
 
 void
@@ -1193,7 +1395,7 @@ sect_variablelist_end_element (Context *context,
 	if (!IS_IN_SECT (context))
 		return;
 
-	sect_print (context, "</OL>");
+	sect_print (context, "</UL>");
 }
 
 void
@@ -1265,9 +1467,9 @@ sect_infobox_start_element (Context *context,
 
 	logo = sect_get_infobox_logo (name);
 
-	sect_print (context, "<TABLE BORDER=\"0\"  WIDTH=\"100%%\">\n<tr><TD WIDTH=\"25%%\" ALIGN=\"CENTER\" VALIGN=\"TOP\">");
+	sect_print (context, "<TABLE BORDER=\"0\"  WIDTH=\"100%%\">\n<tr><TD WIDTH=\"10%%\" ALIGN=\"CENTER\" VALIGN=\"TOP\">");
 	if (logo != NULL)
-		sect_print (context, "<IMG ALT=\"%s\" SRC=\"%s\">", name, logo);
+		sect_print (context, "<IMG ALT=\"%s\" SRC=\"file://%s\">", name, logo);
 	else
 		sect_print (context, "%s", name);
 	sect_print (context, "<TH ALIGN=\"LEFT\" VALIGN=\"CENTER\"></TD>\n");
@@ -1469,3 +1671,353 @@ sect_void_start_element (Context        *context,
 
 	sect_print (context, "void");
 }
+
+
+void
+sect_informaltable_start_element (Context *context,
+			   	  const char *name,
+				  const xmlChar **atrs)
+{
+	if (!IS_IN_SECT (context))
+		return;
+
+	sect_print (context, "<TABLE BORDER=\"0\" BGCOLOR=\"#E0E0E0\"
+                           CELLSPACING=\"0\" CELLPADDING=\"4\">\n");
+}
+
+void
+sect_informaltable_end_element (Context *context,
+				const char *name)
+{
+	if (!IS_IN_SECT (context))
+		return;
+	
+	sect_print (context, "</TABLE>\n");
+}
+
+void
+sect_table_with_border_start_element (Context *context,
+			  	      const char *name,
+			  	      const xmlChar **atrs)
+{
+	SectContext *sect_context;
+
+	sect_context = (SectContext *) context->data;
+	if (!IS_IN_SECT (context))
+		return;
+	
+	sect_print (context, "<TABLE BORDER=\"1\">\n");
+	
+	sect_context->table_count++;
+}
+
+void
+sect_table_without_border_start_element (Context *context,
+					 const char *name,
+					 const xmlChar **atrs)
+{
+	if (!IS_IN_SECT (context))
+		return;
+
+	sect_print (context, "<TABLE BORDER=\"0\">\n");
+}
+
+void
+sect_table_end_element (Context *context,
+		        const char *name)
+{
+	if (!IS_IN_SECT (context))
+		return;
+	
+	sect_print (context,"</TABLE>\n");
+}
+
+void
+sect_row_start_element (Context *context,
+			const char *name,
+			const xmlChar **atrs)
+{
+	if (!IS_IN_SECT (context))
+		return;
+
+	sect_print (context, "<TR>\n");
+}
+
+void
+sect_row_end_element (Context *context,
+		      const char *name)
+{
+	if (!IS_IN_SECT (context))
+		return;
+
+	sect_print (context, "</TR>\n");
+}
+void
+sect_entry_start_element (Context *context,
+			   const char *name,
+			   const xmlChar **atrs)
+{
+	GSList *element_list = NULL;
+	StackElement *stack_el;
+	
+	if (!IS_IN_SECT (context))
+		return;
+	
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (THEAD));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (TBODY));
+	stack_el = find_first_element (context, element_list);
+	g_slist_free (element_list);
+
+	if (stack_el == NULL) {
+		return;
+	}
+
+	switch (stack_el->info->index) {
+		case THEAD:
+			sect_print (context, "<TH ALIGN=\"LEFT\" VALIGN=\"TOP\">");
+			break;
+		case TBODY:
+			sect_print (context, "<TD ALIGN=\"LEFT\" VALIGN=\"TOP\">");
+			break;
+		default:
+			break;
+	};
+}
+
+void
+sect_entry_end_element (Context *context,
+			const char *name)
+{
+	GSList *element_list = NULL;
+	StackElement *stack_el;
+	
+	if (!IS_IN_SECT (context))
+		return;
+
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (THEAD));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (TBODY));
+	stack_el = find_first_element (context, element_list);
+	g_slist_free (element_list);
+	if (stack_el == NULL) {
+		return;
+	}
+
+	if (context->empty_element) {
+		/* We encoutnered an 'empty' element */
+		g_print ("&nbsp;");
+	}
+	
+	switch (stack_el->info->index) {
+		case THEAD:
+			sect_print (context, "</TH>\n");
+			break;
+		case TBODY:
+			sect_print (context, "</TD>\n");
+			break;
+		default:
+			break;
+	}
+}
+
+void
+sect_thead_start_element (Context *context,
+			  const char *name,
+			  const xmlChar **atrs)
+{
+	if (!IS_IN_SECT (context))
+	        return;
+
+        sect_print (context, "<THEAD>");
+}
+
+void
+sect_thead_end_element (Context *context,
+			const char *name)
+{
+        if (!IS_IN_SECT (context))
+		return;
+
+	sect_print (context, "</THEAD>");
+}
+
+void
+sect_tbody_start_element (Context *context,
+                          const char *name,
+			  const xmlChar **atrs)
+{
+        if (!IS_IN_SECT (context))
+                return;
+
+        sect_print (context, "<TBODY>");
+}
+
+void
+sect_tbody_end_element (Context *context,
+                        const char *name)
+{
+        if (!IS_IN_SECT (context))
+                return;
+
+        sect_print (context, "</TBODY>");
+}
+
+void
+sect_member_start_element (Context *context,
+			   const char *name,
+			   const xmlChar **atrs)
+{
+	if (!IS_IN_SECT (context))
+		return;
+
+	sect_print (context, "<TR><TD>");
+}
+
+void
+sect_member_end_element (Context *context,
+			 const char *name)
+{
+	if (!IS_IN_SECT (context))
+		return;
+	
+	sect_print (context, "</TD></TR>\n");
+}
+
+void
+sect_sup_start_element (Context *context,
+			const char *name,
+			const xmlChar **atrs)
+{
+	if (!IS_IN_SECT (context))
+		return;
+	
+	sect_print (context, "<SUP>");
+}
+
+void
+sect_sup_end_element (Context *context,
+		      const char *name)
+{
+	if (!IS_IN_SECT (context))
+		return;
+
+	sect_print (context, "</SUP>");
+}
+
+void
+sect_quote_start_element (Context *context,
+		       	  const char *name,
+		       	  const xmlChar **atrs)
+{
+	if (!IS_IN_SECT (context))
+		return;
+
+	sect_print (context, "\"");
+}
+
+void sect_quote_end_element (Context *context,
+			     const char *name)
+{
+	if (!IS_IN_SECT (context))
+		return;
+
+	sect_print (context, "\"");
+}
+
+void
+sect_blockquote_start_element (Context *context,
+			       const char *name,
+			       const xmlChar **atrs)
+{
+	if (!IS_IN_SECT (context))
+		return;
+
+	sect_print (context, "<BLOCKQUOTE>");
+}
+
+void
+sect_blockquote_end_element (Context *context,
+			     const char *name)
+{
+	if (!IS_IN_SECT (context))
+		return;
+	
+	sect_print (context, "</BLOCKQUOTE>\n");
+}
+
+void
+sect_legalnotice_start_element (Context *context,
+				const char *name,
+				const xmlChar **atrs)
+{
+	SectContext *sect_context;
+
+	sect_context = (SectContext *)context->data;
+
+	if ((strcmp (context->target_section, "legalnotice") != 0) || IS_IN_SECT (context)) {
+		/* If we are not searching for the legalnotice or we are
+		 * currently in a sect then abort */
+		return;
+	}
+
+	sect_context->state = IN_SECT;
+	sect_print (context, "<HEAD>\n<TITLE>Legal Notice</TITLE>\n</HEAD>\n");
+	sect_print (context, "<BODY BGCOLOR=\"#FFFFFF\" TEXT=\"#000000\" LINK=\"#0000FF\" VLINK=\"#840084\" ALINK=\"#0000FF\">\n");
+	sect_print (context, "<H1>Legal Notice</H1>\n");
+
+}
+
+void
+sect_legalnotice_end_element (Context *context,
+			      const char *name)
+{
+	SectContext *sect_context;
+	char *prev_link;
+	
+	sect_context = (SectContext *)context->data;
+
+	if (strcmp (context->target_section, "legalnotice") != 0) {
+		return;
+	}
+
+
+	prev_link = g_strdup_printf ("gnome-help:%s", context->base_file);
+	print_footer (prev_link, prev_link, NULL);
+	g_free (prev_link);
+	
+	sect_context->state = DONE_WITH_SECT;
+}
+
+void
+sect_legalnotice_characters (Context *context,
+			     const char *chars,
+			     int len)
+{
+	if (strcmp (context->target_section, "legalnotice") != 0) {
+		return;
+	}
+
+	sect_write_characters (context, chars, len);
+}
+
+void
+sect_question_start_element (Context *context,
+                             const char *name,
+                             const xmlChar **atrs)
+{
+      if (!IS_IN_SECT (context))
+              return;
+
+      sect_print (context, "<P><B>Q: </B>");
+}
+
+void
+sect_answer_start_element (Context *context,
+                            const char *name,
+                            const xmlChar **atrs)
+{
+      if (!IS_IN_SECT (context))
+              return;
+
+      sect_print (context, "<P><B>A: </B>");
+}
+
