@@ -21,7 +21,7 @@ static void toc_title_characters (Context *context, const gchar *chars, int len)
 
 ElementInfo toc_elements[] = {
 	{ ARTICLE, "article", (startElementSAXFunc) article_start_element, NULL, NULL},
-	{ BOOK, "book", (startElementSAXFunc) article_start_element, NULL, NULL},
+	{ BOOK, "book", (startElementSAXFunc) book_start_element, NULL, NULL},
 	{ SECTION, "section", (startElementSAXFunc) toc_sect_start_element, (endElementSAXFunc) toc_sect_end_element, NULL},
 	{ SECT1, "sect1", (startElementSAXFunc) toc_sect_start_element, (endElementSAXFunc) toc_sect_end_element, NULL},
 	{ SECT2, "sect2", (startElementSAXFunc) toc_sect_start_element, (endElementSAXFunc) toc_sect_end_element, NULL},
@@ -33,6 +33,7 @@ ElementInfo toc_elements[] = {
 	{ BOOKINFO, "bookinfo", (startElementSAXFunc) artheader_start_element, (endElementSAXFunc) toc_artheader_end_element, NULL},
 	{ ARTHEADER, "artheader", (startElementSAXFunc) artheader_start_element, (endElementSAXFunc) toc_artheader_end_element, NULL},
 	{ ARTICLEINFO, "articleinfo", (startElementSAXFunc) artheader_start_element, (endElementSAXFunc) toc_artheader_end_element, NULL},
+	{ GLOSSARYINFO, "glossaryinfo", (startElementSAXFunc) artheader_start_element, (endElementSAXFunc) toc_artheader_end_element, NULL},
 	{ AUTHORGROUP, "authorgroup", NULL, NULL, NULL},
 	{ AUTHOR, "author", (startElementSAXFunc) toc_author_start_element, NULL, NULL},
 	{ FIRSTNAME, "firstname", NULL, NULL, (charactersSAXFunc) toc_author_characters },
@@ -80,8 +81,8 @@ ElementInfo toc_elements[] = {
 	{ GUIMENUITEM, "guimenuitem", NULL, NULL, NULL},
 	{ HARDWARE, "hardware", NULL, NULL, NULL},
 	{ KEYCAP, "keycap", NULL, NULL, NULL},
-	{ KEYCAP, "keycode", NULL, NULL, NULL},
-	{ KEYCAP, "keysym", NULL, NULL, NULL},
+	{ KEYCODE, "keycode", NULL, NULL, NULL},
+	{ KEYSYM, "keysym", NULL, NULL, NULL},
 	{ LITERAL, "literal", NULL, NULL, NULL},
 	{ PARAMETER, "parameter", NULL, NULL, NULL},
 	{ PROMPT, "prompt", NULL, NULL, NULL},
@@ -98,6 +99,36 @@ ElementInfo toc_elements[] = {
 	{ FUNCPARAMS, "funcparams", NULL, NULL, NULL},
 	{ PARAMDEF, "paramdef", NULL, NULL, NULL},
 	{ VOID, "void", NULL, NULL, NULL},
+	{ GUISUBMENU, "guisubmenu", NULL, NULL, NULL},
+	{ INTERFACE, "interface", NULL, NULL, NULL},
+	{ LINK, "link", NULL, NULL, NULL},
+	{ MENUCHOICE, "menuchoice", NULL, NULL, NULL},
+	{ TABLE, "table", NULL, NULL, NULL},
+	{ INFORMALTABLE, "informaltable", NULL, NULL, NULL},
+	{ ROW, "row",  NULL, NULL, NULL},
+	{ ENTRY, "entry", NULL, NULL, NULL},
+	{ THEAD, "thead", NULL, NULL, NULL},
+	{ TBODY, "tbody", NULL, NULL, NULL},
+	{ ACRONYM, "acronym", NULL, NULL, NULL},
+	{ MARKUP, "markup", NULL, NULL, NULL},
+	{ SIMPLELIST, "simplelist", NULL, NULL, NULL},
+	{ MEMBER, "member", NULL, NULL, NULL},
+	{ MOUSEBUTTON, "mousebutton", NULL, NULL, NULL},
+	{ SUPERSCRIPT, "superscript", NULL, NULL, NULL},
+	{ SYSTEMITEM, "systemitem", NULL, NULL, NULL},
+	{ VARNAME, "varname", NULL, NULL, NULL},
+	{ BLOCKQUOTE, "blockquote", NULL, NULL, NULL},
+	{ QUOTE, "quote", NULL, NULL, NULL},
+	{ OPTION, "option", NULL, NULL, NULL},
+	{ ENVAR, "envar", NULL, NULL, NULL},
+	{ COMPUTEROUTPUT, "computeroutput", NULL, NULL, NULL},
+	{ INLINEGRAPHIC, "inlinegraphic", NULL, NULL, NULL},
+	{ LEGALNOTICE, "legalnotice", NULL, NULL, NULL},
+	{ QUESTION, "question", NULL, NULL, NULL},
+	{ ANSWER, "answer", NULL, NULL, NULL},
+	{ CHAPTER, "chapter", (startElementSAXFunc) toc_sect_start_element, (endElementSAXFunc) toc_sect_end_element, NULL},
+	{ PREFACE, "preface", (startElementSAXFunc) toc_sect_start_element, (endElementSAXFunc) toc_sect_end_element, NULL},
+	{ APPENDIX, "appendix", (startElementSAXFunc) toc_sect_start_element, (endElementSAXFunc) toc_sect_end_element, NULL},
 	{ UNDEFINED, NULL, NULL, NULL, NULL}
 };
 
@@ -118,10 +149,44 @@ toc_sect_start_element (Context *context,
 {
 	gchar **atrs_ptr;
 
-	g_return_if_fail (strlen (name) >= 5);
+	if (g_strcasecmp (name, "section") == 0) {
+		return;
+	}
 
 	switch (name[4]) {
+	case 'a':
+		sect1id_stack_add (context, name, atrs);
+		context->preface++;
+		context->chapter = 0;
+		context->sect1 = 0;
+		context->sect2 = 0;
+		context->sect3 = 0;
+		context->sect4 = 0;
+		context->sect5 = 0;
+		break;
+	case 't':
+		sect1id_stack_add (context, name, atrs);
+		context->chapter++;
+		context->sect1 = 0;
+		context->sect2 = 0;
+		context->sect3 = 0;
+		context->sect4 = 0;
+		context->sect5 = 0;
+		break;
+	case 'n':
+		sect1id_stack_add (context, name, atrs);
+		context->appendix++;
+		context->chapter = 0;
+		context->sect1 = 0;
+		context->sect2 = 0;
+		context->sect3 = 0;
+		context->sect4 = 0;
+		context->sect5 = 0;
+		break;
 	case '1':
+		if (context->doctype == ARTICLE) {
+			sect1id_stack_add (context, name, atrs);
+		}
 		context->sect1++;
 		context->sect2 = 0;
 		context->sect3 = 0;
@@ -167,17 +232,37 @@ static void
 toc_sect_end_element (Context *context,
 		      const gchar *name)
 {
-	g_return_if_fail (strlen (name) >= 5);
+	if (g_strcasecmp (name, "section") == 0) {
+		return;
+	}
+	
 
 	switch (name[4]) {
+	case 'a':
+		/* FIXME: should chapter be set to zero? */
+	        context->preface = 0;
+		context->sect1 = 0;
+		break;
+	case 't':
+		context->sect1 = 0;
+		break;
 	case '1':
 		context->sect2 = 0;
+		break;
 	case '2':
 		context->sect3 = 0;
+		break;
 	case '3':
 		context->sect4 = 0;
+		break;
 	case '4':
 		context->sect5 = 0;
+		break;
+	case 'n':
+		/* FIXME: should chapter be set to zero? */
+		context->appendix = 0;
+		context->sect1 = 0;
+		break;
 	default:
 		break;
 	}
@@ -197,7 +282,7 @@ toc_artheader_end_element (Context *context, const gchar *name)
 
 	g_print ("<BODY BGCOLOR=\"#FFFFFF\" TEXT=\"#000000\" LINK=\"#0000FF\" VLINK=\"#840084\" ALINK=\"#0000FF\">\n");
 	g_print ("<TABLE><TR><TD VALIGN=\"TOP\">\n");
-	g_print ("<IMG SRC=\"/usr/share/pixmaps/gnome-logo-icon.png\" BORDER=\"0\" ALT=\"GNOME\">\n");
+	g_print ("<IMG SRC=\"file:///usr/share/pixmaps/gnome-logo-icon.png\" BORDER=\"0\" ALT=\"GNOME\">\n");
 	g_print ("</TD><TD VALIGN=\"BOTTOM\">\n");
 	if (header->title)
 		g_print ("<H1>%s</H1>\n", header->title);
@@ -223,8 +308,8 @@ toc_artheader_end_element (Context *context, const gchar *name)
 	}
 	g_print ("<P>");
 	if ((header->copyright_holder) && (header->copyright_year))
-		g_print ("Copyright &copy; %s by %s", header->copyright_year, header->copyright_holder);
-	g_print ("<HR>\n<H1>Table of Contents</H1>\n\n");
+		g_print ("<A HREF=\"gnome-help:%s?legalnotice\">Copyright</A> &copy; %s by %s", context->base_file, header->copyright_year, header->copyright_holder);
+	g_print ("<HR>\n<H2>Table of Contents</H2>\n\n");
 	g_print ("<P>\n");
 }
 
@@ -327,12 +412,16 @@ toc_title_start_element (Context *context,
 	GSList *element_list = NULL;
 	StackElement *stack_el;
 	gchar **atrs_ptr;
+	gboolean print_link;
 
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (PREFACE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (CHAPTER));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT1));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT2));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT3));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT4));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT5));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (APPENDIX));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECTION));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FIGURE));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FORMALPARA));
@@ -341,6 +430,8 @@ toc_title_start_element (Context *context,
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (WARNING));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (GRAPHIC));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (CAUTION));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (TABLE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (IMPORTANT));
 	stack_el = find_first_element (context, element_list);
 
 	g_slist_free (element_list);
@@ -348,35 +439,100 @@ toc_title_start_element (Context *context,
 		return;
 
 	switch (stack_el->info->index) {
+	case PREFACE:
+	case CHAPTER:
 	case SECT1:
+	case SECTION:
+	case APPENDIX:
+		if (context->sect1 == 0) {
+			g_print ("<DT>");
+		} else if (context->sect2 == 0) {
+			if (context->chapter > 0) {
+				g_print ("<DT>&nbsp;&nbsp;");
+			} else {
+				g_print ("<DT>");
+			}
+
+		} else if (context->sect3 == 0) {
+			g_print ("<DT>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
+		} else {
+			g_print ("<DT>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
+		}
+
+		if (context->preface > 0) {
+		       if (context->sect1 == 0) g_print ("PREFACE:<BR>");
+		       if (context->sect1 > 0) g_print ("&nbsp;&nbsp;%d", context->sect1); 
+		        
+		} else if (context->chapter > 0) {
+			g_print ("%d", context->chapter);
+			if (context->sect1 > 0) g_print (".%d", context->sect1);
+									
+		} else if (context->appendix > 0) {
+			if (context->sect1 == 0) {
+				g_print ("APPENDIX:<BR>");
+			} else if (context->sect1 > 0) {
+				g_print ("&nbsp;&nbsp;%d",context->sect1);
+			}
+		} else {
+			if (context->sect1 > 0) g_print ("%d", context->sect1);
+		}
+		
+		
+		if (context->sect2 > 0) g_print (".%d", context->sect2);
+                if (context->sect3 > 0) g_print (".%d", context->sect3);
+                if (context->sect4 > 0) g_print (".%d", context->sect4);
+                if (context->sect5 > 0) g_print (".%d", context->sect5);
+
+		/* Don't print the "." if you are in the preface or appendix title */
+		if (stack_el->info->index != PREFACE && stack_el->info->index != APPENDIX) {
+		       g_print (".&nbsp;&nbsp;");
+		}
+		
+		/* Only print the link if we are the chapter tag, or the sect1 tag
+		 * (and the document is an article) or preface or appendix */
+		print_link = (((stack_el->info->index == SECT1) && (context->doctype != BOOK_DOC)
+				&& (context->appendix == 0))
+				|| stack_el->info->index == CHAPTER
+		                || stack_el->info->index == PREFACE
+				|| stack_el->info->index == APPENDIX);
+
+		if (print_link) {
+			g_print ("<A href=\"gnome-help:%s", context->base_file);
+
+			atrs_ptr = (stack_el->atrs);
+			while (atrs_ptr && *atrs_ptr) {
+				if (!g_strcasecmp (*atrs_ptr, "id")) {
+					atrs_ptr++;
+					g_print ("?%s", *atrs_ptr);
+					break;
+				}
+				atrs_ptr += 2;
+			}
+			g_print ("\">");
+		}
+		break;
 	case SECT2:
 	case SECT3:
 	case SECT4:
 	case SECT5:
-	case SECTION:
-		if (context->sect2 == 0)
-			g_print ("<H2>");
-		else
-			g_print ("<H3>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
-		if (context->sect1 > 0) g_print ("%d", context->sect1);
+		if (context->sect2 == 0) {
+			g_print ("<DT>");
+		} else if (context->sect3 == 0) {
+			g_print ("<DT>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
+		} else {
+			g_print ("<DT>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
+		}
+		if (context->chapter > 0) {
+			g_print ("%d", context->chapter);
+			if (context->sect1 > 0) g_print (".%d", context->sect1);
+		} else {
+			if (context->sect1 > 0) g_print ("%d", context->sect1);
+		}
 		if (context->sect2 > 0) g_print (".%d", context->sect2);
 		if (context->sect3 > 0) g_print (".%d", context->sect3);
 		if (context->sect4 > 0) g_print (".%d", context->sect4);
 		if (context->sect5 > 0) g_print (".%d", context->sect5);
 		g_print (".&nbsp;&nbsp;");
-		g_print ("<A href=\"help:%s", context->base_file);
-
-		atrs_ptr = (stack_el->atrs);
-		while (atrs_ptr && *atrs_ptr) {
-			if (!g_strcasecmp (*atrs_ptr, "id")) {
-				atrs_ptr++;
-				g_print ("?%s", *atrs_ptr);
-				break;
-			}
-			atrs_ptr += 2;
-		}
-		g_print ("\">");
-		break;
 	default:
 		break;
 	};
@@ -390,11 +546,14 @@ toc_title_end_element (Context *context,
 	GSList *element_list = NULL;
 	ElementIndex index;
 
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (PREFACE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (CHAPTER));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT1));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT2));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT3));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT4));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT5));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (APPENDIX));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECTION));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FIGURE));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (FORMALPARA));
@@ -403,19 +562,34 @@ toc_title_end_element (Context *context,
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (WARNING));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (GRAPHIC));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (CAUTION));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (TABLE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (IMPORTANT));
 
 	index = find_first_parent (context, element_list);
 
 	switch (index) {
+	case PREFACE:
+	case APPENDIX:
+		g_print ("</A></DT>");
+		break;
+	case CHAPTER:
+		g_print ("</A></DT>");
+		break;
 	case SECT1:
 	case SECTION:
-		g_print ("</A></H2>\n");
+		if (context->doctype == ARTICLE_DOC) {
+			g_print ("</A></DT>\n");
+		} else {
+			g_print ("</DT>");
+		}
 		break;
 	case SECT2:
+		g_print ("</DT>\n");
+		break;
 	case SECT3:
 	case SECT4:
 	case SECT5:
-		g_print ("</A></H3>\n");
+		g_print ("</DT>\n");
 		break;
 	default:
 		break;
@@ -431,11 +605,14 @@ toc_title_characters (Context *context, const gchar *chars, int len)
 	ElementIndex index;
 	char *temp;
 
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (PREFACE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (CHAPTER));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT1));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT2));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT3));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT4));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECT5));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (APPENDIX));	
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (SECTION));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (ARTHEADER));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (BOOKINFO));
@@ -446,18 +623,23 @@ toc_title_characters (Context *context, const gchar *chars, int len)
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (WARNING));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (GRAPHIC));
 	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (CAUTION));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (TABLE));
+	element_list = g_slist_prepend (element_list, GINT_TO_POINTER (IMPORTANT));
 
 	index = find_first_parent (context, element_list);
 
 	temp = g_strndup (chars, len);
 
 	switch (index) {
+	case PREFACE:
+	case CHAPTER:
 	case SECT1:
 	case SECT2:
 	case SECT3:
 	case SECT4:
 	case SECT5:
 	case SECTION:
+	case APPENDIX:
 		g_print (temp);
 		g_free (temp);
 		break;
