@@ -42,6 +42,7 @@ enum {
 	UNINSTALL_PROGRESS,
 
 	DOWNLOAD_FAILED,
+	MD5_CHECK_FAILED,
 	INSTALL_FAILED,
 	UNINSTALL_FAILED,
 
@@ -76,13 +77,37 @@ impl_download_progress (impl_POA_Trilobite_Eazel_InstallCallback *servant,
 	gtk_signal_emit (GTK_OBJECT (servant->object), signals[DOWNLOAD_PROGRESS], name, amount, total);
 }
 
-static void
+static CORBA_boolean
 impl_preflight_check (impl_POA_Trilobite_Eazel_InstallCallback *servant,
+		      const CORBA_char *xmlcorbapacks,
 		      const CORBA_long total_bytes,
 		      const CORBA_long total_packages,
 		      CORBA_Environment * ev)
 {
-	gtk_signal_emit (GTK_OBJECT (servant->object), signals[PREFLIGHT_CHECK], total_bytes, total_packages);
+	GList *categories;
+	gboolean result;
+	
+	categories = parse_memory_xml_package_list ((char*)xmlcorbapacks, strlen (xmlcorbapacks));
+	if (categories==NULL) {
+		g_warning ("install_failed called with error in xml.");
+		g_warning ("XML is = \n%s", xmlcorbapacks);
+	} else {
+		CategoryData *cat;
+		cat = (CategoryData*)categories->data;
+		if (cat->packages==NULL) {
+			g_warning ("install_failed called with error in xml.");
+			g_warning ("XML is = \n%s", xmlcorbapacks);
+		} else {
+			gtk_signal_emit (GTK_OBJECT (servant->object), signals[PREFLIGHT_CHECK], 
+					 cat->packages,
+					 total_bytes, 
+					 total_packages,
+					 &result);
+		}
+	}
+	g_list_foreach (categories, (GFunc)categorydata_destroy_foreach, NULL);
+
+	return result ? CORBA_TRUE : CORBA_FALSE;
 }
 
 static void
@@ -103,6 +128,8 @@ impl_dep_check (impl_POA_Trilobite_Eazel_InstallCallback *servant,
 	pack = packagedata_from_corba_packagedatastruct (*corbapack);
 	needs = packagedata_from_corba_packagedatastruct (*corbaneeds);
 	gtk_signal_emit (GTK_OBJECT (servant->object), signals[DEPENDENCY_CHECK], pack, needs);
+	packagedata_destroy (pack, TRUE);
+	packagedata_destroy (needs, TRUE);
 }
 
 static void 
@@ -120,6 +147,7 @@ impl_install_progress (impl_POA_Trilobite_Eazel_InstallCallback *servant,
 			 package_num, num_packages,
 			 package_size_completed, package_size_total,
 			 total_size_completed, total_size);
+	packagedata_destroy (pack, TRUE);
 }
 
 static void 
@@ -132,6 +160,20 @@ impl_uninstall_progress (impl_POA_Trilobite_Eazel_InstallCallback *servant,
 	PackageData *pack;
 	pack = packagedata_from_corba_packagedatastruct (*corbapack);
 	gtk_signal_emit (GTK_OBJECT (servant->object), signals[UNINSTALL_PROGRESS], pack, amount, total);
+	packagedata_destroy (pack, TRUE);
+}
+
+static void 
+impl_md5_check_failed (impl_POA_Trilobite_Eazel_InstallCallback *servant,
+		       const Trilobite_Eazel_PackageDataStruct *corbapack,
+		       const CORBA_char *actual_md5,
+		       CORBA_Environment * ev)
+{
+	PackageData *pack;
+	g_message ("impl_md5_check_failed (..., %s)", actual_md5);
+	pack = packagedata_from_corba_packagedatastruct (*corbapack);
+	gtk_signal_emit (GTK_OBJECT (servant->object), signals[MD5_CHECK_FAILED], pack, actual_md5);
+	packagedata_destroy (pack, TRUE);
 }
 
 static void 
@@ -140,7 +182,7 @@ impl_install_failed (impl_POA_Trilobite_Eazel_InstallCallback *servant,
 		     CORBA_Environment * ev)
 {
 	GList *categories;
-	PackageData *pack;
+
 	categories = parse_memory_xml_package_list ((char*)xmlcorbapack, strlen (xmlcorbapack));
 	if (categories==NULL) {
 		g_warning ("install_failed called with error in xml.");
@@ -157,6 +199,7 @@ impl_install_failed (impl_POA_Trilobite_Eazel_InstallCallback *servant,
 			gtk_signal_emit (GTK_OBJECT (servant->object), signals[INSTALL_FAILED], pack);
 		}
 	}
+	g_list_foreach (categories, (GFunc)categorydata_destroy_foreach, NULL);
 }
 
 static void 
@@ -165,23 +208,24 @@ impl_uninstall_failed (impl_POA_Trilobite_Eazel_InstallCallback *servant,
 		       CORBA_Environment * ev)
 {
 	GList *categories;
-	PackageData *pack;
+
 	categories = parse_memory_xml_package_list ((char*)xmlcorbapack, strlen (xmlcorbapack));
 	if (categories==NULL) {
-		g_warning ("install_failed called with error in xml.");
+		g_warning ("uninstall_failed called with error in xml.");
 		g_warning ("XML is = \n%s", xmlcorbapack);
 	} else {
 		CategoryData *cat;
 		cat = (CategoryData*)categories->data;
 		if (cat->packages==NULL) {
-			g_warning ("install_failed called with error in xml.");
+			g_warning ("uninstall_failed called with error in xml.");
 			g_warning ("XML is = \n%s", xmlcorbapack);
 		} else {
 			PackageData *pack;
 			pack = (PackageData*)cat->packages->data;
-			gtk_signal_emit (GTK_OBJECT (servant->object), signals[INSTALL_FAILED], pack);
+			gtk_signal_emit (GTK_OBJECT (servant->object), signals[UNINSTALL_FAILED], pack);
 		}
 	}
+	g_list_foreach (categories, (GFunc)categorydata_destroy_foreach, NULL);
 }
 
 static CORBA_boolean
@@ -191,14 +235,15 @@ impl_delete_files (impl_POA_Trilobite_Eazel_InstallCallback *servant,
 	gboolean result;
 
 	gtk_signal_emit (GTK_OBJECT (servant->object), signals[DELETE_FILES], &result);
-	return (CORBA_boolean)result;
+	return result ? CORBA_TRUE : CORBA_FALSE;
 }
 
 static void 
 impl_done (impl_POA_Trilobite_Eazel_InstallCallback *servant, 
+	   CORBA_boolean result,
 	   CORBA_Environment * ev)
 {
-	gtk_signal_emit (GTK_OBJECT (servant->object), signals[DONE]);
+	gtk_signal_emit (GTK_OBJECT (servant->object), signals[DONE], result);
 }
 
 POA_Trilobite_Eazel_InstallCallback__epv* 
@@ -212,6 +257,7 @@ eazel_install_callback_get_epv ()
 	epv->dependency_check    = (gpointer)&impl_dep_check;
 	epv->install_progress    = (gpointer)&impl_install_progress;
 	epv->uninstall_progress  = (gpointer)&impl_uninstall_progress;
+	epv->md5_check_failed    = (gpointer)&impl_md5_check_failed;
 	epv->install_failed      = (gpointer)&impl_install_failed;
 	epv->download_failed     = (gpointer)&impl_download_failed;
 	epv->uninstall_failed    = (gpointer)&impl_uninstall_failed;
@@ -258,15 +304,22 @@ eazel_install_callback_create_corba_object (BonoboObject *service) {
 *****************************************/
 
 void
-eazel_install_callback_destroy (GtkObject *object)
+eazel_install_callback_unref (GtkObject *object)
+{
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (EAZEL_INSTALL_CALLBACK (object));
+	
+	bonobo_object_unref (BONOBO_OBJECT (object));
+}
+
+static void
+eazel_install_callback_finalize (GtkObject *object)
 {
 	EazelInstallCallback *service;
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (EAZEL_INSTALL_CALLBACK (object));
 	
-	g_message ("in eazel_install_callback_destroy");
-
 	service = EAZEL_INSTALL_CALLBACK (object);
 
 	if (service->installservice_corba != CORBA_OBJECT_NIL) {
@@ -284,7 +337,6 @@ eazel_install_callback_destroy (GtkObject *object)
 	if (GTK_OBJECT_CLASS (eazel_install_callback_parent_class)->destroy) {
 		GTK_OBJECT_CLASS (eazel_install_callback_parent_class)->destroy (object);
 	}
-	g_message ("out eazel_install_callback_destroy");
 }
 
 static void
@@ -293,7 +345,7 @@ eazel_install_callback_class_initialize (EazelInstallCallbackClass *klass)
 	GtkObjectClass *object_class;
 
 	object_class = (GtkObjectClass*)klass;
-	object_class->destroy = (void(*)(GtkObject*))eazel_install_callback_destroy;
+	object_class->finalize = eazel_install_callback_finalize;
 
 	eazel_install_callback_parent_class = gtk_type_class (bonobo_object_get_type ());
 	klass->servant_vepv = g_new0 (POA_Trilobite_Eazel_InstallCallback__vepv,1);
@@ -314,8 +366,8 @@ eazel_install_callback_class_initialize (EazelInstallCallbackClass *klass)
 				GTK_RUN_LAST,
 				object_class->type,
 				GTK_SIGNAL_OFFSET (EazelInstallCallbackClass, preflight_check),
-				gtk_marshal_NONE__INT_INT,
-				GTK_TYPE_NONE, 2, GTK_TYPE_INT, GTK_TYPE_INT);
+				gtk_marshal_BOOL__POINTER_INT_INT,
+				GTK_TYPE_BOOL, 3, GTK_TYPE_POINTER, GTK_TYPE_INT, GTK_TYPE_INT);
 	signals[INSTALL_PROGRESS] = 
 		gtk_signal_new ("install_progress",
 				GTK_RUN_LAST,
@@ -337,6 +389,13 @@ eazel_install_callback_class_initialize (EazelInstallCallbackClass *klass)
 				GTK_SIGNAL_OFFSET (EazelInstallCallbackClass, download_failed),
 				gtk_marshal_NONE__POINTER,
 				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+	signals[MD5_CHECK_FAILED] = 
+		gtk_signal_new ("md5_check_failed",
+				GTK_RUN_LAST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (EazelInstallCallbackClass, md5_check_failed),
+				gtk_marshal_NONE__POINTER_POINTER,
+				GTK_TYPE_NONE, 2, GTK_TYPE_POINTER, GTK_TYPE_POINTER);
 	signals[INSTALL_FAILED] = 
 		gtk_signal_new ("install_failed",
 				GTK_RUN_LAST,
@@ -370,8 +429,8 @@ eazel_install_callback_class_initialize (EazelInstallCallbackClass *klass)
 				GTK_RUN_LAST,
 				object_class->type,
 				GTK_SIGNAL_OFFSET (EazelInstallCallbackClass, done),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE, 0);
+				gtk_marshal_NONE__BOOL,
+				GTK_TYPE_NONE, 1, GTK_TYPE_BOOL);
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 }
 
@@ -407,7 +466,8 @@ eazel_install_callback_initialize (EazelInstallCallback *service) {
 }
 
 GtkType
-eazel_install_callback_get_type() {
+eazel_install_callback_get_type (void)
+{
 	static GtkType service_type = 0;
 
 	/* g_message ("into eazel_install_callback_get_type");  */
@@ -433,8 +493,8 @@ eazel_install_callback_get_type() {
 	return service_type;
 }
 
-EazelInstallCallback*
-eazel_install_callback_new()
+EazelInstallCallback *
+eazel_install_callback_new (void)
 {
 	EazelInstallCallback *service;
 
@@ -458,12 +518,14 @@ eazel_install_callback_bonobo (EazelInstallCallback *service)
 void 
 eazel_install_callback_install_packages (EazelInstallCallback *service, 
 					 GList *categories,
+					 const char *root,
 					 CORBA_Environment *ev)
 {
 	Trilobite_Eazel_CategoryStructList *corbacats;
 	corbacats = corba_category_list_from_categorydata_list (categories);
 	Trilobite_Eazel_Install_install_packages (service->installservice_corba, 
 						  corbacats, 
+						  root ? root : "",
 						  service->cb, 
 						  ev);
 }
@@ -471,12 +533,14 @@ eazel_install_callback_install_packages (EazelInstallCallback *service,
 void 
 eazel_install_callback_uninstall_packages (EazelInstallCallback *service, 
 					   GList *categories,
+					   const char *root,
 					   CORBA_Environment *ev)
 {
 	Trilobite_Eazel_CategoryStructList *corbacats;
 	corbacats = corba_category_list_from_categorydata_list (categories);
 	Trilobite_Eazel_Install_uninstall_packages (service->installservice_corba, 
 						    corbacats, 
+						    root ? root : "",
 						    service->cb, 
 						    ev);
 }
@@ -484,13 +548,15 @@ eazel_install_callback_uninstall_packages (EazelInstallCallback *service,
 GList*
 eazel_install_callback_simple_query (EazelInstallCallback *service, 
 				     const char *query,
+				     const char *root,
 				     CORBA_Environment *ev)
 {
 	GList *result;
 	Trilobite_Eazel_PackageDataStructList *corbares;
-	
+
 	corbares = Trilobite_Eazel_Install_simple_query (service->installservice_corba,
 							 query,
+							 root ? root : "",
 							 ev);
 	result = packagedata_list_from_corba_packagedatastructlist (*corbares);
 	CORBA_free (corbares); 
@@ -501,6 +567,7 @@ eazel_install_callback_simple_query (EazelInstallCallback *service,
 void 
 eazel_install_callback_revert_transaction (EazelInstallCallback *service, 
 					   const char *xmlfile,
+					   const char *root,
 					   CORBA_Environment *ev)
 {
 	xmlDocPtr doc;
@@ -513,9 +580,17 @@ eazel_install_callback_revert_transaction (EazelInstallCallback *service,
 	arg = CORBA_string_dup (mem);
 	Trilobite_Eazel_Install_revert_transaction (service->installservice_corba, 
 						    arg,
+						    root ? root : "",
 						    service->cb,
 						    ev);
 	CORBA_free (arg);
 	g_free (mem);
 	xmlFreeDoc (doc);
 }
+
+void
+eazel_install_callback_delete_files (EazelInstallCallback *service,
+				     CORBA_Environment *ev)
+{
+	Trilobite_Eazel_Install_delete_files (service->installservice_corba, ev);
+}
