@@ -40,7 +40,6 @@
 #include <libnautilus/nautilus-bonobo-ui.h>
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
 #include <libnautilus-extensions/nautilus-password-dialog.h>
-#include <glade/glade.h>
 
 #include <libtrilobite/libtrilobite.h>
 
@@ -49,8 +48,8 @@
  *
  */
 
-/* FIXME: no way to save the url once you change it */
-#define DEFAULT_SERVER_URL	"http://testmachine.eazel.com:8888/examples/time/current"
+/* FIXME bugzilla.eazel.com 2588: no way to save the url once you change it */
+#define DEFAULT_SERVER  "nist1.sjc.certifiedtime.com"
 #define DEFAULT_TIME_DIFF	"180"
 
 #define OAFIID_TIME_SERVICE	"OAFIID:trilobite_eazel_time_service:13a2dbd9-84f9-4400-bd9e-bb4575b86894"
@@ -59,12 +58,12 @@
 
 /* A NautilusContentView's private information. */
 struct TrilobiteEazelTimeViewDetails {
-	char *		server_url;
+	char *		server;
 	char *		max_time_diff;
 	NautilusView *	nautilus_view;
 	GtkEntry *	p_localtime;
 	GtkEntry *	p_servertime;
-	GtkEntry *	p_url;
+	GtkEntry *	p_server;
 	GtkEntry *	p_maxdiff;
 	GtkLabel *	p_status;
 	Trilobite_Eazel_Time service;
@@ -145,7 +144,7 @@ sync_button_pressed( GtkButton *p_button, TrilobiteEazelTimeView *view )
 
 
 static void
-url_button_pressed( GtkButton *p_button, TrilobiteEazelTimeView *view )
+server_button_pressed( GtkButton *p_button, TrilobiteEazelTimeView *view )
 {
 	CORBA_Environment ev;
 
@@ -155,12 +154,12 @@ url_button_pressed( GtkButton *p_button, TrilobiteEazelTimeView *view )
 
 	if (view->details->service) {
 
-		g_free (view->details->server_url);
-		view->details->server_url = gtk_editable_get_chars (GTK_EDITABLE (view->details->p_url), 0, -1);
+		g_free (view->details->server);
+		view->details->server = gtk_editable_get_chars (GTK_EDITABLE (view->details->p_server), 0, -1);
 			
-		Trilobite_Eazel_Time_set_time_url (view->details->service, view->details->server_url, &ev);
+		Trilobite_Eazel_Time_set_time_server (view->details->service, view->details->server, &ev);
 
-		set_status_text (view, "Server URL changed");
+		set_status_text (view, "Time server changed");
 
 	} else {
 		set_status_text (view, STATUS_ERROR_NO_SERVER);
@@ -180,8 +179,8 @@ timediff_button_pressed( GtkButton *p_button, TrilobiteEazelTimeView *view )
 
 	if (view->details->service) {
 
-		g_free (view->details->server_url);
-		view->details->server_url = gtk_editable_get_chars (GTK_EDITABLE (view->details->p_maxdiff), 0, -1);
+		g_free (view->details->server);
+		view->details->server = gtk_editable_get_chars (GTK_EDITABLE (view->details->p_maxdiff), 0, -1);
 			
 		Trilobite_Eazel_Time_set_max_difference (view->details->service, atoi (view->details->max_time_diff), &ev);
 
@@ -292,7 +291,7 @@ trilobite_eazel_time_view_initialize (TrilobiteEazelTimeView *view)
 	GtkWidget *p_hbox2;
 	GtkWidget *p_hbox3;
 	GtkWidget *p_sync;
-	GtkWidget *p_url_button;
+	GtkWidget *p_server_button;
 	GtkWidget *p_timediff_button;
 	BonoboObjectClient *p_service;
 
@@ -306,19 +305,19 @@ trilobite_eazel_time_view_initialize (TrilobiteEazelTimeView *view)
 
 	view->details->nautilus_view 	= nautilus_view_new (GTK_WIDGET (view));
 
-	view->details->server_url 	= g_strdup(DEFAULT_SERVER_URL);
+	view->details->server 	        = g_strdup(DEFAULT_SERVER);
 	view->details->max_time_diff 	= g_strdup(DEFAULT_TIME_DIFF);
 
 	view->details->p_localtime 	= GTK_ENTRY (gtk_entry_new());
 	view->details->p_servertime 	= GTK_ENTRY (gtk_entry_new());
-	view->details->p_url 		= GTK_ENTRY (gtk_entry_new());
+	view->details->p_server 		= GTK_ENTRY (gtk_entry_new());
 	view->details->p_maxdiff 	= GTK_ENTRY (gtk_entry_new());
 	view->details->p_status 	= GTK_LABEL (gtk_label_new("(no status)"));
 
 	gtk_entry_set_editable (GTK_ENTRY(view->details->p_localtime), FALSE);
 	gtk_entry_set_editable (GTK_ENTRY(view->details->p_servertime), FALSE);
 
-	gtk_entry_set_text (GTK_ENTRY(view->details->p_url), view->details->server_url);
+	gtk_entry_set_text (GTK_ENTRY(view->details->p_server), view->details->server);
 	gtk_entry_set_text (GTK_ENTRY(view->details->p_maxdiff), view->details->max_time_diff);
 
 	p_service	= bonobo_object_activate (OAFIID_TIME_SERVICE, 0);
@@ -334,7 +333,7 @@ trilobite_eazel_time_view_initialize (TrilobiteEazelTimeView *view)
 	/* Initialize service params */
 
 	if (view->details->service) {
-		Trilobite_Eazel_Time_set_time_url (view->details->service, view->details->server_url, &ev);
+		Trilobite_Eazel_Time_set_time_server (view->details->service, view->details->server, &ev);
 		Trilobite_Eazel_Time_set_max_difference (view->details->service, atoi (view->details->max_time_diff), &ev);
 	}
 
@@ -364,14 +363,14 @@ trilobite_eazel_time_view_initialize (TrilobiteEazelTimeView *view)
 
 	gtk_box_pack_start (GTK_BOX (p_vbox), GTK_WIDGET (p_sync), FALSE, FALSE, 0);
 
-	/* Time URL */
+	/* Time SERVER */
 
 	p_hbox2 = gtk_hbox_new (FALSE,0);
-	gtk_box_pack_start (GTK_BOX(p_hbox2), gtk_label_new ("Server URL"), FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX(p_hbox2), GTK_WIDGET (view->details->p_url), FALSE, FALSE, 0);
-	p_url_button = gtk_button_new_with_label ("Set");
-	gtk_signal_connect (GTK_OBJECT (p_url_button), "pressed", GTK_SIGNAL_FUNC (url_button_pressed), view);
-	gtk_box_pack_start (GTK_BOX(p_hbox2), GTK_WIDGET (p_url_button), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX(p_hbox2), gtk_label_new ("Time server"), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX(p_hbox2), GTK_WIDGET (view->details->p_server), FALSE, FALSE, 0);
+	p_server_button = gtk_button_new_with_label ("Set");
+	gtk_signal_connect (GTK_OBJECT (p_server_button), "pressed", GTK_SIGNAL_FUNC (server_button_pressed), view);
+	gtk_box_pack_start (GTK_BOX(p_hbox2), GTK_WIDGET (p_server_button), FALSE, FALSE, 0);
 
 	gtk_box_pack_start (GTK_BOX (p_vbox), GTK_WIDGET (p_hbox2), FALSE, FALSE, 0);
 
@@ -408,19 +407,6 @@ trilobite_eazel_time_view_initialize (TrilobiteEazelTimeView *view)
 		p_service = NULL;
 	}
 
-#if 0
-	/* 
-	 * Get notified when our bonobo control is activated so we
-	 * can merge menu & toolbar items into Nautilus's UI.
-	 */
-        gtk_signal_connect (GTK_OBJECT (nautilus_view_get_bonobo_control
-					(view->details->nautilus_view)),
-                            "activate",
-                            sample_merge_bonobo_items_callback,
-                            view);
-	
-#endif /* 0 */
-
 	update_time_display (view);
 
 	view->details->timeout_slot = gtk_timeout_add( 1000, (GtkFunction)update_time_display, view );
@@ -441,7 +427,7 @@ trilobite_eazel_time_view_destroy (GtkObject *object)
 	
 	view = TRILOBITE_EAZEL_TIME_VIEW (object);
 	
-	g_free (view->details->server_url);
+	g_free (view->details->server);
 	g_free (view->details->max_time_diff);
 	g_free (view->details->remembered_password);
 
@@ -483,16 +469,6 @@ void
 trilobite_eazel_time_view_load_uri (TrilobiteEazelTimeView *view,
 				       const char               *uri)
 {
-#if 0
-	char *label_text;
-	
-	g_free (view->details->uri);
-	view->details->uri = g_strdup (uri);
-
-	label_text = g_strdup_printf (_("%s\n\nThis is a sample Nautilus content view component."), uri);
-	gtk_label_set_text (GTK_LABEL (view), label_text);
-	g_free (label_text);
-#endif /* 0 */
 }
 
 static void
@@ -506,110 +482,3 @@ load_location_callback (NautilusView *nautilus_view,
 	trilobite_eazel_time_view_load_uri (view, location);
 	nautilus_view_report_load_complete (nautilus_view);
 }
-
-#if 0
-static void
-bonobo_sample_callback (BonoboUIHandler *ui_handler, gpointer user_data, const char *path)
-{
- 	TrilobiteEazelTimeView *view;
-#if 0
-	char *label_text;
-#endif /* 0 */
-
-        g_assert (TRILOBITE_IS_EAZEL_TIME_VIEW (user_data));
-
-	view = TRILOBITE_EAZEL_TIME_VIEW (user_data);
-
-#if 0
-	if (strcmp (path, "/File/Sample") == 0) {
-		label_text = g_strdup_printf ("%s\n\nYou selected the Sample menu item.", view->details->uri);
-	} else {
-		g_assert (strcmp (path, "/Main/Sample") == 0);
-		label_text = g_strdup_printf (_("%s\n\nYou clicked the Sample toolbar button."), view->details->uri);
-	}
-	
-	gtk_label_set_text (GTK_LABEL (view), label_text);
-	g_free (label_text);
-#endif /* 0 */
-}
-#endif /* 0 */
-
-#if 0
-
-static void
-sample_merge_bonobo_items_callback (BonoboObject *control, gboolean state, gpointer user_data)
-{
- 	TrilobiteEazelTimeView *view;
-	BonoboUIHandler *local_ui_handler;
-	GdkPixbuf		*pixbuf;
-	BonoboUIHandlerPixmapType pixmap_type;
-	char *path;
-	
-	g_assert (TRILOBITE_IS_EAZEL_TIME_VIEW (user_data));
-
-	view = TRILOBITE_EAZEL_TIME_VIEW (user_data);
-	local_ui_handler = bonobo_control_get_ui_handler (BONOBO_CONTROL (control));
-
-	if (state) {
-		/* Tell the Nautilus window to merge our bonobo_ui_handler items with its ones */
-		bonobo_ui_handler_set_container (local_ui_handler, 
-                                                 bonobo_control_get_remote_ui_handler (BONOBO_CONTROL (control)));
-
-		/* Load test pixbuf */
-		pixbuf = gdk_pixbuf_new_from_file ("/gnome/share/pixmaps/nautilus/i-directory-24.png");		
-		if (pixbuf != NULL)
-			pixmap_type = BONOBO_UI_HANDLER_PIXMAP_PIXBUF_DATA;
-		else
-			pixmap_type = BONOBO_UI_HANDLER_PIXMAP_NONE;
-
-	       /* Create our sample menu item. */ 
-		path = bonobo_ui_handler_build_path (NULL,
-						     "File", 
-						     "Sample",
-						     NULL);
-		bonobo_ui_handler_menu_new_item 
-			(local_ui_handler,					/* BonoboUIHandler */
-	        	 path,							/* menu item path, must start with /some-existing-menu-path and be otherwise unique */
-	                 _("_Sample"),						/* menu item user-displayed label */
-	                 _("This is a sample merged menu item"),		/* hint that appears in status bar */
-	                 bonobo_ui_handler_menu_get_pos 
-	                 	(local_ui_handler, 
-	                         NAUTILUS_MENU_PATH_NEW_WINDOW_ITEM) + 1, 	/* position within menu; -1 means last */
-	                 pixmap_type,						/* pixmap type */
-	                 pixbuf,						/* pixmap data */
-	                 'M',							/* accelerator key, couldn't bear the thought of using Control-S for anything except Save */
-	                 GDK_CONTROL_MASK,					/* accelerator key modifiers */
-	                 bonobo_sample_callback,				/* callback function */
-	                 view);                					/* callback function data */
-		g_free (path);
-
-                /* Create our sample toolbar button. */ 
-                path = bonobo_ui_handler_build_path (NAUTILUS_TOOLBAR_PATH_MAIN_TOOLBAR, 
-                				     "Sample",
-                				     NULL);
-	        bonobo_ui_handler_toolbar_new_item 
-	        	(local_ui_handler,				/* BonoboUIHandler */
-	        	 path,						/* button path, must start with /some-existing-toolbar-path and be otherwise unique */
-	        	 _("Sample"),					/* button user-displayed label */
-	        	 _("This is a sample merged toolbar button"),	/* hint that appears in status bar */
-	        	 -1,						/* position, -1 means last */
-	        	 pixmap_type,					/* pixmap type */
-	        	 pixbuf,					/* pixmap data */
-	        	 0,						/* accelerator key */
-	        	 0,						/* accelerator key modifiers */
-	        	 bonobo_sample_callback,			/* callback function */
-	        	 view);						/* callback function's data */
-	        g_free (path);
-	} else {
-		/* Do nothing. */
-	}
-
-        /* 
-         * Note that we do nothing if state is FALSE. Nautilus content views are activated
-         * when installed, but never explicitly deactivated. When the view changes to another,
-         * the content view object is destroyed, which ends up calling bonobo_ui_handler_unset_container,
-         * which removes its merged menu & toolbar items.
-         */
-}
-
-#endif /* 0 */
