@@ -35,7 +35,8 @@
 #include <gnome-xml/tree.h>
 #include <gnome-xml/parser.h>
 
-char* xml_get_value (xmlNode* node, const char* name);
+const char* xml_get_value (xmlNode* node, const char* name);
+
 gboolean check_for_root_user (void);
 gboolean check_for_redhat (void);
 
