@@ -25,12 +25,14 @@
 
 #include <config.h>
 #include "nautilus-tree-model.h"
+
 #include "nautilus-tree-node-private.h"
-#include <libnautilus-extensions/nautilus-gtk-macros.h>
 #include <gtk/gtksignal.h>
 #include <libgnomevfs/gnome-vfs.h>
-
+#include <libnautilus-extensions/nautilus-file-attributes.h>
+#include <libnautilus-extensions/nautilus-gtk-macros.h>
 #include <stdio.h>
+#include <string.h>
 
 enum {
 	NODE_ADDED,
@@ -44,7 +46,7 @@ static guint signals[LAST_SIGNAL];
 
 
 struct NautilusTreeModelDetails {
-	GHashTable       *uri_to_node_map;
+	GHashTable       *file_to_node_map;
 
 	GList            *monitor_clients;
 
@@ -56,45 +58,39 @@ struct NautilusTreeModelDetails {
 
 
 
-static void               nautilus_tree_model_destroy          (GtkObject   *object);
-static void               nautilus_tree_model_initialize       (gpointer     object,
-								gpointer     klass);
-static void               nautilus_tree_model_initialize_class (gpointer     klass);
-
-NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusTreeModel, nautilus_tree_model, GTK_TYPE_OBJECT)
-
-
-
-static void nautilus_tree_model_set_root_uri                   (NautilusTreeModel *model,
-								const char        *root_uri);
-static char *uri_get_parent_text                               (const char        *uri_text);
-static void  report_root_node_if_possible                      (NautilusTreeModel *model);
-static void  report_node_added                                 (NautilusTreeModel *model,
-								NautilusTreeNode  *node);
-static void  report_node_changed                               (NautilusTreeModel *model,
-								NautilusTreeNode  *node);
-static void  report_node_removed                               (NautilusTreeModel *model,
-								NautilusTreeNode  *node);
-static void  report_done_loading                               (NautilusTreeModel *model,
-								NautilusTreeNode  *node);
-
-static void  nautilus_tree_stop_monitoring_internal            (NautilusTreeModel *model);
+static void nautilus_tree_model_destroy                          (GtkObject         *object);
+static void nautilus_tree_model_initialize                       (gpointer           object,
+								  gpointer           klass);
+static void nautilus_tree_model_initialize_class                 (gpointer           klass);
+static void remove_all_nodes                                     (NautilusTreeModel *model);
+static void nautilus_tree_model_set_root_uri                     (NautilusTreeModel *model,
+								  const char        *root_uri);
+static void report_root_node_if_possible                         (NautilusTreeModel *model);
+static void report_node_changed                                  (NautilusTreeModel *model,
+								  NautilusTreeNode  *node);
+static void report_node_removed_internal                         (NautilusTreeModel *model,
+								  NautilusTreeNode  *node,
+								  gboolean           signal);
+static void report_node_removed                                  (NautilusTreeModel *model,
+								  NautilusTreeNode  *node);
+static void report_done_loading                                  (NautilusTreeModel *model,
+								  NautilusTreeNode  *node);
 
 
 /* signal/monitoring callbacks */
-static void  nautilus_tree_model_root_node_file_monitor          (NautilusFile      *file,
+static void nautilus_tree_model_root_node_file_monitor           (NautilusFile      *file,
+								  NautilusTreeModel *model);
+static void nautilus_tree_model_directory_files_changed_callback (NautilusDirectory *directory,
+								  GList             *added_files,
+								  NautilusTreeModel *model);
+static void nautilus_tree_model_directory_files_added_callback   (NautilusDirectory *directory,
+								  GList             *added_files,
+								  NautilusTreeModel *model);
+static void nautilus_tree_model_directory_done_loading_callback  (NautilusDirectory *directory,
 								  NautilusTreeModel *model);
-static void nautilus_tree_model_directory_files_changed_callback (NautilusDirectory        *directory,
-								  GList                    *added_files,
-								  NautilusTreeModel        *model);
 
-static void nautilus_tree_model_directory_files_added_callback   (NautilusDirectory        *directory,
-								  GList                    *added_files,
-								  NautilusTreeModel        *model);
-
-static void nautilus_tree_model_directory_done_loading_callback  (NautilusDirectory        *directory,
-								  NautilusTreeModel        *model);
 
+NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusTreeModel, nautilus_tree_model, GTK_TYPE_OBJECT)
 
 /* infrastructure stuff */
 
@@ -152,29 +148,27 @@ nautilus_tree_model_initialize (gpointer object, gpointer klass)
 
 	model->details = g_new0 (NautilusTreeModelDetails, 1);
 
-	model->details->uri_to_node_map = g_hash_table_new (g_str_hash, 
-							    g_str_equal);
+	model->details->file_to_node_map = g_hash_table_new (g_direct_hash, 
+							     g_direct_equal);
 }
 
-static void       
+static void
 nautilus_tree_model_destroy (GtkObject *object)
 {
 	NautilusTreeModel *model;
 
-	model = (NautilusTreeModel *) object;
+	model = NAUTILUS_TREE_MODEL (object);
 
-	/* FIXME: Free all the key strings and unref all the value nodes in
-           the uri_to_node_map */
-
-	g_hash_table_destroy (model->details->uri_to_node_map);
-
-	nautilus_tree_stop_monitoring_internal (model);
-
-	if (model->details->root_node != NULL) {
-		gtk_object_unref (GTK_OBJECT (model->details->root_node));
+	if (model->details->root_node_changed_signal_id != 0) {
+		gtk_signal_disconnect (GTK_OBJECT (nautilus_tree_node_get_file (model->details->root_node)),
+				       model->details->root_node_changed_signal_id);
 	}
 
-	g_free (model->details->monitor_clients);
+	remove_all_nodes (model);
+
+	g_list_free (model->details->monitor_clients);
+
+	g_hash_table_destroy (model->details->file_to_node_map);
 
 	g_free (model->details);
 	
@@ -189,7 +183,9 @@ nautilus_tree_model_new (const char *root_uri)
 {
 	NautilusTreeModel *model;
 
-	model = NAUTILUS_TREE_MODEL (gtk_type_new (NAUTILUS_TYPE_TREE_MODEL));
+	model = NAUTILUS_TREE_MODEL (gtk_object_new (NAUTILUS_TYPE_TREE_MODEL, NULL));
+	gtk_object_ref (GTK_OBJECT (model));
+	gtk_object_sink (GTK_OBJECT (model));
 
 	nautilus_tree_model_set_root_uri (model, root_uri);
 
@@ -212,6 +208,77 @@ nautilus_tree_model_set_root_uri (NautilusTreeModel *model,
 }
 
 
+static void
+nautilus_tree_model_unref_callback (NautilusTreeModel *model,
+				    NautilusTreeNode  *node,
+				    gpointer           callback_data)
+{
+	report_node_removed_internal (model, node, FALSE);
+}
+
+static void
+nautilus_tree_model_for_each_postorder (NautilusTreeModel  *model,
+					NautilusTreeModelCallback  callback,
+					gpointer                   callback_data)
+{
+	NautilusTreeNode *current_node;
+	GList *reporting_queue, *link;
+
+	if (model->details->root_node_reported) {
+		reporting_queue = g_list_prepend (NULL, model->details->root_node);
+		
+		while (reporting_queue != NULL) {
+			current_node = (NautilusTreeNode *) reporting_queue->data;
+
+			if (nautilus_tree_node_get_children (current_node) == NULL) {
+				NautilusDirectory *tmp;
+
+				link = reporting_queue;
+				reporting_queue = g_list_remove_link (reporting_queue, link);
+				g_list_free_1 (link);
+				
+				
+#ifdef DEBUG_TREE
+				printf ("XXX unrefing: %s\n", nautilus_tree_node_get_uri
+					(current_node));
+#endif
+
+				tmp = current_node->details->directory;
+
+				(*callback) (model, current_node, callback_data);
+				
+#ifdef DEBUG_TREE
+				printf ("XXX refs: %d\n", ((GtkObject *) tmp)->ref_count);
+#endif
+			} else {
+#ifdef DEBUG_TREE
+				printf ("XXX adding children of: %s\n", nautilus_tree_node_get_uri
+					(current_node));
+#endif
+				reporting_queue = g_list_concat (g_list_copy (nautilus_tree_node_get_children (current_node)),
+								 reporting_queue);
+			}
+#ifdef DEBUG_TREE
+			printf ("XXX queue length: %d\n", g_list_length (reporting_queue));
+#endif
+		}
+	}
+}
+
+static void
+remove_all_nodes (NautilusTreeModel *model)
+{
+	nautilus_tree_model_for_each_postorder (model,
+						nautilus_tree_model_unref_callback,
+						NULL);
+
+	if (model->details->root_node != NULL) {
+		gtk_object_unref (GTK_OBJECT (model->details->root_node));
+		model->details->root_node = NULL;
+	}
+	model->details->root_node_reported = FALSE;
+}
+
 void
 nautilus_tree_model_monitor_add (NautilusTreeModel         *model,
 				 gconstpointer              client,
@@ -219,7 +286,7 @@ nautilus_tree_model_monitor_add (NautilusTreeModel         *model,
 				 gpointer                   callback_data)
 {
 	NautilusTreeNode *current_node;
-	GList *reporting_queue;
+	GList *reporting_queue, *link;
 	GList *monitor_attributes;
 	
 	reporting_queue = NULL;
@@ -228,7 +295,7 @@ nautilus_tree_model_monitor_add (NautilusTreeModel         *model,
            to monitor the root node itself. */
 
 	if (model->details->monitor_clients == NULL) {
-		if (model->details->root_node_reported == FALSE) {
+		if (!model->details->root_node_reported) {
 			report_root_node_if_possible (model);
 		}
 
@@ -238,12 +305,11 @@ nautilus_tree_model_monitor_add (NautilusTreeModel         *model,
 			 nautilus_tree_model_root_node_file_monitor,
 			 model);
 		
-		monitor_attributes = g_list_prepend (NULL, "is directory");
+		monitor_attributes = g_list_prepend (NULL, NAUTILUS_FILE_ATTRIBUTE_IS_DIRECTORY);
 
 		nautilus_file_monitor_add (nautilus_tree_node_get_file (model->details->root_node),
 					   model,
-					   monitor_attributes,
-					   FALSE);
+					   monitor_attributes);
 
 		g_list_free (monitor_attributes);
 	}
@@ -257,7 +323,10 @@ nautilus_tree_model_monitor_add (NautilusTreeModel         *model,
 
 		while (reporting_queue != NULL) {
 			current_node = (NautilusTreeNode *) reporting_queue->data;
-			reporting_queue = g_list_remove_link (reporting_queue, reporting_queue);
+
+			link = reporting_queue;
+			reporting_queue = g_list_remove_link (reporting_queue, link);
+			g_list_free_1 (link);
 
 			(*initial_nodes_callback) (model, current_node, callback_data);
 
@@ -279,65 +348,99 @@ nautilus_tree_model_monitor_remove (NautilusTreeModel         *model,
 {
 	model->details->monitor_clients = g_list_remove (model->details->monitor_clients, (gpointer) client);
 
+	if (model->details->root_node_reported) {
+		nautilus_tree_model_stop_monitoring_node_recursive (model,
+								    model->details->root_node,
+								    client);
+	}
+
+
 	if (model->details->monitor_clients == NULL) {
-		/* FIXME: stop monitoring root node file, dunno what else */
+		if (model->details->root_node_reported) {
+			nautilus_file_monitor_remove
+				(nautilus_tree_node_get_file (model->details->root_node),
+				 model);
+		}
 	}
 }
 
 
+static gboolean
+nautilus_tree_model_node_has_monitor_clients (NautilusTreeModel         *model,
+					      NautilusTreeNode          *node)
+{
+	return (node->details->monitor_clients != NULL);
+}
+				 
+static void
+nautilus_tree_model_node_begin_monitoring (NautilusTreeModel         *model,
+					   NautilusTreeNode          *node)
+{
+	GList             *monitor_attributes;
+
+
+	NautilusDirectory *directory;
+	directory = nautilus_tree_node_get_directory (node);
+
+	/* we must connect to signals */
+	
+	node->details->files_added_id = gtk_signal_connect 
+		(GTK_OBJECT (directory),
+		 "files_added",
+		 nautilus_tree_model_directory_files_added_callback,
+		 model);
+	
+	node->details->files_changed_id = gtk_signal_connect 
+		(GTK_OBJECT (directory),
+		 "files_changed",
+		 nautilus_tree_model_directory_files_changed_callback,
+		 model);
+	
+	node->details->done_loading_id = gtk_signal_connect 
+		(GTK_OBJECT (directory),
+		 "done_loading",
+		 nautilus_tree_model_directory_done_loading_callback,
+		 model);
+
+	monitor_attributes = g_list_prepend (NULL, NAUTILUS_FILE_ATTRIBUTE_IS_DIRECTORY);
+	nautilus_directory_file_monitor_add (directory,
+					     model,
+					     monitor_attributes,
+					     TRUE);
+	g_list_free (monitor_attributes);
+}
+
+static void
+nautilus_tree_model_node_end_monitoring (NautilusTreeModel         *model,
+					 NautilusTreeNode          *node)
+{
+	gtk_signal_disconnect (GTK_OBJECT (node->details->directory), node->details->files_added_id);
+	gtk_signal_disconnect (GTK_OBJECT (node->details->directory), node->details->files_changed_id);
+	gtk_signal_disconnect (GTK_OBJECT (node->details->directory), node->details->done_loading_id);
+	
+	nautilus_directory_file_monitor_remove (node->details->directory,
+						model);
+}
+
+
+
 void
 nautilus_tree_model_monitor_node (NautilusTreeModel         *model,
 				  NautilusTreeNode          *node,
 				  gconstpointer              client)
 {
-	NautilusDirectory *directory;
-	GList             *monitor_attributes;
-
 	if (!nautilus_file_is_directory (nautilus_tree_node_get_file (node))) {
 		report_done_loading (model, node);
 		return;
 	}
 
-	if (node->details->monitor_clients == NULL) {
-		/* we must connect to signals */
-		directory = nautilus_tree_node_get_directory (node);
-		
-		node->details->files_added_id = gtk_signal_connect 
-			(GTK_OBJECT (directory),
-			 "files_added",
-			 nautilus_tree_model_directory_files_added_callback,
-			 model);
-		
-		node->details->files_changed_id = gtk_signal_connect 
-			(GTK_OBJECT (directory),
-			 "files_changed",
-			 nautilus_tree_model_directory_files_changed_callback,
-			 model);
-
-		node->details->done_loading_id = gtk_signal_connect 
-			(GTK_OBJECT (directory),
-			 "done_loading",
-			 nautilus_tree_model_directory_done_loading_callback,
-			 model);
+	if (! nautilus_tree_model_node_has_monitor_clients (model, node)) {
+		nautilus_tree_model_node_begin_monitoring (model, node);
 	}
 
-	monitor_attributes = g_list_prepend (NULL, "is directory");
-	
-	node->details->provisional_children = node->details->children;
-	node->details->children = NULL;
-
-	nautilus_directory_file_monitor_add (directory,
-					     model,
-					     monitor_attributes,
-					     FALSE,
-					     TRUE,
-					     (NautilusDirectoryCallback) nautilus_tree_model_directory_files_added_callback,
-					     model);
-	
-	g_list_free (monitor_attributes);
-
 	if (! g_list_find (node->details->monitor_clients, (gpointer) client)) {
-		node->details->monitor_clients = g_list_prepend (node->details->monitor_clients, (gpointer) client);
+		node->details->monitor_clients = g_list_prepend (node->details->monitor_clients, 
+								 (gpointer) client);
 	}
 }
 
@@ -351,15 +454,29 @@ nautilus_tree_model_stop_monitoring_node (NautilusTreeModel *model,
 		return;
 	}
 
+	if (g_list_find (node->details->monitor_clients, (gpointer) client) == NULL) {
+		return;
+	}
+
 	node->details->monitor_clients = g_list_remove (node->details->monitor_clients, (gpointer) client);
 
-	if (node->details->monitor_clients == NULL) {
-		gtk_signal_disconnect (GTK_OBJECT (node), node->details->files_added_id);
-		gtk_signal_disconnect (GTK_OBJECT (node), node->details->files_changed_id);
-		gtk_signal_disconnect (GTK_OBJECT (node), node->details->done_loading_id);
-		
-		nautilus_directory_file_monitor_remove (node->details->directory,
-							model);
+	if (!nautilus_tree_model_node_has_monitor_clients (model, node)) {
+		nautilus_tree_model_node_end_monitoring (model, node);
+	}
+}
+
+
+void
+nautilus_tree_model_stop_monitoring_node_recursive (NautilusTreeModel *model,
+						    NautilusTreeNode  *node,
+						    gconstpointer      client)
+{
+	GList *p;
+
+	nautilus_tree_model_stop_monitoring_node (model, node, client);
+
+	for (p = nautilus_tree_node_get_children (node); p != NULL; p = p->next) {
+		nautilus_tree_model_stop_monitoring_node_recursive (model, (NautilusTreeNode *) p->data, client);
 	}
 }
 
@@ -368,24 +485,60 @@ NautilusTreeNode *
 nautilus_tree_model_get_node (NautilusTreeModel *model,
 			      const char *uri)
 {
-	return g_hash_table_lookup (model->details->uri_to_node_map, uri);
+	NautilusFile *file;
+	NautilusTreeNode *node;
+
+	file = nautilus_file_get (uri);
+	
+	if (file == NULL) {
+		return NULL;
+	}
+
+	node = g_hash_table_lookup (model->details->file_to_node_map, file);
+	nautilus_file_unref (file);
+
+	return node;
 }
 
+/* Debugging functions to dump current contents of file_to_node_map */
 
-#if 0
+static void
+dump_one_file_node (gpointer key, gpointer value, gpointer user_data)
+{
+	guint *file_number;
+	char *uri;
 
-NautilusTreeNode  *nautilus_tree_model_get_nearest_parent_node  (NautilusTreeModel *model,
-								 const char *uri);
+	g_assert (NAUTILUS_IS_FILE (key));
+	g_assert (NAUTILUS_IS_TREE_NODE (value));
+	g_assert (user_data != NULL);
 
+	file_number = (guint *)user_data;
+	uri = nautilus_file_get_uri (NAUTILUS_FILE (key));
 
-NautilusTreeNode  *nautilus_tree_model_get_root_node            (NautilusTreeModel *model);
+	g_print ("%d: %s (0x%x)|\n", ++(*file_number), uri, (unsigned) key);
+	
+	g_free (uri);
+}
 
-#endif
+void
+nautilus_tree_model_dump_files (NautilusTreeModel *model)
+{
+	guint file_number;
+
+	file_number = 0;
+
+	g_print ("nautilus_tree_model_dump_files: %d files in tree view file_to_node_map hash table:\n", 
+		   g_hash_table_size (model->details->file_to_node_map));
+	g_hash_table_foreach (model->details->file_to_node_map, 
+			      dump_one_file_node, 
+			      &file_number);
+}
 
 
 /* helper functions */
 
-static char *uri_get_parent_text (const char *uri_text)
+static char *
+uri_get_parent_text (const char *uri_text)
 {
 	GnomeVFSURI *uri;
 	GnomeVFSURI *parent;
@@ -412,45 +565,7 @@ report_root_node_if_possible (NautilusTreeModel *model)
 	    != GNOME_VFS_FILE_TYPE_UNKNOWN) {
 		model->details->root_node_reported = TRUE;
 		
-		report_node_added (model, model->details->root_node);
-	}
-}
-
-
-static void
-report_node_added (NautilusTreeModel *model,
-		   NautilusTreeNode  *node)
-{
-	char *uri;
-	char *parent_uri;
-	NautilusTreeNode *parent_node;
-
-	uri = nautilus_file_get_uri (nautilus_tree_node_get_file (node));
-	
-	if (g_hash_table_lookup (model->details->uri_to_node_map, uri) == NULL) {
-		parent_uri = uri_get_parent_text (uri);
-
-		if (parent_uri != NULL) {
-			parent_node = g_hash_table_lookup (model->details->uri_to_node_map, parent_uri);
-			
-			if (parent_node != NULL) {
-				nautilus_tree_node_set_parent (node,
-							       parent_node);
-			}
-
-			g_free (parent_uri);
-		}
-
-
-		g_hash_table_insert (model->details->uri_to_node_map, 
-				     g_strdup (nautilus_file_get_uri (nautilus_tree_node_get_file (node))),
-				     node);
-
-		gtk_signal_emit (GTK_OBJECT (model),
-				 signals[NODE_ADDED],
-				 node);
-	} else {
-		/* FIXME: reconstruct internals and report that it changed?? */
+		report_node_changed (model, model->details->root_node);
 	}
 }
 
@@ -459,19 +574,31 @@ static void
 report_node_changed (NautilusTreeModel *model,
 		     NautilusTreeNode  *node)
 {
-	char *uri;
 	char *parent_uri;
 	NautilusTreeNode *parent_node;
+	char *node_uri;
+	char *file_uri;
 
-	uri = nautilus_file_get_uri (nautilus_tree_node_get_file (node));
+	node_uri = nautilus_tree_node_get_uri (node);
+
+	if (node->details->directory == NULL && nautilus_file_is_directory (node->details->file)) {
+		node->details->directory = nautilus_directory_get (node_uri);
 	
-	if (g_hash_table_lookup (model->details->uri_to_node_map, uri) == NULL) {
+#if 0
+		if (nautilus_tree_model_node_has_monitor_clients (model, node)) {
+			nautilus_tree_model_node_begin_monitoring (model, node);
+		}
+#endif
+	}
+
+	if (g_hash_table_lookup (model->details->file_to_node_map, 
+				 nautilus_tree_node_get_file (node)) == NULL) {
 		/* Actually added, go figure */
 		
-		parent_uri = uri_get_parent_text (uri);
+		parent_uri = uri_get_parent_text (node_uri);
 
 		if (parent_uri != NULL) {
-			parent_node = g_hash_table_lookup (model->details->uri_to_node_map, parent_uri);
+			parent_node = nautilus_tree_model_get_node (model, parent_uri);
 			
 			if (parent_node != NULL) {
 				nautilus_tree_node_set_parent (node,
@@ -481,9 +608,9 @@ report_node_changed (NautilusTreeModel *model,
 			g_free (parent_uri);
 		}
 
-
-		g_hash_table_insert (model->details->uri_to_node_map, 
-				     g_strdup (nautilus_file_get_uri (nautilus_tree_node_get_file (node))),
+		gtk_object_ref (GTK_OBJECT (node));
+		g_hash_table_insert (model->details->file_to_node_map, 
+				     nautilus_tree_node_get_file (node),
 				     node);
 
 		gtk_signal_emit (GTK_OBJECT (model),
@@ -491,64 +618,85 @@ report_node_changed (NautilusTreeModel *model,
 				 node);
 	} else {
 		/* really changed */
-		gtk_signal_emit (GTK_OBJECT (model),
-				 signals[NODE_CHANGED],
-				 node);
+
+		file_uri = nautilus_file_get_uri (nautilus_tree_node_get_file (node));
+
+		if (strcmp (file_uri, node_uri) == 0) {
+			/* A normal change */
+			gtk_signal_emit (GTK_OBJECT (model),
+					 signals[NODE_CHANGED],
+					 node);
+			g_free (file_uri);
+		} else {
+			/* A move or rename - model it as a remove followed by an add */
+
+			gtk_object_ref (GTK_OBJECT (node));
+
+			report_node_removed (model, node);
+			
+			g_free (node->details->uri);
+			node->details->uri = file_uri;
+			
+#if 0
+			if (node->details->directory != NULL) {
+				/* Stop monitoring the old directory */
+				if (nautilus_tree_model_node_has_monitor_clients (model, node)) {
+					nautilus_tree_model_node_end_monitoring (model, node);
+				}
+				nautilus_directory_unref (node->details->directory);
+				node->details->directory = NULL;
+			}
+#endif
+
+			report_node_changed (model, node);
+
+			gtk_object_unref (GTK_OBJECT (node));
+		}
 	}
 	
-	g_free (uri);
+	g_free (node_uri);
 }
 
 static void
-report_node_removed (NautilusTreeModel *model,
-		     NautilusTreeNode  *node)
+report_node_removed_internal (NautilusTreeModel *model,
+			      NautilusTreeNode  *node,
+			      gboolean signal)
 {
-	char *uri;
-	gpointer orig_key;
-	gpointer dummy;
-	char *parent_uri;
 	NautilusTreeNode *parent_node;
 
 	if (node == NULL) {
 		return;
 	}
 
-	uri = nautilus_file_get_uri (nautilus_tree_node_get_file (node));
-	
-	if (g_hash_table_lookup (model->details->uri_to_node_map, uri) != NULL) {
-		parent_uri = uri_get_parent_text (uri);
+	if (g_hash_table_lookup (model->details->file_to_node_map, 
+				 nautilus_tree_node_get_file (node)) != NULL) {
 
-		if (parent_uri != NULL) {
-			parent_node = g_hash_table_lookup (model->details->uri_to_node_map, parent_uri);
+		parent_node = nautilus_tree_node_get_parent (node);
 			
-			if (parent_node != NULL) {
-				nautilus_tree_remove_from_parent (node);
-			}
-
-			g_free (parent_uri);
+		if (parent_node != NULL) {
+			nautilus_tree_node_remove_from_parent (node);
 		}
 
+		g_hash_table_remove (model->details->file_to_node_map, 
+				     nautilus_tree_node_get_file (node));
+	
 
-		g_hash_table_lookup_extended (model->details->uri_to_node_map, 
-					      nautilus_file_get_uri (nautilus_tree_node_get_file (node)),
-					      &orig_key,
-					      &dummy);
-
-		g_hash_table_remove (model->details->uri_to_node_map, 
-				     nautilus_file_get_uri (nautilus_tree_node_get_file (node)));
-		
-		g_free (orig_key);
-				     
-		gtk_signal_emit (GTK_OBJECT (model),
-				 signals[NODE_REMOVED],
-				 node);
+		if (signal) {
+			gtk_signal_emit (GTK_OBJECT (model),
+					 signals[NODE_REMOVED],
+					 node);
+		}
 
 
 		gtk_object_unref (GTK_OBJECT (node));
 	} 
+}
 
-	g_free (uri);
-
+static void
+report_node_removed (NautilusTreeModel *model,
+		     NautilusTreeNode  *node)
+{
+	report_node_removed_internal (model, node, TRUE);
 }
 
 
@@ -562,21 +710,13 @@ report_done_loading (NautilusTreeModel *model,
 }
 
 
-static void
-nautilus_tree_stop_monitoring_internal (NautilusTreeModel *model)
-{
-	/* FIXME: stop monitoring everything; we should probably
-           forget everything in the hash table too. */
-
-}
-
 /* signal/monitoring callbacks */
 
 static void
 nautilus_tree_model_root_node_file_monitor (NautilusFile      *file,
 					    NautilusTreeModel *model)
 {
-	if (model->details->root_node_reported == FALSE) {
+	if (!model->details->root_node_reported) {
 		report_root_node_if_possible (model);
 	} else {
 		report_node_changed (model,
@@ -587,25 +727,29 @@ nautilus_tree_model_root_node_file_monitor (NautilusFile      *file,
 
 static void
 nautilus_tree_model_directory_files_changed_callback (NautilusDirectory        *directory,
-						      GList                    *added_files,
+						      GList                    *changed_files,
 						      NautilusTreeModel        *model)
 {
 	GList *p;
-	NautilusFile     *file;
+	NautilusFile *file;
 	NautilusTreeNode *node;
 	char *uri;
 
-	for (p = added_files; p != NULL; p = p->next) {
-		file = (NautilusFile *) p->data;
+	for (p = changed_files; p != NULL; p = p->next) {
+		file = NAUTILUS_FILE (p->data);
 		
-		uri = nautilus_file_get_uri (file);
-		node = nautilus_tree_model_get_node (model, uri);
-		g_free (uri);
+		node = (NautilusTreeNode *) g_hash_table_lookup (model->details->file_to_node_map, file);
 
-		if (!nautilus_directory_contains_file (directory, file)) {
-			report_node_removed (model, node);
-		} else {			
-			report_node_changed (model, node);
+		if (node == NULL) {
+			/* Do we need to add this node? */
+			uri = nautilus_file_get_uri (file);
+			g_free (uri);
+		} else {
+			if (!nautilus_directory_contains_file (directory, file)) {
+				report_node_removed (model, node);
+			} else {			
+				report_node_changed (model, node);
+			}
 		}
 	} 
 }
@@ -624,7 +768,9 @@ nautilus_tree_model_directory_files_added_callback (NautilusDirectory        *di
 		
 		node = nautilus_tree_node_new (file);
 		
-		report_node_added (model, node);
+		report_node_changed (model, node);
+
+		gtk_object_unref (GTK_OBJECT (node));
 	}
 }
 
