@@ -19,9 +19,13 @@
    write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 
-   Authors: Darin Adler <darin@eazel.com>, Pavel Cisler <pavel@eazel.com>
+   Authors: Darin Adler <darin@eazel.com>, 
+            Pavel Cisler <pavel@eazel.com>,
+            Ramiro Estrugo <ramiro@eazel.com>
 */
 
+#include <stdlib.h>
+
 #include <config.h>
 #include "nautilus-gdk-extensions.h"
 #include <gdk-pixbuf/gdk-pixbuf.h>
@@ -79,73 +83,6 @@ nautilus_fill_rectangle_with_color (GdkDrawable *drawable,
 	gdk_gc_set_foreground (gc, &saved_values.foreground);
 }
 
-/**
- * nautilus_fill_rectangle_with_gradient:
- * @drawable: Target to draw into.
- * @gc: Graphics context (mainly for clip).
- * @rectangle: Rectangle to draw gradient in.
- * @start_color: Color for the left or top; pixel value does not matter.
- * @end_color: Color for the right or bottom; pixel value does not matter.
- * @horizontal: TRUE if the color changes from left to right. FALSE if from top to bottom.
- *
- * Fill the rectangle with a gradient.
- * The color changes from start_color to end_color.
- * This effect works best on true color displays.
- */
-void
-nautilus_fill_rectangle_with_gradient (GdkDrawable *drawable,
-				       GdkGC *gc,
-				       const GdkRectangle *rectangle,
-				       guint32 start_rgb,
-				       guint32 end_rgb,
-				       gboolean horizontal)
-{
-	GdkRectangle band_box;
-	gint16 *position;
-	guint16 *size;
-	gint num_bands;
-	guint16 last_band_size;
-	gdouble multiplier;
-	gint band;
-	guint32 band_rgb;
-
-	g_return_if_fail (drawable != NULL);
-	g_return_if_fail (gc != NULL);
-	g_return_if_fail (rectangle != NULL);
-	g_return_if_fail (horizontal == FALSE || horizontal == TRUE);
-
-	/* Set up the band box so we can access it the same way for horizontal or vertical. */
-	band_box = *rectangle;
-	position = horizontal ? &band_box.x : &band_box.y;
-	size = horizontal ? &band_box.width : &band_box.height;
-
-	/* Figure out how many bands we will need. */
-	num_bands = (*size + GRADIENT_BAND_SIZE - 1) / GRADIENT_BAND_SIZE;
-	last_band_size = GRADIENT_BAND_SIZE - (GRADIENT_BAND_SIZE * num_bands - *size);
-
-	/* Change the band box to be the size of a single band. */
-	*size = GRADIENT_BAND_SIZE;
-
-	/* Set up a multiplier to use to interpolate the colors as we go. */
-	multiplier = num_bands <= 1 ? 0.0 : 1.0 / (num_bands - 1);
-	
-	/* Fill each band with a separate nautilus_draw_rectangle call. */
-	for (band = 0; band < num_bands; band++) {
-		/* Compute a new color value for each band. */
-		band_rgb = nautilus_interpolate_color (band * multiplier, start_rgb, end_rgb);
-
-		/* Last band may need to be a bit smaller to avoid writing outside the box.
-		 * This is more efficient than changing and restoring the clip.
-		 */
-		if (band == num_bands - 1) {
-			*size = last_band_size;
-		}
-		
-		nautilus_fill_rectangle_with_color (drawable, gc, &band_box, band_rgb);
-		*position += *size;
-	}
-}
-
 /**
  * nautilus_rectangle_contains:
  * @rectangle: Rectangle possibly containing a point.
@@ -288,6 +225,56 @@ nautilus_gradient_strip_trailing_direction_if_any (const char *gradient_spec)
 	return g_strndup (gradient_spec, length);
 }
 
+/* For parsing n-point gradients. Successive calls should pass the next_spec value
+ * set by the previous call as their first argument - to continue parsing where the
+ * previous call left off.
+ */
+char *
+nautilus_gradient_parse_one_color_spec (const char *spec, int *percent, const char **next_spec)
+{
+	char *result;
+	const char *rgb_end_ptr;
+	const char *percent_ptr;
+	const char *separator_ptr;
+	
+	percent_ptr   = nautilus_strchr (spec, '%');
+	separator_ptr = nautilus_strchr (spec, '-');
+
+	if (percent_ptr != NULL && (separator_ptr == NULL || percent_ptr < separator_ptr)) {
+		if (percent != NULL) {
+			*percent = (int) strtol (percent_ptr + 1, NULL, 10);
+		}
+		rgb_end_ptr = percent_ptr;
+	} else {
+		if (percent != NULL) {
+			*percent = 100;
+		}
+		rgb_end_ptr = separator_ptr;
+	}
+		
+	if (rgb_end_ptr != NULL) {
+		result = g_strndup (spec, rgb_end_ptr - spec);
+	} else {
+		result = nautilus_gradient_strip_trailing_direction_if_any (spec);
+	}
+
+	/* It's important not to use spec after setting *next_spec because it's
+	 * likely that *next_spec == spec. 
+	 */
+	if (next_spec != NULL) {
+		*next_spec = (separator_ptr != NULL) ? separator_ptr + 1 : NULL;
+	}
+
+	return result;
+}
+
+/* FIXME anyone using nautilus_gradient_get_start_color_spec or
+ * nautilus_gradient_get_end_color_spec is assuming the gradient
+ * is 2 colors which is questionable.
+ * 
+ * Callers should be rewritten and these fns eliminated.
+ */
+ 
 /**
  * nautilus_gradient_get_start_color_spec
  * @gradient_spec: A gradient spec. string.
@@ -298,14 +285,7 @@ nautilus_gradient_strip_trailing_direction_if_any (const char *gradient_spec)
 char *
 nautilus_gradient_get_start_color_spec (const char *gradient_spec)
 {
-	const char *separator;
-	
-	separator = nautilus_strchr (gradient_spec, '-');
-	if (separator == NULL) {
-		return nautilus_gradient_strip_trailing_direction_if_any (gradient_spec);
-	}
-
-	return g_strndup (gradient_spec, separator - gradient_spec);
+	return nautilus_gradient_parse_one_color_spec (gradient_spec, NULL, NULL);
 }
 
 /**
@@ -318,11 +298,14 @@ nautilus_gradient_get_start_color_spec (const char *gradient_spec)
 char *
 nautilus_gradient_get_end_color_spec (const char *gradient_spec)
 {
-	const char *separator;
+	char* color = NULL;
 
-	separator = nautilus_strchr (gradient_spec, '-');
-	return nautilus_gradient_strip_trailing_direction_if_any
-		(separator != NULL ? separator + 1 : gradient_spec);
+	do {
+		g_free (color);
+		color = nautilus_gradient_parse_one_color_spec (gradient_spec, NULL, &gradient_spec);
+	} while (gradient_spec != NULL);
+
+	return color;
 }
 
 /* Do the work shared by all the set_color_spec functions below. */
@@ -458,6 +441,24 @@ nautilus_parse_rgb_with_white_default (const char *color_spec)
 		| ((color.blue >> 8) & NAUTILUS_RGB_COLOR_BLUE);
 }
 
+guint32
+nautilus_rgb16_to_rgb (gushort r, gushort g, gushort b)
+{
+	guint32 result;
+
+	result = (0xff0000 | (r & 0xff00));
+	result <<= 8;
+	result |= ((g & 0xff00) | (b >> 8));
+
+	return result;
+}
+
+guint32
+nautilus_rgb8_to_rgb (guchar r, guchar g, guchar b)
+{
+	return nautilus_rgb16_to_rgb (r << 8, g << 8, b << 8);
+}
+
 /**
  * nautilus_gdk_color_to_rgb
  * @color: A GdkColor style color.
@@ -469,15 +470,33 @@ nautilus_parse_rgb_with_white_default (const char *color_spec)
 guint32
 nautilus_gdk_color_to_rgb (const GdkColor *color)
 {
-	guint32 result;
+	return nautilus_rgb16_to_rgb (color->red, color->green, color->blue);
+}
+
+/**
+ * nautilus_gdk_rgb_to_color
+ * @color: a gdk_rgb style value.
+ *
+ * Converts from a gdk_rgb value style to a GdkColor one.
+ * The gdk_rgb color alpha channel is ignored.
+ * 
+ * Return value: a newly allocated GdkColor.
+ */
+GdkColor *
+nautilus_gdk_rgb_to_color (const guint32 color)
+{
+	GdkColor *result;
 	
-	result = (0xff0000 | (color->red & 0xff00));
-	result <<= 8;
-	result |= ((color->green & 0xff00) | (color->blue >> 8));
+	result = g_new0 (GdkColor, 1);
+	 
+	result->red = (color & 0xff0000) >> 16 ;
+	result->green = (color & 0xff00);
+	result->blue = color << 8;
 
 	return result;
 }
 
+
 static guint32
 nautilus_shift_color_component (guchar component, float shift_by)
 {
@@ -545,7 +564,7 @@ nautilus_get_largest_fitting_font (const char *text_to_format, int width, const
 {
 	int font_index, this_width;
 	char *font_name;
-//	const int font_sizes[5] = { 28, 24, 18, 14, 12 };
+	/*const int font_sizes[5] = { 28, 24, 18, 14, 12 };*/
 	const int font_sizes[4] = { 20, 18, 14, 12 };
 	GdkFont *candidate_font;
 	char *alt_text_to_format = NULL;
@@ -566,18 +585,20 @@ nautilus_get_largest_fitting_font (const char *text_to_format, int width, const
 		}
 		
 		font_name = g_strdup_printf (font_template, font_sizes[font_index]);
-		candidate_font = gdk_font_load (font_name);
+		candidate_font = gdk_fontset_load (font_name);
 		g_free (font_name);
-		
-		this_width = gdk_string_width (candidate_font, temp_str);
-		if (alt_text_to_format != NULL) {
-			int alt_width = gdk_string_width (candidate_font, alt_text_to_format);
-			if (this_width <= width && alt_width <= width) {
-				break;
-			}
-		} else {
-			if (this_width <= width) {
-				break;
+
+		if (candidate_font != NULL) {
+			this_width = gdk_string_width (candidate_font, temp_str);
+			if (alt_text_to_format != NULL) {
+				int alt_width = gdk_string_width (candidate_font, alt_text_to_format);
+				if (this_width <= width && alt_width <= width) {
+					break;
+				}
+			} else {
+				if (this_width <= width) {
+					break;
+				}
 			}
 		}
 	}
@@ -702,7 +723,7 @@ nautilus_gdk_font_get_bold (const GdkFont *plain_font)
 		scanner = strchr (scanner, '-');
 		g_assert (scanner != NULL);
 
-		/* FIXME:
+		/* FIXME bugzilla.eazel.com 2558:
 		 * some fonts have demibold, etc. instead. We should be able to figure out
 		 * which they are and use them here.
 		 */
@@ -716,12 +737,72 @@ nautilus_gdk_font_get_bold (const GdkFont *plain_font)
 		return NULL;
 	}
 	
-	result = gdk_font_load (bold_name);
+	result = gdk_fontset_load (bold_name);
 	g_free (bold_name);
 
 	return result;
 }
 
+GdkGC *
+nautilus_gdk_create_copy_area_gc (GdkWindow	*window)
+{
+	GdkGC *copy_area_gc;
+	
+	g_return_val_if_fail (window != NULL, NULL);
+
+	copy_area_gc = gdk_gc_new (window);
+
+	gdk_gc_set_function (copy_area_gc, GDK_COPY);
+
+	return copy_area_gc;
+}
+
+void
+nautilus_gdk_gc_ref_if_not_null (GdkGC *gc_or_null)
+{
+	if (gc_or_null != NULL) {
+		gdk_gc_ref (gc_or_null);
+	}
+}
+
+void
+nautilus_gdk_gc_unref_if_not_null (GdkGC *gc_or_null)
+{
+	if (gc_or_null != NULL) {
+		gdk_gc_unref (gc_or_null);
+	}
+}
+
+/**
+ * nautilus_gdk_window_set_wm_hints_input:
+ * 
+ * Set the WM_HINTS.input flag to the passed in value
+ */
+void
+nautilus_gdk_window_set_wm_hints_input (GdkWindow *window, gboolean status)
+{
+	Display *dpy;
+	Window id;
+	XWMHints *wm_hints;
+
+	g_return_if_fail (window != NULL);
+
+	dpy = GDK_WINDOW_XDISPLAY (window);
+	id = GDK_WINDOW_XWINDOW (window);
+
+	wm_hints = XGetWMHints (dpy, id);
+	if (wm_hints == 0) {
+		wm_hints = XAllocWMHints ();
+	}
+
+	wm_hints->flags |= InputHint;
+	wm_hints->input = status;
+
+	XSetWMHints (dpy, id, wm_hints);
+	XFree (wm_hints);
+}
+
+
 #if ! defined (NAUTILUS_OMIT_SELF_CHECK)
 
 static char *
