@@ -20,19 +20,21 @@
    Boston, MA 02111-1307, USA.
 
    Authors: Darin Adler <darin@eazel.com>
+            Ramiro Estrugo <ramiro@eazel.com>
 */
 
 #include <config.h>
-#include <math.h>
-#include "nautilus-file-utilities.h"
 #include "nautilus-gdk-pixbuf-extensions.h"
+
 #include "nautilus-gdk-extensions.h"
 #include "nautilus-glib-extensions.h"
 #include "nautilus-string.h"
-
 #include <gdk-pixbuf/gdk-pixbuf-loader.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
+#include <math.h>
+#include <png.h>
 
 #define LOAD_BUFFER_SIZE 4096
 
@@ -58,7 +60,7 @@ static void file_closed_callback (GnomeVFSAsyncHandle      *handle,
 				  gpointer                  callback_data);
 static void load_done            (NautilusPixbufLoadHandle *handle,
 				  GnomeVFSResult            result,
-				  GdkPixbuf                *pixbuf);
+				  gboolean		    get_pixbuf);
 
 /**
  * nautilus_gdk_pixbuf_list_ref
@@ -90,26 +92,12 @@ nautilus_gdk_pixbuf_load (const char *uri)
 	GnomeVFSResult result;
 	GnomeVFSHandle *handle;
 	char buffer[LOAD_BUFFER_SIZE];
-	char *local_path;
 	GnomeVFSFileSize bytes_read;
 	GdkPixbufLoader *loader;
 	GdkPixbuf *pixbuf;	
 
 	g_return_val_if_fail (uri != NULL, NULL);
 
-	/* FIXME #1964: unfortunately, there are bugs in the gdk_pixbuf_loader
-	   stuff that make it not work for various image types like .xpms.
-	   Since gdk_pixbuf_new_from_file uses different, bug-free code,
-	   we call that when the file is local.  This should be fixed
-	   (in gdk_pixbuf) eventually, then this hack can be removed */
-	   
-	local_path = nautilus_get_local_path_from_uri (uri);
-	if (local_path != NULL) {
-		pixbuf = gdk_pixbuf_new_from_file (local_path);
-		g_free (local_path);
-		return pixbuf;
-	}
-	
 	result = gnome_vfs_open (&handle,
 				 uri,
 				 GNOME_VFS_OPEN_READ);
@@ -137,13 +125,14 @@ nautilus_gdk_pixbuf_load (const char *uri)
 		}
 	}
 
-	if (result != GNOME_VFS_OK) {
+	if (result != GNOME_VFS_OK && result != GNOME_VFS_ERROR_EOF) {
 		gtk_object_unref (GTK_OBJECT (loader));
 		gnome_vfs_close (handle);
 		return NULL;
 	}
 
 	gnome_vfs_close (handle);
+	gdk_pixbuf_loader_close (loader);
 
 	pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
 	if (pixbuf != NULL) {
@@ -185,7 +174,7 @@ file_opened_callback (GnomeVFSAsyncHandle *vfs_handle,
 	g_assert (handle->vfs_handle == vfs_handle);
 
 	if (result != GNOME_VFS_OK) {
-		load_done (handle, result, NULL);
+		load_done (handle, result, FALSE);
 		return;
 	}
 
@@ -207,7 +196,6 @@ file_read_callback (GnomeVFSAsyncHandle *vfs_handle,
 		    gpointer callback_data)
 {
 	NautilusPixbufLoadHandle *handle;
-	GdkPixbuf *pixbuf;
 
 	handle = callback_data;
 	g_assert (handle->vfs_handle == vfs_handle);
@@ -227,13 +215,7 @@ file_read_callback (GnomeVFSAsyncHandle *vfs_handle,
 		return;
 	}
 
-	if (result != GNOME_VFS_OK) {
-		pixbuf = NULL;
-	} else {
-		pixbuf = gdk_pixbuf_loader_get_pixbuf (handle->loader);
-	}
-
-	load_done (handle, result, pixbuf);
+	load_done (handle, result, result == GNOME_VFS_OK || result == GNOME_VFS_ERROR_EOF);
 }
 
 static void
@@ -254,14 +236,20 @@ free_pixbuf_load_handle (NautilusPixbufLoadHandle *handle)
 }
 
 static void
-load_done (NautilusPixbufLoadHandle *handle,
-	   GnomeVFSResult result,
-	   GdkPixbuf *pixbuf)
+load_done (NautilusPixbufLoadHandle *handle, GnomeVFSResult result, gboolean get_pixbuf)
 {
+	GdkPixbuf *pixbuf;
+
+	gdk_pixbuf_loader_close (handle->loader);
+
+	pixbuf = get_pixbuf ? gdk_pixbuf_loader_get_pixbuf (handle->loader) : NULL;
+	
 	if (handle->vfs_handle != NULL) {
 		gnome_vfs_async_close (handle->vfs_handle, file_closed_callback, NULL);
 	}
-	(* handle->callback) (result, pixbuf, handle->callback_data);
+
+	handle->callback (result, pixbuf, handle->callback_data);
+
 	free_pixbuf_load_handle (handle);
 }
 
@@ -277,14 +265,15 @@ nautilus_cancel_gdk_pixbuf_load (NautilusPixbufLoadHandle *handle)
 	free_pixbuf_load_handle (handle);
 }
 
+/* FIXME
+ * This fn is only used by some test code, it should probably be removed
+ */
 void
-nautilus_gdk_pixbuf_render_to_drawable_tiled (GdkPixbuf *pixbuf,
-					      GdkDrawable *drawable,
-					      GdkGC *gc,
-					      const GdkRectangle *rect,
-					      GdkRgbDither dither,
-					      int x_dither,
-					      int y_dither)
+nautilus_gdk_pixbuf_render_to_pixbuf_tiled (GdkPixbuf *source_pixbuf,
+					    GdkPixbuf *destination_pixbuf,
+					    const GdkRectangle *rect,
+					    int x_dither,
+					    int y_dither)
 {
 	int x, y;
 	int start_x, start_y;
@@ -295,8 +284,8 @@ nautilus_gdk_pixbuf_render_to_drawable_tiled (GdkPixbuf *pixbuf,
 	int blit_width, blit_height;
 	int tile_offset_x, tile_offset_y;
 
-	tile_width = gdk_pixbuf_get_width (pixbuf);
-	tile_height = gdk_pixbuf_get_height (pixbuf);
+	tile_width = gdk_pixbuf_get_width (source_pixbuf);
+	tile_height = gdk_pixbuf_get_height (source_pixbuf);
 
 	tile_offset_x = (rect->x - x_dither) % tile_width;
 	if (tile_offset_x < 0) {
@@ -326,10 +315,48 @@ nautilus_gdk_pixbuf_render_to_drawable_tiled (GdkPixbuf *pixbuf,
 			tile_y = blit_y - y;
 			blit_height = MIN (tile_height, end_y - y) - tile_y;
 			
-			gdk_pixbuf_render_to_drawable (pixbuf, drawable, gc,
-						       tile_x, tile_y,
-						       blit_x, blit_y, blit_width, blit_height,
-						       dither, x_dither, y_dither);
+// 			gdk_pixbuf_render_to_drawable (pixbuf, drawable, gc,
+// 						       tile_x, tile_y,
+// 						       blit_x, blit_y, blit_width, blit_height,
+// 						       dither, x_dither, y_dither);
+
+// 			gdk_pixbuf_render_to_drawable (pixbuf, drawable, gc,
+// 						       tile_x, tile_y,
+// 						       blit_x, blit_y, blit_width, blit_height,
+// 						       dither, x_dither, y_dither);
+
+			gdk_pixbuf_copy_area (source_pixbuf,
+					      tile_x, 
+					      tile_y,
+					      blit_width,
+					      blit_height,
+					      destination_pixbuf,
+					      blit_x,
+					      blit_y);
+
+// 		/* blit the pixbuf to the drawable */
+// 		gdk_pixbuf_render_to_drawable (pixbuf,
+// 					       drawable,
+// 					       gc,
+// 					       0,
+// 					       0,
+// 					       rectangle->x,
+// 					       rectangle->y,
+// 					       rectangle->width,
+// 					       rectangle->height,
+// 					       GDK_RGB_DITHER_NORMAL,
+// 					       origin_x,
+// 					       origin_y);
+
+// 		gdk_pixbuf_copy_area (pixbuf,
+// 				      0,
+// 				      0,
+// 				      rectangle->width,
+// 				      rectangle->height,
+// 				      destination_pixbuf,
+// 				      rectangle->x,
+// 				      rectangle->y);
+
 		}
 	}
 }
@@ -338,7 +365,7 @@ nautilus_gdk_pixbuf_render_to_drawable_tiled (GdkPixbuf *pixbuf,
 void 
 nautilus_gdk_pixbuf_average_value (GdkPixbuf *pixbuf, GdkColor *color)
 {
-	int red_total, green_total, blue_total, count;
+	uint red_total, green_total, blue_total, count;
 	int row, column;
 	int width, height;
 	int row_stride;
@@ -359,7 +386,7 @@ nautilus_gdk_pixbuf_average_value (GdkPixbuf *pixbuf, GdkColor *color)
 	original_pixels = gdk_pixbuf_get_pixels (pixbuf);
 
 	for (row = 0; row < height; row++) {
-		pixsrc = original_pixels + row * row_stride;
+		pixsrc = original_pixels + (row * row_stride);
 		for (column = 0; column < width; column++) {
 			red_total += *pixsrc++;
 			green_total += *pixsrc++;
@@ -376,38 +403,58 @@ nautilus_gdk_pixbuf_average_value (GdkPixbuf *pixbuf, GdkColor *color)
 	color->blue =  (blue_total  * 256) / count;
 }
 
-/* scale the passed in pixbuf to conform to the passed-in maximum width and height */
-/* utility routine to scale the passed-in pixbuf to be smaller than the maximum allowed size, if necessary */
+double
+nautilus_gdk_scale_to_fit_factor (int width, int height,
+				  int max_width, int max_height,
+				  int *scaled_width, int *scaled_height)
+{
+	double scale_factor;
+	
+	scale_factor = MIN (max_width  / (double) width, max_height / (double) height);
+
+	*scaled_width  = floor (width * scale_factor + .5);
+	*scaled_height = floor (height * scale_factor + .5);
+
+	return scale_factor;
+}
+
+/* Returns a scaled copy of pixbuf, preserving aspect ratio. The copy will
+ * be scaled as large as possible without exceeding the specified width and height.
+ */
 GdkPixbuf *
 nautilus_gdk_pixbuf_scale_to_fit (GdkPixbuf *pixbuf, int max_width, int max_height)
 {
-	double scale_factor;
-	double h_scale = 1.0;
-	double v_scale = 1.0;
+	int scaled_width;
+	int scaled_height;
 
-	int width  = gdk_pixbuf_get_width(pixbuf);
-	int height = gdk_pixbuf_get_height(pixbuf);
+	nautilus_gdk_scale_to_fit_factor (gdk_pixbuf_get_width(pixbuf), gdk_pixbuf_get_height(pixbuf),
+					  max_width, max_height,
+					  &scaled_width, &scaled_height);
+
+	return gdk_pixbuf_scale_simple (pixbuf, scaled_width, scaled_height, GDK_INTERP_BILINEAR);	
+}
+
+/* Returns a copy of pixbuf scaled down, preserving aspect ratio, to fit
+ * within the specified width and height. If it already fits, a copy of
+ * the original, without scaling, is returned.
+ */
+GdkPixbuf *
+nautilus_gdk_pixbuf_scale_down_to_fit (GdkPixbuf *pixbuf, int max_width, int max_height)
+{
+	int scaled_width;
+	int scaled_height;
 	
-	if (width > max_width) {
-		h_scale = max_width / (double) width;
+	double scale_factor;
+
+	scale_factor = nautilus_gdk_scale_to_fit_factor (gdk_pixbuf_get_width(pixbuf), gdk_pixbuf_get_height(pixbuf),
+							 max_width, max_height,
+							 &scaled_width, &scaled_height);
+
+	if (scale_factor >= 1.0) {
+		return gdk_pixbuf_copy (pixbuf);
+	} else {				
+		return gdk_pixbuf_scale_simple (pixbuf, scaled_width, scaled_height, GDK_INTERP_BILINEAR);	
 	}
-	if (height > max_height) {
-		v_scale = max_height  / (double) height;
-	}
-	scale_factor = MIN (h_scale, v_scale);
-	
-	if (scale_factor < 1.0) {
-		GdkPixbuf *scaled_pixbuf;
-		/* the width and scale factor are always > 0, so it's OK to round by adding here */
-		int scaled_width  = floor(width * scale_factor + .5);
-		int scaled_height = floor(height * scale_factor + .5);
-				
-		scaled_pixbuf = gdk_pixbuf_scale_simple (pixbuf, scaled_width, scaled_height, GDK_INTERP_BILINEAR);	
-		gdk_pixbuf_unref (pixbuf);
-		pixbuf = scaled_pixbuf;
-	}
-	
-	return pixbuf;
 }
 
 /**
@@ -426,6 +473,7 @@ nautilus_gdk_pixbuf_draw_text (GdkPixbuf	*pixbuf,
 			       const double	font_scale,
 			       const ArtIRect	*destination_rect,
 			       const char	*text,
+			       guint		text_color,
 			       guint		overall_alpha)
 {
 	ArtIRect	pixbuf_rect;
@@ -472,19 +520,23 @@ nautilus_gdk_pixbuf_draw_text (GdkPixbuf	*pixbuf,
 	width = dest_width / font_scale;
 	height = dest_height / font_scale;
 	
+	/* cut out if it's too small for comfort */
+	if (width <= 8 || height <= 8)
+		return;
+		
 	pixmap = gdk_pixmap_new (NULL, width, height, visual->depth);
 	gc = gdk_gc_new (pixmap);
 
 	/* Set up a white background. */
-	gdk_rgb_gc_set_foreground (gc, NAUTILUS_RGB_COLOR_WHITE);
+	gdk_rgb_gc_set_foreground (gc, text_color == NAUTILUS_RGB_COLOR_WHITE ? NAUTILUS_RGB_COLOR_BLACK : NAUTILUS_RGB_COLOR_WHITE);
 	gdk_draw_rectangle (pixmap, gc, TRUE, 0, 0, width, height);
 	
-	/* Draw black text. */
-	gdk_rgb_gc_set_foreground (gc, NAUTILUS_RGB_COLOR_BLACK);
+	/* Set up the text color. */
+	gdk_rgb_gc_set_foreground (gc, text_color);
 	gdk_gc_set_font (gc, (GdkFont *) font);
 	line = text;
 
-	/* FIXME bugzilla.eazel.com xxxx:
+	/* FIXME bugzilla.eazel.com 2559:
 	 * The iteration code should work with strings that dont have 
 	 * new lines.  Its broken right now for single line strings.  The 
 	 * (y + font->ascent) <= height test always fails and no text is drawn.
@@ -547,129 +599,400 @@ nautilus_gdk_pixbuf_draw_text (GdkPixbuf	*pixbuf,
 	gdk_pixbuf_unref (text_pixbuf_with_alpha);
 }
 
-/* The following function is a carbon copy of the above except it draws 
- * white text.  This is a hack workaround for a demo.  The correct fix is
- * to make the above function work with an arbitrary text color as an rgb 
- * value.  This work is happening on the HEAD of nautilus.
+/**
+ * nautilus_gdk_pixbuf_fill_rectangle:
+ * @pixbuf: Target pixbuf to fill into.
+ * @rectangle: Rectangle to fill.
+ * @color: The color to use.
+ *
+ * Fill the rectangle with the the given color.
+ * Use the given rectangle if not NULL.
+ * If rectangle is NULL, fill the whole pixbuf.
  */
 void
-nautilus_gdk_pixbuf_draw_text_white (GdkPixbuf		*pixbuf,
-				     const GdkFont	*font,
-				     const ArtIRect	*destination_rect,
-				     const char		*text,
-				     guint		overall_alpha) {
+nautilus_gdk_pixbuf_fill_rectangle_with_color (GdkPixbuf		*pixbuf,
+					       const GdkRectangle	*rectangle,
+					       guint32			color)
+{
+	GdkRectangle	area;
+	guchar		red;
+	guchar		green;
+	guchar		blue;
+	guchar		alpha;
 
-	ArtIRect	pixbuf_rect;
-	ArtIRect	text_rect;
-	int		width, height;
-	GdkVisual	*visual;
-	GdkPixmap	*pixmap;
-	GdkGC		*gc;
-	GdkColormap	*colormap;
-	int		y;
-	const char	*line;
-	const char	*end_of_line;
-	int		line_length;
-	GdkPixbuf	*text_pixbuf;
-	GdkPixbuf	*text_pixbuf_with_alpha;
+	guint		width;
+	guint		height;
 	guchar		*pixels;
+	gboolean	has_alpha;
+	guint		pixel_offset;
+	guint		rowstride;
+
+	int		x;
+	int		y;
+
+	int		x1;
+	int		y1;
+	int		x2;
+	int		y2;
+	guchar		*row_offset;
 
 	g_return_if_fail (pixbuf != NULL);
-	g_return_if_fail (font != NULL);
-	g_return_if_fail (destination_rect != NULL);
-	g_return_if_fail (text != NULL);
-	g_return_if_fail (nautilus_strlen (text) > 0);
-	g_return_if_fail (overall_alpha <= 255);
 
-	/* Compute the intersection of the text rectangle with the pixbuf.
-	 * This is only to prevent pathological cases from upsetting the
-	 * GdkPixbuf routines. It should not happen in any normal circumstance.
-	 */
-	pixbuf_rect.x0 = 0;
-	pixbuf_rect.y0 = 0;
-	pixbuf_rect.x1 = gdk_pixbuf_get_width (pixbuf);
-	pixbuf_rect.y1 = gdk_pixbuf_get_height (pixbuf);
-	art_irect_intersect (&text_rect, destination_rect, &pixbuf_rect);
+	width = gdk_pixbuf_get_width (pixbuf);
+	height = gdk_pixbuf_get_height (pixbuf);
+	pixels = gdk_pixbuf_get_pixels (pixbuf);
+	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+	has_alpha = gdk_pixbuf_get_has_alpha (pixbuf);
 
-	/* Get the system visual. I wish I could do this all in 1-bit mode,
-	 * but I can't.
-	 */
-	visual = gdk_visual_get_system ();
+	pixel_offset = has_alpha ? 4 : 3;
 
-	/* Allocate a GdkPixmap of the appropriate size. */
-	width = text_rect.x1 - text_rect.x0;
-	height = text_rect.y1 - text_rect.y0;
-	pixmap = gdk_pixmap_new (NULL, width, height, visual->depth);
-	gc = gdk_gc_new (pixmap);
+	if (rectangle != NULL)
+	{
+		area = *rectangle;
+	}
+	else
+	{
+		area.x = 0;
+		area.y = 0;
+		area.width = width;
+		area.height = height;
+	}
 
-	/* Set up a white background. */
-	gdk_rgb_gc_set_background (gc, NAUTILUS_RGB_COLOR_BLACK);
-	gdk_draw_rectangle (pixmap, gc, TRUE, 0, 0, width, height);
+	red = NAUTILUS_RGBA_COLOR_GET_R (color);
+	green = NAUTILUS_RGBA_COLOR_GET_G (color);
+	blue = NAUTILUS_RGBA_COLOR_GET_B (color);
+	alpha = NAUTILUS_RGBA_COLOR_GET_A (color);
 
-	/* Draw black text. */
-	gdk_rgb_gc_set_foreground (gc, NAUTILUS_RGB_COLOR_WHITE);
-	gdk_gc_set_font (gc, (GdkFont *) font);
-	line = text;
+	x1 = area.x;
+	y1 = area.y;
+	x2 = area.x + area.width;
+	y2 = area.y + area.height;
 
-	/* FIXME bugzilla.eazel.com xxxx:
-	 * The iteration code should work with strings that dont have 
-	 * new lines.  Its broken right now for single line strings.  The 
-	 * (y + font->ascent) <= height test always fails and no text is drawn.
-	 */
-	if (strchr (line, '\n')) {
-		for (y = font->ascent;
-		y + font->descent <= height;
-		y += font->ascent + font->descent) {
-                
-			/* Extract the next line of text. */
-			end_of_line = strchr (line, '\n');
-			line_length = end_of_line == NULL
-				? strlen (line)
-				: end_of_line - line;
-                 
-			/* Draw the next line of text. */
-			gdk_draw_text (pixmap, (GdkFont *) font, gc, 0, y,
-				       line, line_length);
+	row_offset = pixels + y1 * rowstride;
 
-			/* Move on to the subsequent line. */
-			line = end_of_line == NULL
-				? ""
-				: end_of_line + 1;
+	for (y = y1; y < y2; y++)
+	{
+		guchar *offset = row_offset + (x1 * pixel_offset);
+		
+		for (x = x1; x < x2; x++)
+		{
+			*(offset++) = red;
+			*(offset++) = green;
+			*(offset++) = blue;
+			
+			if (has_alpha)
+			{
+				*(offset++) = alpha;
+			}
+			
 		}
+
+		row_offset += rowstride;
 	}
-	else {
-		/* Draw the next line of text. */
-		gdk_draw_text (pixmap, (GdkFont *) font, gc, 0, font->ascent,
-			       line, strlen (line));
-	}
-	gdk_gc_unref (gc);
-
-	/* Convert into a GdkPixbuf with gdk_pixbuf_get_from_drawable. */
-	colormap = gdk_colormap_new (visual, FALSE);
-	text_pixbuf = gdk_pixbuf_get_from_drawable (NULL, pixmap, colormap,
-						    0, 0,
-						    0, 0, width, height);
-	gdk_colormap_unref (colormap);
-	gdk_pixmap_unref (pixmap);
-
-	/* White is not always FF FF FF. So we get the top left corner pixel. */
-	pixels = gdk_pixbuf_get_pixels (text_pixbuf);
-	text_pixbuf_with_alpha = gdk_pixbuf_add_alpha 
-		(text_pixbuf,
-		 TRUE, pixels[0], pixels[1], pixels[2]);
-	gdk_pixbuf_unref (text_pixbuf);
-
-	gdk_pixbuf_composite (text_pixbuf_with_alpha,
-			      pixbuf,
-			      text_rect.x0,
-			      text_rect.y0,
-			      width, height,
-			      text_rect.x0,
-			      text_rect.y0,
-			      1, 1,
-			      GDK_INTERP_BILINEAR,
-			      overall_alpha);
-
-	gdk_pixbuf_unref (text_pixbuf_with_alpha);
 }
 
+void
+nautilus_gdk_pixbuf_render_to_drawable (const GdkPixbuf		*pixbuf,
+					GdkDrawable		*drawable,
+					GdkGC			*gc,
+					const GdkPoint		*source_point,
+					const GdkRectangle	*destination_area,
+					GdkRgbDither		dither)
+{
+	GdkPoint	src;
+	GdkRectangle	dst;
+	GdkPoint	end;
+
+	g_return_if_fail (pixbuf != NULL);
+	g_return_if_fail (drawable != NULL);
+	g_return_if_fail (gc != NULL);
+	g_return_if_fail (source_point != NULL);
+	g_return_if_fail (destination_area != NULL);
+ 	g_return_if_fail (destination_area->width > 0);
+ 	g_return_if_fail (destination_area->height > 0);
+	g_return_if_fail (source_point->x >= 0);
+	g_return_if_fail (source_point->y >= 0);
+
+	g_assert (gdk_pixbuf_get_width (pixbuf) > 0);
+	g_assert (gdk_pixbuf_get_height (pixbuf) > 0);
+
+	src = *source_point;
+	dst = *destination_area;
+
+	/* Clip to the left edge of the drawable */
+	if (dst.x < 0)
+	{
+		src.x += ABS (dst.x);
+		dst.x = 0;
+	}
+
+	/* Clip to the top edge of the drawable */
+	if (dst.y < 0)
+	{
+		src.y += ABS (dst.y);
+		dst.y = 0;
+	}
+	
+	end.x = src.x + dst.width;
+	end.y = src.y + dst.height;
+	
+	if (end.x >= gdk_pixbuf_get_width (pixbuf))
+	{
+		g_assert (dst.width >= (end.x - gdk_pixbuf_get_width (pixbuf)));
+		
+		dst.width -= (end.x - gdk_pixbuf_get_width (pixbuf));
+	}
+
+	if (end.y >= gdk_pixbuf_get_height (pixbuf))
+	{
+		g_assert (dst.height >= (end.y - gdk_pixbuf_get_height (pixbuf)));
+
+		dst.height -= (end.y - gdk_pixbuf_get_height (pixbuf));
+	}
+
+	if (gdk_pixbuf_get_has_alpha (pixbuf)) {
+		gdk_pixbuf_render_to_drawable_alpha ((GdkPixbuf *) pixbuf,
+				       drawable,
+				       src.x,
+				       src.y,
+				       dst.x,
+				       dst.y,
+				       dst.width,
+				       dst.height,
+				       GDK_PIXBUF_ALPHA_FULL,
+				       128,
+				       dither,
+				       0,
+				       0);
+
+	} else {
+		gdk_pixbuf_render_to_drawable ((GdkPixbuf *) pixbuf,
+				       drawable,
+				       gc,
+				       src.x,
+				       src.y,
+				       dst.x,
+				       dst.y,
+				       dst.width,
+				       dst.height,
+				       dither,
+				       0,
+				       0);
+	}
+}
+
+void
+nautilus_gdk_pixbuf_render_to_pixbuf (const GdkPixbuf		*pixbuf,
+				      GdkPixbuf			*destination_pixbuf,
+				      const GdkPoint		*source_point,
+				      const GdkRectangle	*destination_area)
+{
+	GdkPoint	src;
+	GdkRectangle	dst;
+	GdkPoint	end;
+
+	g_return_if_fail (pixbuf != NULL);
+	g_return_if_fail (destination_pixbuf != NULL);
+	g_return_if_fail (source_point != NULL);
+	g_return_if_fail (source_point->x >= 0);
+	g_return_if_fail (source_point->y >= 0);
+	g_return_if_fail (destination_area != NULL);
+	g_return_if_fail (destination_area->width > 0);
+	g_return_if_fail (destination_area->height > 0);
+
+	src = *source_point;
+	dst = *destination_area;
+
+	/* Clip to the left edge of the drawable */
+	if (dst.x < 0)
+	{
+		src.x += ABS (dst.x);
+		dst.x = 0;
+	}
+
+	/* Clip to the top edge of the drawable */
+	if (dst.y < 0)
+	{
+		src.y += ABS (dst.y);
+		dst.y = 0;
+	}
+	
+	end.x = src.x + dst.width;
+	end.y = src.y + dst.height;
+	
+	if (end.x >= gdk_pixbuf_get_width (pixbuf))
+	{
+		g_assert (dst.width >= (end.x - gdk_pixbuf_get_width (pixbuf)));
+
+		dst.width -= (end.x - gdk_pixbuf_get_width (pixbuf));
+	}
+
+	if (end.y >= gdk_pixbuf_get_height (pixbuf))
+	{
+		g_assert (dst.height >= (end.y - gdk_pixbuf_get_height (pixbuf)));
+
+		dst.height -= (end.y - gdk_pixbuf_get_height (pixbuf));
+	}
+
+	gdk_pixbuf_copy_area ((GdkPixbuf *) pixbuf,
+			      src.x,
+			      src.y,
+			      dst.width,
+			      dst.height,
+			      destination_pixbuf,
+			      dst.x,
+			      dst.y);
+}
+
+void
+nautilus_gdk_pixbuf_render_to_pixbuf_alpha (const GdkPixbuf	*pixbuf,
+					    GdkPixbuf		*destination_pixbuf,
+					    const GdkRectangle	*destination_area,
+					    GdkInterpType	interpolation_mode,
+					    guchar		overall_alpha)
+{
+	g_return_if_fail (pixbuf != NULL);
+	g_return_if_fail (destination_pixbuf != NULL);
+	g_return_if_fail (destination_area != NULL);
+	g_return_if_fail (destination_area->width > 0);
+	g_return_if_fail (destination_area->height > 0);
+
+	gdk_pixbuf_composite (pixbuf,
+			      destination_pixbuf,
+			      destination_area->x,
+			      destination_area->y,
+			      destination_area->width,
+			      destination_area->height,
+			      (double) destination_area->x,
+			      (double) destination_area->y,
+			      1.0,
+			      1.0,
+			      interpolation_mode,
+			      overall_alpha);
+}
+
+/* utility routine for saving a pixbuf to a png file.
+ * This was adapted from Iain Holmes' code in gnome-iconedit, and probably
+ * should be in a utility library, possibly in gdk-pixbuf itself.
+ */
+gboolean
+nautilus_gdk_pixbuf_save_to_file (GdkPixbuf                  *pixbuf,
+				  const char                 *file_name)
+{
+	FILE *handle;
+  	char *buffer;
+	gboolean has_alpha;
+	int width, height, depth, rowstride;
+  	guchar *pixels;
+  	png_structp png_ptr;
+  	png_infop info_ptr;
+  	png_text text[2];
+  	int i;
+
+	g_return_val_if_fail (pixbuf != NULL, FALSE);
+	g_return_val_if_fail (file_name != NULL, FALSE);
+	g_return_val_if_fail (file_name[0] != '\0', FALSE);
+
+        handle = fopen (file_name, "wb");
+        if (handle == NULL) {
+        	return FALSE;
+	}
+
+	png_ptr = png_create_write_struct (PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
+	if (png_ptr == NULL) {
+		fclose (handle);
+		return FALSE;
+	}
+
+	info_ptr = png_create_info_struct (png_ptr);
+	if (info_ptr == NULL) {
+		png_destroy_write_struct (&png_ptr, (png_infopp)NULL);
+		fclose (handle);
+	    	return FALSE;
+	}
+
+	if (setjmp (png_ptr->jmpbuf)) {
+		png_destroy_write_struct (&png_ptr, &info_ptr);
+		fclose (handle);
+		return FALSE;
+	}
+
+	png_init_io (png_ptr, handle);
+
+        has_alpha = gdk_pixbuf_get_has_alpha (pixbuf);
+	width = gdk_pixbuf_get_width (pixbuf);
+	height = gdk_pixbuf_get_height (pixbuf);
+	depth = gdk_pixbuf_get_bits_per_sample (pixbuf);
+	pixels = gdk_pixbuf_get_pixels (pixbuf);
+	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+
+	png_set_IHDR (png_ptr, info_ptr, width, height,
+			depth, PNG_COLOR_TYPE_RGB_ALPHA,
+			PNG_INTERLACE_NONE,
+			PNG_COMPRESSION_TYPE_DEFAULT,
+			PNG_FILTER_TYPE_DEFAULT);
+
+	/* Some text to go with the png image */
+	text[0].key = "Title";
+	text[0].text = (char *) file_name;
+	text[0].compression = PNG_TEXT_COMPRESSION_NONE;
+	text[1].key = "Software";
+	text[1].text = "Nautilus Thumbnail";
+	text[1].compression = PNG_TEXT_COMPRESSION_NONE;
+	png_set_text (png_ptr, info_ptr, text, 2);
+
+	/* Write header data */
+	png_write_info (png_ptr, info_ptr);
+
+	/* if there is no alpha in the data, allocate buffer to expand into */
+	if (has_alpha) {
+		buffer = NULL;
+	} else {
+		buffer = g_malloc(4 * width);
+	}
+	
+	/* pump the raster data into libpng, one scan line at a time */	
+	for (i = 0; i < height; i++) {
+		if (has_alpha) {
+			png_bytep row_pointer = pixels;
+			png_write_row (png_ptr, row_pointer);
+		} else {
+			/* expand RGB to RGBA using an opaque alpha value */
+			int x;
+			char *buffer_ptr = buffer;
+			char *source_ptr = pixels;
+			for (x = 0; x < width; x++) {
+				*buffer_ptr++ = *source_ptr++;
+				*buffer_ptr++ = *source_ptr++;
+				*buffer_ptr++ = *source_ptr++;
+				*buffer_ptr++ = 255;
+			}
+			png_write_row (png_ptr, (png_bytep) buffer);		
+		}
+		pixels += rowstride;
+	}
+	
+	png_write_end (png_ptr, info_ptr);
+	png_destroy_write_struct (&png_ptr, &info_ptr);
+	
+	g_free (buffer);
+		
+	fclose (handle);
+	return TRUE;
+}
+
+void
+nautilus_gdk_pixbuf_ref_if_not_null (GdkPixbuf *pixbuf_or_null)
+{
+	if (pixbuf_or_null != NULL) {
+		gdk_pixbuf_ref (pixbuf_or_null);
+	}
+}
+
+void
+nautilus_gdk_pixbuf_unref_if_not_null (GdkPixbuf *pixbuf_or_null)
+{
+	if (pixbuf_or_null != NULL) {
+		gdk_pixbuf_unref (pixbuf_or_null);
+	}
+}
