@@ -23,58 +23,44 @@
 */
 
 #include <config.h>
-#include <liboaf/liboaf.h>
 #include "nautilus-global-preferences.h"
 
-#include <gtk/gtkbox.h>
-
-#include "nautilus-preferences-group.h"
-#include "nautilus-preferences-item.h"
-#include "nautilus-preferences-dialog.h"
-#include "nautilus-user-level-manager.h"
+#include "nautilus-file-utilities.h"
 #include "nautilus-glib-extensions.h"
 #include "nautilus-gtk-extensions.h"
-#include "nautilus-file-utilities.h"
-#include "nautilus-view-identifier.h"
+#include "nautilus-preferences-dialog.h"
+#include "nautilus-preferences-group.h"
+#include "nautilus-preferences-item.h"
 #include "nautilus-string.h"
+#include "nautilus-user-level-manager.h"
+#include "nautilus-view-identifier.h"
+#include <gconf/gconf.h>
+#include <gconf/gconf-client.h>
+#include <gtk/gtkbox.h>
+#include <libgnome/gnome-i18n.h>
+#include <liboaf/liboaf.h>
 
 /* Constants */
 #define GLOBAL_PREFERENCES_DIALOG_TITLE _("Nautilus Preferences")
 
-/* default web search uri - this will soon be changed to point to our service */
-#define DEFAULT_SEARCH_WEB_URI "http://www.eazel.com/websearch.html"
+/* default web search uri */
+#define DEFAULT_SEARCH_WEB_URI "http://services.eazel.com/websearch"
 
-/* Private stuff */
-static GtkWidget *global_preferences_create_dialog                      (void);
-static GtkWidget *global_preferences_get_dialog                         (void);
-static void       global_preferences_register                           (void);
+static const char PROXY_KEY[] = "/system/gnome-vfs/http-proxy";
+static const char USE_PROXY_KEY[] = "/system/gnome-vfs/use-http-proxy";
+
+/* Forward declarations */
 static char *     global_preferences_get_sidebar_panel_key              (const char             *panel_iid);
-static gboolean   global_preferences_is_sidebar_panel_enabled           (NautilusViewIdentifier *panel_identifier,
-									 gpointer                ignore);
+static gboolean   global_preferences_is_sidebar_panel_enabled_cover     (gpointer                data,
+									 gpointer                callback_data);
 static GList *    global_preferences_get_sidebar_panel_view_identifiers (void);
 static gboolean   global_preferences_close_dialog_callback              (GtkWidget              *dialog,
 									 gpointer                user_data);
-static void       global_preferences_register_with_defaults             (const char             *name,
-									 const char             *description,
-									 NautilusPreferenceType  type,
-									 gconstpointer           novice_default,
-									 gconstpointer           intermediate_default,
-									 gconstpointer           hacker_default);
 static void       global_preferences_register_boolean_with_defaults     (const char             *name,
 									 const char             *description,
 									 gboolean                novice_default,
 									 gboolean                intermediate_default,
 									 gboolean                hacker_default);
-static void       global_preferences_register_string_with_defaults      (const char             *name,
-									 const char             *description,
-									 const char             *novice_default,
-									 const char             *intermediate_default,
-									 const char             *hacker_default);
-static void       global_preferences_register_enum_with_defaults        (const char             *name,
-									 const char             *description,
-									 int                     novice_default,
-									 int                     intermediate_default,
-									 int                     hacker_default);
 
 static GtkWidget *global_prefs_dialog = NULL;
 
@@ -129,8 +115,8 @@ global_preferences_create_dialog (void)
 	 * Directory Views pane
 	 */
 	directory_views_pane = nautilus_preferences_box_add_pane (preference_box,
-								 _("Directory Views"),
-								 _("Directory Views Settings"));
+								 _("Folder Views"),
+								 _("Folder Views Settings"));
 	
 	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane), _("Window Behavior"));
 	
@@ -146,15 +132,27 @@ global_preferences_create_dialog (void)
 							 NAUTILUS_PREFERENCES_CLICK_POLICY,
 							 NAUTILUS_PREFERENCE_ITEM_ENUM);
 
-	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane), _("Display"));
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane), _("Trash Behavior"));
 	
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane),
 							 2,
+							 NAUTILUS_PREFERENCES_CONFIRM_TRASH,
+							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
+	
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane), _("Display"));
+	
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane),
+							 3,
 							 NAUTILUS_PREFERENCES_SHOW_HIDDEN_FILES,
 							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
 
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane),
-							 2,
+							 3,
+							 NAUTILUS_PREFERENCES_SHOW_BACKUP_FILES,
+							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
+
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane),
+							 3,
 							 NAUTILUS_PREFERENCES_SHOW_SPECIAL_FLAGS,
 							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
 
@@ -210,7 +208,7 @@ global_preferences_create_dialog (void)
 	
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (appearance_pane),
 							 0,
-							 NAUTILUS_PREFERENCES_ANTI_ALIASED_CANVAS,
+							 NAUTILUS_PREFERENCES_SMOOTH_GRAPHICS_MODE,
 							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
 	
 	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (appearance_pane), _("Fonts"));
@@ -222,21 +220,26 @@ global_preferences_create_dialog (void)
 	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (appearance_pane), _("Views"));
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (appearance_pane),
 							 2,
-							 NAUTILUS_PREFERENCES_DISPLAY_TOOLBAR,
+							 NAUTILUS_PREFERENCES_START_WITH_TOOL_BAR,
 							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (appearance_pane),
 							 2,
-							 NAUTILUS_PREFERENCES_DISPLAY_LOCATIONBAR,
+							 NAUTILUS_PREFERENCES_START_WITH_LOCATION_BAR,
 							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (appearance_pane),
 							 2,
-							 NAUTILUS_PREFERENCES_DISPLAY_STATUSBAR,
+							 NAUTILUS_PREFERENCES_START_WITH_STATUS_BAR,
 							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (appearance_pane),
 							 2,
-							 NAUTILUS_PREFERENCES_DISPLAY_SIDEBAR,
+							 NAUTILUS_PREFERENCES_START_WITH_SIDEBAR,
 							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
 	
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (appearance_pane), _("Desktop"));
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (appearance_pane),
+							 3,
+							 NAUTILUS_PREFERENCES_SHOW_DESKTOP,
+							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
 
 	
 	/*
@@ -260,8 +263,8 @@ global_preferences_create_dialog (void)
 							 NAUTILUS_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS,
 							 NAUTILUS_PREFERENCE_ITEM_SHORT_ENUM);
 
-	/* FIXME: This title phrase needs improvement. */
-	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (tradeoffs_pane), _("Make Directory Appearance Details Public"));
+	/* FIXME bugzilla.eazel.com 2560: This title phrase needs improvement. */
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (tradeoffs_pane), _("Make Folder Appearance Details Public"));
 	
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (tradeoffs_pane),
 							 2,
@@ -310,6 +313,31 @@ global_preferences_create_dialog (void)
 							 NAUTILUS_PREFERENCES_HOME_URI,
 							 NAUTILUS_PREFERENCE_ITEM_EDITABLE_STRING);
 
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (navigation_pane), _("Proxy Settings"));
+
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (navigation_pane),
+							 1,
+							 NAUTILUS_PREFERENCES_HTTP_USE_PROXY,
+							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
+
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (navigation_pane),
+							 1,
+							 NAUTILUS_PREFERENCES_HTTP_PROXY,
+							 NAUTILUS_PREFERENCE_ITEM_EDITABLE_STRING);
+
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (navigation_pane),
+							 1,
+							 NAUTILUS_PREFERENCES_HTTP_PROXY_PORT,
+							 NAUTILUS_PREFERENCE_ITEM_EDITABLE_STRING);
+
+	/* built-in bookmarks */
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (navigation_pane),
+					     _("Built-in Bookmarks"));
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (navigation_pane),
+							 2,
+							 NAUTILUS_PREFERENCES_HIDE_BUILT_IN_BOOKMARKS,
+							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
+
 	/* all done */
 
 	return prefs_dialog;
@@ -341,6 +369,7 @@ global_preferences_get_sidebar_panel_view_identifiers (void)
 	oaf_result = oaf_query (query, NULL, &ev);
 		
 	view_identifiers = NULL;
+
         if (ev._major == CORBA_NO_EXCEPTION && oaf_result != NULL) {
 		for (i = 0; i < oaf_result->_length; i++) {
 			id = nautilus_view_identifier_new_from_sidebar_panel
@@ -367,66 +396,110 @@ nautilus_global_preferences_get_enabled_sidebar_panel_view_identifiers (void)
         
 	enabled_view_identifiers = global_preferences_get_sidebar_panel_view_identifiers ();
         
-        enabled_view_identifiers = nautilus_g_list_partition (enabled_view_identifiers,
-                                                      (NautilusGPredicateFunc) global_preferences_is_sidebar_panel_enabled,
-                                                      NULL,
-                                                      &disabled_view_identifiers);
+        enabled_view_identifiers = nautilus_g_list_partition
+		(enabled_view_identifiers,
+		 global_preferences_is_sidebar_panel_enabled_cover,
+		 NULL,
+		 &disabled_view_identifiers);
 	
         nautilus_view_identifier_list_free (disabled_view_identifiers);
 
         return enabled_view_identifiers;
 }
 
-GList *
-nautilus_global_preferences_get_disabled_sidebar_panel_view_identifiers (void)
+static void
+destroy_global_prefs_dialog (void)
 {
-	GList *enabled_view_identifiers;
-	GList *disabled_view_identifiers;
-        
-	enabled_view_identifiers = global_preferences_get_sidebar_panel_view_identifiers ();
-        
-        enabled_view_identifiers = nautilus_g_list_partition (enabled_view_identifiers,
-							      (NautilusGPredicateFunc) global_preferences_is_sidebar_panel_enabled,
-							      NULL,
-							      &disabled_view_identifiers);
-	
-        nautilus_view_identifier_list_free (enabled_view_identifiers);
-	
-        return disabled_view_identifiers;
+	/* Free the dialog first, cause it has refs to preferences */
+	if (global_prefs_dialog != NULL) {
+		/* Since it's a top-level window, it's OK to destroy rather than unref'ing. */
+		gtk_widget_destroy (global_prefs_dialog);
+	}
 }
 
 static GtkWidget *
 global_preferences_get_dialog (void)
 {
+	static gboolean set_up_exit = FALSE;
+
 	nautilus_global_preferences_initialize ();
 
-	if (!global_prefs_dialog)
-	{
+	if (global_prefs_dialog == NULL) {
 		global_prefs_dialog = global_preferences_create_dialog ();
 	}
 
+	if (!set_up_exit) {
+		g_atexit (destroy_global_prefs_dialog);
+		set_up_exit = TRUE;
+	}
+
 	return global_prefs_dialog;
 }
 
 /* FIXME bugzilla.eazel.com 1275: 
  * The actual defaults need to be user level specific.
  */
-static const char *default_sidebar_panel_iids[] =
+static const char *novice_default_sidebar_panel_iids[] =
+{
+	"OAFIID:nautilus_notes_view:7f04c3cb-df79-4b9a-a577-38b19ccd4185",
+	"OAFIID:hyperbola_navigation_tree:57542ce0-71ff-442d-a764-462c92514234",
+	"OAFIID:nautilus_history_view:a7a85bdd-2ecf-4bc1-be7c-ed328a29aacb",
+	NULL
+};
+
+static const char *intermediate_default_sidebar_panel_iids[] =
 {
 	"OAFIID:nautilus_history_view:a7a85bdd-2ecf-4bc1-be7c-ed328a29aacb",
 	"OAFIID:nautilus_notes_view:7f04c3cb-df79-4b9a-a577-38b19ccd4185",
 	"OAFIID:nautilus_tree_view:2d826a6e-1669-4a45-94b8-23d65d22802d",
-	"OAFIID:hyperbola_navigation_tree:57542ce0-71ff-442d-a764-462c92514234"
+	"OAFIID:hyperbola_navigation_tree:57542ce0-71ff-442d-a764-462c92514234",
+	NULL
+
 };
 
+static const char *hacker_default_sidebar_panel_iids[] = 
+{
+	"OAFIID:nautilus_notes_view:7f04c3cb-df79-4b9a-a577-38b19ccd4185",
+	"OAFIID:nautilus_history_view:a7a85bdd-2ecf-4bc1-be7c-ed328a29aacb",
+	"OAFIID:nautilus_tree_view:2d826a6e-1669-4a45-94b8-23d65d22802d",
+	"OAFIID:hyperbola_navigation_tree:57542ce0-71ff-442d-a764-462c92514234",
+	NULL
+};
+
+static const char **
+get_default_sidebar_iids_for_user_level (guint user_level)
+{
+
+	switch (user_level) {
+	case NAUTILUS_USER_LEVEL_NOVICE:
+		return novice_default_sidebar_panel_iids;
+		break;
+	case NAUTILUS_USER_LEVEL_INTERMEDIATE:
+		return intermediate_default_sidebar_panel_iids;
+		break;
+	case NAUTILUS_USER_LEVEL_HACKER:
+		return hacker_default_sidebar_panel_iids;
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
+	}
+
+	g_assert_not_reached ();
+	return NULL;
+}
+
 static gboolean
-sidebar_panel_iid_is_in_default_list (const char *iid)
+sidebar_panel_iid_is_in_default_list (const char *iid,
+				      guint user_level)
 {
 	guint i;
+	const char **default_sidebar_panel_iids;
 
 	g_return_val_if_fail (iid != NULL, FALSE);
+	default_sidebar_panel_iids = get_default_sidebar_iids_for_user_level (user_level);
 	
-	for (i = 0; i < NAUTILUS_N_ELEMENTS (default_sidebar_panel_iids); i++) {
+	for (i = 0; default_sidebar_panel_iids[i] != NULL; i++) {
 		if (strcmp (iid, default_sidebar_panel_iids[i]) == 0) {
 			return TRUE;
 		}
@@ -446,21 +519,27 @@ global_preferences_register_sidebar_panels (void)
 	for (p = view_identifiers; p != NULL; p = p->next) {
 		NautilusViewIdentifier	 *identifier;
 		char			 *preference_key;
-		gboolean		 default_value;
+		gboolean		 novice_default_value;
+		gboolean                 intermediate_default_value;
+		gboolean                 hacker_default_value;
 
 		identifier = (NautilusViewIdentifier *) (p->data);
 		g_assert (identifier != NULL);
 		
 		preference_key = global_preferences_get_sidebar_panel_key (identifier->iid);
 		g_assert (preference_key != NULL);
-
-		default_value = sidebar_panel_iid_is_in_default_list (identifier->iid);
 		
+		novice_default_value = sidebar_panel_iid_is_in_default_list (identifier->iid,
+									    NAUTILUS_USER_LEVEL_NOVICE);
+		intermediate_default_value = sidebar_panel_iid_is_in_default_list (identifier->iid,
+										   NAUTILUS_USER_LEVEL_INTERMEDIATE);
+		hacker_default_value = sidebar_panel_iid_is_in_default_list (identifier->iid,
+									     NAUTILUS_USER_LEVEL_HACKER);
 		global_preferences_register_boolean_with_defaults (preference_key,
 								   identifier->name,
-								   default_value,
-								   default_value,
-								   default_value);
+								   novice_default_value,
+								   intermediate_default_value,
+								   hacker_default_value);
 
 		g_free (preference_key);
 	}
@@ -477,8 +556,7 @@ global_preferences_get_sidebar_panel_key (const char *panel_iid)
 }
 
 static gboolean
-global_preferences_is_sidebar_panel_enabled (NautilusViewIdentifier *panel_identifier,
-					     gpointer ignore)
+global_preferences_is_sidebar_panel_enabled (NautilusViewIdentifier *panel_identifier)
 {
 	gboolean enabled;
         gchar	 *key;
@@ -487,7 +565,7 @@ global_preferences_is_sidebar_panel_enabled (NautilusViewIdentifier *panel_ident
 	g_return_val_if_fail (panel_identifier->iid != NULL, FALSE);
 
 	key = global_preferences_get_sidebar_panel_key (panel_identifier->iid);
-	
+
 	g_assert (key != NULL);
 
         enabled = nautilus_preferences_get_boolean (key, FALSE);
@@ -497,6 +575,12 @@ global_preferences_is_sidebar_panel_enabled (NautilusViewIdentifier *panel_ident
         return enabled;
 }
 
+static gboolean
+global_preferences_is_sidebar_panel_enabled_cover (gpointer data, gpointer callback_data)
+{
+	return global_preferences_is_sidebar_panel_enabled (data);
+}
+
 static void
 global_preferences_register_with_defaults (const char			*name,
 					   const char			*description,
@@ -594,6 +678,125 @@ global_preferences_register_speed_tradeoff_with_defaults (const char		     *name
 	
 }							  
 
+
+/* These three callbacks should go away in the future when we can handle system wide prefs */
+static void
+proxy_changed (gpointer user_data)
+{
+	char *proxy, *port, *new_proxy;
+	gboolean use_proxy;
+
+	/* Don't write to the system preference if use_proxy is FALSE */
+	use_proxy = nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_HTTP_USE_PROXY, FALSE);
+	if (!use_proxy) {
+		return;
+	}
+
+	proxy = nautilus_preferences_get (NAUTILUS_PREFERENCES_HTTP_PROXY, "");
+	port = nautilus_preferences_get (NAUTILUS_PREFERENCES_HTTP_PROXY_PORT, "");
+
+	new_proxy = g_strdup_printf ("%s:%s", proxy, port);
+	
+	nautilus_preferences_set (PROXY_KEY, new_proxy);
+
+	g_free (proxy);
+	g_free (port);
+	g_free (new_proxy);
+}
+
+static void
+use_proxy_changed (gpointer user_data)
+{
+	gboolean use_proxy;
+	GConfClient *gconf_client;
+
+	use_proxy = nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_HTTP_USE_PROXY, FALSE);
+	gconf_client = gconf_client_get_default ();
+
+	if (gconf_client != NULL) {
+		gconf_client_set_bool (gconf_client, USE_PROXY_KEY, use_proxy, NULL);
+	}
+
+	if (use_proxy) {
+		proxy_changed (NULL);
+	} else {
+		/* Unset the system key */		
+		if (gconf_client != NULL) {			
+			gconf_client_unset (gconf_client, PROXY_KEY, NULL);
+		}
+	}
+}
+
+static void
+register_proxy_preferences (void)
+{
+	gboolean use_proxy;
+	char *proxy_string, *port, *proxy;
+	GConfClient *gconf_client;
+	
+	use_proxy = FALSE;
+	proxy = NULL;
+	port = NULL;
+	proxy_string = NULL;
+	
+	gconf_client = gconf_client_get_default ();
+	if (gconf_client != NULL) {
+		/* Get system level proxy values from gconf */
+		use_proxy = gconf_client_get_bool (gconf_client, USE_PROXY_KEY, NULL);
+		proxy_string = gconf_client_get_string (gconf_client, PROXY_KEY, NULL);
+		if (proxy_string != NULL) {			
+			port = strchr (proxy_string, ':');
+			if (port != NULL) {
+				proxy = g_strdup (proxy_string);
+				proxy [port - proxy_string] = '\0';
+				port++;								
+			}
+			
+		}						
+		gtk_object_unref (GTK_OBJECT (gconf_client));
+	}
+	
+	if (proxy == NULL) {
+		proxy = g_strdup ("");
+	}
+	if (port == NULL) {
+		port = "8080";
+	}
+
+	nautilus_preferences_set_boolean (NAUTILUS_PREFERENCES_HTTP_USE_PROXY, use_proxy);
+	nautilus_preferences_set (NAUTILUS_PREFERENCES_HTTP_PROXY, proxy);
+	nautilus_preferences_set (NAUTILUS_PREFERENCES_HTTP_PROXY_PORT, port);
+	
+	global_preferences_register_boolean_with_defaults (NAUTILUS_PREFERENCES_HTTP_USE_PROXY,
+							   _("Use HTTP Proxy"),
+							   use_proxy,
+							   use_proxy,
+							   use_proxy);
+	
+	global_preferences_register_string_with_defaults (NAUTILUS_PREFERENCES_HTTP_PROXY,
+							  _("HTTP Proxy"),
+							  proxy,
+							  proxy,
+							  proxy);
+	
+	global_preferences_register_string_with_defaults (NAUTILUS_PREFERENCES_HTTP_PROXY_PORT,
+							  _("HTTP Proxy Port"),
+							  port,
+							  port,
+							  port);
+	
+	g_free (proxy);
+	g_free (proxy_string);
+
+	/* Add a callbacks to update the system setting with the new local setting. This should go away when the
+	 * prefs mechanism can handle system wide prefs.
+	 */
+	nautilus_preferences_add_callback (NAUTILUS_PREFERENCES_HTTP_USE_PROXY, use_proxy_changed, NULL);
+	nautilus_preferences_add_callback (NAUTILUS_PREFERENCES_HTTP_PROXY, proxy_changed, NULL);
+	nautilus_preferences_add_callback (NAUTILUS_PREFERENCES_HTTP_PROXY_PORT, proxy_changed, NULL);
+}
+
+
 /**
  * global_preferences_register
  *
@@ -605,17 +808,41 @@ global_preferences_register_speed_tradeoff_with_defaults (const char		     *name
 static void
 global_preferences_register (void)
 {
-	static gboolean preference_registered = FALSE;
+	static gboolean preferences_registered = FALSE;
 
-	if (preference_registered)
+	if (preferences_registered) {
 		return;
+	}
 
-	preference_registered = TRUE;
+	preferences_registered = TRUE;
 
 	/*
 	 * In the soon to come star trek future, the following information
 	 * will be fetched using the latest xml techniques.
 	 */
+
+
+	/* 
+	 * Non user level specific preferences:
+	 */
+
+	/* FIXME bugzilla.eazel.com 2654: 
+	 * Cannot set default values for non user level preferecens.  Yes, this
+	 * is bad.  The right way to fix this problem (and many other default
+	 * value problems) is to use gconf schemas.  See the bug for info.
+	 */
+
+#if 0
+	global_preferences_register_string_with_defaults (NAUTILUS_PREFERENCES_THEME,
+							  _("current theme"),
+							  "default",
+							  "default",
+							  "default");
+#endif
+
+	/* 
+	 * User user level specific preferences:
+	 */
 	
 	/* Window create new */
 	global_preferences_register_boolean_with_defaults (NAUTILUS_PREFERENCES_WINDOW_ALWAYS_NEW,
@@ -624,12 +851,19 @@ global_preferences_register (void)
 							   FALSE,
 							   FALSE);
 	
+	/* Trash confirm */
+	global_preferences_register_boolean_with_defaults (NAUTILUS_PREFERENCES_CONFIRM_TRASH,
+							   _("Ask before deleting items from the trash"),
+							   TRUE,
+							   TRUE,
+							   TRUE);
+	
 	/* Click activation type */
 	global_preferences_register_enum_with_defaults (NAUTILUS_PREFERENCES_CLICK_POLICY,
 							_("Click policy"),
-							NAUTILUS_CLICK_POLICY_SINGLE,
-							NAUTILUS_CLICK_POLICY_SINGLE,
-							NAUTILUS_CLICK_POLICY_SINGLE);
+							NAUTILUS_CLICK_POLICY_DOUBLE,
+							NAUTILUS_CLICK_POLICY_DOUBLE,
+							NAUTILUS_CLICK_POLICY_DOUBLE);
 	
 	nautilus_preference_enum_add_entry_by_name (NAUTILUS_PREFERENCES_CLICK_POLICY,
 						    _("single"),
@@ -655,7 +889,7 @@ global_preferences_register (void)
 							   	  NAUTILUS_SPEED_TRADEOFF_ALWAYS);
 	
 	global_preferences_register_speed_tradeoff_with_defaults (NAUTILUS_PREFERENCES_USE_PUBLIC_METADATA,
-							   	  _("Read and write metadata in each directory"),
+							   	  _("Read and write metadata in each folder"),
 							   	  NAUTILUS_SPEED_TRADEOFF_ALWAYS,
 							   	  NAUTILUS_SPEED_TRADEOFF_ALWAYS,
 							   	  NAUTILUS_SPEED_TRADEOFF_ALWAYS);
@@ -664,18 +898,12 @@ global_preferences_register (void)
 	global_preferences_register_sidebar_panels ();
 
 	/* Appearance options */
-	global_preferences_register_boolean_with_defaults (NAUTILUS_PREFERENCES_ANTI_ALIASED_CANVAS,
+	global_preferences_register_boolean_with_defaults (NAUTILUS_PREFERENCES_SMOOTH_GRAPHICS_MODE,
 							   _("Use smoother (but slower) graphics"),
-							   TRUE,
-							   TRUE,
-							   TRUE);
+							   FALSE,
+							   FALSE,
+							   FALSE);
 
-	global_preferences_register_string_with_defaults (NAUTILUS_PREFERENCES_THEME,
-							  _("current theme"),
-							  "default",
-							  "default",
-							  "default");
-	
 	global_preferences_register_string_with_defaults (NAUTILUS_PREFERENCES_DIRECTORY_VIEW_FONT_FAMILY,
 							  _("Font family used to display file names"),
 							  "helvetica",
@@ -683,30 +911,35 @@ global_preferences_register (void)
 							  "helvetica");
 
 
-	global_preferences_register_boolean_with_defaults (NAUTILUS_PREFERENCES_DISPLAY_TOOLBAR,
-							   _("Display tool bar"),
+	global_preferences_register_boolean_with_defaults (NAUTILUS_PREFERENCES_START_WITH_TOOL_BAR,
+							   _("Display tool bar in new windows"),
 							   TRUE,
 							   TRUE,
 							   TRUE);
 
-	global_preferences_register_boolean_with_defaults (NAUTILUS_PREFERENCES_DISPLAY_LOCATIONBAR,
-							   _("Display location bar"),
+	global_preferences_register_boolean_with_defaults (NAUTILUS_PREFERENCES_START_WITH_LOCATION_BAR,
+							   _("Display location bar in new windows"),
 							   TRUE,
 							   TRUE,
 							   TRUE);
 
-	global_preferences_register_boolean_with_defaults (NAUTILUS_PREFERENCES_DISPLAY_STATUSBAR,
-							   _("Display status bar"),
+	global_preferences_register_boolean_with_defaults (NAUTILUS_PREFERENCES_START_WITH_STATUS_BAR,
+							   _("Display status bar in new windows"),
 							   TRUE,
 							   TRUE,
 							   TRUE);
 
-	global_preferences_register_boolean_with_defaults (NAUTILUS_PREFERENCES_DISPLAY_SIDEBAR,
-							   _("Display Sidebar"),
+	global_preferences_register_boolean_with_defaults (NAUTILUS_PREFERENCES_START_WITH_SIDEBAR,
+							   _("Display sidebar in new windows"),
 							   TRUE,
 							   TRUE,
 							   TRUE);
 
+	global_preferences_register_boolean_with_defaults (NAUTILUS_PREFERENCES_SHOW_DESKTOP,
+							   _("Use Nautilus to draw the desktop"),
+							   FALSE,
+							   FALSE,
+							   FALSE);
 								  
 	/* search tradeoffs */
 	global_preferences_register_boolean_with_defaults (NAUTILUS_PREFERENCES_SEARCH_METHOD,
@@ -741,7 +974,13 @@ global_preferences_register (void)
 
 	
 	global_preferences_register_boolean_with_defaults (NAUTILUS_PREFERENCES_SHOW_HIDDEN_FILES,
-							   _("Show hidden files"),
+							   _("Show hidden files (starting with \".\")"),
+							   FALSE,
+							   FALSE,
+							   TRUE);
+
+	global_preferences_register_boolean_with_defaults (NAUTILUS_PREFERENCES_SHOW_BACKUP_FILES,
+							   _("Show backup files (ending with \"~\")"),
 							   FALSE,
 							   FALSE,
 							   TRUE);
@@ -756,15 +995,13 @@ global_preferences_register (void)
 							   _("Can add Content"),
 							   FALSE,
 							   TRUE,
-							   TRUE);
-
- 	/* Default sidebar width */
-	global_preferences_register_enum_with_defaults (NAUTILUS_PREFERENCES_SIDEBAR_WIDTH,
-							_("Sidebar Width"),
-							148,
-							148,
-							148);
-	
+							   TRUE);	
+	/* built-in bookmarks */
+	global_preferences_register_boolean_with_defaults (NAUTILUS_PREFERENCES_HIDE_BUILT_IN_BOOKMARKS,
+							   _("Don't include the built-in bookmarks"),
+							   FALSE,
+							   FALSE,
+							   FALSE);
 
 	{
 		char	*user_main_directory;
@@ -789,6 +1026,8 @@ global_preferences_register (void)
 		g_free (intermediate_home_location);
 		g_free (hacker_home_location);
 	}
+
+	register_proxy_preferences ();
 }
 
 static gboolean
@@ -841,9 +1080,10 @@ nautilus_global_preferences_dialog_update (void)
 		was_showing = GTK_WIDGET_VISIBLE (global_prefs_dialog);
 
 		gtk_widget_destroy (global_prefs_dialog);
+		global_prefs_dialog = NULL;
 	}
 	
-	global_prefs_dialog = global_preferences_create_dialog ();
+	global_preferences_get_dialog ();
 
 	if (was_showing) {
 		nautilus_global_preferences_show_dialog ();
@@ -863,16 +1103,3 @@ nautilus_global_preferences_initialize (void)
 
 	initialized = TRUE;
 }
-
-void
-nautilus_global_preferences_shutdown (void)
-{
-	/* Free the dialog first, cause it has refs to preferences */
-	if (global_prefs_dialog != NULL) {
-		gtk_widget_destroy (global_prefs_dialog);
-	}
-
-	/* Now free the preferences tables and stuff */
-	nautilus_preferences_shutdown ();
-}
-
