@@ -32,25 +32,35 @@
 #include <gtk/gtksignal.h>
 #include <libgnomeui/gnome-geometry.h>
 #include "nautilus-glib-extensions.h"
+#include "nautilus-string.h"
 
 /* This number should be large enough to be visually noticeable,
  * but small enough to not allow the user to perform other actions.
  */
 #define BUTTON_AUTO_HIGHLIGHT_MILLISECONDS	100
 
+/* This number is fairly arbitrary. Long enough to show a pretty long
+ * menu title, but not so long to make a menu grotesquely wide.
+ */
+#define MAXIMUM_MENU_TITLE_LENGTH	48
+
+/* Used for window position & size sanity-checking. The sizes are big enough to prevent
+ * at least normal-sized gnome panels from obscuring the window at the screen edges. 
+ */
+#define MINIMUM_ON_SCREEN_WIDTH		100
+#define MINIMUM_ON_SCREEN_HEIGHT	100
+
 static gboolean
 finish_button_activation (gpointer data)
 {
 	GtkButton *button;
 	
-	g_assert (GTK_IS_BUTTON (data));
-
 	button = GTK_BUTTON (data);
 
 	if (!button->in_button) {
-		gtk_button_clicked (GTK_BUTTON (data));
+		gtk_button_clicked (button);
 	}
-	gtk_button_released (GTK_BUTTON (data));
+	gtk_button_released (button);
 
 	return FALSE;	
 }
@@ -67,11 +77,15 @@ nautilus_gtk_button_auto_click (GtkButton *button)
 {
 	g_return_if_fail (GTK_IS_BUTTON (button));
 
+	if (!GTK_WIDGET_IS_SENSITIVE (GTK_WIDGET (button))) {
+		return;
+	}
+
 	button->in_button = TRUE;
 	gtk_button_pressed (button);
 	button->in_button = FALSE;
 
-	/* FIXME:
+	/* FIXME bugzilla.eazel.com 2562:
 	 * Nothing is preventing other events from occuring between
 	 * now and when this timeout function fires, which means in
 	 * theory the user could click on a different row or otherwise
@@ -298,6 +312,129 @@ nautilus_gtk_window_present (GtkWindow *window) {
 	gtk_widget_show (GTK_WIDGET (window));
 }
 
+static void
+sanity_check_window_geometry (int *left, int *top, int *width, int *height)
+{
+	g_assert (left != NULL);
+	g_assert (top != NULL);
+	g_assert (width != NULL);
+	g_assert (height != NULL);
+
+	/* Pin the size of the window to the screen, so we don't end up in
+	 * a state where the window is so big essential parts of it can't
+	 * be reached (might not be necessary with all window managers,
+	 * but seems reasonable anyway).
+	 */
+	*width = MIN (*width, gdk_screen_width());
+	*height = MIN (*height, gdk_screen_height());
+	
+	/* Make sure the top of the window is on screen, for
+	 * draggability (might not be necessary with all window managers,
+	 * but seems reasonable anyway). Make sure the top of the window
+	 * isn't off the bottom of the screen, or so close to the bottom
+	 * that it might be obscured by the panel.
+	 */
+	*top = CLAMP (*top, 0, gdk_screen_height() - MINIMUM_ON_SCREEN_HEIGHT);
+	
+	/* FIXME bugzilla.eazel.com 669: 
+	 * If window has negative left coordinate, set_uposition sends it
+	 * somewhere else entirely. Not sure what level contains this bug (XWindows?).
+	 * Hacked around by pinning the left edge to zero, which just means you
+	 * can't set a window to be partly off the left of the screen using
+	 * this routine.
+	 */
+	/* Make sure the left edge of the window isn't off the right edge of
+	 * the screen, or so close to the right edge that it might be
+	 * obscured by the panel.
+	 */
+	*left = CLAMP (*left, 0, gdk_screen_width() - MINIMUM_ON_SCREEN_WIDTH);
+}
+
+
+/**
+ * nautilus_gtk_window_set_initial_geometry:
+ * 
+ * Sets the position and size of a GtkWindow before the
+ * GtkWindow is shown. It is an error to call this on a window that
+ * is already on-screen. Takes into account screen size, and does
+ * some sanity-checking on the passed-in values.
+ * 
+ * @window: A non-visible GtkWindow
+ * @left: pixel coordinate for left of window
+ * @top: pixel coordinate for top of window
+ * @width: width of window in pixels
+ * @height: height of window in pixels
+ */
+void
+nautilus_gtk_window_set_initial_geometry (GtkWindow *window, 
+					  int left,
+					  int top,
+					  int width,
+					  int height)
+{
+	g_return_if_fail (GTK_IS_WINDOW (window));
+	g_return_if_fail (width > 0 && height > 0);
+
+	/* Setting the default size doesn't work when the window is already showing.
+	 * Someday we could make this move an already-showing window, but we don't
+	 * need that functionality yet. 
+	 */
+	g_return_if_fail (!GTK_WIDGET_VISIBLE (window));
+
+	sanity_check_window_geometry (&left, &top, &width, &height);
+			    		
+	gtk_widget_set_uposition (GTK_WIDGET (window), left, top);
+	gtk_window_set_default_size (GTK_WINDOW (window), width, height);
+}
+
+/**
+ * nautilus_gtk_window_set_initial_geometry_from_string:
+ * 
+ * Sets the position and size of a GtkWindow before the
+ * GtkWindow is shown. The geometry is passed in as a string. 
+ * It is an error to call this on a window that
+ * is already on-screen. Takes into account screen size, and does
+ * some sanity-checking on the passed-in values.
+ * 
+ * @window: A non-visible GtkWindow
+ * @geometry_string: A string suitable for use with gnome_parse_geometry
+ * @minimum_width: If the width from the string is smaller than this,
+ * use this for the width.
+ * @minimum_height: If the height from the string is smaller than this,
+ * use this for the height.
+ */
+void
+nautilus_gtk_window_set_initial_geometry_from_string (GtkWindow *window, 
+					  	      const char *geometry_string,
+					  	      guint minimum_width,
+					  	      guint minimum_height)
+{
+	int left, top, width, height;
+	gboolean parsed;
+
+	g_return_if_fail (GTK_IS_WINDOW (window));
+	g_return_if_fail (geometry_string != NULL);
+
+	/* Setting the default size doesn't work when the window is already showing.
+	 * Someday we could make this move an already-showing window, but we don't
+	 * need that functionality yet. 
+	 */
+	g_return_if_fail (!GTK_WIDGET_VISIBLE (window));
+
+	parsed = gnome_parse_geometry (geometry_string, &left, &top, &width, &height);
+
+	/* Bogus string, dude. */
+	g_return_if_fail (parsed);
+
+	/* Make sure the window isn't smaller than makes sense for this window.
+	 * Other sanity checks are performed in set_initial_geometry.
+	 */
+	width = MAX (width, minimum_width);
+	height = MAX (height, minimum_height);
+
+	nautilus_gtk_window_set_initial_geometry (window, left, top, width, height);
+}
+
 /**
  * nautilus_gtk_selection_data_copy_deep:
  * 
@@ -357,7 +494,7 @@ nautilus_popup_menu_position_func (GtkMenu   *menu,
 	g_assert (offset != NULL);
 
 	/*
-	 * FIXME: The cast from gint16 might cause problems.  
+	 * FIXME bugzilla.eazel.com 2561: The cast from gint16 might cause problems.  
 	 * Unfortunately, GdkPoint uses gint16.
 	 */
 	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
@@ -366,6 +503,19 @@ nautilus_popup_menu_position_func (GtkMenu   *menu,
 	*y = CLAMP (*y + (int) offset->y, 0, MAX (0, gdk_screen_height () - requisition.height));
 }
 
+/**
+ * nautilus_truncate_text_for_menu_item:
+ * 
+ * Given an arbitrary string, returns a newly-allocated string
+ * suitable for use as a menu item label. Truncates long strings 
+ * in the middle.
+ */
+char *
+nautilus_truncate_text_for_menu_item (const char *text)
+{
+	return nautilus_str_middle_truncate (text, MAXIMUM_MENU_TITLE_LENGTH);
+}
+
 /**
  * nautilus_pop_up_context_menu:
  * 
@@ -413,6 +563,45 @@ nautilus_pop_up_context_menu (GtkMenu	*menu,
 	gtk_object_sink (GTK_OBJECT(menu));
 }
 
+GtkMenuItem *
+nautilus_gtk_menu_append_separator (GtkMenu *menu)
+{
+	return nautilus_gtk_menu_insert_separator (menu, -1);
+}
+
+GtkMenuItem *
+nautilus_gtk_menu_insert_separator (GtkMenu *menu, int index)
+{
+	GtkWidget *menu_item;
+
+	menu_item = gtk_menu_item_new ();
+	gtk_widget_set_sensitive (menu_item, FALSE);
+	gtk_widget_show (menu_item);
+	gtk_menu_insert (menu, menu_item, index);
+
+	return GTK_MENU_ITEM (menu_item);
+}
+
+void
+nautilus_gtk_menu_set_item_visibility (GtkMenu *menu, int index, gboolean visible)
+{
+	GList *children;
+	GtkWidget *menu_item;
+
+	g_return_if_fail (GTK_IS_MENU (menu));
+
+	children = gtk_container_children (GTK_CONTAINER (menu));
+	g_return_if_fail (index >= 0 && index < g_list_length (children));
+
+	menu_item = GTK_WIDGET (g_list_nth_data (children, index));
+	if (visible) {
+		gtk_widget_show (menu_item);
+	} else {
+		gtk_widget_hide (menu_item);
+	}
+
+	g_list_free (children);
+}
 
 void
 nautilus_gtk_marshal_NONE__POINTER_INT_INT_DOUBLE (GtkObject *object,
@@ -489,6 +678,41 @@ nautilus_gtk_marshal_NONE__POINTER_POINTER_INT_INT_INT (GtkObject *object,
 		 func_data);
 }
 
+void
+nautilus_gtk_marshal_BOOL__INT_POINTER_INT_INT_UINT (GtkObject *object,
+						     GtkSignalFunc func,
+						     gpointer func_data,
+						     GtkArg *args)
+{
+	* GTK_RETLOC_BOOL (args[5]) = (* (gboolean (*)(GtkObject *, int, gpointer, int, int, guint, gpointer)) func)
+		(object,
+		 GTK_VALUE_INT (args[0]),
+		 GTK_VALUE_POINTER (args[1]),
+		 GTK_VALUE_INT (args[2]),
+		 GTK_VALUE_INT (args[3]),
+		 GTK_VALUE_UINT (args[4]),
+		 func_data);
+}
+
+
+void
+nautilus_gtk_marshal_NONE__INT_POINTER_INT_INT_UINT (GtkObject *object,
+						     GtkSignalFunc func,
+						     gpointer func_data,
+						     GtkArg *args)
+{
+	(* (void (*)(GtkObject *, int, gpointer, int, int, guint, gpointer)) func)
+		(object,
+		 GTK_VALUE_INT (args[0]),
+		 GTK_VALUE_POINTER (args[1]),
+		 GTK_VALUE_INT (args[2]),
+		 GTK_VALUE_INT (args[3]),
+		 GTK_VALUE_UINT (args[4]),
+		 func_data);
+
+}
+
+
 void
 nautilus_gtk_marshal_NONE__POINTER_POINTER_POINTER_INT_INT_INT (GtkObject *object,
 								GtkSignalFunc func,
@@ -506,6 +730,24 @@ nautilus_gtk_marshal_NONE__POINTER_POINTER_POINTER_INT_INT_INT (GtkObject *objec
 		 func_data);
 }
 
+void
+nautilus_gtk_marshal_NONE__POINTER_POINTER_POINTER_POINTER_INT_INT_UINT (GtkObject *object,
+									 GtkSignalFunc func,
+									 gpointer func_data,
+									 GtkArg *args)
+{
+	(* (void (*)(GtkObject *, gpointer, gpointer, gpointer, gpointer, int, int, guint, gpointer)) func)
+		(object,
+		 GTK_VALUE_POINTER (args[0]),
+		 GTK_VALUE_POINTER (args[1]),
+		 GTK_VALUE_POINTER (args[2]),
+		 GTK_VALUE_POINTER (args[3]),
+		 GTK_VALUE_INT (args[4]),
+		 GTK_VALUE_INT (args[5]),
+		 GTK_VALUE_INT (args[6]),
+		 func_data);
+}
+
 void
 nautilus_gtk_marshal_NONE__POINTER_INT_INT_DOUBLE_DOUBLE (GtkObject *object,
 							  GtkSignalFunc func,
@@ -818,7 +1060,7 @@ nautilus_gtk_widget_set_font_by_name (GtkWidget *widget, const char *font_name)
 	g_return_if_fail (GTK_IS_WIDGET (widget));
 	g_return_if_fail (font_name != NULL);
 	
-	font = gdk_font_load (font_name);
+	font = gdk_fontset_load (font_name);
 	nautilus_gtk_widget_set_font (widget, font);
 	gdk_font_unref (font);
 }
@@ -1002,15 +1244,254 @@ nautilus_gtk_label_make_bold (GtkLabel *label)
 	GdkFont *bold_font;
 
 	g_return_if_fail (GTK_IS_LABEL (label));
-	style = gtk_widget_get_style (GTK_WIDGET(label));
+
+	style = gtk_widget_get_style (GTK_WIDGET (label));
 
 	bold_font = nautilus_gdk_font_get_bold (style->font);
-
 	if (bold_font == NULL) {
 		return;
 	}
-
-	nautilus_gtk_widget_set_font (GTK_WIDGET(label), bold_font);
+	nautilus_gtk_widget_set_font (GTK_WIDGET (label), bold_font);
 	gdk_font_unref (bold_font);
 }
 
+void
+nautilus_gtk_widget_set_background_color (GtkWidget              *widget,
+					  const char		*color_spec)
+{
+	GtkStyle	*style;
+	GdkColor	color;
+
+	g_return_if_fail (GTK_IS_WIDGET (widget));
+
+	style = gtk_widget_get_style (widget);
+
+	/* Make a copy of the style. */
+	style = gtk_style_copy (style);
+
+	nautilus_gdk_color_parse_with_white_default (color_spec, &color);
+	style->bg[GTK_STATE_NORMAL] = color;
+	style->base[GTK_STATE_NORMAL] = color;
+	style->bg[GTK_STATE_ACTIVE] = color;
+	style->base[GTK_STATE_ACTIVE] = color;
+
+	/* Put the style in the widget. */
+	gtk_widget_set_style (widget, style);
+	gtk_style_unref (style);
+}
+
+void
+nautilus_gtk_widget_set_foreground_color (GtkWidget              *widget,
+					  const char		*color_spec)
+{
+	GtkStyle	*style;
+	GdkColor	color;
+
+	g_return_if_fail (GTK_IS_WIDGET (widget));
+
+	style = gtk_widget_get_style (widget);
+
+	/* Make a copy of the style. */
+	style = gtk_style_copy (style);
+
+	nautilus_gdk_color_parse_with_white_default (color_spec, &color);
+	style->fg[GTK_STATE_NORMAL] = color;
+	style->fg[GTK_STATE_ACTIVE] = color;
+
+	/* Put the style in the widget. */
+	gtk_widget_set_style (widget, style);
+	gtk_style_unref (style);
+}
+
+GtkWidget *
+nautilus_gtk_widget_find_windowed_ancestor (GtkWidget *widget)
+{
+	g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
+
+	while (widget && GTK_WIDGET_NO_WINDOW (widget)) {
+		widget = widget->parent;
+	}
+
+	return widget;
+}
+
+
+
+/*following code shamelessly stolen from gtk*/
+static void
+rgb_to_hls (gdouble *r,
+            gdouble *g,
+            gdouble *b)
+{
+	gdouble min;
+	gdouble max;
+	gdouble red;
+	gdouble green;
+	gdouble blue;
+	gdouble h, l, s;
+	gdouble delta;
+
+	red = *r;
+	green = *g;
+	blue = *b;
+
+	if (red > green) {
+		if (red > blue)
+			max = red;
+		else
+			max = blue;
+		
+		if (green < blue)
+			min = green;
+		else
+			min = blue;
+	} else {
+		if (green > blue)
+			max = green;
+		else
+			max = blue;
+		
+		if (red < blue)
+			min = red;
+		else
+			min = blue;
+	}
+
+	l = (max + min) / 2;
+	s = 0;
+	h = 0;
+
+	if (max != min) {
+		if (l <= 0.5)
+			s = (max - min) / (max + min);
+		else
+			s = (max - min) / (2 - max - min);
+		
+		delta = max -min;
+		if (red == max)
+			h = (green - blue) / delta;
+		else if (green == max)
+			h = 2 + (blue - red) / delta;
+		else if (blue == max)
+			h = 4 + (red - green) / delta;
+		
+		h *= 60;
+		if (h < 0.0)
+			h += 360;
+	}
+	
+	*r = h;
+	*g = l;
+	*b = s;
+}
+
+static void
+hls_to_rgb (gdouble *h,
+            gdouble *l,
+            gdouble *s)
+{
+	gdouble hue;
+	gdouble lightness;
+	gdouble saturation;
+	gdouble m1, m2;
+	gdouble r, g, b;
+	
+	lightness = *l;
+	saturation = *s;
+
+	if (lightness <= 0.5)
+		m2 = lightness * (1 + saturation);
+	else
+		m2 = lightness + saturation - lightness * saturation;
+	m1 = 2 * lightness - m2;
+
+	if (saturation == 0) {
+		*h = lightness;
+		*l = lightness;
+		*s = lightness;
+	} else {
+		hue = *h + 120;
+		while (hue > 360)
+			hue -= 360;
+		while (hue < 0)
+			hue += 360;
+		
+		if (hue < 60)
+			r = m1 + (m2 - m1) * hue / 60;
+		else if (hue < 180)
+			r = m2;
+		else if (hue < 240)
+			r = m1 + (m2 - m1) * (240 - hue) / 60;
+		else
+			r = m1;
+
+		hue = *h;
+		while (hue > 360)
+			hue -= 360;
+		while (hue < 0)
+			hue += 360;
+		
+		if (hue < 60)
+			g = m1 + (m2 - m1) * hue / 60;
+		else if (hue < 180)
+			g = m2;
+		else if (hue < 240)
+			g = m1 + (m2 - m1) * (240 - hue) / 60;
+		else
+			g = m1;
+		
+		hue = *h - 120;
+		while (hue > 360)
+			hue -= 360;
+		while (hue < 0)
+			hue += 360;
+		
+		if (hue < 60)
+			b = m1 + (m2 - m1) * hue / 60;
+		else if (hue < 180)
+			b = m2;
+		else if (hue < 240)
+			b = m1 + (m2 - m1) * (240 - hue) / 60;
+		else
+			b = m1;
+		
+		*h = r;
+		*l = g;
+		*s = b;
+	}
+}
+
+void
+nautilus_gtk_style_shade (GdkColor *a,
+			  GdkColor *b,
+			  gdouble   k)
+{
+
+	gdouble red;
+	gdouble green;
+	gdouble blue;
+	
+	red = (gdouble) a->red / 65535.0;
+	green = (gdouble) a->green / 65535.0;
+	blue = (gdouble) a->blue / 65535.0;
+	
+	rgb_to_hls (&red, &green, &blue);
+
+	green *= k;
+	if (green > 1.0)
+		green = 1.0;
+	else if (green < 0.0)
+		green = 0.0;
+
+	blue *= k;
+	if (blue > 1.0)
+		blue = 1.0;
+	else if (blue < 0.0)
+		blue = 0.0;
+
+	hls_to_rgb (&red, &green, &blue);
+
+	b->red = red * 65535.0;
+	b->green = green * 65535.0;
+	b->blue = blue * 65535.0;
+}
