@@ -30,26 +30,32 @@
 #include <config.h>
 #include "nautilus-icon-dnd.h"
 
-#include <math.h>
-#include <string.h>
-#include <stdio.h>
-#include <gdk/gdkx.h>
-#include <gdk/gdkkeysyms.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkmain.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomeui/gnome-stock.h>
-#include <libgnomeui/gnome-canvas-rect-ellipse.h>
+#include "nautilus-background.h"
+#include "nautilus-file-utilities.h"
+#include "nautilus-gdk-pixbuf-extensions.h"
 #include "nautilus-glib-extensions.h"
+#include "nautilus-gnome-extensions.h"
+#include "nautilus-graphic-effects.h"
 #include "nautilus-gtk-extensions.h"
 #include "nautilus-gtk-macros.h"
-#include "nautilus-gnome-extensions.h"
-#include "nautilus-background.h"
-#include "nautilus-graphic-effects.h"
-#include "nautilus-stock-dialogs.h"
-
 #include "nautilus-icon-private.h"
+#include "nautilus-link.h"
+#include "nautilus-stock-dialogs.h"
+#include "nautilus-string.h"
+#include <gdk/gdkkeysyms.h>
+#include <gdk/gdkx.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtksignal.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnome/gnome-mime.h>
+#include <libgnomeui/gnome-canvas-rect-ellipse.h>
+#include <libgnomeui/gnome-stock.h>
+#include <libgnomeui/gnome-uidefs.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
+#include <math.h>
+#include <stdio.h>
+#include <string.h>
 
 static gboolean drag_drop_callback                   (GtkWidget             *widget,
 						      GdkDragContext        *context,
@@ -67,6 +73,24 @@ static gboolean drag_motion_callback                 (GtkWidget             *wid
 						      int                    y,
 						      guint32                time);
 
+static void     nautilus_icon_container_receive_dropped_icons    (NautilusIconContainer *container,
+								  GdkDragContext *context,
+								  int x, int y);
+static void     receive_dropped_tile_image                       (NautilusIconContainer *container, 
+								  GtkSelectionData *data);
+static void     receive_dropped_keyword                          (NautilusIconContainer *container, 
+								  char* keyword, 
+								  int x, 
+								  int y);
+static void     receive_dropped_uri_list                          (NautilusIconContainer *container, 
+								  char* keyword, 
+								  int x, 
+								  int y);
+static void     nautilus_icon_container_free_drag_data           (NautilusIconContainer *container);
+static void     set_drop_target                                  (NautilusIconContainer *container,
+								  NautilusIcon *icon);
+
+
 static GtkTargetEntry drag_types [] = {
 	{ NAUTILUS_ICON_DND_GNOME_ICON_LIST_TYPE, 0, NAUTILUS_ICON_DND_GNOME_ICON_LIST },
 	{ NAUTILUS_ICON_DND_URI_LIST_TYPE, 0, NAUTILUS_ICON_DND_URI_LIST },
@@ -82,24 +106,9 @@ static GtkTargetEntry drop_types [] = {
 	{ NAUTILUS_ICON_DND_KEYWORD_TYPE, 0, NAUTILUS_ICON_DND_KEYWORD }
 };
 
-#define AUTOSCROLL_TIMEOUT_INTERVAL 100
-	/* in milliseconds */
 
-#define AUTOSCROLL_INITIAL_DELAY 600000
-	/* in microseconds */
-
-#define AUTO_SCROLL_MARGIN 20
-	/* drag this close to the view edge to start auto scroll*/
-
-#define MIN_AUTOSCROLL_DELTA 5
-	/* the smallest amount of auto scroll used when we just enter the autoscroll
-	 * margin
-	 */
-	 
-#define MAX_AUTOSCROLL_DELTA 50
-	/* the largest amount of auto scroll used when we are right over the view
-	 * edge
-	 */
+/* special reserved name for the erase emblem */
+#define ERASE_KEYWORD "erase"
 
 static GnomeCanvasItem *
 create_selection_shadow (NautilusIconContainer *container,
@@ -270,7 +279,7 @@ nautilus_icon_container_each_selected_icon (NautilusIconContainer *container,
 }
 
 /* Adaptor function used with nautilus_icon_container_each_selected_icon
- * to help iterate over all selected items, passing uris, x,y,w and h
+ * to help iterate over all selected items, passing uris, x, y, w and h
  * values to the iteratee
  */
 static void
@@ -324,9 +333,9 @@ nautilus_icon_container_position_shadow (NautilusIconContainer *container,
 	if (shadow == NULL) {
 		return;
 	}
-
 	gnome_canvas_window_to_world (GNOME_CANVAS (container),
 				      x, y, &world_x, &world_y);
+
 	set_shadow_position (shadow, world_x, world_y);
 	gnome_canvas_item_show (shadow);
 }
@@ -348,7 +357,8 @@ nautilus_icon_container_dropped_icon_feedback (GtkWidget *widget,
 
 	/* Delete old shadow if any. */
 	if (dnd_info->shadow != NULL) {
-		/* FIXME: Is a destroy really sufficient here? Who does the unref? */
+		/* FIXME bugzilla.eazel.com 2484: 
+		 * Is a destroy really sufficient here? Who does the unref? */
 		gtk_object_destroy (GTK_OBJECT (dnd_info->shadow));
 	}
 
@@ -358,6 +368,14 @@ nautilus_icon_container_dropped_icon_feedback (GtkWidget *widget,
 	nautilus_icon_container_position_shadow (container, x, y);
 }
 
+/** this callback is called in 2 cases.
+    It is called upon drag_motion events to get the actual data 
+    In that case, it just makes sure it gets the data.
+    It is called upon drop_drop events to execute the actual 
+    actions on the received action. In that case, it actually fist makes sure
+    that we have got the data then processes it.
+*/
+
 static void
 drag_data_received_callback (GtkWidget *widget,
 			     GdkDragContext *context,
@@ -368,12 +386,12 @@ drag_data_received_callback (GtkWidget *widget,
 			     guint32 time,
 			     gpointer user_data)
 {
-    	NautilusIconDndInfo *dnd_info;
+    	NautilusDragInfo *drag_info;
 
-	dnd_info = NAUTILUS_ICON_CONTAINER (widget)->details->dnd_info;
+	drag_info = &(NAUTILUS_ICON_CONTAINER (widget)->details->dnd_info->drag_info);
 
-	dnd_info->drag_info.got_drop_data_type = TRUE;
-	dnd_info->drag_info.data_type = info;
+	drag_info->got_drop_data_type = TRUE;
+	drag_info->data_type = info;
 
 	switch (info) {
 	case NAUTILUS_ICON_DND_GNOME_ICON_LIST:
@@ -382,13 +400,65 @@ drag_data_received_callback (GtkWidget *widget,
 	case NAUTILUS_ICON_DND_COLOR:
 	case NAUTILUS_ICON_DND_BGIMAGE:	
 	case NAUTILUS_ICON_DND_KEYWORD:	
+	case NAUTILUS_ICON_DND_URI_LIST:
 		/* Save the data so we can do the actual work on drop. */
-		g_assert (dnd_info->drag_info.selection_data == NULL);
-		dnd_info->drag_info.selection_data = nautilus_gtk_selection_data_copy_deep (data);
+		g_assert (drag_info->selection_data == NULL);
+		drag_info->selection_data = nautilus_gtk_selection_data_copy_deep (data);
 		break;
 	default:
 		break;
 	}
+
+
+	/* this is the second use case of this callback.
+	 * we have to do the actual work for the drop.
+	 */
+	if (drag_info->drop_occured) {
+
+		switch (info) {
+		case NAUTILUS_ICON_DND_GNOME_ICON_LIST:
+			nautilus_icon_container_receive_dropped_icons
+				(NAUTILUS_ICON_CONTAINER (widget),
+				 context, x, y);
+			gtk_drag_finish (context, TRUE, FALSE, time);
+			break;
+		case NAUTILUS_ICON_DND_COLOR:
+			nautilus_background_receive_dropped_color
+				(nautilus_get_widget_background (widget),
+				 widget, x, y, data);
+			gtk_drag_finish (context, TRUE, FALSE, time);
+			break;
+		case NAUTILUS_ICON_DND_BGIMAGE:
+			receive_dropped_tile_image
+				(NAUTILUS_ICON_CONTAINER (widget),
+				 data);
+			gtk_drag_finish (context, FALSE, FALSE, time);
+			break;
+		case NAUTILUS_ICON_DND_KEYWORD:
+			receive_dropped_keyword
+				(NAUTILUS_ICON_CONTAINER (widget),
+				 (char*) data->data, x, y);
+			gtk_drag_finish (context, FALSE, FALSE, time);
+			break;
+		case NAUTILUS_ICON_DND_URI_LIST:
+			receive_dropped_uri_list
+				(NAUTILUS_ICON_CONTAINER (widget),
+				 (char*) data->data, x, y);
+			gtk_drag_finish (context, FALSE, FALSE, time);
+			break;
+
+		default:
+			gtk_drag_finish (context, FALSE, FALSE, time);
+		}
+		
+		nautilus_icon_container_free_drag_data (NAUTILUS_ICON_CONTAINER (widget));
+		
+		set_drop_target (NAUTILUS_ICON_CONTAINER (widget), NULL);
+
+		/* reinitialise it for the next dnd */
+		drag_info->drop_occured = FALSE;
+	}
+
 }
 
 static void
@@ -454,9 +524,7 @@ get_container_uri (const NautilusIconContainer *container)
 
 	/* get the URI associated with the container */
 	uri = NULL;
-	gtk_signal_emit_by_name (GTK_OBJECT (container),
-			 "get_container_uri",
-			 &uri);
+	gtk_signal_emit_by_name (GTK_OBJECT (container), "get_container_uri", &uri);
 	return uri;
 }
 
@@ -482,18 +550,17 @@ nautilus_icon_container_selection_items_local (const NautilusIconContainer *cont
 
 /* handle dropped tile images */
 static void
-receive_dropped_tile_image (NautilusIconContainer *container, gpointer data)
+receive_dropped_tile_image (NautilusIconContainer *container, GtkSelectionData *data)
 {
 	g_assert (data != NULL);
 	nautilus_background_receive_dropped_background_image
-		(nautilus_get_widget_background (GTK_WIDGET (container)), data);
+		(nautilus_get_widget_background (GTK_WIDGET (container)), data->data);
 }
 
 /* handle dropped keywords */
 static void
 receive_dropped_keyword (NautilusIconContainer *container, char* keyword, int x, int y)
 {
-	GList *keywords, *word;
 	char *uri;
 	double world_x, world_y;
 
@@ -509,7 +576,8 @@ receive_dropped_keyword (NautilusIconContainer *container, char* keyword, int x,
 		return;
 	}
 
-	/* FIXME: This does not belong in the icon code.
+	/* FIXME bugzilla.eazel.com 2485: 
+	 * This does not belong in the icon code.
 	 * It has to be in the file manager.
 	 * The icon code has no right to deal with the file directly.
 	 * But luckily there's no issue of not getting a file object,
@@ -519,20 +587,42 @@ receive_dropped_keyword (NautilusIconContainer *container, char* keyword, int x,
 	file = nautilus_file_get (uri);
 	g_free (uri);
 	
-	keywords = nautilus_file_get_keywords (file);
-	word = g_list_find_custom (keywords, keyword, (GCompareFunc) strcmp);
-	if (word == NULL) {
-		keywords = g_list_append (keywords, g_strdup (keyword));
-	} else {
-		keywords = g_list_remove_link (keywords, word);
-		g_free (word->data);
-		g_list_free (word);
-	}
+	nautilus_drag_file_receive_dropped_keyword (file, keyword);
 
-	nautilus_file_set_keywords (file, keywords);
 	nautilus_file_unref (file);
 	nautilus_icon_container_update_icon (container, drop_target_icon);
+}
 
+/* handle dropped uri list */
+static void
+receive_dropped_uri_list (NautilusIconContainer *container, char *uri_list, int x, int y)
+{
+	/* FIXME:
+	 * this needs a better name - it's link/desktop specific 
+	 */
+	GList *li, *files;
+	int argc;
+	char **argv;
+	int i;
+	
+	g_assert (uri_list != NULL);
+
+	files = gnome_uri_list_extract_filenames (uri_list);
+	argc = g_list_length (files);
+	argv = g_new (char *, argc + 1);
+	argv[argc] = NULL;
+
+	for (i=0, li = files; li; i++, li = g_list_next (li)) {
+		argv[i] = li->data;
+	}
+
+	/* Extract .desktop info and create link/links */
+	gtk_signal_emit_by_name (GTK_OBJECT (container), "create_nautilus_links",
+				 files,
+				 x, y);
+
+	gnome_uri_list_free_strings (files);
+	g_free(argv);
 }
 
 static int
@@ -540,7 +630,6 @@ auto_scroll_timeout_callback (gpointer data)
 {
 	NautilusIconContainer *container;
 	GtkWidget *widget;
-	int x, y;
 	float x_scroll_delta, y_scroll_delta;
 	GdkRectangle exposed_area;
 
@@ -556,67 +645,18 @@ auto_scroll_timeout_callback (gpointer data)
 
 	container->details->waiting_to_autoscroll = FALSE;
 
-	gdk_window_get_pointer (widget->window, &x, &y, NULL);
-
-	/* Find out if we are anywhere close to the container view edges
-	 * to see if we need to autoscroll.
-	 */
-	x_scroll_delta = 0;
-	y_scroll_delta = 0;
-	
-	if (x < AUTO_SCROLL_MARGIN) {
-		x_scroll_delta = (float)(x - AUTO_SCROLL_MARGIN);
-	}
-
-	if (x > widget->allocation.width - AUTO_SCROLL_MARGIN) {
-		if (x_scroll_delta != 0) {
-			/* Already trying to scroll because of being too close to 
-			 * the top edge -- must be the window is really short,
-			 * don't autoscroll.
-			 */
-			return TRUE;
-		}
-		x_scroll_delta = (float)(x - (widget->allocation.width - AUTO_SCROLL_MARGIN));
-	}
-
-	if (y < AUTO_SCROLL_MARGIN) {
-		y_scroll_delta = (float)(y - AUTO_SCROLL_MARGIN);
-	}
-
-	if (y > widget->allocation.height - AUTO_SCROLL_MARGIN) {
-		if (y_scroll_delta != 0) {
-			/* Already trying to scroll because of being too close to 
-			 * the top edge -- must be the window is really narrow,
-			 * don't autoscroll.
-			 */
-			return TRUE;
-		}
-		y_scroll_delta = (float)(y - (widget->allocation.height - AUTO_SCROLL_MARGIN));
-	}
-
+	nautilus_drag_autoscroll_calculate_delta (widget, &x_scroll_delta, &y_scroll_delta);
 	if (x_scroll_delta == 0 && y_scroll_delta == 0) {
 		/* no work */
 		return TRUE;
 	}
 
-	/* Adjust the scroll delta to the proper acceleration values depending on how far
-	 * into the sroll margins we are.
-	 * FIXME:
-	 * we could use an exponential acceleration factor here for better feel
-	 */
-	if (x_scroll_delta != 0) {
-		x_scroll_delta /= AUTO_SCROLL_MARGIN;
-		x_scroll_delta *= (MAX_AUTOSCROLL_DELTA - MIN_AUTOSCROLL_DELTA);
-		x_scroll_delta += MIN_AUTOSCROLL_DELTA;
+	if (!nautilus_icon_container_scroll (container, (int)x_scroll_delta, (int)y_scroll_delta)) {
+		/* the scroll value got pinned to a min or max adjustment value,
+		 * we ended up not scrolling
+		 */
+		return TRUE;
 	}
-	
-	if (y_scroll_delta != 0) {
-		y_scroll_delta /= AUTO_SCROLL_MARGIN;
-		y_scroll_delta *= (MAX_AUTOSCROLL_DELTA - MIN_AUTOSCROLL_DELTA);
-		y_scroll_delta += MIN_AUTOSCROLL_DELTA;
-	}
-
-	nautilus_icon_container_scroll (container, (int)x_scroll_delta, (int)y_scroll_delta);
 
 	/* update cached drag start offsets */
 	container->details->dnd_info->drag_info.start_x -= x_scroll_delta;
@@ -679,6 +719,7 @@ static gboolean
 confirm_switch_to_manual_layout (NautilusIconContainer *container)
 {
 	const char *message;
+	GnomeDialog *dialog;
 
 	/* FIXME bugzilla.eazel.com 915: Use of the word "directory"
 	 * makes this FMIconView specific. Move these messages into
@@ -687,28 +728,30 @@ confirm_switch_to_manual_layout (NautilusIconContainer *container)
 	 */
 	if (nautilus_icon_container_has_stored_icon_positions (container)) {
 		if (nautilus_g_list_exactly_one_item (container->details->dnd_info->drag_info.selection_list)) {
-			message = _("This directory uses automatic layout. "
+			message = _("This folder uses automatic layout. "
 			"Do you want to switch to manual layout and leave this item where you dropped it? "
 			"This will clobber the stored manual layout.");
 		} else {
-			message = _("This directory uses automatic layout. "
+			message = _("This folder uses automatic layout. "
 			"Do you want to switch to manual layout and leave these items where you dropped them? "
 			"This will clobber the stored manual layout.");
 		}
 	} else {
 		if (nautilus_g_list_exactly_one_item (container->details->dnd_info->drag_info.selection_list)) {
-			message = _("This directory uses automatic layout. "
+			message = _("This folder uses automatic layout. "
 			"Do you want to switch to manual layout and leave this item where you dropped it?");
 		} else {
-			message = _("This directory uses automatic layout. "
+			message = _("This folder uses automatic layout. "
 			"Do you want to switch to manual layout and leave these items where you dropped them?");
 		}
 	}
 
-	return nautilus_simple_dialog
-		(GTK_WIDGET (container), message,
-		 _("Switch to Manual Layout?"),
-		 _("Switch"), GNOME_STOCK_BUTTON_CANCEL, NULL) == 0;
+	dialog = nautilus_yes_no_dialog (message, _("Switch to Manual Layout?"),
+					 _("Switch"), GNOME_STOCK_BUTTON_CANCEL,
+					 GTK_WINDOW (gtk_widget_get_ancestor 
+					 	(GTK_WIDGET (container), GTK_TYPE_WINDOW)));
+
+	return gnome_dialog_run (dialog) == GNOME_OK;
 }
 
 static void
@@ -733,17 +776,23 @@ handle_local_move (NautilusIconContainer *container,
 		
 		icon = nautilus_icon_container_get_icon_by_uri
 			(container, item->uri);
+
 		if (item->got_icon_position) {
 			nautilus_icon_container_move_icon
 				(container, icon,
 				 world_x + item->icon_x, world_y + item->icon_y,
 				 icon->scale_x, icon->scale_y,
-				 TRUE);
+				 TRUE, TRUE);
 		}
 		moved_icons = g_list_prepend (moved_icons, icon);
 	}		
 	nautilus_icon_container_select_list_unselect_others
 		(container, moved_icons);
+	/* Might have been moved in a way that requires adjusting scroll region.
+	 * Include visible area so coordinate-adjustment "auto-scrolling" doesn't
+	 * occur after the drop. (see bugzilla.eazel.com 2068)
+	 */
+	nautilus_icon_container_update_scroll_region_include_visible_area (container);
 	g_list_free (moved_icons);
 }
 
@@ -777,9 +826,6 @@ handle_nonlocal_move (NautilusIconContainer *container,
 		source_item_locations = g_new (GdkPoint, g_list_length (source_uris));
 		for (i = 0, p = container->details->dnd_info->drag_info.selection_list;
 		     p != NULL; i++, p = p->next) {
-			/* FIXME bugzilla.eazel.com 626:
-			 * subtract the original click coordinates from each point here
-			 */
 			source_item_locations[i].x = ((DragSelectionItem *)p->data)->icon_x;
 			source_item_locations[i].y = ((DragSelectionItem *)p->data)->icon_y;
 		}
@@ -804,6 +850,8 @@ nautilus_icon_container_find_drop_target (NautilusIconContainer *container,
 {
 	NautilusIcon *drop_target_icon;
 	double world_x, world_y;
+	NautilusFile *file;
+	char *icon_uri;
 
 	if (container->details->dnd_info->drag_info.selection_list == NULL) {
 		return NULL;
@@ -812,33 +860,68 @@ nautilus_icon_container_find_drop_target (NautilusIconContainer *container,
   	gnome_canvas_window_to_world (GNOME_CANVAS (container),
 				      x, y, &world_x, &world_y);
 
-	/* FIXME: These "can_accept_items" tests need to be done by
+	/* FIXME bugzilla.eazel.com 2485: 
+	 * These "can_accept_items" tests need to be done by
 	 * the icon view, not here. This file is not supposed to know
 	 * that the target is a file.
 	 */
 
-	/* Find the item we hit with our drop, if any */
+	/* Find the item we hit with our drop, if any */	
 	drop_target_icon = nautilus_icon_container_item_at (container, world_x, world_y);
-	if (drop_target_icon != NULL 
-		&& !nautilus_drag_can_accept_items 
-			(nautilus_file_get (
-				nautilus_icon_container_get_icon_uri 
-					(container, drop_target_icon)), 
-			container->details->dnd_info->drag_info.selection_list)) {
-		/* the item we dropped our selection on cannot accept the items,
-		 * do the same thing as if we just dropped the items on the canvas
-		 */
-		drop_target_icon = NULL;
+	if (drop_target_icon != NULL) {
+		icon_uri = nautilus_icon_container_get_icon_uri (container, drop_target_icon);
+		if (icon_uri != NULL) {
+			file = nautilus_file_get (icon_uri);
+
+			if ( !nautilus_drag_can_accept_items (file, 
+					container->details->dnd_info->drag_info.selection_list)) {
+			 	/* the item we dropped our selection on cannot accept the items,
+			 	 * do the same thing as if we just dropped the items on the canvas
+				 */
+				drop_target_icon = NULL;
+			}
+			
+			g_free (icon_uri);
+			nautilus_file_unref (file);
+		}
 	}
 
-	if (!drop_target_icon) {
+	if (drop_target_icon == NULL) {
 		*icon_hit = FALSE;
 		return get_container_uri (container);
 	}
-
 	
 	*icon_hit = TRUE;
-	return nautilus_icon_container_get_icon_uri (container, drop_target_icon);
+
+	return nautilus_icon_container_get_icon_drop_target_uri (container, drop_target_icon);
+}
+
+/* FIXME bugzilla.eazel.com 2485: This belongs in FMDirectoryView, not here. */
+static gboolean
+selection_includes_special_link (GList *selection_list)
+{
+	GList *node;
+	char *uri, *local_path;
+	gboolean link_in_selection;
+
+	link_in_selection = FALSE;
+
+	for (node = selection_list; node != NULL; node = node->next) {
+		uri = ((DragSelectionItem *) node->data)->uri;
+
+		/* FIXME bugzilla.eazel.com 3020: This does sync. I/O and works only locally. */
+		local_path = gnome_vfs_get_local_path_from_uri (uri);
+		link_in_selection = local_path != NULL
+			&& (nautilus_link_local_is_trash_link (local_path) || nautilus_link_local_is_home_link (local_path) ||
+			nautilus_link_local_is_volume_link (local_path));
+		g_free (local_path);
+		
+		if (link_in_selection) {
+			break;
+		}
+	}
+	
+	return link_in_selection;
 }
 
 static void
@@ -850,6 +933,7 @@ nautilus_icon_container_receive_dropped_icons (NautilusIconContainer *container,
 	gboolean local_move_only;
 	double world_x, world_y;
 	gboolean icon_hit;
+	GdkDragAction action;
 
 	drop_target = NULL;
 
@@ -858,8 +942,15 @@ nautilus_icon_container_receive_dropped_icons (NautilusIconContainer *container,
 	}
 
 	if (context->action == GDK_ACTION_ASK) {
-		context->action = nautilus_drag_drop_action_ask 
-			(GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK);
+		/* FIXME bugzilla.eazel.com 2485: This belongs in FMDirectoryView, not here. */
+		/* Check for special case items in selection list */
+		if (selection_includes_special_link (container->details->dnd_info->drag_info.selection_list)) {
+			/* We only want to move the trash */
+			action = GDK_ACTION_MOVE;
+		} else {
+			action = GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK;
+		}
+		context->action = nautilus_drag_drop_action_ask (action);
 	}
 
 	if (context->action > 0) {
@@ -881,7 +972,7 @@ nautilus_icon_container_receive_dropped_icons (NautilusIconContainer *container,
 		if (local_move_only) {
 			handle_local_move (container, world_x, world_y);
 		} else {
-			handle_nonlocal_move (container, context, x, y, drop_target, icon_hit);
+			handle_nonlocal_move (container, context, world_x, world_y, drop_target, icon_hit);
 		}
 	}
 
@@ -927,10 +1018,11 @@ nautilus_icon_container_get_drop_action (NautilusIconContainer *container,
 	case NAUTILUS_ICON_DND_COLOR:
 	case NAUTILUS_ICON_DND_BGIMAGE:
 	case NAUTILUS_ICON_DND_KEYWORD:
+	case NAUTILUS_ICON_DND_URI_LIST:
 		*default_action = context->suggested_action;
 		*non_default_action = context->suggested_action;
 		break;
-
+	
 	default:
 	}
 
@@ -976,7 +1068,8 @@ nautilus_icon_dnd_update_drop_target (NautilusIconContainer *container,
 	/* Find the item we hit with our drop, if any. */
 	icon = nautilus_icon_container_item_at (container, world_x, world_y);
 
-	/* FIXME: These "can_accept_items" tests need to be done by
+	/* FIXME bugzilla.eazel.com 2485: 
+	 * These "can_accept_items" tests need to be done by
 	 * the icon view, not here. This file is not supposed to know
 	 * that the target is a file.
 	 */
@@ -1004,7 +1097,6 @@ nautilus_icon_container_free_drag_data (NautilusIconContainer *container)
 	dnd_info->drag_info.got_drop_data_type = FALSE;
 	
 	if (dnd_info->shadow != NULL) {
-		/* FIXME: Is a destroy really sufficient here? Who does the unref? */
 		gtk_object_destroy (GTK_OBJECT (dnd_info->shadow));
 		dnd_info->shadow = NULL;
 	}
@@ -1081,7 +1173,8 @@ nautilus_icon_dnd_fini (NautilusIconContainer *container)
 
 	stop_auto_scroll (container);
 	if (container->details->dnd_info->shadow != NULL) {
-		/* FIXME: Is a destroy really sufficient here? Who does the unref? */
+		/* FIXME bugzilla.eazel.com 2484: 
+		 * Is a destroy really sufficient here? Who does the unref? */
 		gtk_object_destroy (GTK_OBJECT (container->details->dnd_info->shadow));
 	}
 
@@ -1097,7 +1190,7 @@ nautilus_icon_dnd_begin_drag (NautilusIconContainer *container,
 	NautilusIconDndInfo *dnd_info;
 	GnomeCanvas *canvas;
 	GdkDragContext *context;
-	GdkPixbuf *pixbuf, *transparent_pixbuf;
+	GdkPixbuf *pixbuf;
 	GdkPixmap *pixmap_for_dragged_file;
 	GdkBitmap *mask_for_dragged_file;
 	int x_offset, y_offset;
@@ -1128,26 +1221,15 @@ nautilus_icon_dnd_begin_drag (NautilusIconContainer *container,
 
         /* create a pixmap and mask to drag with */
         pixbuf = nautilus_icon_canvas_item_get_image (container->details->drag_icon->item);
-        
-	/* unfortunately, X is very slow when using a stippled mask,
-	   so only use the stipple for relatively small pixbufs. */
-	/* FIXME bugzilla.eazel.com 914: Eventually, we may have to
-	 * remove this entirely for UI consistency reasons.
-	 */
-	
-	if (gdk_pixbuf_get_width(pixbuf) * gdk_pixbuf_get_height(pixbuf) < 4096) {
-		transparent_pixbuf = nautilus_make_semi_transparent (pixbuf);
-	} else {
-		gdk_pixbuf_ref (pixbuf);
-		transparent_pixbuf = pixbuf;
-	}
-		
-	gdk_pixbuf_render_pixmap_and_mask (transparent_pixbuf,
+    
+    	/* we want to drag semi-transparent pixbufs, but X is too slow dealing with
+	   stippled masks, so we had to remove the code; this comment is left as a memorial
+	   to it, with the hope that we get it back someday as X Windows improves */
+	       
+	gdk_pixbuf_render_pixmap_and_mask (pixbuf,
 					   &pixmap_for_dragged_file,
 					   &mask_for_dragged_file,
-					   128);
-
-	gdk_pixbuf_unref (transparent_pixbuf);
+					   NAUTILUS_STANDARD_ALPHA_THRESHHOLD);
 	
         /* compute the image's offset */
 	nautilus_icon_canvas_item_get_icon_rectangle
@@ -1173,7 +1255,7 @@ drag_motion_callback (GtkWidget *widget,
 {
 	int default_action, non_default_action;
 	int resulting_action;
-
+	
 	nautilus_icon_container_ensure_drag_data (NAUTILUS_ICON_CONTAINER (widget), context, time);
 	nautilus_icon_container_position_shadow (NAUTILUS_ICON_CONTAINER (widget), x, y);
 	nautilus_icon_dnd_update_drop_target (NAUTILUS_ICON_CONTAINER (widget), context, x, y);
@@ -1182,12 +1264,12 @@ drag_motion_callback (GtkWidget *widget,
 	 * the drop target.
 	 */
 	nautilus_icon_container_get_drop_action (NAUTILUS_ICON_CONTAINER (widget), context, x, y,
-		&default_action, &non_default_action);
+						 &default_action, &non_default_action);
 
 	/* set the right drop action, choose based on modifier key state
 	 */
 	resulting_action = nautilus_drag_modifier_based_action (default_action, 
-		non_default_action);
+								non_default_action);
 	gdk_drag_status (context, resulting_action, time);
 
 	return TRUE;
@@ -1205,42 +1287,16 @@ drag_drop_callback (GtkWidget *widget,
 
 	dnd_info = NAUTILUS_ICON_CONTAINER (widget)->details->dnd_info;
 
-	nautilus_icon_container_ensure_drag_data
-		(NAUTILUS_ICON_CONTAINER (widget), context, time);
 
-	g_assert (dnd_info->drag_info.got_drop_data_type);
-	switch (dnd_info->drag_info.data_type) {
-	case NAUTILUS_ICON_DND_GNOME_ICON_LIST:
-		nautilus_icon_container_receive_dropped_icons
-			(NAUTILUS_ICON_CONTAINER (widget),
-			 context, x, y);
-		gtk_drag_finish (context, TRUE, FALSE, time);
-		break;
-	case NAUTILUS_ICON_DND_COLOR:
-		nautilus_background_receive_dropped_color
-			(nautilus_get_widget_background (widget),
-			 widget, x, y, dnd_info->drag_info.selection_data);
-		gtk_drag_finish (context, TRUE, FALSE, time);
-		break;
-	case NAUTILUS_ICON_DND_BGIMAGE:
-		receive_dropped_tile_image
-			(NAUTILUS_ICON_CONTAINER (widget),
-			 dnd_info->drag_info.selection_data->data);
-		gtk_drag_finish (context, FALSE, FALSE, time);
-		break;
-	case NAUTILUS_ICON_DND_KEYWORD:
-		receive_dropped_keyword
-			(NAUTILUS_ICON_CONTAINER (widget),
-			 dnd_info->drag_info.selection_data->data, x, y);
-		gtk_drag_finish (context, FALSE, FALSE, time);
-		break;
-	default:
-		gtk_drag_finish (context, FALSE, FALSE, time);
-	}
-
-	nautilus_icon_container_free_drag_data (NAUTILUS_ICON_CONTAINER (widget));
-
-	set_drop_target (NAUTILUS_ICON_CONTAINER (widget), NULL);
+	/* tell the drag_data_received callback that
+	   the drop occured and that it can actually
+	   process the actions.
+	   make sure it is going to be called at least once.
+	*/
+	dnd_info->drag_info.drop_occured = TRUE;
+	gtk_drag_get_data (GTK_WIDGET (widget), context,
+			   GPOINTER_TO_INT (context->targets->data),
+			   time);
 
 	return FALSE;
 }
