@@ -14,6 +14,8 @@
 #include "nautilus-icon-text-item.h"
 
 #include "nautilus-entry.h"
+#include "nautilus-theme.h"
+
 #include <libnautilus/nautilus-undo.h>
 #include "nautilus-gdk-extensions.h"
 
@@ -382,6 +384,20 @@ iti_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags)
 	priv->need_state_update = FALSE;
 }
 
+/* utility to fetch a color from a theme */
+static void
+fetch_themed_color (const char *property_name, GdkColor *color)
+{
+	char *color_string;
+	
+	color_string = nautilus_theme_get_theme_data ("icon", property_name);
+	if (color_string == NULL) {
+		color_string = g_strdup ("rgb:FFFF/FFFF/FFFF");
+	}
+	gdk_color_parse (color_string, color);
+	g_free (color_string);
+}
+
 /* Draw the icon text item's text when it is being edited */
 static void
 iti_paint_text (Iti *iti, GdkDrawable *drawable, int x, int y)
@@ -399,6 +415,8 @@ iti_paint_text (Iti *iti, GdkDrawable *drawable, int x, int y)
 	int cursor, offset, i;
 	GnomeCanvasItem *canvas_item;
 
+	xpos = 0;
+	offset = 0;
 	priv = iti->priv;
 	style = GTK_WIDGET (GNOME_CANVAS_ITEM (iti)->canvas)->style;
 
@@ -417,11 +435,11 @@ iti_paint_text (Iti *iti, GdkDrawable *drawable, int x, int y)
 	sgc = style->fg_gc [GTK_STATE_SELECTED];
 	bsgc = style->bg_gc [GTK_STATE_SELECTED];
 
-	/* FIXME: Get these values from prefs when they are saved there */
-	gdk_color_parse ("rgb:00/00/00", &highlight_background_color);
-	gdk_color_parse ("rgb:FF/FF/FF", &highlight_text_color);
-	gdk_color_parse ("rgb:FF/FF/FF", &fill_color);
-
+	/* fetch the colors from the theme */
+	fetch_themed_color ("HIGHLIGHT_BACKGROUND_COLOR",  &highlight_background_color);
+	fetch_themed_color ("HIGHLIGHT_TEXT_COLOR",  &highlight_text_color);
+	fetch_themed_color ("TEXT_FILL_COLOR",  &fill_color);
+	
 	/* Set up user defined colors */
 	canvas_item = GNOME_CANVAS_ITEM (iti);
 	gdk_colormap_alloc_color
@@ -525,7 +543,7 @@ iti_paint_text (Iti *iti, GdkDrawable *drawable, int x, int y)
 	/* The i-beam should only be drawn at the end of a line of text if that line is the
 	 * only or last line of text in a label.  We subtract one from the x position
 	 * so the i-beam is not visually jammed against the edge of the bounding rect. */
-	if (cursor == i){
+	if (cursor == i) {
 		int px = x + xpos + offset;
 		y -= ti->baseline_skip;
 		
@@ -1018,8 +1036,9 @@ iti_event (GnomeCanvasItem *item, GdkEvent *event)
 		return TRUE;
 
 	case GDK_FOCUS_CHANGE:
-		if (iti->editing && event->focus_change.in == FALSE)
+		if (iti->editing && !event->focus_change.in) {
 			iti_edition_accept (iti);
+		}
 
 		return TRUE;
 
@@ -1065,77 +1084,69 @@ iti_class_init (NautilusIconTextItemClass *text_item_class)
 
 	parent_class = gtk_type_class (gnome_canvas_item_get_type ());
 
-	iti_signals [TEXT_CHANGED] =
-		gtk_signal_new (
-			"text_changed",
-			GTK_RUN_LAST,
-			object_class->type,
-			GTK_SIGNAL_OFFSET (NautilusIconTextItemClass, text_changed),
-			gtk_marshal_BOOL__NONE,
-			GTK_TYPE_BOOL, 0);
+	iti_signals [TEXT_CHANGED] = gtk_signal_new
+		("text_changed",
+		 GTK_RUN_LAST,
+		 object_class->type,
+		 GTK_SIGNAL_OFFSET (NautilusIconTextItemClass, text_changed),
+		 gtk_marshal_BOOL__NONE,
+		 GTK_TYPE_BOOL, 0);
 
-	iti_signals [TEXT_EDITED] =
-		gtk_signal_new (
-			"text_edited",
-			GTK_RUN_LAST,
-			object_class->type,
-			GTK_SIGNAL_OFFSET (NautilusIconTextItemClass, text_edited),
-			gtk_marshal_NONE__NONE,
-			GTK_TYPE_NONE, 0);
+	iti_signals [TEXT_EDITED] = gtk_signal_new
+		("text_edited",
+		 GTK_RUN_LAST,
+		 object_class->type,
+		 GTK_SIGNAL_OFFSET (NautilusIconTextItemClass, text_edited),
+		 gtk_marshal_NONE__NONE,
+		 GTK_TYPE_NONE, 0);
 
-	iti_signals [HEIGHT_CHANGED] =
-		gtk_signal_new (
-			"height_changed",
-			GTK_RUN_LAST,
-			object_class->type,
-			GTK_SIGNAL_OFFSET (NautilusIconTextItemClass, height_changed),
-			gtk_marshal_NONE__NONE,
-			GTK_TYPE_NONE, 0);
+	iti_signals [HEIGHT_CHANGED] = gtk_signal_new
+		("height_changed",
+		 GTK_RUN_LAST,
+		 object_class->type,
+		 GTK_SIGNAL_OFFSET (NautilusIconTextItemClass, height_changed),
+		 gtk_marshal_NONE__NONE,
+		 GTK_TYPE_NONE, 0);
 
-	iti_signals [WIDTH_CHANGED] =
-		gtk_signal_new (
-			"width_changed",
-			GTK_RUN_LAST,
-			object_class->type,
-			GTK_SIGNAL_OFFSET (NautilusIconTextItemClass, width_changed),
-			gtk_marshal_NONE__NONE,
-			GTK_TYPE_NONE, 0);
+	iti_signals [WIDTH_CHANGED] = gtk_signal_new
+		("width_changed",
+		 GTK_RUN_LAST,
+		 object_class->type,
+		 GTK_SIGNAL_OFFSET (NautilusIconTextItemClass, width_changed),
+		 gtk_marshal_NONE__NONE,
+		 GTK_TYPE_NONE, 0);
 
-	iti_signals[EDITING_STARTED] =
-		gtk_signal_new (
-			"editing_started",
-			GTK_RUN_LAST,
-			object_class->type,
-			GTK_SIGNAL_OFFSET (NautilusIconTextItemClass, editing_started),
-			gtk_marshal_NONE__NONE,
-			GTK_TYPE_NONE, 0);
+	iti_signals[EDITING_STARTED] = gtk_signal_new
+		("editing_started",
+		 GTK_RUN_LAST,
+		 object_class->type,
+		 GTK_SIGNAL_OFFSET (NautilusIconTextItemClass, editing_started),
+		 gtk_marshal_NONE__NONE,
+		 GTK_TYPE_NONE, 0);
 
-	iti_signals[EDITING_STOPPED] =
-		gtk_signal_new (
-			"editing_stopped",
-			GTK_RUN_LAST,
-			object_class->type,
-			GTK_SIGNAL_OFFSET (NautilusIconTextItemClass, editing_stopped),
-			gtk_marshal_NONE__NONE,
-			GTK_TYPE_NONE, 0);
+	iti_signals[EDITING_STOPPED] = gtk_signal_new
+		("editing_stopped",
+		 GTK_RUN_LAST,
+		 object_class->type,
+		 GTK_SIGNAL_OFFSET (NautilusIconTextItemClass, editing_stopped),
+		 gtk_marshal_NONE__NONE,
+		 GTK_TYPE_NONE, 0);
 
-	iti_signals[SELECTION_STARTED] =
-		gtk_signal_new (
-			"selection_started",
-			GTK_RUN_FIRST,
-			object_class->type,
-			GTK_SIGNAL_OFFSET (NautilusIconTextItemClass, selection_started),
-			gtk_marshal_NONE__NONE,
-			GTK_TYPE_NONE, 0);
+	iti_signals[SELECTION_STARTED] = gtk_signal_new
+		("selection_started",
+		 GTK_RUN_LAST,
+		 object_class->type,
+		 GTK_SIGNAL_OFFSET (NautilusIconTextItemClass, selection_started),
+		 gtk_marshal_NONE__NONE,
+		 GTK_TYPE_NONE, 0);
 
-	iti_signals[SELECTION_STOPPED] =
-		gtk_signal_new (
-			"selection_stopped",
-			GTK_RUN_FIRST,
-			object_class->type,
-			GTK_SIGNAL_OFFSET (NautilusIconTextItemClass, selection_stopped),
-			gtk_marshal_NONE__NONE,
-			GTK_TYPE_NONE, 0);
+	iti_signals[SELECTION_STOPPED] = gtk_signal_new
+		("selection_stopped",
+		 GTK_RUN_LAST,
+		 object_class->type,
+		 GTK_SIGNAL_OFFSET (NautilusIconTextItemClass, selection_stopped),
+		 gtk_marshal_NONE__NONE,
+		 GTK_TYPE_NONE, 0);
 
 	gtk_object_class_add_signals (object_class, iti_signals, LAST_SIGNAL);
 
@@ -1485,9 +1496,9 @@ register_rename_undo (NautilusIconTextItem *item)
 		 g_strdup (gtk_entry_get_text (GTK_ENTRY (priv->entry))),
 		 (GDestroyNotify) g_free,
 		 _("Rename"),
-		 _("Undo Rename"),
+		 _("Undo Typing"),
 		 _("Restore the old name"),
-		 _("Redo Rename"),
+		 _("Redo Typing"),
 		 _("Restore the changed name"));
 }
 
