@@ -31,6 +31,7 @@
 #include <libgnomevfs/gnome-vfs.h>
 
 #include <gtk/gtkcheckbutton.h>
+#include <gtk/gtksignal.h>
 
 #include "nautilus-radio-button-group.h"
 #include "nautilus-string-picker.h"
@@ -45,12 +46,6 @@ enum
 	ARG_TITLE_STRING
 };
 
-enum
-{
-	ACTIVATE,
-	LAST_SIGNAL
-};
-
 static const guint PREFERENCES_ITEM_TITLE_SPACING = 4;
 static const guint PREFERENCES_ITEM_FRAME_BORDER_WIDTH = 6;
 static const gint PREFERENCES_ITEM_UNDEFINED_ITEM = -1;
@@ -473,13 +468,12 @@ preferences_item_create_font_family (NautilusPreferencesItem	*item,
 	g_free (description);
 
 	/* FIXME bugzilla.eazel.com 1274: Need to query system for available fonts */
-	font_list = nautilus_string_list_new ();
+	font_list = nautilus_string_list_new (TRUE);
 
 	nautilus_string_list_insert (font_list, "helvetica");
 	nautilus_string_list_insert (font_list, "times");
 	nautilus_string_list_insert (font_list, "courier");
 	nautilus_string_list_insert (font_list, "lucida");
-	nautilus_string_list_insert (font_list, "fixed");
 
 	nautilus_string_picker_set_string_list (NAUTILUS_STRING_PICKER (item->details->child), font_list);
 
@@ -488,7 +482,7 @@ preferences_item_create_font_family (NautilusPreferencesItem	*item,
 	g_assert (current_value != NULL);
 	g_assert (nautilus_string_list_contains (font_list, current_value));
 
-	nautilus_string_picker_set_text (NAUTILUS_STRING_PICKER (item->details->child), current_value);
+	nautilus_string_picker_set_selected_string (NAUTILUS_STRING_PICKER (item->details->child), current_value);
 
 	g_free (current_value);
 
@@ -555,7 +549,7 @@ add_icon_themes(NautilusStringList *theme_list, char *required_file)
 	pixmap_directory = nautilus_get_pixmap_directory ();
 
 	/* get the uri for the images directory */
-	directory_uri = nautilus_get_uri_from_local_path (pixmap_directory);
+	directory_uri = gnome_vfs_get_uri_from_local_path (pixmap_directory);
 	g_free (pixmap_directory);
 			
 	result = gnome_vfs_directory_list_load (&list, directory_uri,
@@ -601,7 +595,7 @@ preferences_item_create_theme (NautilusPreferencesItem	*item,
 	
 	g_free (description);
 
-	theme_list = nautilus_string_list_new ();
+	theme_list = nautilus_string_list_new (TRUE);
 	nautilus_string_list_insert (theme_list, "default");
 	add_icon_themes(theme_list, "i-directory");
 	
@@ -612,7 +606,7 @@ preferences_item_create_theme (NautilusPreferencesItem	*item,
 	g_assert (current_value != NULL);
 	g_assert (nautilus_string_list_contains (theme_list, current_value));
 	
-	nautilus_string_picker_set_text (NAUTILUS_STRING_PICKER (item->details->child), current_value);
+	nautilus_string_picker_set_selected_string (NAUTILUS_STRING_PICKER (item->details->child), current_value);
 
 	g_free (current_value);
 
@@ -633,7 +627,8 @@ nautilus_preferences_item_new (const gchar			*preference_name,
 
 	g_return_val_if_fail (preference_name != NULL, NULL);
 
-	item = gtk_type_new (nautilus_preferences_item_get_type ());
+	item = NAUTILUS_PREFERENCES_ITEM
+		(gtk_widget_new (nautilus_preferences_item_get_type (), NULL));
 
 	/* Cast away the constness so that the preferences object can be
 	 * refed in this object. */
@@ -694,7 +689,7 @@ text_item_changed_callback (GtkWidget *button, gpointer user_data)
 	g_assert (item->details->child != NULL);
 	g_assert (NAUTILUS_IS_STRING_PICKER (item->details->child));
 
-	text = nautilus_string_picker_get_text (NAUTILUS_STRING_PICKER (item->details->child));
+	text = nautilus_string_picker_get_selected_string (NAUTILUS_STRING_PICKER (item->details->child));
 
 	if (text != NULL)
 	{
