@@ -26,6 +26,7 @@
 #include <string.h>
 
 #include "nautilus-preferences.h"
+#include "nautilus-preferences-private.h"
 #include "nautilus-user-level-manager.h"
 
 #include "nautilus-gtk-macros.h"
@@ -37,8 +38,15 @@
 
 #include <gtk/gtksignal.h>
 
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-dialog.h>
+#include <libgnomeui/gnome-dialog-util.h>
+ 
 static const char PREFERENCES_GCONF_PATH[] = "/apps/nautilus";
 
+#include <gtk/gtksignal.h>
+
 /*
  * PreferencesHashNode:
  *
@@ -47,9 +55,9 @@ static const char PREFERENCES_GCONF_PATH[] = "/apps/nautilus";
  * (a string).  The  hash value is a pointer of the following struct:
  */
 typedef struct {
-	GList			*callback_list;
-	int			gconf_connections[3];
-	char *name;
+	GList	*callback_list;
+	int	gconf_connections[3];
+	char	*name;
 } PreferencesHashNode;
 
 /*
@@ -81,47 +89,50 @@ typedef struct {
 static PreferencesGlobalData GLOBAL = { NULL, NULL, 0 };
 
 /* PreferencesHashNode functions */
-static PreferencesHashNode *    preferences_hash_node_alloc                    (const char                        *name);
-static void                     preferences_hash_node_free                     (PreferencesHashNode         *node);
-static void                     preferences_hash_node_free_func                (gpointer                     key,
-										gpointer                     value,
-										gpointer                     callback_data);
-static void                     preferences_hash_node_check_changes_func       (gpointer                     key,
-										gpointer                     value,
-										gpointer                     callback_data);
-
-
+static PreferencesHashNode *    preferences_hash_node_alloc                       (const char                  *name);
+static void                     preferences_hash_node_free                        (PreferencesHashNode         *node);
+static void                     preferences_hash_node_free_func                   (gpointer                     key,
+										   gpointer                     value,
+										   gpointer                     callback_data);
+static void                     preferences_hash_node_check_changes_func          (gpointer                     key,
+										   gpointer                     value,
+										   gpointer                     callback_data);
 
 /* PreferencesCallbackNode functions */
-static PreferencesCallbackNode *preferences_callback_node_alloc                (NautilusPreferencesCallback  callback_proc,
-										gpointer                     callback_data,
-										const PreferencesHashNode   *hash_node);
-static void                     preferences_callback_node_free                 (PreferencesCallbackNode     *node);
-static void                     preferences_callback_node_free_func            (gpointer                     data,
-										gpointer                     callback_data);
-static void                     preferences_callback_node_invoke_func          (gpointer                     data,
-										gpointer                     callback_data);
-static void                     preferences_hash_node_add_callback             (PreferencesHashNode         *node,
-										NautilusPreferencesCallback  callback_proc,
-										gpointer                     callback_data);
-static void                     preferences_hash_node_remove_callback          (PreferencesHashNode         *node,
-										NautilusPreferencesCallback  callback_proc,
-										gpointer                     callback_data);
-
+static PreferencesCallbackNode *preferences_callback_node_alloc                   (NautilusPreferencesCallback  callback_proc,
+										   gpointer                     callback_data,
+										   const PreferencesHashNode   *hash_node);
+static void                     preferences_callback_node_free                    (PreferencesCallbackNode     *node);
+static void                     preferences_callback_node_free_func               (gpointer                     data,
+										   gpointer                     callback_data);
+static void                     preferences_callback_node_invoke_func             (gpointer                     data,
+										   gpointer                     callback_data);
+static void                     preferences_hash_node_add_by_user_level_callbacks (PreferencesHashNode         *node,
+										   NautilusPreferencesCallback  callback_proc,
+										   gpointer                     callback_data);
+static void                     preferences_hash_node_add_callback                (PreferencesHashNode         *node,
+										   NautilusPreferencesCallback  callback_proc,
+										   gpointer                     callback_data);
+static void                     preferences_hash_node_remove_callback             (PreferencesHashNode         *node,
+										   NautilusPreferencesCallback  callback_proc,
+										   gpointer                     callback_data);
 
 /* Private stuff */
-static PreferencesHashNode *    preferences_hash_node_lookup                   (const char                  *name);
-static PreferencesHashNode *    preferences_hash_node_lookup_with_registration (const char                  *pref_name);
-static void                     preferences_register                           (const char                        *name);
+static PreferencesHashNode *    preferences_hash_node_lookup                      (const char                  *name);
+static void                     preferences_register                              (const char                  *name);
+static gboolean                 preferences_initialize_if_needed                  (void);
+static void                     preferences_shutdown                              (void);
+static char *                   preferences_make_make_gconf_key                   (const char                  *preference_name);
 
-/* Gconf callbacks */
-static void                     preferences_gconf_callback                     (GConfClient                 *client,
-										guint                        cnxn_id,
-										const gchar                 *key,
-										GConfValue                  *value,
-										gboolean                     is_default,
-										gpointer                     user_data);
-static gboolean                 preferences_initialize_if_needed               (void);
+/* GConf callbacks */
+static void                     preferences_gconf_by_user_level_callback          (GConfClient                 *client,
+										   guint                        cnxn_id,
+										   GConfEntry                  *value,
+										   gpointer                     user_data);
+static void                     preferences_gconf_callback                        (GConfClient                 *client,
+										   guint                        cnxn_id,
+										   GConfEntry                  *value,
+										   gpointer                     user_data);
 
 /**
  * preferences_hash_node_alloc
@@ -188,7 +199,7 @@ preferences_hash_node_free (PreferencesHashNode *node)
 }
 
 /**
- * preferences_hash_node_add_callback
+ * preferences_hash_node_add_by_user_level_callbacks
  *
  * Add a callback to a pref node.  Callbacks are fired whenever
  * the pref value changes.
@@ -197,9 +208,9 @@ preferences_hash_node_free (PreferencesHashNode *node)
  * @callback_data: The user supplied closure.
  **/
 static void
-preferences_hash_node_add_callback (PreferencesHashNode		*node,
-				    NautilusPreferencesCallback	callback_proc,
-				    gpointer			callback_data)
+preferences_hash_node_add_by_user_level_callbacks (PreferencesHashNode		*node,
+						   NautilusPreferencesCallback	callback_proc,
+						   gpointer			callback_data)
 {
 	PreferencesCallbackNode	*preferences_callback_node;
 	guint i;
@@ -226,6 +237,8 @@ preferences_hash_node_add_callback (PreferencesHashNode		*node,
 	{
 		if (node->gconf_connections[i] == 0) {
 			char *key;
+			GError *error = NULL;
+
 			g_assert (node->name != NULL);
 			g_assert (node->gconf_connections[i] == 0);
 			
@@ -234,16 +247,70 @@ preferences_hash_node_add_callback (PreferencesHashNode		*node,
 
 			node->gconf_connections[i] = gconf_client_notify_add (GLOBAL.gconf_client,
 									      key,
-									      preferences_gconf_callback,
+									      preferences_gconf_by_user_level_callback,
 									      node,
 									      NULL,
-									      NULL);
+									      &error);
+			if (nautilus_preferences_handle_error (&error)) {
+				node->gconf_connections[i] = 0;
+			}
 			
 			g_free (key);
 		}
 	}
 }
 
+/**
+ * preferences_hash_node_add_callback
+ *
+ * Add a callback to a pref node.  Callbacks are fired whenever
+ * the pref value changes.
+ * @preferences_hash_node: The hash node.
+ * @callback_proc: The user supplied callback.
+ * @callback_data: The user supplied closure.
+ **/
+static void
+preferences_hash_node_add_callback (PreferencesHashNode		*node,
+				    NautilusPreferencesCallback	callback_proc,
+				    gpointer			callback_data)
+{
+	PreferencesCallbackNode	*preferences_callback_node;
+
+	g_assert (node != NULL);
+	g_assert (callback_proc != NULL);
+
+	preferences_callback_node = preferences_callback_node_alloc (callback_proc, 
+								     callback_data,
+								     node);
+	
+	g_assert (preferences_callback_node != NULL);
+	
+	node->callback_list = g_list_append (node->callback_list, 
+					     (gpointer) preferences_callback_node);
+
+	/*
+	 * We install only one gconf notification for each preference node.
+	 * Otherwise, we would invoke the installed callbacks more than once
+	 * per registered callback.
+	 */
+	if (node->gconf_connections[0] == 0) {
+		GError *error = NULL;
+
+		g_assert (node->name != NULL);
+		g_assert (node->gconf_connections[0] == 0);
+
+		node->gconf_connections[0] = gconf_client_notify_add (GLOBAL.gconf_client,
+								      node->name,
+								      preferences_gconf_callback,
+								      node,
+								      NULL,
+								      &error);
+		if (nautilus_preferences_handle_error (&error)) {
+			node->gconf_connections[0] = 0;
+		}
+	}
+}
+
 /**
  * preferences_hash_node_remove_callback
  *
@@ -282,6 +349,8 @@ preferences_hash_node_remove_callback (PreferencesHashNode		*node,
 			preferences_callback_node_free (callback_info);
 		}
 	}
+
+	g_list_free (new_list);
 	
 	/*
 	 * If there are no callbacks left in the node, remove the gconf 
@@ -292,10 +361,12 @@ preferences_hash_node_remove_callback (PreferencesHashNode		*node,
 
 		for (i = 0; i < 3; i++) 
 		{
-			g_assert (node->gconf_connections[i] != 0);
-			
-			gconf_client_notify_remove (GLOBAL.gconf_client,
-						    node->gconf_connections[i]);
+			/* we don't assert here. if we had trouble with gconf
+			 * notify addition, this would be 0 */
+			if (node->gconf_connections[i] != 0) {
+				gconf_client_notify_remove (GLOBAL.gconf_client,
+							    node->gconf_connections[i]);
+			}
 			
 			node->gconf_connections[i] = 0;
 		}
@@ -333,6 +404,12 @@ preferences_hash_node_check_changes_func (gpointer key,
 	g_assert (value != NULL);
 	
 	node = (PreferencesHashNode *) value;
+
+	/* Ignore preferences that are not coupled to user level */
+	if (nautilus_str_has_prefix (node->name, "/")) {
+		return;
+	}
+	
 	old_user_level = GPOINTER_TO_UINT (user_data);
 	new_user_level = nautilus_user_level_manager_get_user_level ();
 
@@ -437,6 +514,16 @@ preferences_callback_node_invoke_func (gpointer	data,
  	(* callback_node->callback_proc) (callback_node->callback_data);
 }
 
+static char *
+preferences_make_make_gconf_key (const char *preference_name)
+{
+	if (nautilus_str_has_prefix (preference_name, "/")) {
+		return g_strdup (preference_name);
+	}
+
+	return nautilus_user_level_manager_make_current_gconf_key (preference_name);
+}
+
 static void
 preferences_register (const char			*name)
 {
@@ -472,41 +559,23 @@ preferences_hash_node_lookup (const char *name)
 	return (PreferencesHashNode *) hash_value;
 }
 
-static PreferencesHashNode *
-preferences_hash_node_lookup_with_registration (const char		*name)
-{
-	PreferencesHashNode * node;
-
-	g_assert (name != NULL);
-
-	preferences_initialize_if_needed ();
-
-	node = preferences_hash_node_lookup (name);
-
-	if (!node) {
-		preferences_register (name);
-		
-		node = preferences_hash_node_lookup (name);
-	}
-	
-	g_assert (node != NULL);
-
-	return node;
-}
-
 static void
-preferences_gconf_callback (GConfClient	*client, 
-			    guint	connection_id, 
-			    const gchar	*key, 
-			    GConfValue	*value, 
-			    gboolean	is_default, 
-			    gpointer	user_data)
+preferences_gconf_by_user_level_callback (GConfClient	*client, 
+					  guint		connection_id, 
+					  GConfEntry	*entry, 
+					  gpointer	user_data)
 {
 	PreferencesHashNode	*node;
 	const char		*expected_name;
 	char			*expected_key;
+	GError			*error = NULL;
+	const char		*key;
 
 	g_return_if_fail (user_data != NULL);
+	g_return_if_fail (entry != NULL);
+	g_return_if_fail (entry->key != NULL);
+
+	key = entry->key;
 
 	node = (PreferencesHashNode *) user_data;
 
@@ -536,13 +605,13 @@ preferences_gconf_callback (GConfClient	*client,
 	if (strcmp (key, expected_key) != 0) {
 
 		/* The prefix should be the same */
-		if (strncmp (key, expected_key, strlen (expected_key)) != 0) {
+		if (strncmp (entry->key, expected_key, strlen (expected_key)) != 0) {
 
 			/* FIXME bugzilla.eazel.com 1272: This is triggering the first time the beast runs
 			 * without an existing ~/.gconf directory.
 			 */
 #if 0
- 			g_warning ("preferences_gconf_callback: Wrong prefix!  This indicates a bug.\n");
+ 			g_warning ("preferences_gconf_by_user_level_callback: Wrong prefix!  This indicates a bug.\n");
 #endif
 			g_free (expected_key);
 			return;
@@ -557,7 +626,8 @@ preferences_gconf_callback (GConfClient	*client,
 	
 	g_assert (node != NULL);
 
-	gconf_client_suggest_sync (GLOBAL.gconf_client, NULL);
+	gconf_client_suggest_sync (GLOBAL.gconf_client, &error);
+	nautilus_preferences_handle_error (&error);
 
 	/* Invoke callbacks for this node */
 	if (node->callback_list) {
@@ -569,6 +639,38 @@ preferences_gconf_callback (GConfClient	*client,
 	g_free (expected_key);
 }
 
+static void
+preferences_gconf_callback (GConfClient	*client, 
+			    guint	connection_id, 
+			    GConfEntry	*entry, 
+			    gpointer	user_data)
+{
+	PreferencesHashNode	*node;
+	GError			*error = NULL;
+	const char	        *key;
+
+	g_return_if_fail (user_data != NULL);
+	g_return_if_fail (entry != NULL);
+	g_return_if_fail (entry->key != NULL);
+	
+	node = (PreferencesHashNode *) user_data;
+	g_assert (node != NULL);
+
+	key = entry->key;
+	
+	g_assert (nautilus_str_is_equal (node->name, key));
+
+	gconf_client_suggest_sync (GLOBAL.gconf_client, &error);
+	nautilus_preferences_handle_error (&error);
+
+	/* Invoke callbacks for this node */
+	if (node->callback_list) {
+		g_list_foreach (node->callback_list,
+				preferences_callback_node_invoke_func,
+				(gpointer) NULL);
+	}
+}
+
 static void
 user_level_changed_callback (GtkObject	*user_level_manager,
 			     gpointer	user_data)
@@ -587,23 +689,19 @@ user_level_changed_callback (GtkObject	*user_level_manager,
 static gboolean
 preferences_initialize_if_needed (void)
 {
+	GError	  *error = NULL;
+
 	if (GLOBAL.preference_table != NULL && GLOBAL.gconf_client != NULL) {
 		return TRUE;
 	}
 	
 	if (!gconf_is_initialized ()) {
-		GConfError	  *error = NULL;
 		char		  *argv[] = { "nautilus", NULL };
 		
 		if (!gconf_init (1, argv, &error)) {
 			g_assert (error != NULL);
-			
-			/* FIXME bugzilla.eazel.com 672: Need better error reporting here */
-			g_warning ("GConf init failed:\n  %s", error->str);
-			
-			gconf_error_destroy (error);
-			
-			error = NULL;
+
+			nautilus_preferences_handle_error (&error);
 			
 			return FALSE;
 		}
@@ -618,13 +716,14 @@ preferences_initialize_if_needed (void)
 
  	GLOBAL.gconf_client = gconf_client_get_default ();
 
+	g_assert (GLOBAL.gconf_client != NULL);
+
 	/* Let gconf know about ~/.gconf/nautilus */
 	gconf_client_add_dir (GLOBAL.gconf_client,
 			      PREFERENCES_GCONF_PATH,
-			      GCONF_CLIENT_PRELOAD_RECURSIVE,
-			      NULL);
-
-	g_assert (GLOBAL.gconf_client != NULL);
+			      GCONF_CLIENT_PRELOAD_NONE,
+			      &error);
+	nautilus_preferences_handle_error (&error);
 
 	GLOBAL.old_user_level = nautilus_user_level_manager_get_user_level ();
 	
@@ -634,6 +733,8 @@ preferences_initialize_if_needed (void)
 			    user_level_changed_callback,
 			    NULL);
 
+	g_atexit (preferences_shutdown);
+
 	return TRUE;
 }
 
@@ -653,14 +754,27 @@ nautilus_preferences_add_callback (const char			*name,
 
 	preferences_initialize_if_needed ();
 
-	node = preferences_hash_node_lookup_with_registration (name);
+	node = preferences_hash_node_lookup (name);
+
+	if (!node) {
+		preferences_register (name);
+		
+		node = preferences_hash_node_lookup (name);
+	}
+
+	g_assert (node != NULL);
 
 	if (node == NULL) {
 		g_warning ("trying to add a callback for an unregistered preference");
 		return FALSE;
 	}
 
-	preferences_hash_node_add_callback (node, callback_proc, callback_data);
+	if (nautilus_str_has_prefix (name, "/")) {
+		preferences_hash_node_add_callback (node, callback_proc, callback_data);
+	}
+	else {
+		preferences_hash_node_add_by_user_level_callbacks (node, callback_proc, callback_data);
+	}
 
 	return TRUE;
 }
@@ -690,28 +804,31 @@ void
 nautilus_preferences_set_boolean (const char	*name,
 				  gboolean	boolean_value)
 {
+	GError      *error = NULL;
 	char		*key;
-
-	gboolean gconf_result;
+	gboolean	 old_value;
 
 	g_return_if_fail (name != NULL);
 
 	preferences_initialize_if_needed ();
-
-	key = nautilus_user_level_manager_make_current_gconf_key (name);
+	
+	key = preferences_make_make_gconf_key (name);
 	g_assert (key != NULL);
 
 	/* Make sure the preference value is indeed different */
-	if (gconf_client_get_bool (GLOBAL.gconf_client, key, NULL) == boolean_value) {
+	old_value = gconf_client_get_bool (GLOBAL.gconf_client, key, &error);
+	if (error == NULL
+	    && old_value == boolean_value) {
 		g_free (key);
 		return;
 	}
+	nautilus_preferences_handle_error (&error);
 
-	gconf_result = gconf_client_set_bool (GLOBAL.gconf_client, key, boolean_value, NULL);
+	gconf_client_set_bool (GLOBAL.gconf_client, key, boolean_value, &error);
+	nautilus_preferences_handle_error (&error);
 
-	g_assert (gconf_result);
-
-	gconf_client_suggest_sync (GLOBAL.gconf_client, NULL);
+	gconf_client_suggest_sync (GLOBAL.gconf_client, &error);
+	nautilus_preferences_handle_error (&error);
 
 	g_free (key);
 }
@@ -720,48 +837,110 @@ gboolean
 nautilus_preferences_get_boolean (const char	*name,
 				  gboolean	default_value)
 {
-	gboolean result;
+	GError      *error = NULL;
+	gboolean         result;
 	char		*key;
 
 	g_return_val_if_fail (name != NULL, FALSE);
 
 	preferences_initialize_if_needed ();
 
-	key = nautilus_user_level_manager_make_current_gconf_key (name);
+	key = preferences_make_make_gconf_key (name);
 	g_assert (key != NULL);
 
-	result = gconf_client_get_bool (GLOBAL.gconf_client, key, NULL);
+	result = gconf_client_get_bool (GLOBAL.gconf_client, key, &error);
+	if (nautilus_preferences_handle_error (&error)) {
+		result = default_value;
+	}
 
 	g_free (key);
 
 	return result;
 }
 
+
 void
-nautilus_preferences_set_enum (const char    *name,
-			       int           enum_value)
+nautilus_preferences_set_string_list (const char	*name,
+				      GSList            *string_list_value)
 {
-	gboolean gconf_result;
+	GError      *error = NULL;
 	char		*key;
 
 	g_return_if_fail (name != NULL);
 
 	preferences_initialize_if_needed ();
 
-	key = nautilus_user_level_manager_make_current_gconf_key (name);
+	key = preferences_make_make_gconf_key (name);
+	g_assert (key != NULL);
+
+	/* FIXME bugzilla.eazel.com 2543: Make sure the preference value is indeed different
+           before setting, like the other functions */
+
+	gconf_client_set_list (GLOBAL.gconf_client, key,
+			       GCONF_VALUE_STRING,
+			       string_list_value,
+			       &error);
+	nautilus_preferences_handle_error (&error);
+
+	gconf_client_suggest_sync (GLOBAL.gconf_client, &error);
+	nautilus_preferences_handle_error (&error);
+
+	g_free (key);
+}
+
+GSList *
+nautilus_preferences_get_string_list (const char	*name)
+{
+	GError      *error = NULL;
+	GSList          *result;
+	char            *key;
+
+	g_return_val_if_fail (name != NULL, FALSE);
+
+	preferences_initialize_if_needed ();
+
+	key = preferences_make_make_gconf_key (name);
+	g_assert (key != NULL);
+
+	result = gconf_client_get_list (GLOBAL.gconf_client, key, 
+					GCONF_VALUE_STRING, &error);
+	nautilus_preferences_handle_error (&error);
+
+	g_free (key);
+
+	return result;
+}
+
+
+void
+nautilus_preferences_set_enum (const char    *name,
+			       int           enum_value)
+{
+	GError      *error = NULL;
+	int		 old_value;
+	char		*key;
+
+	g_return_if_fail (name != NULL);
+
+	preferences_initialize_if_needed ();
+
+	key = preferences_make_make_gconf_key (name);
 	g_assert (key != NULL);
 
 	/* Make sure the preference value is indeed different */
-	if (gconf_client_get_int (GLOBAL.gconf_client, key, NULL) == enum_value) {
+	old_value = gconf_client_get_int (GLOBAL.gconf_client, key, &error);
+	if (error == NULL
+	    && old_value == enum_value) {
 		g_free (key);
 		return;
 	}
+	nautilus_preferences_handle_error (&error);
 
-	gconf_result = gconf_client_set_int (GLOBAL.gconf_client, key, enum_value, NULL);
+	gconf_client_set_int (GLOBAL.gconf_client, key, enum_value, &error);
+	nautilus_preferences_handle_error (&error);
 
-	g_assert (gconf_result);
-
-	gconf_client_suggest_sync (GLOBAL.gconf_client, NULL);
+	gconf_client_suggest_sync (GLOBAL.gconf_client, &error);
+	nautilus_preferences_handle_error (&error);
 
 	g_free (key);
 }
@@ -770,17 +949,21 @@ int
 nautilus_preferences_get_enum (const char	*name,
 			       int		default_value)
 {
-	int result;
+	GError      *error = NULL;
+	int              result;
 	char		*key;
 
 	g_return_val_if_fail (name != NULL, FALSE);
 
 	preferences_initialize_if_needed ();
 
-	key = nautilus_user_level_manager_make_current_gconf_key (name);
+	key = preferences_make_make_gconf_key (name);
 	g_assert (key != NULL);
 
-	result = gconf_client_get_int (GLOBAL.gconf_client, key, NULL);
+	result = gconf_client_get_int (GLOBAL.gconf_client, key, &error);
+	if (nautilus_preferences_handle_error (&error)) {
+		result = default_value;
+	}
 
 	g_free (key);
 
@@ -791,20 +974,27 @@ void
 nautilus_preferences_set (const char *name, 
 			  const char *value)
 {
-	gboolean gconf_result;
+	GError      *error = NULL;
 	char		*key;
 
 	g_return_if_fail (name != NULL);
 
 	preferences_initialize_if_needed ();
 
-	key = nautilus_user_level_manager_make_current_gconf_key (name);
+	key = preferences_make_make_gconf_key (name);
 	g_assert (key != NULL);
 
 	/* Make sure the preference value is indeed different */
 	if (value) {
-		char *current_value = gconf_client_get_string (GLOBAL.gconf_client, key, NULL);
-		int result = nautilus_strcmp (current_value, value);
+		char *current_value;
+		int result;
+
+		current_value = gconf_client_get_string (GLOBAL.gconf_client, key, &error);
+		if (nautilus_preferences_handle_error (&error)) {
+			result = ! 0;
+		} else {
+			result = nautilus_strcmp (current_value, value);
+		}
 		
 		if (current_value) {
 			g_free (current_value);
@@ -816,11 +1006,11 @@ nautilus_preferences_set (const char *name,
 		}
 	}
 
-	gconf_result = gconf_client_set_string (GLOBAL.gconf_client, key, value, NULL);
+	gconf_client_set_string (GLOBAL.gconf_client, key, value, &error);
+	nautilus_preferences_handle_error (&error);
 
-	g_assert (gconf_result);
-
-	gconf_client_suggest_sync (GLOBAL.gconf_client, NULL);
+	gconf_client_suggest_sync (GLOBAL.gconf_client, &error);
+	nautilus_preferences_handle_error (&error);
 
 	g_free (key);
 }
@@ -829,17 +1019,23 @@ char *
 nautilus_preferences_get (const char	*name,
 			  const char	*default_value)
 {
-	gchar *value = NULL;
+	GError	*error = NULL;
+	gchar		*value = NULL;
 	char		*key;
 
 	g_return_val_if_fail (name != NULL, FALSE);
 
 	preferences_initialize_if_needed ();
 
-	key = nautilus_user_level_manager_make_current_gconf_key (name);
+	key = preferences_make_make_gconf_key (name);
 	g_assert (key != NULL);
 
-	value = gconf_client_get_string (GLOBAL.gconf_client, key, NULL);
+	value = gconf_client_get_string (GLOBAL.gconf_client, key, &error);
+	if (nautilus_preferences_handle_error (&error)) {
+		/* note: g_free and g_strdup handle NULL correctly */
+		g_free (value);
+		value = g_strdup (default_value);
+	}
 
 	if (!value && default_value) {
 		value = g_strdup (default_value);
@@ -850,8 +1046,8 @@ nautilus_preferences_get (const char	*name,
 	return value;
 }
 
-void
-nautilus_preferences_shutdown (void)
+static void
+preferences_shutdown (void)
 {
 	if (GLOBAL.preference_table == NULL && GLOBAL.gconf_client == NULL) {
 		return;
@@ -879,3 +1075,33 @@ nautilus_preferences_shutdown (void)
 
 }
 
+gboolean
+nautilus_preferences_handle_error (GError **error)
+{
+	static gboolean shown_dialog = FALSE;
+
+	g_return_val_if_fail (error != NULL, FALSE);
+
+	if (*error != NULL) {
+		g_warning (_("GConf error:\n  %s"), (*error)->message);
+		if ( ! shown_dialog) {
+			char *message;
+			GtkWidget *dialog;
+
+			shown_dialog = TRUE;
+
+			message = g_strdup_printf (_("GConf error:\n  %s\n"
+						     "All further errors shown "
+						     "only on terminal"),
+						   (*error)->message);
+
+			dialog = gnome_error_dialog (message);
+		}
+		g_error_free (*error);
+		*error = NULL;
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
