@@ -102,7 +102,7 @@ program_file_pair_compute_status (ProgramFilePair *pair)
 {
 	ProgramFileStatus new_status;
 
-	/* FIXME: Need to check whether it's the default or in short list for the supertype */
+	/* FIXME bugzilla.eazel.com 1459: Need to check whether it's the default or in short list for the supertype */
 	if (program_file_pair_is_default_for_file_type (pair)) {
 		new_status = PROGRAM_DEFAULT_FOR_TYPE;
 	} else if (program_file_pair_is_default_for_file (pair)) {
@@ -194,7 +194,7 @@ program_file_pair_get_program_name_for_display (ProgramFilePair *pair)
 static char *
 get_supertype_from_file (NautilusFile *file)
 {
-	/* FIXME: Needs implementation */
+	/* FIXME bugzilla.eazel.com 1459: Needs implementation */
 	return nautilus_file_get_string_attribute (file, "type");
 }
 
@@ -209,8 +209,10 @@ program_file_pair_get_short_status_text (ProgramFilePair *pair)
 	supertype = get_supertype_from_file (pair->file);
 
 	switch (pair->status) {
+	default:
 	case PROGRAM_STATUS_UNKNOWN:
 		g_assert_not_reached ();
+		result = "error";
 		break;
 	case PROGRAM_NOT_IN_PREFERRED_LIST:
 		result = g_strdup (_("not in menu"));
@@ -254,8 +256,10 @@ program_file_pair_get_long_status_text (ProgramFilePair *pair)
 	file_name = nautilus_file_get_name (pair->file);
 
 	switch (pair->status) {
+	default:
 	case PROGRAM_STATUS_UNKNOWN:
 		g_assert_not_reached ();
+		result = "error";
 		break;
 	case PROGRAM_NOT_IN_PREFERRED_LIST:
 		result = g_strdup_printf (_("Is not in the menu for \"%s\" items."), file_type);
@@ -306,22 +310,22 @@ repopulate_program_list (GnomeDialog *program_chooser,
 		         GtkCList *clist)
 {
 	char **text;
-	char *uri;
 	GList *programs, *program;
 	ProgramFilePair *pair;
 	int new_row;
 	GnomeVFSMimeActionType type;
 
+	nautilus_mime_actions_wait_for_full_file_attributes (file);
+
 	type = nautilus_program_chooser_get_type (program_chooser);
 
 	g_assert (type == GNOME_VFS_MIME_ACTION_TYPE_COMPONENT
 		  || type == GNOME_VFS_MIME_ACTION_TYPE_APPLICATION);
+	
 
-	uri = nautilus_file_get_uri (file);
 	programs = type == GNOME_VFS_MIME_ACTION_TYPE_COMPONENT
-		? nautilus_mime_get_all_components_for_uri (uri)
-		: nautilus_mime_get_all_applications_for_uri (uri);
-	g_free (uri);
+		? nautilus_mime_get_all_components_for_file (file)
+		: nautilus_mime_get_all_applications_for_file (file);
 
 	gtk_clist_clear (clist);
 		
@@ -476,14 +480,15 @@ is_component_default_for_type (NautilusViewIdentifier *identifier, const char *m
 }
 
 static gboolean
-is_application_default_for_uri (GnomeVFSMimeApplication *application, const char *uri)
+is_application_default_for_file (GnomeVFSMimeApplication *application, 
+				 NautilusFile *file)
 {
 	GnomeVFSMimeApplication *default_application;
 	gboolean result;
 
 	g_assert (application != NULL);
 
-	default_application = nautilus_mime_get_default_application_for_uri (uri);
+	default_application = nautilus_mime_get_default_application_for_file (file);
 	result = (default_application != NULL && strcmp (default_application->id, application->id) == 0);
 
 	gnome_vfs_mime_application_free (default_application);
@@ -492,14 +497,16 @@ is_application_default_for_uri (GnomeVFSMimeApplication *application, const char
 }
 
 static gboolean
-is_component_default_for_uri (NautilusViewIdentifier *identifier, const char *uri)
+is_component_default_for_file (NautilusViewIdentifier *identifier, NautilusFile *file)
 {
 	OAF_ServerInfo *default_component;
 	gboolean result;
 
 	g_assert (identifier != NULL);
 
-	default_component = nautilus_mime_get_default_component_for_uri (uri);
+	nautilus_mime_actions_wait_for_full_file_attributes (file);
+
+	default_component = nautilus_mime_get_default_component_for_file (file);
 	result = (default_component != NULL && strcmp (default_component->iid, identifier->iid) == 0);
 
 	CORBA_free (default_component);
@@ -524,12 +531,13 @@ is_component_in_short_list (NautilusViewIdentifier *identifier, const char *mime
 }
 
 static gboolean
-is_component_in_short_list_for_uri (NautilusViewIdentifier *identifier, const char *uri)
+is_component_in_short_list_for_file (NautilusViewIdentifier *identifier, 
+				     NautilusFile *file)
 {
 	GList *list;
 	gboolean result;
 
-	list = nautilus_mime_get_short_list_components_for_uri (uri);
+	list = nautilus_mime_get_short_list_components_for_file (file);
 	result = g_list_find_custom (list, 
 				     identifier, 
 				     (GCompareFunc)compare_component_with_view) 
@@ -556,18 +564,18 @@ is_application_in_short_list (GnomeVFSMimeApplication *application, const char *
 }
 
 static gboolean
-is_application_in_short_list_for_uri (GnomeVFSMimeApplication *application, const char *uri)
+is_application_in_short_list_for_file (GnomeVFSMimeApplication *application, NautilusFile *file)
 {
 	GList *list;
 	gboolean result;
 
-	list = nautilus_mime_get_short_list_applications_for_uri (uri);
+	list = nautilus_mime_get_short_list_applications_for_file (file);
 	result = g_list_find_custom (list, 
 				     application, 
 				     (GCompareFunc)compare_mime_applications) 
 		 != NULL;
 	gnome_vfs_mime_application_list_free (list);
-
+	
 	return result;
 }
 
@@ -600,26 +608,23 @@ program_file_pair_is_default_for_file_type (ProgramFilePair *pair)
 static gboolean
 program_file_pair_is_default_for_file (ProgramFilePair *pair)
 {
-	char *uri;
 	gboolean result;
-	
+
 	g_assert (pair != NULL);
 	g_assert (NAUTILUS_IS_FILE (pair->file));
 
-	uri = nautilus_file_get_uri (pair->file);
+	nautilus_mime_actions_wait_for_full_file_attributes (pair->file);
 
-	if (pair->action_type != nautilus_mime_get_default_action_type_for_uri (uri)) {
+	if (pair->action_type != nautilus_mime_get_default_action_type_for_file (pair->file)) {
 		return FALSE;
 	}
 
 	if (pair->action_type == GNOME_VFS_MIME_ACTION_TYPE_COMPONENT) {
-		result = is_component_default_for_uri (pair->view_identifier, uri);
+		result = is_component_default_for_file (pair->view_identifier, pair->file);
 	} else {
-		result = is_application_default_for_uri (pair->application, uri);
+		result = is_application_default_for_file (pair->application, pair->file);
 	}
 
-	g_free (uri);
-
 	return result;
 }
 
@@ -648,22 +653,19 @@ program_file_pair_is_in_short_list_for_file_type (ProgramFilePair *pair)
 static gboolean
 program_file_pair_is_in_short_list_for_file (ProgramFilePair *pair) 
 {
-	char *uri;
 	gboolean result;
 
 	g_assert (pair != NULL);
 	g_assert (NAUTILUS_IS_FILE (pair->file));
 
-	uri = nautilus_file_get_uri (pair->file);
+	nautilus_mime_actions_wait_for_full_file_attributes (pair->file);
 
 	if (pair->action_type == GNOME_VFS_MIME_ACTION_TYPE_COMPONENT) {
-		result = is_component_in_short_list_for_uri (pair->view_identifier, uri);
+		result = is_component_in_short_list_for_file (pair->view_identifier, pair->file);
 	} else {
-		result = is_application_in_short_list_for_uri (pair->application, uri);
+		result = is_application_in_short_list_for_file (pair->application, pair->file);
 	}
 
-	g_free (uri);
-
 	return result;
 }
 
@@ -773,33 +775,25 @@ pack_radio_button (GtkBox *box, const char *label_text, GtkRadioButton *group)
 static void
 add_to_short_list_for_file (ProgramFilePair *pair)
 {
-	char *uri;
+	nautilus_mime_actions_wait_for_full_file_attributes (pair->file);
 
-	uri = nautilus_file_get_uri (pair->file);
-	
 	if (pair->action_type == GNOME_VFS_MIME_ACTION_TYPE_APPLICATION) {
-		nautilus_mime_add_application_to_short_list_for_uri (uri, pair->application->id);
+		nautilus_mime_add_application_to_short_list_for_file (pair->file, pair->application->id);
 	} else {
-		nautilus_mime_add_component_to_short_list_for_uri (uri, pair->view_identifier->iid);
+		nautilus_mime_add_component_to_short_list_for_file (pair->file, pair->view_identifier->iid);
 	}
-
-	g_free (uri);
 }
 
 static void
 remove_from_short_list_for_file (ProgramFilePair *pair)
 {
-	char *uri;
+	nautilus_mime_actions_wait_for_full_file_attributes (pair->file);
 
-	uri = nautilus_file_get_uri (pair->file);
-	
 	if (pair->action_type == GNOME_VFS_MIME_ACTION_TYPE_APPLICATION) {
-		nautilus_mime_remove_application_from_short_list_for_uri (uri, pair->application->id);
+		nautilus_mime_remove_application_from_short_list_for_file (pair->file, pair->application->id);
 	} else {
-		nautilus_mime_remove_component_from_short_list_for_uri (uri, pair->view_identifier->iid);
+		nautilus_mime_remove_component_from_short_list_for_file (pair->file, pair->view_identifier->iid);
 	}
-
-	g_free (uri);
 }
 
 static void
@@ -857,31 +851,27 @@ remove_default_for_type (ProgramFilePair *pair)
 static void
 remove_default_for_item (ProgramFilePair *pair)
 {
-	char *uri;
+	nautilus_mime_actions_wait_for_full_file_attributes (pair->file);
 
-	uri = nautilus_file_get_uri (pair->file);
-	
 	if (pair->action_type == GNOME_VFS_MIME_ACTION_TYPE_APPLICATION) {
 	    	/* If the default is just falling through to the default for this type,
 	    	 * don't do anything here.
 	    	 */
-	    	if (nautilus_mime_is_default_application_for_uri_user_chosen (uri)) {
-	    		if (is_application_default_for_uri (pair->application, uri)) {
-				nautilus_mime_set_default_application_for_uri (uri, NULL);
+	    	if (nautilus_mime_is_default_application_for_file_user_chosen (pair->file)) {
+	    		if (is_application_default_for_file (pair->application, pair->file)) {
+				nautilus_mime_set_default_application_for_file (pair->file, NULL);
 		        }
 	    	}
 	} else {
 	    	/* If the default is just falling through to the default for this type,
 	    	 * don't do anything here.
 	    	 */
-	    	if (nautilus_mime_is_default_component_for_uri_user_chosen (uri)) {
-	    		if (is_component_default_for_uri (pair->view_identifier, uri)) {
-				nautilus_mime_set_default_component_for_uri (uri, NULL);
+	    	if (nautilus_mime_is_default_component_for_file_user_chosen (pair->file)) {
+	    		if (is_component_default_for_file (pair->view_identifier, pair->file)) {
+				nautilus_mime_set_default_component_for_file (pair->file, NULL);
 		        }
 	    	}
 	}
-
-	g_free (uri);
 }
 
 static void
@@ -905,19 +895,15 @@ set_default_for_type (ProgramFilePair *pair)
 static void
 set_default_for_item (ProgramFilePair *pair)
 {
-	char *uri;
-
-	uri = nautilus_file_get_uri (pair->file);
+	nautilus_mime_actions_wait_for_full_file_attributes (pair->file);
 
 	if (pair->action_type == GNOME_VFS_MIME_ACTION_TYPE_APPLICATION) {
-		nautilus_mime_set_default_application_for_uri (uri, pair->application->id);
+		nautilus_mime_set_default_application_for_file (pair->file, pair->application->id);
 	} else {
-		nautilus_mime_set_default_component_for_uri (uri, pair->view_identifier->iid);
+		nautilus_mime_set_default_component_for_file (pair->file, pair->view_identifier->iid);
 	}
 
-	nautilus_mime_set_default_action_type_for_uri (uri, pair->action_type);
-	
-	g_free (uri);
+	nautilus_mime_set_default_action_type_for_file (pair->file, pair->action_type);
 }
 
 static void
@@ -1244,12 +1230,12 @@ nautilus_program_chooser_new (GnomeVFSMimeActionType action_type,
 
 	switch (action_type) {
 	case GNOME_VFS_MIME_ACTION_TYPE_APPLICATION:
-		title = _("Nautilus: Open with Other");
+		title = _("Open with Other");
 		prompt = g_strdup_printf (_("Choose an application with which to open \"%s\"."), file_name);
 		break;
 	case GNOME_VFS_MIME_ACTION_TYPE_COMPONENT:
 	default:
-		title = _("Nautilus: View as Other");
+		title = _("View as Other");
 		prompt = g_strdup_printf (_("Choose a view for \"%s\"."), file_name);
 		break;
 	}
@@ -1351,7 +1337,7 @@ nautilus_program_chooser_new (GnomeVFSMimeActionType action_type,
 	gtk_box_pack_end (GTK_BOX (capplet_hbox), capplet_button, FALSE, FALSE, 0);
 
 	capption = gtk_label_new (_("You can configure which programs are offered "
-				    "for which file types in the Gnome Control Center."));
+				    "for which file types in the GNOME Control Center."));
 	gtk_widget_show (capption);
 	gtk_label_set_line_wrap (GTK_LABEL (capption), TRUE);
 	gtk_box_pack_start (GTK_BOX (capplet_hbox), capption, FALSE, FALSE, 0);				    
@@ -1459,15 +1445,18 @@ nautilus_program_chooser_show_no_choices_message (GnomeVFSMimeActionType action_
 	char *prompt;
 	char *unavailable_message;
 	char *file_name;
+	char *dialog_title;
 	GnomeDialog *dialog;
 
 	file_name = nautilus_file_get_name (file);
 
 	if (action_type == GNOME_VFS_MIME_ACTION_TYPE_COMPONENT) {
 		unavailable_message = g_strdup_printf ("No viewers are available for %s.", file_name);		
+		dialog_title = g_strdup ("No Viewers Available");
 	} else {
 		g_assert (action_type == GNOME_VFS_MIME_ACTION_TYPE_APPLICATION);
 		unavailable_message = g_strdup_printf ("No applications are available for %s.", file_name);		
+		dialog_title = g_strdup ("No Applications Available");
 	}
 
 	/* Note: This might be misleading in the components case, since the
@@ -1477,14 +1466,15 @@ nautilus_program_chooser_show_no_choices_message (GnomeVFSMimeActionType action_
 	prompt = g_strdup_printf ("%s\n\n"
 				  "You can configure which programs are offered "
 				  "for which file types with the \"File Types and "
-				  "Programs\" part of the Gnome Control Center. Do "
+				  "Programs\" part of the GNOME Control Center. Do "
 				  "you want to go there now?", unavailable_message);
 	dialog = nautilus_yes_no_dialog 
-		(prompt, GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, parent_window);
+		(prompt, dialog_title, GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, parent_window);
 
 	gnome_dialog_button_connect (dialog, GNOME_OK, launch_mime_capplet, NULL);
 
 	g_free (unavailable_message);
 	g_free (file_name);
 	g_free (prompt);
+	g_free (dialog_title);
 }
