@@ -35,43 +35,51 @@ static gboolean supress_out_of_bounds_warning;
 
 struct _NautilusStringList
 {
-	GList *strings;
+	GList		*strings;
+	GCompareFunc	compare_function;
 };
 
+static gboolean str_is_equal (const char *a,
+			      const char *b,
+			      gboolean    case_sensitive);
+
 /**
  * nautilus_string_list_new:
  *
- * Construct an empty string list.
+ * @case_sensitive: Flag indicating whether the new string list is case sensitive.
  *
- * Returns the string list.
+ * Return value: A newly constructed string list.
  */
 NautilusStringList *
-nautilus_string_list_new (void)
+nautilus_string_list_new (gboolean case_sensitive)
 {
 	NautilusStringList * string_list;
 
 	string_list = g_new (NautilusStringList, 1);
 
 	string_list->strings = NULL;
+	string_list->compare_function = case_sensitive ? nautilus_str_compare : nautilus_istr_compare;
 
 	return string_list;
 }
 
 /**
- * nautilus_string_list_new:
+ * nautilus_string_list_new_from_string:
  *
- * Construct an empty string list.
+ * @other_or_null: A NautilusStringList or NULL
+ * @case_sensitive: Flag indicating whether the new string list is case sensitive.
  *
- * Returns the string list.
+ * Return value: A newly constructed string list with one entry 'string'.
  */
 NautilusStringList *
-nautilus_string_list_new_from_string (const char *string)
+nautilus_string_list_new_from_string (const char	*string,
+				      gboolean          case_sensitive)
 {
 	NautilusStringList * string_list;
 
 	g_return_val_if_fail (string != NULL, NULL);
 
-	string_list = nautilus_string_list_new ();
+	string_list = nautilus_string_list_new (case_sensitive);
 
 	nautilus_string_list_insert (string_list, string);
 
@@ -79,71 +87,94 @@ nautilus_string_list_new_from_string (const char *string)
 }
 
 /**
- * nautilus_string_list_new:
+ * nautilus_string_list_new_from_string_list:
  *
- * Construct an empty string list.
+ * @other_or_null: A NautilusStringList or NULL
+ * @case_sensitive: Flag indicating whether the new string list is case sensitive.
  *
- * Returns the string list.
+ * Return value: A newly allocated string list that is equal to other.
  */
 NautilusStringList *
-nautilus_string_list_new_from_string_list (const NautilusStringList *other)
+nautilus_string_list_new_from_string_list (const NautilusStringList	*other_or_null,
+					   gboolean			case_sensitive)
 {
 	NautilusStringList *string_list;
-	GList *other_iterator;
-	const char *other_string;
 
-	g_return_val_if_fail (other != NULL, NULL);
-
-	string_list = nautilus_string_list_new ();
-
-	for (other_iterator = other->strings; 
-	     other_iterator != NULL; 
-	     other_iterator = other_iterator->next) {
-
-		other_string = (const char *) other_iterator->data;
-
-		g_assert (other_string != NULL);
-
-		nautilus_string_list_insert (string_list, other_string);
+	if (other_or_null == NULL) {
+		return NULL;
 	}
 
+	string_list = nautilus_string_list_new (case_sensitive);
+
+	nautilus_string_list_assign_from_string_list (string_list, other_or_null);
+
 	return string_list;
 }
 
 /* Construct a string list from tokens delimited by the given string and delimiter */
 NautilusStringList *
-nautilus_string_list_new_from_tokens (const char *string,
-				      const char *delimiter)
+nautilus_string_list_new_from_tokens (const char	*string,
+				      const char	*delimiter,
+				      gboolean          case_sensitive)
 {
 	NautilusStringList *string_list;
-	char		   **string_array;
-	guint		   i;
 
-	g_return_val_if_fail (string != NULL, NULL);
 	g_return_val_if_fail (delimiter != NULL, NULL);
 
-	string_list = nautilus_string_list_new ();
+	string_list = nautilus_string_list_new (case_sensitive);
 
-	string_array = g_strsplit (string, delimiter, -1);
+	if (string != NULL) {
+		char  **string_array;
+		guint i;
 
-	if (string_array) {
-		for (i = 0; string_array[i]; i++) {
-			nautilus_string_list_insert (string_list, string_array[i]);
-		}
+		string_array = g_strsplit (string, delimiter, -1);
 		
-		g_strfreev (string_array);
+		if (string_array) {
+			for (i = 0; string_array[i]; i++) {
+				nautilus_string_list_insert (string_list, string_array[i]);
+			}
+			
+			g_strfreev (string_array);
+		}
 	}
 
 	return string_list;
 }
 
 void
-nautilus_string_list_free (NautilusStringList *string_list)
+nautilus_string_list_assign_from_string_list (NautilusStringList       *string_list,
+					      const NautilusStringList *other_or_null)
 {
+	GList		*other_iterator;
+	const char	*other_string;
+
 	g_return_if_fail (string_list != NULL);
 
 	nautilus_string_list_clear (string_list);
-	g_free (string_list);
+
+	if (other_or_null == NULL) {
+		return;
+	}
+
+	for (other_iterator = other_or_null->strings; 
+	     other_iterator != NULL; 
+	     other_iterator = other_iterator->next) {
+		
+		other_string = (const char *) other_iterator->data;
+
+		nautilus_string_list_insert (string_list, other_string);
+	}
+}
+
+void
+nautilus_string_list_free (NautilusStringList *string_list_or_null)
+{
+	if (string_list_or_null == NULL) {
+		return;
+	}
+	
+	nautilus_string_list_clear (string_list_or_null);
+	g_free (string_list_or_null);
 }
 
 void
@@ -176,15 +207,18 @@ nautilus_string_list_nth (const NautilusStringList *string_list, guint n)
 }
 
 gboolean
-nautilus_string_list_contains (const NautilusStringList *string_list,
-			       const char	  *string)
+nautilus_string_list_contains (const NautilusStringList	*string_list,
+			       const char		*string)
 {
 	GList *find;
 
-	g_return_val_if_fail (string_list != NULL, FALSE);
+	if (string_list == NULL) {
+		return FALSE;
+	}
+
 	g_return_val_if_fail (string != NULL, FALSE);
 
-	find = g_list_find_custom (string_list->strings, (gpointer) string, (GCompareFunc) strcmp);
+	find = g_list_find_custom (string_list->strings, (gpointer) string, string_list->compare_function);
 
 	return find == NULL ? FALSE : TRUE;
 }
@@ -202,8 +236,7 @@ nautilus_string_list_clear (NautilusStringList *string_list)
 {
 	g_return_if_fail (string_list != NULL);
 
-	nautilus_g_list_free_deep_custom (string_list->strings, (GFunc) g_free, NULL);
-
+	nautilus_g_list_free_deep (string_list->strings);
 	string_list->strings = NULL;
 }
 
@@ -213,8 +246,6 @@ nautilus_string_list_equals (const NautilusStringList *a,
 {
 	GList *a_iterator;
 	GList *b_iterator;
-	const char * a_string;
-	const char * b_string;
 
 	g_return_val_if_fail (a != NULL, FALSE);
 	g_return_val_if_fail (b != NULL, FALSE);
@@ -222,15 +253,11 @@ nautilus_string_list_equals (const NautilusStringList *a,
 	for (a_iterator = a->strings, b_iterator = b->strings; 
 	     a_iterator != NULL && b_iterator != NULL;
 	     a_iterator = a_iterator->next, b_iterator = b_iterator->next) {
-
-		a_string = (const char *) a_iterator->data;
-		b_string = (const char *) b_iterator->data;
-
-		if (strcmp (a_string, b_string) != 0) {
+		if (!str_is_equal ((const char *) a_iterator->data, (const char *) b_iterator->data, a->compare_function == b->compare_function)) {
 			return FALSE;
 		}
 	}
-
+	
 	return a_iterator == NULL && b_iterator == NULL;
 }
 
@@ -277,14 +304,10 @@ nautilus_string_list_get_index_for_string (const NautilusStringList	*string_list
 	g_return_val_if_fail (string != NULL, NAUTILUS_STRING_LIST_NOT_FOUND);
 
 	for (iterator = string_list->strings; iterator != NULL; iterator = iterator->next) {
-		const char *current = (const char *) iterator->data;
-		
-		g_assert (current != NULL);
-
-		if (strcmp (current, string) == 0) {
+		if (str_is_equal ((const char *) iterator->data, string, string_list->compare_function == nautilus_str_compare)) {
 			return n;
 		}
-
+		
 		n++;
 	}
 
@@ -341,6 +364,118 @@ nautilus_string_list_as_concatenated_string (const NautilusStringList *string_li
 	return result;
 }
 
+void
+nautilus_string_list_sort (NautilusStringList *string_list)
+{
+	g_return_if_fail (string_list != NULL);
+
+	string_list->strings = g_list_sort (string_list->strings, string_list->compare_function);
+}
+
+void
+nautilus_string_list_remove_duplicates (NautilusStringList *string_list)
+{
+	GList	*new_strings = NULL;
+	GList	*iterator;
+
+	g_return_if_fail (string_list != NULL);
+	
+	for (iterator = string_list->strings; iterator != NULL; iterator = iterator->next) {
+		const char *string = (const char *) iterator->data;
+		g_assert (string != NULL);
+
+		if (g_list_find_custom (new_strings, (gpointer) string, string_list->compare_function) == NULL) {
+			new_strings = g_list_append (new_strings, g_strdup (string));
+		}
+	}
+
+	nautilus_string_list_clear (string_list);
+
+	string_list->strings = new_strings;
+}
+
+void
+nautilus_string_list_for_each (const NautilusStringList *string_list,
+			       GFunc                     function,
+			       gpointer                  user_data)
+{
+	g_return_if_fail (string_list != NULL);
+
+	g_list_foreach (string_list->strings, function, user_data);
+}
+
+/**
+ * nautilus_string_list_get_longest_string:
+ *
+ * @string_list: A NautilusStringList
+ *
+ * Return value: A copy of the longest string in the collection.  Need to g_free() it.
+ */
+char *
+nautilus_string_list_get_longest_string (const NautilusStringList *string_list)
+{
+	guint	longest_length = 0;
+	guint	longest_index = 0;
+	GList	*iterator;
+	guint	i;
+
+	g_return_val_if_fail (string_list != NULL, NULL);
+
+	if (string_list->strings == NULL) {
+		return NULL;
+	}
+	
+	for (iterator = string_list->strings, i = 0; iterator != NULL; iterator = iterator->next, i++) {
+		guint current_length = nautilus_strlen ((const char *) iterator->data);
+		
+		if (current_length > longest_length) {
+			longest_index = i;
+			longest_length = current_length;
+		}
+	}
+
+	return nautilus_string_list_nth (string_list, longest_index);
+}
+
+/**
+ * nautilus_string_list_get_longest_string_length:
+ *
+ * @string_list: A NautilusStringList
+ *
+ * Return value: The length of the longest string in the collection.
+ */
+guint
+nautilus_string_list_get_longest_string_length (const NautilusStringList *string_list)
+{
+	guint	longest_length = 0;
+	GList	*iterator;
+	guint	i;
+
+	g_return_val_if_fail (string_list != NULL, 0);
+
+	if (string_list->strings == NULL) {
+		return 0;
+	}
+	
+	for (iterator = string_list->strings, i = 0; iterator != NULL; iterator = iterator->next, i++) {
+		guint current_length = nautilus_strlen ((const char *) iterator->data);
+		
+		if (current_length > longest_length) {
+			longest_length = current_length;
+		}
+	}
+
+	return longest_length;
+}
+
+static gboolean
+str_is_equal (const char	*a,
+	      const char	*b,
+	      gboolean		case_sensitive)
+{
+	return case_sensitive ? nautilus_str_is_equal (a, b) : nautilus_istr_is_equal (a, b);
+}
+
 #if !defined (NAUTILUS_OMIT_SELF_CHECK)
 
 void
@@ -350,40 +485,49 @@ nautilus_self_check_string_list (void)
 	NautilusStringList *cities;
 	NautilusStringList *cities_copy;
 	NautilusStringList *empty;
-	NautilusStringList *tokens;
 	NautilusStringList *single;
 
-	const char token_string[] = "london:paris:rome";
-	const char token_string_thick[] = "london####paris####rome";
-
-	empty = nautilus_string_list_new ();
+	/*
+	 * nautilus_string_list_contains
+	 */
+	empty = nautilus_string_list_new (TRUE);
 
 	NAUTILUS_CHECK_INTEGER_RESULT (nautilus_string_list_get_length (empty), 0);
 	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_string_list_contains (empty, "something"), FALSE);
+	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_string_list_contains (NULL, "something"), FALSE);
 
-	/********/
-
-	cities = nautilus_string_list_new ();
+	/*
+	 * nautilus_string_list_new
+	 */
+	cities = nautilus_string_list_new (TRUE);
 
  	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_string_list_equals (cities, empty), TRUE);
-
+	
 	nautilus_string_list_insert (cities, "london");
 	nautilus_string_list_insert (cities, "paris");
 	nautilus_string_list_insert (cities, "rome");
 
  	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_string_list_equals (cities, empty), FALSE);
 
-	/********/
-
-	cities_copy = nautilus_string_list_new_from_string_list (cities);
+	/*
+	 * nautilus_string_list_new_from_string_list
+	 */
+	cities_copy = nautilus_string_list_new_from_string_list (cities, TRUE);
 
  	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_string_list_equals (cities, cities_copy), TRUE);
 
 	nautilus_string_list_free (cities_copy);
+	nautilus_string_list_free (cities);
 
-	/********/
+	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_string_list_new_from_string_list (NULL, TRUE) == NULL, TRUE);
 
-	fruits = nautilus_string_list_new ();
+	/*
+	 * nautilus_string_list_insert,
+	 * nautilus_string_list_nth,
+	 * nautilus_string_list_contains,
+	 * nautilus_string_list_get_length
+	 */
+	fruits = nautilus_string_list_new (TRUE);
 
  	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_string_list_equals (fruits, empty), TRUE);
 
@@ -420,24 +564,10 @@ nautilus_self_check_string_list (void)
 	nautilus_string_list_free (fruits);
 	nautilus_string_list_free (empty);
 
-	/********/
-
-	tokens = nautilus_string_list_new_from_tokens (token_string, ":");
-
- 	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_string_list_equals (cities, tokens), TRUE);
-
-	nautilus_string_list_free (tokens);
-
-	tokens = nautilus_string_list_new_from_tokens (token_string_thick, "####");
-
- 	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_string_list_equals (cities, tokens), TRUE);
-
-	nautilus_string_list_free (cities);
-	nautilus_string_list_free (tokens);
-
-	/********/
-
-	single = nautilus_string_list_new_from_string ("something");
+	/*
+	 * nautilus_string_list_new_from_string
+	 */
+	single = nautilus_string_list_new_from_string ("something", TRUE);
 
  	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_string_list_contains (single, "something"), TRUE);
 	NAUTILUS_CHECK_INTEGER_RESULT (nautilus_string_list_get_length (single), 1);
@@ -454,7 +584,7 @@ nautilus_self_check_string_list (void)
 		GList			*glist_iterator;
 		NautilusStringList	*string_list;
 
-		string_list = nautilus_string_list_new ();
+		string_list = nautilus_string_list_new (TRUE);
 		
 		nautilus_string_list_insert (string_list, "orange");
 		nautilus_string_list_insert (string_list, "apple");
@@ -492,7 +622,7 @@ nautilus_self_check_string_list (void)
 	{
 		NautilusStringList *fruits;
 
-		fruits = nautilus_string_list_new ();
+		fruits = nautilus_string_list_new (TRUE);
 		
 		nautilus_string_list_insert (fruits, "orange");
 		nautilus_string_list_insert (fruits, "apple");
@@ -520,7 +650,7 @@ nautilus_self_check_string_list (void)
 	{
 		NautilusStringList *l;
 
-		fruits = nautilus_string_list_new ();
+		l = nautilus_string_list_new (TRUE);
 		
 		nautilus_string_list_insert (l, "x");
 
@@ -537,6 +667,192 @@ nautilus_self_check_string_list (void)
 
 		nautilus_string_list_free (l);
 	}
+
+
+	/*
+	 * nautilus_string_list_sort
+	 *
+	 */
+	{
+		NautilusStringList *l;
+
+		l = nautilus_string_list_new (TRUE);
+		
+		nautilus_string_list_insert (l, "dog");
+		nautilus_string_list_insert (l, "cat");
+		nautilus_string_list_insert (l, "bird");
+
+		nautilus_string_list_sort (l);
+
+		NAUTILUS_CHECK_STRING_RESULT (nautilus_string_list_nth (l, 0), "bird");
+		NAUTILUS_CHECK_STRING_RESULT (nautilus_string_list_nth (l, 1), "cat");
+		NAUTILUS_CHECK_STRING_RESULT (nautilus_string_list_nth (l, 2), "dog");
+
+		nautilus_string_list_free (l);
+	}
+
+	/*
+	 * nautilus_string_list_remove_duplicates
+	 *
+	 */
+	{
+		NautilusStringList *l;
+
+		l = nautilus_string_list_new (TRUE);
+
+		nautilus_string_list_remove_duplicates (l);
+		NAUTILUS_CHECK_INTEGER_RESULT (nautilus_string_list_get_length (l), 0);
+		
+		nautilus_string_list_insert (l, "foo");
+		nautilus_string_list_insert (l, "bar");
+		nautilus_string_list_insert (l, "bar");
+		nautilus_string_list_insert (l, "foo");
+		nautilus_string_list_insert (l, "foo");
+
+		nautilus_string_list_remove_duplicates (l);
+
+		NAUTILUS_CHECK_INTEGER_RESULT (nautilus_string_list_get_length (l), 2);
+		NAUTILUS_CHECK_STRING_RESULT (nautilus_string_list_nth (l, 0), "foo");
+		NAUTILUS_CHECK_STRING_RESULT (nautilus_string_list_nth (l, 1), "bar");
+
+		nautilus_string_list_clear (l);
+
+		nautilus_string_list_remove_duplicates (l);
+		NAUTILUS_CHECK_INTEGER_RESULT (nautilus_string_list_get_length (l), 0);
+
+		nautilus_string_list_insert (l, "single");
+		nautilus_string_list_remove_duplicates (l);
+		NAUTILUS_CHECK_INTEGER_RESULT (nautilus_string_list_get_length (l), 1);
+		NAUTILUS_CHECK_STRING_RESULT (nautilus_string_list_nth (l, 0), "single");
+
+		nautilus_string_list_clear (l);
+
+		nautilus_string_list_free (l);
+	}
+
+	/*
+	 * nautilus_string_list_assign_from_string_list
+	 *
+	 */
+	{
+		NautilusStringList *l;
+		NautilusStringList *other;
+
+		l = nautilus_string_list_new (TRUE);
+		other = nautilus_string_list_new (TRUE);
+
+		/* assign an other with some items */
+		nautilus_string_list_insert (other, "dog");
+		nautilus_string_list_insert (other, "cat");
+		nautilus_string_list_insert (other, "mouse");
+		nautilus_string_list_assign_from_string_list (l, other);
+		NAUTILUS_CHECK_INTEGER_RESULT (nautilus_string_list_get_length (l), 3);
+		NAUTILUS_CHECK_STRING_RESULT (nautilus_string_list_nth (l, 0), "dog");
+		NAUTILUS_CHECK_STRING_RESULT (nautilus_string_list_nth (l, 1), "cat");
+		NAUTILUS_CHECK_STRING_RESULT (nautilus_string_list_nth (l, 2), "mouse");
+
+		/* assign an other with 1 item */
+		nautilus_string_list_clear (other);
+		nautilus_string_list_insert (other, "something");
+		nautilus_string_list_assign_from_string_list (l, other);
+		NAUTILUS_CHECK_INTEGER_RESULT (nautilus_string_list_get_length (l), 1);
+		NAUTILUS_CHECK_STRING_RESULT (nautilus_string_list_nth (l, 0), "something");
+		
+		/* assign an empty other */
+		nautilus_string_list_clear (other);
+		nautilus_string_list_assign_from_string_list (l, other);
+		NAUTILUS_CHECK_INTEGER_RESULT (nautilus_string_list_get_length (l), 0);
+
+		nautilus_string_list_free (l);
+		nautilus_string_list_free (other);
+	}
+
+
+	/*
+	 * nautilus_string_list_get_longest_string
+	 *
+	 */
+	{
+		NautilusStringList *l;
+
+		l = nautilus_string_list_new (TRUE);
+
+		NAUTILUS_CHECK_STRING_RESULT (nautilus_string_list_get_longest_string (l), NULL);
+		NAUTILUS_CHECK_INTEGER_RESULT (nautilus_string_list_get_longest_string_length (l), 0);
+
+		nautilus_string_list_insert (l, "a");
+		nautilus_string_list_insert (l, "bb");
+		nautilus_string_list_insert (l, "ccc");
+		nautilus_string_list_insert (l, "dddd");
+
+		NAUTILUS_CHECK_STRING_RESULT (nautilus_string_list_get_longest_string (l), "dddd");
+		NAUTILUS_CHECK_INTEGER_RESULT (nautilus_string_list_get_longest_string_length (l), strlen ("dddd"));
+
+		nautilus_string_list_clear (l);
+
+		nautilus_string_list_insert (l, "foo");
+		NAUTILUS_CHECK_STRING_RESULT (nautilus_string_list_get_longest_string (l), "foo");
+		NAUTILUS_CHECK_INTEGER_RESULT (nautilus_string_list_get_longest_string_length (l), strlen ("foo"));
+
+		nautilus_string_list_free (l);
+
+	}
+
+	/*
+	 * case insensitive tests
+	 *
+	 */
+	{
+		NautilusStringList *l;
+
+		l = nautilus_string_list_new (FALSE);
+
+		nautilus_string_list_insert (l, "foo");
+		nautilus_string_list_insert (l, "bar");
+
+		NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_string_list_contains (l, "Foo"), TRUE);
+		NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_string_list_contains (l, "foO"), TRUE);
+		NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_string_list_contains (l, "fOo"), TRUE);
+		NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_string_list_contains (l, "foo"), TRUE);
+
+		nautilus_string_list_clear (l);
+
+		nautilus_string_list_insert (l, "Foo");
+		nautilus_string_list_insert (l, "Foo");
+		nautilus_string_list_insert (l, "fOo");
+		nautilus_string_list_insert (l, "foO");
+		nautilus_string_list_remove_duplicates (l);
+		NAUTILUS_CHECK_INTEGER_RESULT (nautilus_string_list_get_length (l), 1);
+
+		nautilus_string_list_free (l);
+	}
+
+	/*
+	 * nautilus_string_list_new_from_tokens
+	 */
+	{
+		NautilusStringList *lines;
+		NautilusStringList *thick_lines;
+
+		const char lines_string[] = "This\nAre\nSome\n\nLines";
+		const char thick_lines_string[] = "This####Are####Some########Lines";
+		const guint num_lines = nautilus_str_count_characters (lines_string, '\n') + 1;
+
+		lines = nautilus_string_list_new_from_tokens (lines_string, "\n", TRUE);
+		thick_lines = nautilus_string_list_new_from_tokens (thick_lines_string, "####", TRUE);
+
+		NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_string_list_equals (lines, thick_lines), TRUE);
+
+		NAUTILUS_CHECK_INTEGER_RESULT (nautilus_string_list_get_length (lines), num_lines);
+		NAUTILUS_CHECK_STRING_RESULT (nautilus_string_list_nth (lines, 0), "This");
+		NAUTILUS_CHECK_STRING_RESULT (nautilus_string_list_nth (lines, 1), "Are");
+		NAUTILUS_CHECK_STRING_RESULT (nautilus_string_list_nth (lines, 2), "Some");
+		NAUTILUS_CHECK_STRING_RESULT (nautilus_string_list_nth (lines, 3), "");
+		NAUTILUS_CHECK_STRING_RESULT (nautilus_string_list_nth (lines, 4), "Lines");
+		
+		nautilus_string_list_free (lines);
+		nautilus_string_list_free (thick_lines);
+	}
 }
 
 #endif /* !NAUTILUS_OMIT_SELF_CHECK */
