@@ -57,12 +57,36 @@ nautilus_strcasecmp (const char *string_a, const char *string_b)
 		             string_b == NULL ? "" : string_b);
 }
 
+int
+nautilus_strcmp_case_breaks_ties (const char *string_a, const char *string_b)
+{
+	int casecmp_result;
+
+	casecmp_result = nautilus_strcasecmp (string_a, string_b);
+	if (casecmp_result != 0) {
+		return casecmp_result;
+	}
+	return nautilus_strcmp (string_a, string_b);
+}
+
 gboolean
 nautilus_str_is_empty (const char *string_or_null)
 {
 	return nautilus_strcmp (string_or_null, NULL) == 0;
 }
 
+gboolean
+nautilus_str_is_equal (const char *string_a, const char *string_b)
+{
+	return nautilus_strcmp (string_a, string_b) == 0;
+}
+
+gboolean
+nautilus_istr_is_equal (const char *string_a, const char *string_b)
+{
+	return nautilus_strcasecmp (string_a, string_b) == 0;
+}
+
 int
 nautilus_str_compare (gconstpointer string_a, gconstpointer string_b)
 {
@@ -70,6 +94,13 @@ nautilus_str_compare (gconstpointer string_a, gconstpointer string_b)
 				(const char *) string_b);
 }
 
+int
+nautilus_istr_compare (gconstpointer string_a, gconstpointer string_b)
+{
+	return nautilus_strcasecmp ((const char *) string_a,
+				    (const char *) string_b);
+}
+
 gboolean
 nautilus_str_has_prefix (const char *haystack, const char *needle)
 {
@@ -308,7 +339,7 @@ nautilus_str_strip_chr (const char *source, char remove_this)
 		return NULL;
 	}
 	
-	result = g_malloc (strlen (source) + 1);
+	result = g_new (char, strlen (source) + 1);
 	in = source;
 	out = result;
 	do {
@@ -378,59 +409,6 @@ nautilus_eat_str_to_int (char *source, int *integer)
 	return result;
 }
 
-/* To use a directory name as a file name, we need to escape any slashes.
-   This means that "/" is replaced by "%2F" and "%" is replaced by "%25".
-   Later we might share the escaping code with some more generic escaping
-   function, but this should do for now.
-*/
-char *
-nautilus_str_escape_slashes (const char *string)
-{
-	char c;
-	const char *in;
-	guint length;
-	char *result;
-	char *out;
-
-	/* Figure out how long the result needs to be. */
-	in = string;
-	length = 0;
-	while ((c = *in++) != '\0')
-		switch (c) {
-		case '/':
-		case '%':
-			length += 3;
-			break;
-		default:
-			length += 1;
-		}
-
-	/* Create the result string. */
-	result = g_malloc (length + 1);
-	in = string;
-	out = result;	
-	while ((c = *in++) != '\0')
-		switch (c) {
-		case '/':
-			*out++ = '%';
-			*out++ = '2';
-			*out++ = 'F';
-			break;
-		case '%':
-			*out++ = '%';
-			*out++ = '2';
-			*out++ = '5';
-			break;
-		default:
-			*out++ = c;
-		}
-	g_assert (out == result + length);
-	*out = '\0';
-
-	return result;
-}
-
-
 char *
 nautilus_str_double_underscores (const char *string)
 {
@@ -452,7 +430,7 @@ nautilus_str_double_underscores (const char *string)
 		return g_strdup (string);
 	}
 
-	escaped = g_malloc (strlen (string) + underscores + 1);
+	escaped = g_new (char, strlen (string) + underscores + 1);
 	for (p = string, q = escaped; *p != '\0'; p++, q++) {
 		/* Add an extra underscore. */
 		if (*p == '_') {
@@ -483,6 +461,77 @@ nautilus_str_capitalize (const char *string)
 	return capitalized;
 }
 
+char *
+nautilus_str_middle_truncate (const char *string,
+			      guint truncate_length)
+{
+	char *truncated;
+	guint length;
+	guint num_left_chars;
+	guint num_right_chars;
+
+	const char delimter[] = "...";
+	const guint delimter_length = strlen (delimter);
+	const guint min_truncate_length = delimter_length + 2;
+
+	if (string == NULL) {
+		return NULL;
+	}
+
+	/* It doesnt make sense to truncate strings to less than
+	 * the size of the delimter plus 2 characters (one on each
+	 * side)
+	 */
+	if (truncate_length < min_truncate_length) {
+		return g_strdup (string);
+	}
+
+	length = strlen (string);
+
+	/* Make sure the string is not already small enough. */
+	if (length <= truncate_length) {
+		return g_strdup (string);
+	}
+
+	/* Find the 'middle' where the truncation will occur. */
+	num_left_chars = (truncate_length - delimter_length) / 2;
+	num_right_chars = truncate_length - num_left_chars - delimter_length + 1;
+
+	truncated = g_new (char, truncate_length + 1);
+
+	strncpy (truncated, string, num_left_chars);
+	strncpy (truncated + num_left_chars, delimter, delimter_length);
+	strncpy (truncated + num_left_chars + delimter_length, string + length - num_right_chars + 1, num_right_chars);
+	
+	return truncated;
+}
+
+/**
+ * nautilus_str_count_characters:
+ * Count the number of 'c' characters that occur in 'string.
+ * 
+ * @string: The string to be scanned.
+ * @c: The char to count.
+ * 
+ * Return value: @count, the 'c' occurance count.
+ */
+guint
+nautilus_str_count_characters (const char	*string,
+			       char		c)
+{
+	guint count = 0;
+
+	while (string && *string != '\0') {
+		if (*string == c) {
+			count++;
+		}
+
+		string++;
+	}
+
+	return count;
+}
+
 #if !defined (NAUTILUS_OMIT_SELF_CHECK)
 
 static int
@@ -614,6 +663,38 @@ nautilus_self_check_string (void)
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_capitalize ("foo"), "Foo");
 	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_capitalize ("Foo"), "Foo");
 
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("foo", 0), "foo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("foo", 1), "foo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("foo", 3), "foo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("foo", 4), "foo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("foo", 5), "foo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("foo", 6), "foo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("foo", 7), "foo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("a_much_longer_foo", 0), "a_much_longer_foo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("a_much_longer_foo", 1), "a_much_longer_foo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("a_much_longer_foo", 2), "a_much_longer_foo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("a_much_longer_foo", 3), "a_much_longer_foo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("a_much_longer_foo", 4), "a_much_longer_foo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("a_much_longer_foo", 5), "a...o");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("a_much_longer_foo", 6), "a...oo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("a_much_longer_foo", 7), "a_...oo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("a_much_longer_foo", 8), "a_...foo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("a_much_longer_foo", 9), "a_m...foo");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("something_even", 8), "so...ven");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("something_odd", 8), "so...odd");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("something_even", 9), "som...ven");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("something_odd", 9), "som...odd");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("something_even", 10), "som...even");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("something_odd", 10), "som..._odd");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("something_even", 11), "some...even");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("something_odd", 11), "some..._odd");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("something_even", 12), "some..._even");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("something_odd", 12), "some...g_odd");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("something_even", 13), "somet..._even");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("something_odd", 13), "something_odd");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("something_even", 14), "something_even");
+	NAUTILUS_CHECK_STRING_RESULT (nautilus_str_middle_truncate ("something_odd", 13), "something_odd");
+
 	#define TEST_INTEGER_CONVERSION_FUNCTIONS(string, boolean_result, integer_result) \
 		NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_str_to_int (string, &integer), boolean_result); \
 		NAUTILUS_CHECK_INTEGER_RESULT (call_str_to_int (string), integer_result); \
@@ -641,6 +722,36 @@ nautilus_self_check_string (void)
 	TEST_INTEGER_CONVERSION_FUNCTIONS ("21474836470", FALSE, 9999)
 	TEST_INTEGER_CONVERSION_FUNCTIONS ("+21474836470", FALSE, 9999)
 	TEST_INTEGER_CONVERSION_FUNCTIONS ("-21474836480", FALSE, 9999)
+
+	/* nautilus_str_is_equal */
+	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_str_is_equal (NULL, NULL), TRUE);
+	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_str_is_equal (NULL, ""), TRUE);
+	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_str_is_equal ("", ""), TRUE);
+	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_str_is_equal ("", NULL), TRUE);
+	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_str_is_equal ("", ""), TRUE);
+	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_str_is_equal ("foo", "foo"), TRUE);
+	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_str_is_equal ("foo", "bar"), FALSE);
+
+	/* nautilus_istr_is_equal */
+	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_istr_is_equal (NULL, NULL), TRUE);
+	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_istr_is_equal (NULL, ""), TRUE);
+	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_istr_is_equal ("", ""), TRUE);
+	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_istr_is_equal ("", NULL), TRUE);
+	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_istr_is_equal ("", ""), TRUE);
+	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_istr_is_equal ("foo", "foo"), TRUE);
+	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_istr_is_equal ("foo", "bar"), FALSE);
+	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_istr_is_equal ("Foo", "foo"), TRUE);
+	NAUTILUS_CHECK_BOOLEAN_RESULT (nautilus_istr_is_equal ("foo", "Foo"), TRUE);
+
+	/* nautilus_str_count_characters */
+	NAUTILUS_CHECK_INTEGER_RESULT (nautilus_str_count_characters (NULL, 'x'), 0);
+	NAUTILUS_CHECK_INTEGER_RESULT (nautilus_str_count_characters ("", 'x'), 0);
+	NAUTILUS_CHECK_INTEGER_RESULT (nautilus_str_count_characters (NULL, '\0'), 0);
+	NAUTILUS_CHECK_INTEGER_RESULT (nautilus_str_count_characters ("", '\0'), 0);
+	NAUTILUS_CHECK_INTEGER_RESULT (nautilus_str_count_characters ("foo", 'x'), 0);
+	NAUTILUS_CHECK_INTEGER_RESULT (nautilus_str_count_characters ("foo", 'f'), 1);
+	NAUTILUS_CHECK_INTEGER_RESULT (nautilus_str_count_characters ("foo", 'o'), 2);
+	NAUTILUS_CHECK_INTEGER_RESULT (nautilus_str_count_characters ("xxxx", 'x'), 4);
 }
 
 #endif /* !NAUTILUS_OMIT_SELF_CHECK */
