@@ -1,4 +1,5 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
+
 /* fm-directory-view.h
  *
  * Copyright (C) 1999, 2000  Free Software Foundaton
@@ -28,7 +29,19 @@
 #include <gtk/gtkeditable.h>
 #include <bonobo/bonobo-control.h>
 
-void nautilus_clipboard_set_up_editable (GtkEditable *target,
-					 BonoboControl *control);
+/* This makes this editable put clipboard commands into the passed UI
+ * container when the editable is in focus. Callers in Nautilus
+ * normally get the UI container from
+ * nautilus_window_get_bonobo_ui_container.
+ */
+void nautilus_clipboard_set_up_editable            (GtkEditable        *target,
+						    Bonobo_UIContainer  container);
 
-#endif
+/* Components should use this convenient cover instead of the call
+ * above. This cover waits until the UI container shows up, so it can be
+ * called even before the control has a UI container.
+ */
+void nautilus_clipboard_set_up_editable_in_control (GtkEditable        *target,
+						    BonoboControl      *control);
+
+#endif /* NAUTILUS_CLIPBOARD_H */
