@@ -79,45 +79,45 @@ typedef struct {
 
 void                 nautilus_zoomable_real_set_bonobo_control  (NautilusZoomable           *view,
 								 BonoboControl              *bonobo_control);
-static CORBA_double  impl_Nautilus_Zoomable__get_zoom_level     (impl_POA_Nautilus_Zoomable *servant,
+static CORBA_float  impl_Nautilus_Zoomable__get_zoom_level     (PortableServer_Servant      servant,
 								 CORBA_Environment          *ev);
-static void          impl_Nautilus_Zoomable__set_zoom_level     (impl_POA_Nautilus_Zoomable *servant,
-								 const CORBA_double          zoom_level,
+static void          impl_Nautilus_Zoomable__set_zoom_level     (PortableServer_Servant      servant,
+								 const CORBA_float           zoom_level,
 								 CORBA_Environment          *ev);
-static CORBA_double  impl_Nautilus_Zoomable__get_min_zoom_level (impl_POA_Nautilus_Zoomable *servant,
+static CORBA_float  impl_Nautilus_Zoomable__get_min_zoom_level (PortableServer_Servant      servant,
 								 CORBA_Environment          *ev);
-static CORBA_double  impl_Nautilus_Zoomable__get_max_zoom_level (impl_POA_Nautilus_Zoomable *servant,
+static CORBA_float  impl_Nautilus_Zoomable__get_max_zoom_level (PortableServer_Servant      servant,
 								 CORBA_Environment          *ev);
-static CORBA_boolean impl_Nautilus_Zoomable__get_is_continuous (impl_POA_Nautilus_Zoomable *servant,
+static CORBA_boolean impl_Nautilus_Zoomable__get_is_continuous (PortableServer_Servant      servant,
 								 CORBA_Environment          *ev);
 static Nautilus_ZoomLevelList* impl_Nautilus_Zoomable__get_preferred_zoom_level_list
-							        (impl_POA_Nautilus_Zoomable *servant,
+							        (PortableServer_Servant      servant,
 								 CORBA_Environment          *ev);
-static void          impl_Nautilus_Zoomable_zoom_in             (impl_POA_Nautilus_Zoomable *servant,
+static void          impl_Nautilus_Zoomable_zoom_in             (PortableServer_Servant      servant,
 								 CORBA_Environment          *ev);
-static void          impl_Nautilus_Zoomable_zoom_out            (impl_POA_Nautilus_Zoomable *servant,
+static void          impl_Nautilus_Zoomable_zoom_out            (PortableServer_Servant      servant,
 								 CORBA_Environment          *ev);
-static void          impl_Nautilus_Zoomable_zoom_to_fit         (impl_POA_Nautilus_Zoomable *servant,
+static void          impl_Nautilus_Zoomable_zoom_to_fit         (PortableServer_Servant      servant,
 								 CORBA_Environment          *ev);
 
 POA_Nautilus_Zoomable__epv libnautilus_Nautilus_Zoomable_epv =
 {
 	NULL,			/* _private */
-	(gpointer) &impl_Nautilus_Zoomable__get_zoom_level,
-	(gpointer) &impl_Nautilus_Zoomable__set_zoom_level,
-	(gpointer) &impl_Nautilus_Zoomable__get_min_zoom_level,
-	(gpointer) &impl_Nautilus_Zoomable__get_max_zoom_level,
-	(gpointer) &impl_Nautilus_Zoomable__get_is_continuous,
-	(gpointer) &impl_Nautilus_Zoomable__get_preferred_zoom_level_list,
-	(gpointer) &impl_Nautilus_Zoomable_zoom_in,
-	(gpointer) &impl_Nautilus_Zoomable_zoom_out,
-	(gpointer) &impl_Nautilus_Zoomable_zoom_to_fit
+	&impl_Nautilus_Zoomable__get_zoom_level,
+	&impl_Nautilus_Zoomable__set_zoom_level,
+	&impl_Nautilus_Zoomable__get_min_zoom_level,
+	&impl_Nautilus_Zoomable__get_max_zoom_level,
+	&impl_Nautilus_Zoomable__get_is_continuous,
+	&impl_Nautilus_Zoomable__get_preferred_zoom_level_list,
+	&impl_Nautilus_Zoomable_zoom_in,
+	&impl_Nautilus_Zoomable_zoom_out,
+	&impl_Nautilus_Zoomable_zoom_to_fit
 };
 
-//static PortableServer_ServantBase__epv base_epv;
+/* static PortableServer_ServantBase__epv base_epv; */
 static POA_Nautilus_Zoomable__vepv impl_Nautilus_Zoomable_vepv =
 {
-//	&base_epv,
+/*	&base_epv, */
 	NULL,
 	NULL,
 	&libnautilus_Nautilus_Zoomable_epv
@@ -155,52 +155,76 @@ nautilus_ZoomLevelListBuffer_from_zoom_levels (const double *zoom_levels, int nu
 	return buffer;
 }
 
-static CORBA_double
-impl_Nautilus_Zoomable__get_zoom_level (impl_POA_Nautilus_Zoomable *servant,
+static CORBA_float
+impl_Nautilus_Zoomable__get_zoom_level (PortableServer_Servant      servant,
 					CORBA_Environment          *ev)
 {
-	return servant->gtk_object->details->zoom_level;
+	impl_POA_Nautilus_Zoomable *zoomable;
+
+	zoomable = (impl_POA_Nautilus_Zoomable *)servant;
+
+	return zoomable->gtk_object->details->zoom_level;
 }
 
 static void 
-impl_Nautilus_Zoomable__set_zoom_level (impl_POA_Nautilus_Zoomable *servant,
-					const CORBA_double          zoom_level,
+impl_Nautilus_Zoomable__set_zoom_level (PortableServer_Servant      servant,
+					const CORBA_float          zoom_level,
 					CORBA_Environment          *ev)
 {
-	gtk_signal_emit (GTK_OBJECT (servant->gtk_object), signals[SET_ZOOM_LEVEL], zoom_level);
+	impl_POA_Nautilus_Zoomable *zoomable;
+
+	zoomable = (impl_POA_Nautilus_Zoomable *)servant;
+
+	gtk_signal_emit (GTK_OBJECT (zoomable->gtk_object), signals[SET_ZOOM_LEVEL], zoom_level);
 	
 }
 
-static CORBA_double
-impl_Nautilus_Zoomable__get_min_zoom_level (impl_POA_Nautilus_Zoomable *servant,
+static CORBA_float
+impl_Nautilus_Zoomable__get_min_zoom_level (PortableServer_Servant      servant,
 					    CORBA_Environment      *ev)
 {
-	return servant->gtk_object->details->min_zoom_level;
+	impl_POA_Nautilus_Zoomable *zoomable;
+
+	zoomable = (impl_POA_Nautilus_Zoomable *)servant;
+
+	return zoomable->gtk_object->details->min_zoom_level;
 }
-static CORBA_double
-impl_Nautilus_Zoomable__get_max_zoom_level (impl_POA_Nautilus_Zoomable *servant,
+
+static CORBA_float
+impl_Nautilus_Zoomable__get_max_zoom_level (PortableServer_Servant      servant,
 					    CORBA_Environment      *ev)
 {
-	return servant->gtk_object->details->max_zoom_level;
+	impl_POA_Nautilus_Zoomable *zoomable;
+
+	zoomable = (impl_POA_Nautilus_Zoomable *)servant;
+
+	return zoomable->gtk_object->details->max_zoom_level;
 }
 
 static CORBA_boolean
-impl_Nautilus_Zoomable__get_is_continuous (impl_POA_Nautilus_Zoomable *servant,
+impl_Nautilus_Zoomable__get_is_continuous (PortableServer_Servant      servant,
 					   CORBA_Environment      *ev)
 {
-	return servant->gtk_object->details->is_continuous;
+	impl_POA_Nautilus_Zoomable *zoomable;
+
+	zoomable = (impl_POA_Nautilus_Zoomable *)servant;
+
+	return zoomable->gtk_object->details->is_continuous;
 }
 
 static Nautilus_ZoomLevelList *
-impl_Nautilus_Zoomable__get_preferred_zoom_level_list (impl_POA_Nautilus_Zoomable *servant,
+impl_Nautilus_Zoomable__get_preferred_zoom_level_list (PortableServer_Servant      servant,
 					    		  CORBA_Environment      *ev)
 {
 	Nautilus_ZoomLevelList *list;
+	impl_POA_Nautilus_Zoomable *zoomable;
+
+	zoomable = (impl_POA_Nautilus_Zoomable *)servant;
 
 	list = Nautilus_ZoomLevelList__alloc ();
-	list->_maximum = servant->gtk_object->details->num_preferred_zoom_levels;
-	list->_length = servant->gtk_object->details->num_preferred_zoom_levels;
-	list->_buffer = servant->gtk_object->details->preferred_zoom_levels;
+	list->_maximum = zoomable->gtk_object->details->num_preferred_zoom_levels;
+	list->_length = zoomable->gtk_object->details->num_preferred_zoom_levels;
+	list->_buffer = zoomable->gtk_object->details->preferred_zoom_levels;
 	
 	/*  set_release defaults to FALSE - CORBA_sequence_set_release (list, FALSE) */ 
 
@@ -208,67 +232,76 @@ impl_Nautilus_Zoomable__get_preferred_zoom_level_list (impl_POA_Nautilus_Zoomabl
 }
 
 static void
-impl_Nautilus_Zoomable_zoom_in (impl_POA_Nautilus_Zoomable *servant,
+impl_Nautilus_Zoomable_zoom_in (PortableServer_Servant      servant,
 				CORBA_Environment          *ev)
 {	
-	gtk_signal_emit (GTK_OBJECT (servant->gtk_object), signals[ZOOM_IN]);
+	impl_POA_Nautilus_Zoomable *zoomable;
+
+	zoomable = (impl_POA_Nautilus_Zoomable *)servant;
+
+	gtk_signal_emit (GTK_OBJECT (zoomable->gtk_object), signals[ZOOM_IN]);
 }
 
 static void
-impl_Nautilus_Zoomable_zoom_out (impl_POA_Nautilus_Zoomable *servant,
-				 CORBA_Environment          *ev)
+impl_Nautilus_Zoomable_zoom_out (PortableServer_Servant      servant,
+				 CORBA_Environment *ev)
 {
-	gtk_signal_emit (GTK_OBJECT (servant->gtk_object), signals[ZOOM_OUT]);
+	impl_POA_Nautilus_Zoomable *zoomable;
+
+	zoomable = (impl_POA_Nautilus_Zoomable *)servant;
+
+	gtk_signal_emit (GTK_OBJECT (zoomable->gtk_object), signals[ZOOM_OUT]);
 }
 
 static void
-impl_Nautilus_Zoomable_zoom_to_fit (impl_POA_Nautilus_Zoomable *servant,
-				    CORBA_Environment      *ev)
+impl_Nautilus_Zoomable_zoom_to_fit (PortableServer_Servant      servant,
+				    CORBA_Environment *ev)
 {
-	gtk_signal_emit (GTK_OBJECT (servant->gtk_object), signals[ZOOM_TO_FIT]);
+	impl_POA_Nautilus_Zoomable *zoomable;
+
+	zoomable = (impl_POA_Nautilus_Zoomable *)servant;
+
+	gtk_signal_emit (GTK_OBJECT (zoomable->gtk_object), signals[ZOOM_TO_FIT]);
 }
 
 static void
-impl_Nautilus_Zoomable__destroy(BonoboObject *obj, impl_POA_Nautilus_Zoomable *servant)
+impl_Nautilus_Zoomable__destroy (BonoboObject *obj,
+				 PortableServer_Servant      servant)
 {
 	PortableServer_ObjectId *objid;
 	CORBA_Environment ev;
-	void (*servant_destroy_func) (PortableServer_Servant servant, CORBA_Environment *ev);
 	
 	CORBA_exception_init(&ev);
 	
-	servant_destroy_func = NAUTILUS_ZOOMABLE_CLASS (GTK_OBJECT (servant->gtk_object)->klass)->servant_destroy_func;
 	objid = PortableServer_POA_servant_to_id (bonobo_poa (), servant, &ev);
 	PortableServer_POA_deactivate_object (bonobo_poa (), objid, &ev);
 	CORBA_free (objid);
 	obj->servant = NULL;
 	
-	servant_destroy_func ((PortableServer_Servant) servant, &ev);
+	POA_Nautilus_Zoomable__fini (servant, &ev);
 	g_free (servant);
 	CORBA_exception_free(&ev);
 }
 
 static Nautilus_Zoomable
-impl_Nautilus_Zoomable__create(NautilusZoomable *zoomable, CORBA_Environment * ev)
+impl_Nautilus_Zoomable__create (NautilusZoomable *zoomable, CORBA_Environment * ev)
 {
 	Nautilus_Zoomable retval;
 	impl_POA_Nautilus_Zoomable *servant;
-	void (*servant_init_func) (PortableServer_Servant servant, CORBA_Environment *ev);
 	
 	NautilusZoomableClass *zoomable_class = NAUTILUS_ZOOMABLE_CLASS (GTK_OBJECT(zoomable)->klass);
 	
-	servant_init_func = zoomable_class->servant_init_func;
 	servant = g_new0 (impl_POA_Nautilus_Zoomable, 1);
 	servant->servant.vepv = zoomable_class->vepv;
 	if (!servant->servant.vepv->Bonobo_Unknown_epv)
 		servant->servant.vepv->Bonobo_Unknown_epv = bonobo_object_get_epv ();
-	servant_init_func ((PortableServer_Servant) servant, ev);
+	POA_Nautilus_Zoomable__init ((PortableServer_Servant) servant, ev);
 	
 	servant->gtk_object = zoomable;
 	
 	retval = bonobo_object_activate_servant (BONOBO_OBJECT (zoomable), servant);
 	
-	gtk_signal_connect (GTK_OBJECT (zoomable), "destroy", GTK_SIGNAL_FUNC (impl_Nautilus_Zoomable__destroy), servant);
+	gtk_signal_connect (GTK_OBJECT (zoomable), "destroy", impl_Nautilus_Zoomable__destroy, servant);
 	
 	return retval;
 }
@@ -310,8 +343,6 @@ nautilus_zoomable_initialize_class (NautilusZoomableClass *klass)
 	
 	parent_class = gtk_type_class (gtk_type_parent (object_class->type));
 	
-	klass->servant_init_func = POA_Nautilus_Zoomable__init;
-	klass->servant_destroy_func = POA_Nautilus_Zoomable__fini;
 	klass->vepv = &impl_Nautilus_Zoomable_vepv;
 	
 	signals[SET_ZOOM_LEVEL] =
