@@ -50,11 +50,29 @@ typedef struct _RsvgFTFontCacheEntry RsvgFTFontCacheEntry;
 typedef struct _RsvgFTGlyphDesc RsvgFTGlyphDesc;
 typedef struct _RsvgFTGlyphCacheEntry RsvgFTGlyphCacheEntry;
 
+#define SUBPIXEL_FRACTION 4
+
 struct _RsvgFTCtx {
 	FT_Library ftlib;
 
 	GHashTable *font_hash_table; /* map filename to RsvgFTFontCacheEntry */
 
+	/* Notes on the font list:
+
+	   There is a font list entry for each font that is "intern"ed.
+	   This entry persists for the life of the RsvgFTCtx structure.
+	   These entry correspond one-to-one with font handles, which
+	   begin at 0 and allocate upwards.
+
+	   Each entry in the font list may be either loaded or not, as
+	   indicated by the non-NULL status of the font element of the
+	   RsvgFTFontCacheEntry.
+
+	   The lru list (first and last pointers here, prev and next
+	   pointers in the RsvgFTFontCacheEntry) contains only loaded
+	   fonts. This should be considered an invariant.
+	*/
+
 	int n_font_list;
 	RsvgFTFontCacheEntry **font_list;
 	RsvgFTFontCacheEntry *first, *last;
@@ -95,7 +113,9 @@ struct _RsvgFTGlyphDesc {
 
 struct _RsvgFTGlyphCacheEntry {
 	RsvgFTGlyphCacheEntry *prev, *next; /* for lru list */
+	int x0, y0; /* relative to affine */
 	RsvgFTGlyph *glyph;
+	RsvgFTGlyphDesc *desc;
 };
 
 /* Glyph cache fun stuff */
@@ -120,11 +140,11 @@ rsvg_ft_glyph_desc_equal (gconstpointer v, gconstpointer v2)
 			sizeof(RsvgFTGlyphDesc));
 }
 
-#if 0
 /**
  * rsvg_ft_glyph_lookup: Look up a glyph in the glyph cache.
  * @ctx: The Rsvg FT context.
  * @desc: Glyph descriptor.
+ * @xy: Where to store relative coordinates of glyph.
  *
  * Looks up the glyph in the glyph cache. If found, moves it to the front
  * of the LRU list. It does not bump the refcount on the glyph - most
@@ -133,7 +153,8 @@ rsvg_ft_glyph_desc_equal (gconstpointer v, gconstpointer v2)
  * Return value: The glyph if found, otherwise NULL.
  **/
 static RsvgFTGlyph *
-rsvg_ft_glyph_lookup (RsvgFTCtx *ctx, const RsvgFTGlyphDesc *desc)
+rsvg_ft_glyph_lookup (RsvgFTCtx *ctx, const RsvgFTGlyphDesc *desc,
+		      int glyph_xy[2])
 {
 	RsvgFTGlyphCacheEntry *entry;
 
@@ -155,9 +176,114 @@ rsvg_ft_glyph_lookup (RsvgFTCtx *ctx, const RsvgFTGlyphDesc *desc)
 		ctx->glyph_first = entry;
 
 	}
+	glyph_xy[0] = entry->x0;
+	glyph_xy[1] = entry->y0;
 	return entry->glyph;
 }
-#endif
+
+/**
+ * rsvg_ft_glyph_bytes: Determine the number of bytes in a glyph.
+ * @glyph: Glyph.
+ *
+ * Determines the number of bytes used by @glyph, for the purposes of
+ * caching. Note: I'm not counting malloc overhead. Maybe I should.
+ *
+ * Return value: Number of bytes used by glyph.
+ **/
+static int
+rsvg_ft_glyph_bytes (RsvgFTGlyph *glyph)
+{
+	return glyph->rowstride * glyph->height + sizeof (RsvgFTGlyph);
+}
+
+/**
+ * rsvg_gt_glyph_evict: Evict lru glyph from glyph cache.
+ * @ctx: The RsvgFT context.
+ *
+ * Chooses the least recently used glyph that without a refcount, and
+ * evicts it.
+ *
+ * Return value: true if a glyph was successfully evicted.
+ **/
+static gboolean
+rsvg_ft_glyph_evict (RsvgFTCtx *ctx)
+{
+	RsvgFTGlyphCacheEntry *victim;
+	RsvgFTGlyph *glyph;
+
+	for (victim = ctx->glyph_last; victim != NULL; victim = victim->prev)
+		if (victim->glyph->refcnt == 1)
+			break;
+
+	if (victim == NULL)
+		return FALSE;
+
+	if (victim->prev != NULL)
+		victim->prev->next = victim->next;
+	else
+		ctx->glyph_first = victim->next;
+	if (victim->next != NULL)
+		victim->next->prev = victim->prev;
+	else
+		ctx->glyph_last = victim->prev;
+
+	glyph = victim->glyph;
+	ctx->glyph_bytes -= rsvg_ft_glyph_bytes (glyph);
+	rsvg_ft_glyph_unref (glyph);
+
+	g_hash_table_remove (ctx->glyph_hash_table, victim->desc);
+	g_free (victim->desc);
+	g_free (victim);
+
+	return TRUE;
+}
+
+/**
+ * rsvg_ft_glyph_insert: Insert a glyph into the glyph cache.
+ * @ctx: The RsvgFT context.
+ * @desc: Glyph descriptor.
+ * @glyph: The glyph itself.
+ * @x0: Relative x0 of glyph.
+ * @y0: Relative y0 of glyph.
+ *
+ * Inserts @glyph into the glyph cache under the glyph descriptor @desc.
+ * This routine also takes care of evicting glyphs when the cache
+ * becomes full.
+ **/
+static void
+rsvg_ft_glyph_insert (RsvgFTCtx *ctx, const RsvgFTGlyphDesc *desc,
+		      RsvgFTGlyph *glyph, int x0, int y0)
+{
+	RsvgFTGlyphDesc *new_desc;
+	RsvgFTGlyphCacheEntry *entry;
+
+	ctx->glyph_bytes += rsvg_ft_glyph_bytes (glyph);
+
+	/* check for full cache and evict if so */
+	while (ctx->glyph_bytes > ctx->glyph_bytes_max) {
+		if (!rsvg_ft_glyph_evict (ctx)) {
+			g_warning ("rsvg_ft_glyph_insert: unable to free any glyph cache entry, suggesting resource leak.");
+			break;
+		}
+	}
+
+	new_desc = g_new (RsvgFTGlyphDesc, 1);
+	memcpy (new_desc, desc, sizeof (RsvgFTGlyphDesc));
+	entry = g_new (RsvgFTGlyphCacheEntry, 1);
+	entry->prev = NULL;
+	entry->next = ctx->glyph_first;
+	if (entry->next != NULL) {
+		entry->next->prev = entry;
+	} else {
+		ctx->glyph_last = entry;
+	}
+	ctx->glyph_first = entry;
+	entry->glyph = glyph;
+	entry->desc = new_desc;
+	entry->x0 = x0;
+	entry->y0 = y0;
+	g_hash_table_insert (ctx->glyph_hash_table, new_desc, entry);
+}
 
 RsvgFTCtx *
 rsvg_ft_ctx_new (void) {
@@ -191,7 +317,9 @@ rsvg_ft_ctx_new (void) {
 void
 rsvg_ft_ctx_done (RsvgFTCtx *ctx) {
 	int i;
+	RsvgFTGlyphCacheEntry *glyph_ce, *next;
 
+	g_hash_table_destroy (ctx->font_hash_table);
 	for (i = 0; i < ctx->n_font_list; i++) {
 		RsvgFTFontCacheEntry *entry = ctx->font_list[i];
 		RsvgFTFont *font = entry->font;
@@ -201,8 +329,20 @@ rsvg_ft_ctx_done (RsvgFTCtx *ctx) {
 			FT_Done_Face (font->face);
 			g_free (font);
 		}
+		g_free (entry);
 	}
 	g_free (ctx->font_list);
+
+	/* Free glyph cache. */
+	g_hash_table_destroy (ctx->glyph_hash_table);
+	for (glyph_ce = ctx->glyph_first; glyph_ce != NULL; glyph_ce = next) {
+		g_free (glyph_ce->desc);
+		g_free (glyph_ce->glyph->buf);
+		g_free (glyph_ce->glyph);
+		next = glyph_ce->next;
+		g_free (glyph_ce);
+	}
+
 	FT_Done_FreeType (ctx->ftlib);
 	g_free (ctx);
 }
@@ -251,23 +391,7 @@ rsvg_ft_intern (RsvgFTCtx *ctx, const char *font_file_name)
 	RsvgFTFontCacheEntry *entry;
 
 	entry = g_hash_table_lookup (ctx->font_hash_table, font_file_name);
-	if (entry != NULL) {
-		/* found in font list */
-
-		/* move entry to front of LRU list */
-		if (entry->prev != NULL) {
-			entry->prev->next = entry->next;
-			if (entry->next != NULL) {
-				entry->next->prev = entry->prev;
-			} else {
-				ctx->last = entry->prev;
-			}
-			entry->prev = NULL;
-			entry->next = ctx->first;
-			ctx->first->prev = entry;
-			ctx->first = entry;
-		}
-	} else {
+	if (entry == NULL) {
 		/* not found in font list */
 		int n_font_list;
 
@@ -278,12 +402,7 @@ rsvg_ft_intern (RsvgFTCtx *ctx, const char *font_file_name)
 		entry->handle = n_font_list;
 		entry->font = NULL;
 		entry->prev = NULL;
-		entry->next = ctx->first;
-		if (ctx->first != NULL)
-			ctx->first->prev = entry;
-		else
-			ctx->last = entry;
-		ctx->first = entry;
+		entry->next = NULL;
 		if (n_font_list == 0) {
 			ctx->font_list = g_new (RsvgFTFontCacheEntry *, 1);
 		} else if (!(n_font_list & (n_font_list - 1))) {
@@ -310,7 +429,7 @@ rsvg_ft_intern (RsvgFTCtx *ctx, const char *font_file_name)
  **/
 void
 rsvg_ft_font_attach (RsvgFTCtx *ctx, RsvgFTFontHandle fh,
-		     char *font_file_name)
+		     const char *font_file_name)
 {
 	RsvgFTFontCacheEntry *entry;
 	RsvgFTFont *font;
@@ -328,6 +447,53 @@ rsvg_ft_font_attach (RsvgFTCtx *ctx, RsvgFTFontHandle fh,
 	}
 }
 
+/**
+ * rsvg_ft_font_evict: Evict least recently used font from font cache.
+ * @ctx: Rsvg FT context.
+ *
+ * Removes the least recently used font from the font cache.
+ **/
+static void
+rsvg_ft_font_evict (RsvgFTCtx *ctx)
+{
+	RsvgFTFontCacheEntry *victim;
+	RsvgFTFont *font;
+
+#ifdef DEBUG
+	g_print ("rsvg_ft_font_evict: evicting!\n");
+#endif
+
+	victim = ctx->last;
+	if (victim == NULL) {
+		/* We definitely shouldn't get here, but if we do,
+		   print an error message that helps explain why we
+		   did. */
+		if (ctx->n_loaded_fonts > 0) {
+			g_error ("rsvg_ft_font_evict: no font in loaded font list to evict, but ctx->n_loaded_fonts = %d, internal invariant violated",
+				 ctx->n_loaded_fonts);
+		} else {
+			g_error ("rsvg_ft_font_evict: ctx->n_loaded_fonts_max = %d, it must be positive",
+				 ctx->n_loaded_fonts_max);
+		}
+	}
+
+	if (victim->prev == NULL)
+		ctx->first = NULL;
+	else
+		victim->prev->next = NULL;
+	if (victim->next != NULL) {
+		g_warning ("rsvg_ft_font_evict: last font in LRU font list has non-NULL next field, suggesting corruption of data structure");
+	}
+	ctx->last = victim->prev;
+
+	font = victim->font;
+	FT_Done_Face (font->face);
+	g_free (font);
+	victim->font = NULL;
+
+	ctx->n_loaded_fonts--;
+}
+
 /**
  * rsvg_ft_font_resolve: Resolve a font handle.
  * @ctx: Rsvg FT context.
@@ -349,23 +515,46 @@ rsvg_ft_font_resolve (RsvgFTCtx *ctx, RsvgFTFontHandle fh)
 		return NULL;
 	entry = ctx->font_list[fh];
 	if (entry->font == NULL) {
-		ctx->n_loaded_fonts++;
-		while (ctx->n_loaded_fonts > ctx->n_loaded_fonts_max) {
-			/* todo: evict lru */
-			break;
+		while (ctx->n_loaded_fonts >= ctx->n_loaded_fonts_max) {
+			rsvg_ft_font_evict (ctx);
 		}
 		font = rsvg_ft_load (ctx, entry->fn);
-		if (entry->fn_attached != NULL) {
-			FT_Error error;
+		if (font != NULL) {
+			if (entry->fn_attached != NULL) {
+				FT_Error error;
 
-			error = FT_Attach_File (font->face,
-						entry->fn_attached);
+				error = FT_Attach_File (font->face,
+							entry->fn_attached);
+			}
+			entry->font = font;
+			ctx->n_loaded_fonts++;
+
+			/* insert entry at front of list */
+			entry->next = ctx->first;
+			if (ctx->first != NULL)
+				ctx->first->prev = entry;
+			else
+				ctx->last = entry;
+			ctx->first = entry;
 		}
-		entry->font = font;
-	}
-	else {
+	} else {
 		font = entry->font;
+
+		/* move entry to front of LRU list */
+		if (entry->prev != NULL) {
+			entry->prev->next = entry->next;
+			if (entry->next != NULL) {
+				entry->next->prev = entry->prev;
+			} else {
+				ctx->last = entry->prev;
+			}
+			entry->prev = NULL;
+			entry->next = ctx->first;
+			ctx->first->prev = entry;
+			ctx->first = entry;
+		}
 	}
+
 	return font;
 }
 
@@ -440,6 +629,8 @@ rsvg_ft_get_glyph (RsvgFTFont *font, FT_UInt glyph_ix, double sx, double sy,
 	FT_Vector delta;
 	double expansion, scale;
 
+	result = NULL;
+
 	if (glyph_ix == 0)
 		return NULL;
 
@@ -495,8 +686,8 @@ rsvg_ft_get_glyph (RsvgFTFont *font, FT_UInt glyph_ix, double sx, double sy,
 		xy[1] = -glyph->bitmap_top;
 		result->width = bitmap->width;
 		result->height = bitmap->rows;
-		result->xpen = FT_TOFLOAT (delta.x + glyph->advance.x) - xy[0];
-		result->ypen = -FT_TOFLOAT (delta.y + glyph->advance.y) - xy[1];
+		result->xpen = FT_TOFLOAT (glyph->advance.x);
+		result->ypen = -FT_TOFLOAT (glyph->advance.y);
 		result->rowstride = bitmap->pitch;
 		bufsize = bitmap->pitch * bitmap->rows;
 		buf = g_malloc (bufsize);
@@ -507,7 +698,70 @@ rsvg_ft_get_glyph (RsvgFTFont *font, FT_UInt glyph_ix, double sx, double sy,
 }
 
 /**
- * rsvg_ft_render_string: Render a string into a glyph image.
+ * rsvg_ft_get_glyph_cached: Get a rendered glyph, trying the cache.
+ * @ctx: The RsvgFT context.
+ * @fh: Font handle for the font.
+ * @glyph_ix: Glyph index.
+ * @sx: Width of em in pixels.
+ * @sy: Height of em in pixels.
+ * @affine: Affine transformation.
+ * @xy: Where to store the top left coordinates.
+ *
+ * Note: The nominal resolution is 72 dpi. For rendering at other resolutions,
+ * scale the @affine by resolution/(72 dpi).
+ *
+ * Return value: The rendered glyph.
+ **/
+static RsvgFTGlyph *
+rsvg_ft_get_glyph_cached (RsvgFTCtx *ctx, RsvgFTFontHandle fh,
+			  FT_UInt glyph_ix, double sx, double sy,
+			  const double affine[6], int xy[2])
+{
+	RsvgFTGlyphDesc desc;
+	RsvgFTFont *font;
+	RsvgFTGlyph *result;
+	int x_sp;
+
+	if (affine[1] != 0 || affine[2] != 0 || affine[0] != affine[3]) {
+		font = rsvg_ft_font_resolve (ctx, fh);
+		return rsvg_ft_get_glyph (font, glyph_ix, sx, sy, affine, xy);
+	}
+	desc.fh = fh;
+	desc.char_width = floor (sx * 64 + 0.5);
+	desc.char_height = floor (sy * 64 + 0.5);
+	desc.glyph_index = glyph_ix;
+	x_sp = floor (SUBPIXEL_FRACTION * (affine[4] - floor (affine[4])));
+	desc.x_subpixel = x_sp;
+	desc.y_subpixel = 0;
+#ifdef VERBOSE
+	g_print ("affine[4] = %g, x subpix = %x\n",
+		 affine[4], desc.x_subpixel);
+#endif
+	result = rsvg_ft_glyph_lookup (ctx, &desc, xy);
+	if (result == NULL) {
+		int x0, y0;
+		double my_affine[6];
+
+		memcpy (my_affine, affine, sizeof(my_affine));
+		my_affine[4] = floor (affine[4]) +
+			(1.0 / SUBPIXEL_FRACTION) * x_sp;
+		font = rsvg_ft_font_resolve (ctx, fh);
+		result = rsvg_ft_get_glyph (font, glyph_ix, sx, sy, my_affine, xy);
+		if (result == NULL)
+			return NULL;
+		x0 = xy[0] - floor (affine[4]);
+		y0 = xy[1] - floor (affine[5]);
+		rsvg_ft_glyph_insert (ctx, &desc, result, x0, y0);
+	} else {
+		xy[0] += floor (affine[4]);
+		xy[1] += floor (affine[5]);
+	}
+	result->refcnt++;
+	return result;
+}
+
+/**
+ * rsvg_ft_measure_or_render_string: Render a string into a glyph image.
  * @ctx: The Rsvg FT context.
  * @fh: Font handle for the font.
  * @str: String, in ISO-8859-1 encoding.
@@ -515,17 +769,30 @@ rsvg_ft_get_glyph (RsvgFTFont *font, FT_UInt glyph_ix, double sx, double sy,
  * @sy: Height of em in pixels.
  * @affine: Affine transformation.
  * @xy: Where to store the top left coordinates.
+ * &do_render: A boolean value indicating whether we should render the string.
+ * &dimensions: Where to store the resulting glyph's dimensions.
  *
  * Return value: A glyph containing the rendered string.
+ *
+ * This function does the rendering of a string into a glyph.  It can
+ * also work in measure only mode, so that no time is spent compositing
+ * bitmaps.  This is useful for callers that only want to measure a
+ * string.
+ * 
  **/
-RsvgFTGlyph *
-rsvg_ft_render_string (RsvgFTCtx *ctx, RsvgFTFontHandle fh,
-		       const char *str, double sx, double sy,
-		       const double affine[6], int xy[2])
+static RsvgFTGlyph *
+rsvg_ft_measure_or_render_string (RsvgFTCtx *ctx, 
+				  RsvgFTFontHandle fh,
+				  const char *str, 
+				  unsigned int length,
+				  double sx, double sy,
+				  const double affine[6], 
+				  int xy[2],
+				  gboolean do_render,
+				  unsigned int dimensions[2])
 {
 	RsvgFTFont *font;
 	RsvgFTGlyph *result;
-	const int len = strlen (str);
 	RsvgFTGlyph **glyphs;
 	int *glyph_xy;
 	int i, j;
@@ -535,6 +802,15 @@ rsvg_ft_render_string (RsvgFTCtx *ctx, RsvgFTFontHandle fh,
 	double glyph_affine[6];
 	FT_UInt glyph_index;
 	FT_UInt last_glyph = 0; /* for kerning */
+	int n_glyphs;
+	double init_x, init_y;
+
+	g_return_val_if_fail (ctx != NULL, NULL);
+	g_return_val_if_fail (str != NULL, NULL);
+	g_return_val_if_fail (length <= strlen (str), NULL);
+
+	dimensions[0] = 0;
+	dimensions[1] = 0;
 
 	font = rsvg_ft_font_resolve (ctx, fh);
 	if (font == NULL)
@@ -542,18 +818,30 @@ rsvg_ft_render_string (RsvgFTCtx *ctx, RsvgFTFontHandle fh,
 
 	bbox.x0 = bbox.x1 = 0;
 
-	glyphs = g_new (RsvgFTGlyph *, len);
-	glyph_xy = g_new (int, len * 2);
+	glyphs = g_new (RsvgFTGlyph *, length);
+	glyph_xy = g_new (int, length * 2);
 
 	for (j = 0; j < 6; j++)
 		glyph_affine[j] = affine[j];
 
-	for (i = 0; i < len; i++) {
+	init_x = affine[4];
+	init_y = affine[5];
+	n_glyphs = 0;
+	for (i = 0; i < length; i++) {
 		RsvgFTGlyph *glyph;
 
 		glyph_index = FT_Get_Char_Index (font->face,
 						 ((unsigned char *)str)[i]);
 
+		/* FIXME bugzilla.eazel.com 2775: Need a better way to deal
+		 * with unknown characters.
+		 *
+		 * The following is just a band aid fix.
+		 */
+		if (glyph_index == 0) {
+			glyph_index = FT_Get_Char_Index (font->face, '?');
+		}
+
 		if (last_glyph != 0 && glyph_index != 0) {
 			FT_Vector kern;
 			double kx, ky;
@@ -577,21 +865,51 @@ rsvg_ft_render_string (RsvgFTCtx *ctx, RsvgFTFontHandle fh,
 		if (glyph_index != 0)
 			last_glyph = glyph_index;
 
-		glyph = rsvg_ft_get_glyph (font, glyph_index,
-					   sx, sy, glyph_affine,
-					   glyph_xy + i * 2);
-		/* todo: handle glyph == NULL */
-		glyphs[i] = glyph;
+		glyph = rsvg_ft_get_glyph_cached (ctx, fh, glyph_index,
+						  sx, sy, glyph_affine,
+						  glyph_xy + n_glyphs * 2);
+		if (glyph != NULL) {
+			glyphs[n_glyphs] = glyph;
 
-		glyph_bbox.x0 = glyph_xy[i * 2];
-		glyph_bbox.y0 = glyph_xy[i * 2 + 1];
-		glyph_bbox.x1 = glyph_bbox.x0 + glyph->width;
-		glyph_bbox.y1 = glyph_bbox.y0 + glyph->height;
+			glyph_bbox.x0 = glyph_xy[i * 2];
+			glyph_bbox.y0 = glyph_xy[i * 2 + 1];
+			glyph_bbox.x1 = glyph_bbox.x0 + glyph->width;
+			glyph_bbox.y1 = glyph_bbox.y0 + glyph->height;
 
-		art_irect_union (&bbox, &bbox, &glyph_bbox);
+			art_irect_union (&bbox, &bbox, &glyph_bbox);
+#ifdef VERBOSE
+			g_print ("char '%c' bbox: (%d, %d) - (%d, %d)\n",
+				 str[i],
+				 glyph_bbox.x0, glyph_bbox.y0,
+				 glyph_bbox.x1, glyph_bbox.y1);
+#endif
 
-		glyph_affine[4] = glyph->xpen + glyph_bbox.x0;
-		glyph_affine[5] = glyph->ypen + glyph_bbox.y0;
+			glyph_affine[4] += glyph->xpen;
+			glyph_affine[5] += glyph->ypen;
+
+			n_glyphs++;
+		} else {
+			g_print ("no glyph loaded for character '%c'\n",
+				 str[i]);
+		}
+	}
+
+	xy[0] = bbox.x0;
+	xy[1] = bbox.y0;
+
+	dimensions[0] = (bbox.x1 - bbox.x0);
+	dimensions[1] = (bbox.y1 - bbox.y0);
+	
+	/* Skip the glyph compositing loop for the case when all we
+	 * are doing is measuring strings.
+	 */
+	if (!do_render) {
+		for (i = 0; i < n_glyphs; i++) {
+			rsvg_ft_glyph_unref (glyphs[i]);
+		}
+		g_free (glyphs);
+		g_free (glyph_xy);
+		return NULL;
 	}
 
 	rowstride = (bbox.x1 - bbox.x0 + 3) & -4;
@@ -601,14 +919,12 @@ rsvg_ft_render_string (RsvgFTCtx *ctx, RsvgFTFontHandle fh,
 	result->refcnt = 1;
 	result->width = bbox.x1 - bbox.x0;
 	result->height = bbox.y1 - bbox.y0;
-	xy[0] = bbox.x0;
-	xy[1] = bbox.y0;
-	result->xpen = glyph_affine[4] - bbox.x0;
-	result->ypen = glyph_affine[5] - bbox.y0;
+	result->xpen = glyph_affine[4] - init_x;
+	result->ypen = glyph_affine[5] - init_y;
 	result->rowstride = rowstride;
 	result->buf = buf;
 
-	for (i = 0; i < len; i++) {
+	for (i = 0; i < n_glyphs; i++) {
 		rsvg_ft_glyph_composite (result, glyphs[i],
 					 glyph_xy[i * 2] - bbox.x0,
 					 glyph_xy[i * 2 + 1] - bbox.y0);
@@ -621,6 +937,55 @@ rsvg_ft_render_string (RsvgFTCtx *ctx, RsvgFTFontHandle fh,
 	return result;
 }
 
+/**
+ * rsvg_ft_render_string: Render a string into a glyph image.
+ * @ctx: The Rsvg FT context.
+ * @fh: Font handle for the font.
+ * @str: String, in ISO-8859-1 encoding.
+ * @sx: Width of em in pixels.
+ * @sy: Height of em in pixels.
+ * @affine: Affine transformation.
+ * @xy: Where to store the top left coordinates.
+ *
+ * Return value: A glyph containing the rendered string.
+ **/
+RsvgFTGlyph *
+rsvg_ft_render_string (RsvgFTCtx *ctx, RsvgFTFontHandle fh,
+		       const char *str, 
+		       unsigned int length,
+		       double sx, double sy,
+		       const double affine[6], int xy[2])
+{
+	unsigned int unused[2];
+	return rsvg_ft_measure_or_render_string (ctx, fh, str, length,
+						 sx, sy, affine, xy,
+						 TRUE, unused);
+}
+
+/**
+ * rsvg_ft_measure_string: Measure a string.
+ * @ctx: The Rsvg FT context.
+ * @fh: Font handle for the font.
+ * @str: String, in ISO-8859-1 encoding.
+ * @sx: Width of em in pixels.
+ * @sy: Height of em in pixels.
+ * @affine: Affine transformation.
+ * @xy: Where to store the top left coordinates.
+ * @dimensions: Where to store the string dimensions.
+ **/
+void
+rsvg_ft_measure_string (RsvgFTCtx *ctx, RsvgFTFontHandle fh,
+			const char *str, 
+			unsigned int length,
+			double sx, double sy,
+			const double affine[6], 
+			int xy[2],
+			unsigned int dimensions[2])
+{
+	rsvg_ft_measure_or_render_string (ctx, fh, str, length, sx, sy,
+					  affine, xy, FALSE, dimensions);
+}
+
 #if 0
 void
 rsvg_ft_font_ref (RsvgFTFont *font)
@@ -646,4 +1011,3 @@ rsvg_ft_glyph_unref (RsvgFTGlyph *glyph)
 		g_free (glyph);
 	}
 }
-
