@@ -32,6 +32,7 @@
 #include <gtk/gtkhseparator.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtknotebook.h>
+#include <gtk/gtkoptionmenu.h>
 #include <gtk/gtkpixmap.h>
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtksignal.h>
@@ -44,13 +45,17 @@
 #include <libnautilus-extensions/nautilus-entry.h>
 #include <libnautilus-extensions/nautilus-file-attributes.h>
 #include <libnautilus-extensions/nautilus-file-utilities.h>
+#include <libnautilus-extensions/nautilus-gdk-pixbuf-extensions.h>
 #include <libnautilus-extensions/nautilus-glib-extensions.h>
 #include <libnautilus-extensions/nautilus-global-preferences.h>
 #include <libnautilus-extensions/nautilus-gtk-extensions.h>
 #include <libnautilus-extensions/nautilus-icon-factory.h>
+#include <libnautilus-extensions/nautilus-image.h>
+#include <libnautilus-extensions/nautilus-link.h>
 #include <libnautilus-extensions/nautilus-stock-dialogs.h>
 #include <libnautilus-extensions/nautilus-string.h>
 #include <libnautilus-extensions/nautilus-undo-signal-handlers.h>
+#include <libnautilus/nautilus-undo.h>
 #include <string.h>
 
 static GHashTable *windows;
@@ -104,6 +109,8 @@ enum {
 	COLUMN_COUNT
 };
 
+#define ERASE_EMBLEM_FILENAME	"erase.png"
+
 static void cancel_group_change_callback (gpointer callback_data);
 static void cancel_owner_change_callback (gpointer callback_data);
 
@@ -163,48 +170,63 @@ add_prompt_and_separator (GtkVBox *vbox, const char *prompt_text)
   	gtk_box_pack_end (GTK_BOX (vbox), separator_line, TRUE, TRUE, GNOME_PAD_BIG);
 }		   
 
-static void
-get_pixmap_and_mask_for_properties_window (NautilusFile *file,
-					   GdkPixmap **pixmap_return,
-					   GdkBitmap **mask_return)
+static GdkPixbuf *
+get_pixbuf_for_properties_window (NautilusFile *file)
 {
-	GdkPixbuf *pixbuf;
+	g_assert (NAUTILUS_IS_FILE (file));
+	
+	return nautilus_icon_factory_get_pixbuf_for_file (file, NULL, NAUTILUS_ICON_SIZE_STANDARD, TRUE);
+}
+
+static void
+update_properties_window_icon (NautilusImage *image)
+{
+	GdkPixbuf	*pixbuf;
+	NautilusFile	*file;
+
+	g_assert (NAUTILUS_IS_IMAGE (image));
+
+	file = gtk_object_get_data (GTK_OBJECT (image), "nautilus_file");
 
 	g_assert (NAUTILUS_IS_FILE (file));
 	
-	pixbuf = nautilus_icon_factory_get_pixbuf_for_file (file, NULL, NAUTILUS_ICON_SIZE_STANDARD, FALSE);
-        gdk_pixbuf_render_pixmap_and_mask (pixbuf, pixmap_return, mask_return, 128);
+	pixbuf = get_pixbuf_for_properties_window (file);
+
+	nautilus_image_set_pixbuf (image, pixbuf);
+	
 	gdk_pixbuf_unref (pixbuf);
 }
 
-static void
-update_properties_window_icon (GtkPixmap *pixmap_widget)
-{
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;
-	NautilusFile *file;
-
-	g_assert (GTK_IS_PIXMAP (pixmap_widget));
-
-	file = gtk_object_get_data (GTK_OBJECT (pixmap_widget), "nautilus_file");
-
-	g_assert (NAUTILUS_IS_FILE (file));
-
-	get_pixmap_and_mask_for_properties_window (file, &pixmap, &mask);
-	gtk_pixmap_set (pixmap_widget, pixmap, mask);
-}
-
 static GtkWidget *
-create_pixmap_widget_for_file (NautilusFile *file)
+create_image_widget_for_file (NautilusFile *file)
 {
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;
-	GtkWidget *widget;
+	gboolean	smooth_graphics;
+ 	GtkWidget	*image;
+	GdkPixbuf	*pixbuf;
+	
+	pixbuf = get_pixbuf_for_properties_window (file);
+	
+	image = nautilus_image_new ();
 
-	get_pixmap_and_mask_for_properties_window (file, &pixmap, &mask);
-	widget = gtk_pixmap_new (pixmap, mask);
+	/* Set the alpha mode of the image according to the SMOOTH_GRAPHICS preference.
+	 * Note that we dont keep track of changes in this preference to update the 
+	 * alpha more of the image.  There really is no need for this since the properties 
+	 * dialog always gets recreated.
+	 */
+	smooth_graphics = nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_SMOOTH_GRAPHICS_MODE, TRUE);
 
-	gtk_object_set_data_full (GTK_OBJECT (widget),
+	if (smooth_graphics) {
+		nautilus_image_set_alpha_mode (NAUTILUS_IMAGE (image), NAUTILUS_IMAGE_FULL_ALPHA);
+	} else {
+		nautilus_image_set_alpha_mode (NAUTILUS_IMAGE (image), NAUTILUS_IMAGE_THRESHOLD_ALPHA);
+	}
+
+	nautilus_image_set_pixbuf (NAUTILUS_IMAGE (image), pixbuf);
+
+	gdk_pixbuf_unref (pixbuf);
+
+	nautilus_file_ref (file);
+	gtk_object_set_data_full (GTK_OBJECT (image),
 				  "nautilus_file",
 				  file,
 				  (GtkDestroyNotify) nautilus_file_unref);
@@ -213,21 +235,22 @@ create_pixmap_widget_for_file (NautilusFile *file)
 	gtk_signal_connect_object_while_alive (nautilus_icon_factory_get (),
 					       "icons_changed",
 					       update_properties_window_icon,
-					       GTK_OBJECT (widget));
+					       GTK_OBJECT (image));
 
 	/* Name changes can also change icon (since name is determined by MIME type) */
 	gtk_signal_connect_object_while_alive (GTK_OBJECT (file),
 					       "changed",
 					       update_properties_window_icon,
-					       GTK_OBJECT (widget));
-	return widget;
+					       GTK_OBJECT (image));
+	return image;
 }
 
 static void
 name_field_update_to_match_file (NautilusEntry *name_field)
 {
 	NautilusFile *file;
-	char *original_name, *current_name, *displayed_name;
+	const char *original_name;
+	char *current_name, *displayed_name;
 
 	file = gtk_object_get_data (GTK_OBJECT (name_field), "nautilus_file");
 
@@ -237,8 +260,8 @@ name_field_update_to_match_file (NautilusEntry *name_field)
 		return;
 	}
 
-	original_name = (char *) gtk_object_get_data (GTK_OBJECT (name_field),
-						      "original_name");
+	original_name = (const char *) gtk_object_get_data (GTK_OBJECT (name_field),
+						      	    "original_name");
 
 	/* If the file name has changed since the original name was stored,
 	 * update the text in the text field, possibly (deliberately) clobbering
@@ -276,6 +299,23 @@ name_field_update_to_match_file (NautilusEntry *name_field)
 				  nautilus_file_can_rename (file));
 }
 
+static void
+name_field_restore_original_name (NautilusEntry *name_field)
+{
+	const char *original_name;
+	char *displayed_name;
+
+	original_name = (const char *) gtk_object_get_data (GTK_OBJECT (name_field),
+						      	    "original_name");
+	displayed_name = gtk_editable_get_chars (GTK_EDITABLE (name_field), 0, -1);
+
+	if (strcmp (original_name, displayed_name) != 0) {
+		gtk_entry_set_text (GTK_ENTRY (name_field), original_name);
+	}
+
+	g_free (displayed_name);
+}
+
 static void
 rename_callback (NautilusFile *file, GnomeVFSResult result, gpointer callback_data)
 {
@@ -305,7 +345,7 @@ name_field_done_editing (NautilusEntry *name_field)
 
 	/* Special case: silently revert text if new text is empty. */
 	if (strlen (new_name) == 0) {
-		name_field_update_to_match_file (NAUTILUS_ENTRY (name_field));
+		name_field_restore_original_name (NAUTILUS_ENTRY (name_field));
 	} else {
 		nautilus_file_rename (file, new_name,
 				      rename_callback, g_strdup (new_name));
@@ -319,8 +359,10 @@ name_field_focus_out (NautilusEntry *name_field,
 		      GdkEventFocus *event,
 		      gpointer user_data)
 {
-	name_field_done_editing (name_field);
-	gtk_editable_select_region (GTK_EDITABLE (name_field), -1, -1);
+	if (GTK_WIDGET_SENSITIVE (name_field)) {
+		name_field_done_editing (name_field);
+		gtk_editable_select_region (GTK_EDITABLE (name_field), -1, -1);
+	}
 
 	return TRUE;
 }
@@ -330,7 +372,9 @@ name_field_focus_in (NautilusEntry *name_field,
 		      GdkEventFocus *event,
 		      gpointer user_data)
 {
-	nautilus_entry_select_all (name_field);
+	if (GTK_WIDGET_SENSITIVE (name_field)) {
+		nautilus_entry_select_all (name_field);
+	}
 	return TRUE;
 }
 
@@ -393,9 +437,11 @@ property_button_toggled (GtkToggleButton *button)
 	} else {
 		if (word != NULL) {
 			keywords = g_list_remove_link (keywords, word);
+			nautilus_g_list_free_deep (word);
 		}
 	}
 	nautilus_file_set_keywords (file, keywords);
+	nautilus_g_list_free_deep (keywords);
 }
 
 static void
@@ -407,7 +453,7 @@ update_properties_window_title (GtkWindow *window, NautilusFile *file)
 	g_assert (GTK_IS_WINDOW (window));
 
 	name = nautilus_file_get_name (file);
-	title = g_strdup_printf (_("Nautilus: %s Properties"), name);
+	title = g_strdup_printf (_("%s Properties"), name);
   	gtk_window_set_title (window, title);
 
 	g_free (name);	
@@ -541,7 +587,7 @@ activate_group_callback (GtkMenuItem *menu_item, FileNamePair *pair)
 		 pair->file,
 		 _("Cancel Group Change?"),
 		 _("Changing group"),
-		 NULL); /* FIXME: Parent this? */
+		 NULL); /* FIXME bugzilla.eazel.com 2397: Parent this? */
 	nautilus_file_set_group
 		(pair->file, pair->name,
 		 group_change_callback, NULL);
@@ -630,7 +676,8 @@ attach_group_menu (GtkTable *table,
 	option_menu = gtk_option_menu_new ();
 	gtk_widget_show (option_menu);
 
-	/* FIXME: for reasons I don't understand, passing
+	/* FIXME bugzilla.eazel.com 1454: 
+	 * for reasons I don't understand, passing
 	 * GTK_FILL here is not making the option menu
 	 * minimally-sized horizontally. Might have to pack
 	 * it in an hbox or something.
@@ -686,7 +733,7 @@ activate_owner_callback (GtkMenuItem *menu_item, FileNamePair *pair)
 		 pair->file,
 		 _("Cancel Owner Change?"),
 		 _("Changing owner"),
-		 NULL); /* FIXME: Parent this? */
+		 NULL); /* FIXME bugzilla.eazel.com 2397: Parent this? */
 	nautilus_file_set_owner
 		(pair->file, pair->name,
 		 owner_change_callback, NULL);
@@ -786,7 +833,8 @@ attach_owner_menu (GtkTable *table,
 	option_menu = gtk_option_menu_new ();
 	gtk_widget_show (option_menu);
 
-	/* FIXME: for reasons I don't understand, passing
+	/* FIXME bugzilla.eazel.com 1454: 
+	 * for reasons I don't understand, passing
 	 * GTK_FILL here is not making the option menu
 	 * minimally-sized horizontally. Might have to pack
 	 * it in an hbox or something.
@@ -847,8 +895,7 @@ directory_contents_value_field_update (GtkLabel *label, NautilusFile *file)
 			}
 			break;
 		default:
-			text = g_strdup (_("--"));
-						
+			text = g_strdup ("--");
 		}
 	} else {
 		size_string = gnome_vfs_format_file_size_for_display (total_size);
@@ -1018,7 +1065,8 @@ create_basic_page (GtkNotebook *notebook, NautilusFile *file)
 	is_directory = nautilus_file_is_directory (file);
 
 	/* We hide MIME type for directories, so we need one fewer row */
-	/* FIXME: Without the && FALSE here, the last row in the directory
+	/* FIXME bugzilla.eazel.com 1454: 
+	 * Without the && FALSE here, the last row in the directory
 	 * case is too close to the 2nd to last row. Haven't figured out why.
 	 */
 	if (is_directory && FALSE) {
@@ -1034,7 +1082,8 @@ create_basic_page (GtkNotebook *notebook, NautilusFile *file)
 					NULL);
 
 	/* Icon pixmap */
-	icon_pixmap_widget = create_pixmap_widget_for_file (file);
+// 	icon_pixmap_widget = create_pixmap_widget_for_file (file);
+	icon_pixmap_widget = create_image_widget_for_file (file);
 	gtk_widget_show (icon_pixmap_widget);
 	gtk_table_attach_defaults (GTK_TABLE (table),
 				   icon_pixmap_widget,
@@ -1064,7 +1113,7 @@ create_basic_page (GtkNotebook *notebook, NautilusFile *file)
 	name_field_update_to_match_file (NAUTILUS_ENTRY (name_field));
 
 	/* Set up name field for undo */
-	nautilus_undo_setup_nautilus_entry_for_undo ( NAUTILUS_ENTRY (name_field));
+	nautilus_undo_set_up_nautilus_entry_for_undo ( NAUTILUS_ENTRY (name_field));
 	nautilus_undo_editable_set_undo_key (GTK_EDITABLE (name_field), TRUE);
 
 	gtk_signal_connect (GTK_OBJECT (name_field), "focus_in_event",
@@ -1079,8 +1128,10 @@ create_basic_page (GtkNotebook *notebook, NautilusFile *file)
       	              	    GTK_SIGNAL_FUNC (name_field_activate),
                             NULL);
 
-        /* Start with name field selected. */
-        gtk_widget_grab_focus (GTK_WIDGET (name_field));
+        /* Start with name field selected, if it's sensitive. */
+        if (GTK_WIDGET_SENSITIVE (name_field)) {
+	        gtk_widget_grab_focus (GTK_WIDGET (name_field));
+        }
                       			    
 	/* React to name changes from elsewhere. */
 	gtk_signal_connect_object_while_alive (GTK_OBJECT (file),
@@ -1128,7 +1179,7 @@ create_image_widget_for_emblem (const char *emblem_name)
 		 NULL);
 
 	nautilus_scalable_icon_unref (icon);
-	gdk_pixbuf_render_pixmap_and_mask (pixbuf, &pixmap, &mask, 128);
+	gdk_pixbuf_render_pixmap_and_mask (pixbuf, &pixmap, &mask, NAUTILUS_STANDARD_ALPHA_THRESHHOLD);
 	gdk_pixbuf_unref (pixbuf);
 
 	image_widget = gtk_pixmap_new (pixmap, mask);
@@ -1165,6 +1216,11 @@ get_property_names_from_uri (const char *directory_uri, GList *property_list)
 	for (current_file_info = gnome_vfs_directory_list_first(list); current_file_info != NULL; 
 	    current_file_info = gnome_vfs_directory_list_next(list)) {
 		if (nautilus_istr_has_prefix (current_file_info->mime_type, "image/")) {
+			/* skip the special "erase" emblem */
+			if (nautilus_strcmp (current_file_info->name, 
+					     ERASE_EMBLEM_FILENAME) == 0)
+				continue;
+				
 			keyword = g_strdup (current_file_info->name);
 			
 			/* strip image type suffix */
@@ -1625,10 +1681,12 @@ create_permissions_page (GtkNotebook *notebook, NautilusFile *file)
 					  PERMISSIONS_CHECKBOXES_EXECUTE_COLUMN,
 					  GNOME_VFS_PERM_OTHER_EXEC);
 
-		/* FIXME: Would be better to show/hide this info dynamically, so if the
+		/* FIXME bugzilla.eazel.com 2396: 
+		 * Would be better to show/hide this info dynamically, so if the
 		 * preference is changed while the window is open it would react.
 		 */
-		/* FIXME: Spacing is wrong if you just leave empty table cells; need
+		/* FIXME bugzilla.eazel.com 1454: 
+		 * Spacing is wrong if you just leave empty table cells; need
 		 * to size table appropriately in advance.
 		 */
 		if (nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_SPECIAL_FLAGS, 
@@ -1666,7 +1724,9 @@ create_properties_window (NautilusFile *file)
 		attributes = g_list_prepend (attributes,
 					     NAUTILUS_FILE_ATTRIBUTE_DEEP_COUNTS);
 	}
-	nautilus_file_monitor_add (file, window, attributes, TRUE);
+	attributes = g_list_prepend (attributes,
+				     NAUTILUS_FILE_ATTRIBUTE_METADATA);
+	nautilus_file_monitor_add (file, window, attributes);
 	g_list_free (attributes);
 
 	/* React to future property changes and file deletions. */
@@ -1700,34 +1760,105 @@ forget_properties_window (gpointer data, gpointer user_data)
 	window = GTK_WINDOW (data);
 	file = NAUTILUS_FILE (user_data);
 
+	nautilus_file_monitor_remove (file, window);
 	g_hash_table_remove (windows, file);
 	nautilus_file_unref (file);
-	nautilus_file_monitor_remove (file, window);
 }
 
-GtkWindow *
-fm_properties_window_get_or_create (NautilusFile *file)
+static NautilusFile *
+get_and_ref_file_to_display (NautilusFile *file)
 {
-	GtkWindow *window;
+	NautilusFile *file_to_display;
+	char *uri;
+	char *uri_to_display;
+	char *local_path;
+
+	file_to_display = NULL;
+	if (nautilus_file_is_nautilus_link (file)) {
+		/* Note: This will only work on local files. For now
+		 * that seems fine since the links we care about are
+		 * all on the desktop.
+		 */
+		uri = nautilus_file_get_uri (file);
+		local_path = gnome_vfs_get_local_path_from_uri (uri);
+		if (local_path != NULL) {
+			switch (nautilus_link_local_get_link_type (local_path)) {
+			case NAUTILUS_LINK_MOUNT:
+			case NAUTILUS_LINK_TRASH:
+			case NAUTILUS_LINK_HOME:
+				/* map to linked URI for these types of links */
+				uri_to_display = nautilus_link_local_get_link_uri (local_path);
+				file_to_display = nautilus_file_get (uri_to_display);
+				g_free (uri_to_display);
+				break;
+			case NAUTILUS_LINK_GENERIC:
+				/* don't for these types */
+				break;
+			}
+			g_free (local_path);
+		}
+		
+		g_free (uri);
+	}
+
+	if (file_to_display != NULL) {
+		return file_to_display;
+	}
+
+	/* Ref passed-in file here since we've decided to use it. */
+	nautilus_file_ref (file);
+	return file;
+}
+
+static void
+create_properties_window_callback (NautilusFile *file, gpointer data)
+{
+	GtkWindow *new_window;
+
+	g_assert (FM_IS_DIRECTORY_VIEW (data));
+
+	new_window = create_properties_window (file);
+	g_hash_table_insert (windows, file, new_window);
+
+	gtk_signal_connect (GTK_OBJECT (new_window),
+			    "destroy",
+			    forget_properties_window,
+			    file);
+
+	nautilus_undo_share_undo_manager (GTK_OBJECT (new_window), GTK_OBJECT (data));
+	nautilus_gtk_window_present (new_window);
+}
+
+void
+fm_properties_window_present (NautilusFile *file, FMDirectoryView *directory_view)
+{
+	GtkWindow *existing_window;
+	NautilusFile *file_to_display;
+	GList attribute_list;
+
+	g_return_if_fail (NAUTILUS_IS_FILE (file));
+	g_return_if_fail (FM_IS_DIRECTORY_VIEW (directory_view));
 
 	/* Create the hash table first time through. */
 	if (windows == NULL) {
 		windows = g_hash_table_new (g_direct_hash, g_direct_equal);
 	}
 
-	/* Look to see if object is already in the hash table. */
-	window = g_hash_table_lookup (windows, file);
+	file_to_display = get_and_ref_file_to_display (file);
 
-	if (window == NULL) {
-		window = create_properties_window (file);
-		nautilus_file_ref (file);
-		g_hash_table_insert (windows, file, window);
-	
-		gtk_signal_connect (GTK_OBJECT (window),
-				    "destroy",
-				    forget_properties_window,
-				    file);
+	/* Look to see if object is already in the hash table. */
+	existing_window = g_hash_table_lookup (windows, file_to_display);
+	if (existing_window != NULL) {
+		nautilus_file_unref (file_to_display);
+		nautilus_gtk_window_present (existing_window);
+		return;
 	}
 
-	return window;
+	/* Wait until we can tell whether it's a directory before showing. */
+	attribute_list.data = NAUTILUS_FILE_ATTRIBUTE_IS_DIRECTORY;
+	attribute_list.next = NULL;
+	attribute_list.prev = NULL;
+	nautilus_file_call_when_ready
+		(file_to_display, &attribute_list,
+		 create_properties_window_callback, directory_view);
 }
