@@ -6,12 +6,12 @@
  *  Copyright (C) 1999, 2000 Red Hat, Inc.
  *  Copyright (C) 2000 Eazel, Inc.
  *
- *  This program is free software; you can redistribute it and/or
+ *  Nautilus is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License as
  *  published by the Free Software Foundation; either version 2 of the
  *  License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Nautilus is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  *  General Public License for more details.
@@ -27,24 +27,27 @@
 #include <config.h>
 #include "nautilus-application.h"
 
-/* FIXME: This is a workaround for ORBit bug where including idl files
- * in other idl files causes trouble.
- */
-#include "nautilus-shell-interface.h"
-#define nautilus_view_component_H
-
-#include "file-manager/fm-icon-view.h"
 #include "file-manager/fm-desktop-icon-view.h"
+#include "file-manager/fm-icon-view.h"
 #include "file-manager/fm-list-view.h"
 #include "file-manager/fm-search-list-view.h"
 #include "nautilus-desktop-window.h"
 #include "nautilus-first-time-druid.h"
+#include "nautilus-main.h"
+#include "nautilus-shell-interface.h"
 #include "nautilus-shell.h"
-#include <bonobo.h>
+#include <bonobo/bonobo-main.h>
+#include <bonobo/bonobo-object.h>
+#include <gtk/gtksignal.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnome/gnome-util.h>
+#include <libgnomeui/gnome-messagebox.h>
+#include <libgnomeui/gnome-stock.h>
 #include <libnautilus-extensions/nautilus-file-utilities.h>
 #include <libnautilus-extensions/nautilus-global-preferences.h>
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
 #include <libnautilus-extensions/nautilus-icon-factory.h>
+#include <libnautilus-extensions/nautilus-sound.h>
 #include <libnautilus-extensions/nautilus-stock-dialogs.h>
 #include <libnautilus-extensions/nautilus-string-list.h>
 #include <libnautilus-extensions/nautilus-undo-manager.h>
@@ -69,6 +72,7 @@ static void          nautilus_application_destroy                (GtkObject
 static void          nautilus_application_check_user_directories (NautilusApplication      *application);
 static gboolean	     check_for_and_run_as_super_user 		 (void);
 static gboolean	     need_to_show_first_time_druid		 (void);
+static void	     desktop_changed_callback 		         (gpointer 		   user_data);
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusApplication, nautilus_application, BONOBO_OBJECT_TYPE)
 
@@ -141,7 +145,7 @@ create_factory (PortableServer_POA poa,
 }
 
 /* Keeps track of the one and only desktop window. */
-static NautilusDesktopWindow *nautilus_application_desktop;
+static NautilusDesktopWindow *nautilus_application_desktop_window;
 
 /* Keeps track of all the nautilus windows. */
 static GSList *nautilus_application_window_list;
@@ -169,17 +173,16 @@ nautilus_application_initialize (NautilusApplication *application)
 		g_error ("could not create factory");
 	}
 	CORBA_exception_free (&ev);
-
+	
 	bonobo_object_construct (BONOBO_OBJECT (application), corba_object);
-
-	/* Initialize preferences. This is needed so that proper 
-	 * defaults are available before any preference peeking 
-	 * happens.
-	 */
-	nautilus_global_preferences_initialize ();
-
+	
 	/* Create an undo manager */
 	application->undo_manager = nautilus_undo_manager_new ();
+
+	/* monitor the desktop preference */
+	nautilus_preferences_add_callback (NAUTILUS_PREFERENCES_SHOW_DESKTOP,
+					   desktop_changed_callback,
+					   application);	
 }
 
 NautilusApplication *
@@ -191,16 +194,17 @@ nautilus_application_new (void)
 static void
 nautilus_application_destroy (GtkObject *object)
 {
-	/* Shut down preferences. This is needed so that the global
-	 * preferences object and all its allocations are freed. Not
-	 * calling this function would NOT cause the user to lose
-	 * preferences.  The only effect would be to leak those
-	 * objects - which would be collected at exit() time anyway,
-	 * but it adds noise to memory profile tool runs.
-	 */
-	nautilus_global_preferences_shutdown ();
+	NautilusApplication *application;
+
+	application = NAUTILUS_APPLICATION (object);
 
 	nautilus_bookmarks_exiting ();
+	
+	nautilus_preferences_remove_callback (NAUTILUS_PREFERENCES_SHOW_DESKTOP,
+					      desktop_changed_callback,
+					      application);
+
+	bonobo_object_unref (BONOBO_OBJECT (application->undo_manager));
 
 	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
 }
@@ -219,24 +223,25 @@ nautilus_application_check_user_directories (NautilusApplication *application)
 	user_main_directory = nautilus_get_user_main_directory ();
 	desktop_directory = nautilus_get_desktop_directory ();
 
-	dir_list = nautilus_string_list_new ();
+	dir_list = nautilus_string_list_new (TRUE);
 	
 	/* FIXME bugzilla.eazel.com 1115: Need better name for "User Directory"
-	 * and "User Data Directory".
+	 * and "User Data Directory". Perhaps it's OK to use the word "directory"
+	 * instead of "folder" in this technical error case.
 	 */
 
 	if (!g_file_test (user_directory, G_FILE_TEST_ISDIR)) {
-		nautilus_string_list_insert (dir_list, "User Directory");
+		nautilus_string_list_insert (dir_list, _("User Directory"));
 	}
 	g_free (user_directory);
 	    
 	if (!g_file_test (user_main_directory, G_FILE_TEST_ISDIR)) {
-		nautilus_string_list_insert (dir_list, "User Main Directory");
+		nautilus_string_list_insert (dir_list, _("User Main Directory"));
 	}
 	g_free (user_main_directory);
 	    
 	if (!g_file_test (desktop_directory, G_FILE_TEST_ISDIR)) {
-		nautilus_string_list_insert (dir_list, "Desktop Directory");
+		nautilus_string_list_insert (dir_list, _("Desktop Directory"));
 	}
 	g_free (desktop_directory);
 
@@ -251,7 +256,7 @@ nautilus_application_check_user_directories (NautilusApplication *application)
 						dir_list_concatenated,
 						"Please restart Nautilus to fix this problem.");
 
-		nautilus_error_dialog (error_string, NULL);
+		nautilus_error_dialog (error_string, _("Missing Directories"), NULL);
 
 		g_free (dir_list_concatenated);
 		g_free (error_string);
@@ -289,9 +294,10 @@ nautilus_make_uri_list_from_strv (const char * const *strv)
 	return uri_list;
 }
 
-gboolean
+void
 nautilus_application_startup (NautilusApplication *application,
 			      gboolean kill_shell,
+			      gboolean restart_shell,
 			      gboolean stop_desktop,
 			      gboolean start_desktop,
 			      const char *urls[])
@@ -302,17 +308,16 @@ nautilus_application_startup (NautilusApplication *application,
 	const char *message, *detailed_message;
 	GnomeDialog *dialog;
 	Nautilus_URIList *url_list;
-	gboolean need_main_loop;
 
 	/* Perform check for nautilus being run as super user */
 	if (!check_for_and_run_as_super_user ()) {
-		return FALSE;
+		return;
 	}
 
 	/* Run the first time startup druid if needed. */
 	if (need_to_show_first_time_druid ()) {
 		nautilus_first_time_druid_show (application, start_desktop, urls);
-		return TRUE;
+		return;
 	}
 	
 	/* Check the user's ~/.nautilus directories and post warnings
@@ -320,6 +325,9 @@ nautilus_application_startup (NautilusApplication *application,
 	 */
 	nautilus_application_check_user_directories (application);
 
+	/* initialize the sound machinery */
+	nautilus_sound_initialize ();
+	
 	CORBA_exception_init (&ev);
 
 	/* Start up the factory. */
@@ -334,6 +342,7 @@ nautilus_application_startup (NautilusApplication *application,
 		case OAF_REG_ALREADY_ACTIVE:
 			/* Another copy of . */
 			message = NULL;
+			detailed_message = NULL;
 			break;
 		case OAF_REG_NOT_LISTED:
 			/* Can't register myself due to trouble locating the
@@ -349,7 +358,7 @@ nautilus_application_startup (NautilusApplication *application,
 			message = _("Nautilus can't be used now. "
 				    "Rebooting the computer or installing "
 				    "Nautilus again may fix the problem.");
-			/* FIXME: The guesses and stuff here are lame. */
+			/* FIXME bugzilla.eazel.com 2536: The guesses and stuff here are lame. */
 			detailed_message = _("Nautilus can't be used now. "
 					     "Rebooting the computer or installing "
 					     "Nautilus again may fix the problem.\n\n"
@@ -370,7 +379,7 @@ nautilus_application_startup (NautilusApplication *application,
 			 * version of OAF). Show dialog and terminate the
 			 * program.
 			 */
-			/* FIXME: Looks like this does happen with the
+			/* FIXME bugzilla.eazel.com 2537: Looks like this does happen with the
 			 * current OAF. I guess I read the code
 			 * wrong. Need to figure out when and make a
 			 * good message.
@@ -392,7 +401,7 @@ nautilus_application_startup (NautilusApplication *application,
 			 * we better stop looping.
 			 */
 			if (result == OAF_REG_SUCCESS) {
-				/* FIXME: When can this happen? */
+				/* FIXME bugzilla.eazel.com 2538: When can this happen? */
 				message = _("Nautilus can't be used now, due to an unexpected error.");
 				detailed_message = _("Nautilus can't be used now, due to an unexpected error "
 						     "from OAF when attempting to locate the factory.");
@@ -400,17 +409,17 @@ nautilus_application_startup (NautilusApplication *application,
 		}
 
 		if (message != NULL) {
-			dialog = nautilus_error_dialog_with_details
-				(message, detailed_message, NULL);
-			gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
-					    gtk_main_quit, NULL);
-			need_main_loop = TRUE;
+			dialog = nautilus_error_dialog_with_details (message, NULL, detailed_message, NULL);
+			/* We need the main event loop so the user has a chance to see the dialog. */
+			nautilus_main_event_loop_register (GTK_OBJECT (dialog));
 			goto out;
 		}
 	}
 
 	if (kill_shell) {
 		Nautilus_Shell_quit (shell, &ev);
+	} else if (restart_shell) {
+		Nautilus_Shell_restart (shell, &ev);
 	} else {
 		if (start_desktop) {
 			Nautilus_Shell_start_desktop (shell, &ev);
@@ -431,62 +440,37 @@ nautilus_application_startup (NautilusApplication *application,
 	}
 
 	/* We're done with the shell now, so let it go. */
-	Nautilus_Shell_unref (shell, &ev);
-	CORBA_Object_release (shell, &ev);
-
-	need_main_loop = nautilus_application_window_list != NULL
-		|| nautilus_application_desktop != NULL;
+	bonobo_object_release_unref (shell, &ev);
 
  out:
 	CORBA_exception_free (&ev);
-	return need_main_loop;
 }
 
 static void
-nautilus_application_destroy_desktop_window (GtkObject *obj, NautilusApplication *application)
-{
-	nautilus_application_desktop = NULL;
-}
-
-static NautilusDesktopWindow *
 nautilus_application_create_desktop_window (NautilusApplication *application)
 {
-	NautilusDesktopWindow *window;
-	
-	g_return_val_if_fail (NAUTILUS_IS_APPLICATION (application), NULL);
+	g_return_if_fail (nautilus_application_desktop_window == NULL);
+	g_return_if_fail (NAUTILUS_IS_APPLICATION (application));
 
-	window = nautilus_desktop_window_new (application);
-
-	gtk_signal_connect (GTK_OBJECT (window),
-			    "destroy", nautilus_application_destroy_desktop_window,
-			    application);
-
-	nautilus_application_desktop = window;
-
-	return window;
+	nautilus_application_desktop_window = nautilus_desktop_window_new (application);
+	gtk_widget_show (GTK_WIDGET (nautilus_application_desktop_window));
 }
 
 void
 nautilus_application_open_desktop (NautilusApplication *application)
 {
-	NautilusDesktopWindow *desktop_window;
-
-	if (nautilus_application_desktop == NULL) {
-		desktop_window = nautilus_application_create_desktop_window (application);
+	if (nautilus_application_desktop_window == NULL) {
+		nautilus_application_create_desktop_window (application);
 	}
-	gtk_widget_show (GTK_WIDGET (desktop_window));
 }
 
 void
 nautilus_application_close_desktop (void)
 {
-	if (nautilus_application_desktop != NULL) {
-		gtk_widget_destroy (GTK_WIDGET (nautilus_application_desktop));
-	}
-	
-	if (nautilus_application_window_list == NULL && gtk_main_level () > 0) {
-		gtk_main_quit ();
-	}
+	if (nautilus_application_desktop_window != NULL) {
+		gtk_widget_destroy (GTK_WIDGET (nautilus_application_desktop_window));
+		nautilus_application_desktop_window = NULL;
+	}	
 }
 
 void
@@ -497,16 +481,25 @@ nautilus_application_close_all_windows (void)
 	}
 }
 
-
 static void
-nautilus_application_destroy_window (GtkObject *obj, NautilusApplication *application)
+nautilus_application_destroyed_window (GtkObject *object, NautilusApplication *application)
 {
-	nautilus_application_window_list = g_slist_remove (nautilus_application_window_list, obj);
-	if (nautilus_application_window_list == NULL && nautilus_application_desktop == NULL) {
-		gtk_main_quit ();
-	}
+	nautilus_application_window_list = g_slist_remove (nautilus_application_window_list, object);
 }
 
+static gboolean
+nautilus_window_delete_event_callback (GtkWidget *widget,
+				       GdkEvent *event,
+				       gpointer user_data)
+{
+	NautilusWindow *window;
+
+	window = NAUTILUS_WINDOW (widget);
+	nautilus_window_close (window);
+
+	return TRUE;
+}				       
+
 NautilusWindow *
 nautilus_application_create_window (NautilusApplication *application)
 {
@@ -514,11 +507,16 @@ nautilus_application_create_window (NautilusApplication *application)
 
 	g_return_val_if_fail (NAUTILUS_IS_APPLICATION (application), NULL);
 	
-	window = NAUTILUS_WINDOW (gtk_object_new (nautilus_window_get_type (),
+	window = NAUTILUS_WINDOW (gtk_widget_new (nautilus_window_get_type (),
 						  "app", GTK_OBJECT (application),
 						  "app_id", "nautilus", NULL));
+	
+	gtk_signal_connect (GTK_OBJECT (window), 
+			    "delete_event", GTK_SIGNAL_FUNC (nautilus_window_delete_event_callback),
+                    	    NULL);
+
 	gtk_signal_connect (GTK_OBJECT (window),
-			    "destroy", nautilus_application_destroy_window,
+			    "destroy", nautilus_application_destroyed_window,
 			    application);
 
 	nautilus_application_window_list = g_slist_prepend (nautilus_application_window_list, window);
@@ -562,6 +560,20 @@ check_for_and_run_as_super_user (void)
 	return TRUE;
 }
 
+/* callback for showing or hiding the desktop based on the user's preference */
+static void
+desktop_changed_callback (gpointer user_data)
+{
+	NautilusApplication *application;
+	
+	application = NAUTILUS_APPLICATION (user_data);
+	if ( nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_DESKTOP, FALSE)) {
+		nautilus_application_open_desktop (application);
+	} else {
+		nautilus_application_close_desktop ();
+	}
+}
+
 /*
  * need_to_show_first_time_druid
  *
@@ -596,30 +608,7 @@ need_to_show_first_time_druid (void)
 						"first-time-wizard-flag");
 	g_free (user_directory);
 
-	result = !g_file_exists (druid_flag_file_name);
-
-	/* Touch the file so that next time around out states changes */
-	if (result) {
-		FILE *stream;
-		
-		stream = fopen (druid_flag_file_name, "w");
-
-		/* If for some crazy reason we cant touch the file,
-		 * then the worst that will happen is that the user
-		 * will get the startup druid again next time, so we
-		 * dont report this failure to the user.
-		 */
-		if (stream) {
-			const char *blurb =
-				_("Existence of this file indicates that the Nautilus configuration wizard\n"
-				  "has been presented.\n\n"
-				  "You can manually erase this file to present the wizard again.\n\n");
-			
-			fwrite (blurb, sizeof (char), strlen (blurb), stream);
-			fclose (stream);
-		}
-	}
-	
+	result = !g_file_exists (druid_flag_file_name);	
 	g_free (druid_flag_file_name);
 
 	return result;
