@@ -1,26 +1,29 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 
+/*
+ * Nautilus
+ *
+ * Copyright (C) 1999, 2000 Eazel, Inc.
+ *
+ * Nautilus is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * Nautilus is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * Authors: John Sullivan <sullivan@eazel.com>
+ */
+
 /* nautilus-bookmarks-window.c - implementation of bookmark-editing window.
-
-   Copyright (C) 1999, 2000 Eazel, Inc.
-
-   The Gnome Library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   The Gnome Library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public
-   License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
-
-   Authors: John Sullivan <sullivan@eazel.com>
-*/
+ */
 
 #include <config.h>
 #include "nautilus-bookmarks-window.h"
@@ -31,7 +34,7 @@
 #include <libnautilus-extensions/nautilus-undo-signal-handlers.h>
 #include <gnome.h>
 
-
+
 /* Static variables to keep track of window state. If there were
  * more than one bookmark-editing window, these would be struct or
  * class fields. 
@@ -42,12 +45,10 @@ static GtkWidget	    *bookmark_list_widget = NULL; /* awkward name to distinguis
 static GtkWidget	    *name_field = NULL;
 static int		     name_field_changed_signalID;
 static GtkWidget	    *remove_button = NULL;
-static GtkWidget	    *show_static_checkbox = NULL;
 static gboolean		     text_changed = FALSE;
 static GtkWidget	    *uri_field = NULL;
 static int		     uri_field_changed_signalID;
 
-
 /* forward declarations */
 static NautilusBookmark *get_selected_bookmark (void);
 static guint	get_selected_row	      (void);
@@ -80,12 +81,6 @@ static gboolean on_window_delete_event 	      (GtkWidget *,
 					       gpointer user_data);
 static void	repopulate		      (void);
 
-static void	update_built_in_bookmarks_checkbox_to_match_preference (gpointer user_data);
-static void	update_built_in_bookmarks_preference_to_match_checkbox (gpointer user_data);
-
-
-
-
 #define BOOKMARK_LIST_COLUMN_ICON		0
 #define BOOKMARK_LIST_COLUMN_NAME		1
 #define BOOKMARK_LIST_COLUMN_COUNT		2
@@ -100,13 +95,7 @@ static void	update_built_in_bookmarks_preference_to_match_checkbox (gpointer use
 #define BOOKMARKS_WINDOW_INITIAL_WIDTH	500
 #define BOOKMARKS_WINDOW_INITIAL_HEIGHT	200
 
-/* Used for window position & size sanity-checking. The sizes are big enough to prevent
- * at least normal-sized gnome panels from obscuring the window at the screen edges. 
- */
-#define MINIMUM_ON_SCREEN_WIDTH		100
-#define MINIMUM_ON_SCREEN_HEIGHT	100
 
-
 /**
  * create_bookmarks_window:
  * 
@@ -133,7 +122,7 @@ create_bookmarks_window (NautilusBookmarkList *list, GtkObject *undo_manager_sou
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	nautilus_undo_share_undo_manager (GTK_OBJECT (window), undo_manager_source);
 	gtk_container_set_border_width (GTK_CONTAINER (window), GNOME_PAD);
-	gtk_window_set_title (GTK_WINDOW (window), _("Nautilus: Bookmarks"));
+	gtk_window_set_title (GTK_WINDOW (window), _("Bookmarks"));
 	gtk_widget_set_usize (window, 
 			      BOOKMARKS_WINDOW_MIN_WIDTH, 
 			      BOOKMARKS_WINDOW_MIN_HEIGHT);
@@ -200,19 +189,6 @@ create_bookmarks_window (NautilusBookmarkList *list, GtkObject *undo_manager_sou
 	gtk_widget_show (remove_button);
 	gtk_box_pack_start (GTK_BOX (hbox2), remove_button, TRUE, FALSE, 0);
 
-	show_static_checkbox = gtk_check_button_new_with_label (_("Include built-in bookmarks in menu"));
-	gtk_widget_show (show_static_checkbox);
-	gtk_box_pack_end (GTK_BOX (right_side), show_static_checkbox, FALSE, FALSE, 0);
-
-	update_built_in_bookmarks_checkbox_to_match_preference (GTK_CHECK_BUTTON (show_static_checkbox));
-	nautilus_preferences_add_callback (NAUTILUS_PREFERENCES_SHOW_BUILT_IN_BOOKMARKS,
-					   update_built_in_bookmarks_checkbox_to_match_preference,
-					   show_static_checkbox);
-
-	gtk_signal_connect (GTK_OBJECT (show_static_checkbox), "toggled",
-			    update_built_in_bookmarks_preference_to_match_checkbox,
-			    show_static_checkbox);
-
 	bookmark_list_changed_signalID =
 		gtk_signal_connect (GTK_OBJECT(bookmarks), "contents_changed",
 				    GTK_SIGNAL_FUNC(on_bookmark_list_changed),
@@ -338,46 +314,22 @@ nautilus_bookmarks_window_restore_geometry (GtkWidget *window)
 	g_return_if_fail (GTK_IS_WINDOW (window));
 	g_return_if_fail (NAUTILUS_IS_BOOKMARK_LIST (bookmarks));
 
-	window_geometry = nautilus_bookmark_list_get_window_geometry(bookmarks);
+	window_geometry = nautilus_bookmark_list_get_window_geometry (bookmarks);
 
 	if (window_geometry != NULL) 
 	{	
-		int left, top, width, height;
+		nautilus_gtk_window_set_initial_geometry_from_string 
+			(GTK_WINDOW (window), window_geometry, 
+			 BOOKMARKS_WINDOW_MIN_WIDTH, BOOKMARKS_WINDOW_MIN_HEIGHT);
 
-		if (gnome_parse_geometry (window_geometry, &left, &top, &width, &height))
-		{
-			/* Adjust for sanity, in case screen size has changed or
-			 * stored numbers are bogus. Make the entire window fit
-			 * on the screen, so all controls can be reached. Also
-			 * make sure the window isn't ridiculously small. Also
-			 * make sure the top of the window is on screen, for
-			 * draggability (perhaps not absolutely required, depending
-			 * on window manager, but seems like a sensible rule anyway).
-			 */
-			width = CLAMP (width, BOOKMARKS_WINDOW_MIN_WIDTH, gdk_screen_width());
-			height = CLAMP (height, BOOKMARKS_WINDOW_MIN_HEIGHT, gdk_screen_height());
+	} else {
+		/* use default since there was no stored geometry */
+		gtk_window_set_default_size (GTK_WINDOW (window), 
+					     BOOKMARKS_WINDOW_INITIAL_WIDTH, 
+					     BOOKMARKS_WINDOW_INITIAL_HEIGHT);
 
-			top = CLAMP (top, 0, gdk_screen_height() - MINIMUM_ON_SCREEN_HEIGHT);
-			/* FIXME bugzilla.eazel.com 669: 
-			 * If window has negative left coordinate, set_uposition sends it
-			 * somewhere else entirely. Not sure what level contains this bug (XWindows?).
-			 * Hacked around by pinning the left edge to zero.
-			 */
-			left = CLAMP (left, 0, gdk_screen_width() - MINIMUM_ON_SCREEN_WIDTH);
-					    		
-			gtk_widget_set_uposition (window, left, top);
-			gtk_window_set_default_size (GTK_WINDOW (window), width, height);
-
-			return;
-		}
+		/* Let window manager handle default position if no position stored */
 	}
-
-	/* fall through to default if necessary */
-	gtk_window_set_default_size (GTK_WINDOW (window), 
-				     BOOKMARKS_WINDOW_INITIAL_WIDTH, 
-				     BOOKMARKS_WINDOW_INITIAL_HEIGHT);
-
-	/* Let window manager handle default position if no position stored */
 }
 
 /**
@@ -662,39 +614,3 @@ repopulate (void)
 	  
 	gtk_clist_thaw (GTK_CLIST (bookmark_list_widget));
 }
-
-static void
-synch_built_in_bookmarks_preference_and_checkbox (GtkCheckButton *checkbox, gboolean trust_checkbox)
-{
-	gboolean preference_setting, checkbox_setting;
-
-	g_assert (GTK_IS_CHECK_BUTTON (checkbox));
-
-	preference_setting = nautilus_preferences_get_boolean 
-		(NAUTILUS_PREFERENCES_SHOW_BUILT_IN_BOOKMARKS, TRUE);
-
-	checkbox_setting = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbox));
-
-	if (preference_setting == checkbox_setting) {
-		return;
-	}
-
-	if (trust_checkbox) {
-		nautilus_preferences_set_boolean
-			(NAUTILUS_PREFERENCES_SHOW_BUILT_IN_BOOKMARKS, checkbox_setting);
-	} else {
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbox), preference_setting);
-	}
-}
-
-static void
-update_built_in_bookmarks_preference_to_match_checkbox (gpointer user_data)
-{
-	synch_built_in_bookmarks_preference_and_checkbox (user_data, TRUE);
-}
-
-static void
-update_built_in_bookmarks_checkbox_to_match_preference (gpointer user_data)
-{
-	synch_built_in_bookmarks_preference_and_checkbox (user_data, FALSE);
-}
