@@ -1,41 +1,49 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 
+/*
+ * Nautilus
+ *
+ * Copyright (C) 2000 Eazel, Inc.
+ *
+ * Nautilus is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * Nautilus is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * Authors: Rebecca Schulman <rebecka@eazel.com>
+ */
+
 /* nautilus-complex-search-bar.c - Search bar containing many attributes
+ */
 
-   Copyright (C) 2000 Eazel, Inc.
-
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public
-   License along with this program; see the file COPYING.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
-
-   Author: Rebecca Schulman <rebecka@eazel.com>
-*/
-
-
-
+#include <config.h>
 #include "nautilus-complex-search-bar.h"
 
-
+#include "nautilus-search-bar-criterion-private.h"
+#include "nautilus-search-bar-criterion.h"
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <glib.h>
 #include <gtk/gtkeventbox.h>
+#include <gtk/gtktable.h>
+#include <gtk/gtkvbox.h>
+#include <libgnomeui/gnome-dock.h>
+#include <libgnomeui/gnome-uidefs.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
-
+#include <libnautilus-extensions/nautilus-gdk-pixbuf-extensions.h>
+#include <libnautilus-extensions/nautilus-global-preferences.h>
+#include <libnautilus-extensions/nautilus-gtk-extensions.h>
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
-#include <libnautilus-extensions/nautilus-search-bar-criterion.h>
-#include <libnautilus-extensions/nautilus-search-bar-criterion-private.h>
 #include <libnautilus-extensions/nautilus-string.h>
+#include <libnautilus-extensions/nautilus-undo-signal-handlers.h>
 
 struct NautilusComplexSearchBarDetails {
 	GtkVBox *bar_container;
@@ -44,12 +52,14 @@ struct NautilusComplexSearchBarDetails {
 	GtkWidget *more_options;
 	GtkWidget *fewer_options;
 	GtkWidget *find_them;
+	
+	/* For use in setting up clipboard */
+	NautilusWindow *window;
 
 	GSList *search_criteria;
-	gchar *undo_text;
-	gboolean undo_registered;
 };
 
+static void  real_activate				  (NautilusNavigationBar	       *bar);
 static char *nautilus_complex_search_bar_get_location     (NautilusNavigationBar         *bar);
 static void  nautilus_complex_search_bar_set_location     (NautilusNavigationBar         *bar,
 							   const char                    *location);
@@ -65,61 +75,90 @@ static void  more_options_callback                        (GtkObject
 							   gpointer                       data);
 static void  fewer_options_callback                       (GtkObject                     *object,
 							   gpointer                       data);
-static void  criterion_callback                           (NautilusSearchBarCriterion *old_criterion,
-							   NautilusSearchBarCriterion *new_criterion,
+static void  search_bar_criterion_type_changed_callback   (GtkObject *old_criterion_object,
 							   gpointer data);
 static GtkWidget * load_find_them_pixmap_widget           (void);
 
 static void	   update_options_buttons_state 	  (NautilusComplexSearchBar *bar);
 static void	   update_find_button_state 	  	  (NautilusComplexSearchBar *bar);
 static void	   update_dynamic_buttons_state 	  (NautilusComplexSearchBar *bar);
-
+static void        update_criteria_choices                (gpointer list_item,
+							   gpointer data);
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusComplexSearchBar, nautilus_complex_search_bar, NAUTILUS_TYPE_SEARCH_BAR)
 
 
-     /* called by the criterion when the user choosed 
-	a new criterion type */
-static void criterion_callback (NautilusSearchBarCriterion *old_criterion,
-				NautilusSearchBarCriterion *new_criterion,
-				gpointer data)
+/* called by the criterion when the user chooses
+   a new criterion type */
+static void 
+search_bar_criterion_type_changed_callback (GtkObject *old_criterion_object,
+					    gpointer data)
 { 
+	NautilusSearchBarCriterionType new_type;
+	NautilusSearchBarCriterion *criterion, *new_criterion;
+	GSList *old_criterion_location;
 	NautilusComplexSearchBar *bar;
-	int row;
-	GSList *criterion_list;
-	GSList *list;
 
-	bar = (NautilusComplexSearchBar *) data;
+	g_return_if_fail (NAUTILUS_IS_SEARCH_BAR_CRITERION (old_criterion_object));
+	g_return_if_fail (NAUTILUS_IS_COMPLEX_SEARCH_BAR (data));
+
+	criterion = NAUTILUS_SEARCH_BAR_CRITERION (old_criterion_object);
+	bar = NAUTILUS_COMPLEX_SEARCH_BAR (data);
+
+	/* First create the new criterion with the type that was activated */
+	new_type = GPOINTER_TO_INT (gtk_object_get_data (old_criterion_object,
+							 "type"));
+	new_criterion = nautilus_search_bar_criterion_new_with_type (new_type, 
+								     bar);
+	gtk_signal_connect (GTK_OBJECT (new_criterion),
+			    "criterion_type_changed",
+			    search_bar_criterion_type_changed_callback,
+			    (gpointer) bar);
+	old_criterion_location = g_slist_find (bar->details->search_criteria,
+					       criterion);
+	old_criterion_location->data = new_criterion;
+	unattach_criterion_from_search_bar (bar,
+					    criterion);
+	gtk_object_destroy (GTK_OBJECT (criterion));
+	attach_criterion_to_search_bar (bar,
+					new_criterion,
+					g_slist_position (bar->details->search_criteria,
+							  old_criterion_location) + 1);
 
-	unattach_criterion_from_search_bar (bar, old_criterion);
 	
-	criterion_list = g_slist_find (bar->details->search_criteria, 
-				       old_criterion);
-	row = g_slist_position (bar->details->search_criteria,
-				criterion_list);
-	g_print ("row %d\n", row);
-
-	list = g_slist_remove (bar->details->search_criteria,
-			       old_criterion);
-			
-	list = g_slist_insert (list, new_criterion, row);
-
-	nautilus_search_bar_criterion_set_callback (new_criterion, 
-						    criterion_callback,
-						    (gpointer) bar);
-
-	attach_criterion_to_search_bar (bar, new_criterion, row + 1);
+	/* Then tell the other criteria to take update their
+	   menus to reflect the other criteria you can really choose now */
+	g_slist_foreach (bar->details->search_criteria,
+			 update_criteria_choices,
+			 bar);
 
+	update_dynamic_buttons_state (bar);
 }
 
 
-     
+static void
+update_criteria_choices (gpointer list_item,
+			 gpointer data)
+{
+	NautilusSearchBarCriterion *criterion;
+	NautilusComplexSearchBar *bar;
+
+	g_return_if_fail (NAUTILUS_IS_SEARCH_BAR_CRITERION (list_item));
+	g_return_if_fail (NAUTILUS_IS_COMPLEX_SEARCH_BAR (data));
+	criterion = NAUTILUS_SEARCH_BAR_CRITERION (list_item);
+	bar = NAUTILUS_COMPLEX_SEARCH_BAR (data);
+
+	nautilus_search_bar_criterion_update_valid_criteria_choices (criterion,
+								     bar->details->search_criteria);
+	
+}
      
 static void
 nautilus_complex_search_bar_initialize_class (NautilusComplexSearchBarClass *klass)
 {
 	GTK_OBJECT_CLASS (klass)->destroy = nautilus_complex_search_bar_destroy;
 
+	NAUTILUS_NAVIGATION_BAR_CLASS (klass)->activate = real_activate;
 	NAUTILUS_NAVIGATION_BAR_CLASS (klass)->get_location = nautilus_complex_search_bar_get_location;
 	NAUTILUS_NAVIGATION_BAR_CLASS (klass)->set_location = nautilus_complex_search_bar_set_location;
 }
@@ -138,14 +177,21 @@ nautilus_complex_search_bar_initialize (NautilusComplexSearchBar *bar)
 	bar->details->table = GTK_TABLE (gtk_table_new (1, 3, TRUE));
 	gtk_table_set_col_spacings (bar->details->table,
 				    1);
+	gtk_table_set_row_spacings (bar->details->table,
+				    1);
 
 	gtk_container_set_resize_mode (GTK_CONTAINER (bar->details->table),
 				       GTK_RESIZE_IMMEDIATE);
-	file_name_criterion = nautilus_search_bar_criterion_first_new ();
-	nautilus_search_bar_criterion_set_callback (file_name_criterion, 
-						    criterion_callback,
-						    (gpointer) bar);
 
+	/* Create button before criterion so we text fields can hook to criterion's signal */
+	bar->details->find_them = gtk_button_new ();
+
+	file_name_criterion = nautilus_search_bar_criterion_first_new (bar);
+	g_assert (GTK_IS_OBJECT (GTK_OBJECT (file_name_criterion)));
+	gtk_signal_connect (GTK_OBJECT (file_name_criterion),
+			    "criterion_type_changed",
+			    search_bar_criterion_type_changed_callback,
+			    (gpointer) bar);
 	bar->details->search_criteria = g_slist_append (NULL,
 							file_name_criterion);
 	attach_criterion_to_search_bar (bar, file_name_criterion, 1);
@@ -161,7 +207,6 @@ nautilus_complex_search_bar_initialize (NautilusComplexSearchBar *bar)
 	hbox = gtk_hbox_new (FALSE, 1);
 
 
-	bar->details->find_them = gtk_button_new ();
 	find_them_box = gtk_hbox_new (FALSE, 1);
 	find_them_pixmap_widget = load_find_them_pixmap_widget ();
 	if (find_them_pixmap_widget != NULL)
@@ -178,7 +223,7 @@ nautilus_complex_search_bar_initialize (NautilusComplexSearchBar *bar)
 			    1);
 
 	gtk_container_add (GTK_CONTAINER (bar->details->find_them), find_them_box);
-	gtk_signal_connect_object (GTK_OBJECT (bar->details->find_them), "pressed",
+	gtk_signal_connect_object (GTK_OBJECT (bar->details->find_them), "clicked",
 				   nautilus_navigation_bar_location_changed,
 				   GTK_OBJECT (bar));
 
@@ -227,6 +272,37 @@ nautilus_complex_search_bar_destroy (GtkObject *object)
 	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
 }
 
+static GtkWidget *
+get_first_text_field (NautilusComplexSearchBar *bar)
+{
+	GSList *node;
+	NautilusSearchBarCriterion *criterion;
+
+	for (node = bar->details->search_criteria; node != NULL; node = node->next) {
+		criterion = NAUTILUS_SEARCH_BAR_CRITERION (node->data);
+		if (criterion->details->use_value_entry) {
+			return GTK_WIDGET (criterion->details->value_entry);
+		}
+	}
+
+	return NULL;
+}
+
+static void
+real_activate (NautilusNavigationBar *navigation_bar)
+{
+	NautilusComplexSearchBar *bar;
+	GtkWidget *initial_focus_widget;
+
+	bar = NAUTILUS_COMPLEX_SEARCH_BAR (navigation_bar);
+
+	/* Put the keyboard focus in a text field when switching to search mode */
+	initial_focus_widget = get_first_text_field (bar);
+	if (initial_focus_widget != NULL) {
+		gtk_widget_grab_focus (initial_focus_widget);
+	}
+}
+
 /* returned string should be g_freed by the caller */
 static char *
 nautilus_complex_search_bar_get_location (NautilusNavigationBar *navigation_bar)
@@ -250,12 +326,20 @@ nautilus_complex_search_bar_get_location (NautilusNavigationBar *navigation_bar)
 		g_free (criterion_text);
 		criteria_text = temp_criterion;
 	}
+
 	trimmed_fragment = nautilus_str_strip_trailing_str (criteria_text, " & ");
 	g_free (criteria_text);
 
 	escaped_fragment = gnome_vfs_escape_string (trimmed_fragment);
-	search_uri = g_strconcat ("search:", escaped_fragment, NULL);
+	g_free (trimmed_fragment);
+
+	if (nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_SEARCH_METHOD, FALSE)) {
+		search_uri = g_strconcat ("search:index-with-backup", escaped_fragment, NULL);
+	} else {
+		search_uri = g_strconcat ("search:index-only", escaped_fragment, NULL);
+	}
 	g_free (escaped_fragment);
+
 	return search_uri;
 }
 
@@ -267,7 +351,7 @@ nautilus_complex_search_bar_set_location (NautilusNavigationBar *navigation_bar,
 
 	bar = NAUTILUS_COMPLEX_SEARCH_BAR (navigation_bar);
 
-	/* FIXME: Not implemented. */
+	/* FIXME bugzilla.eazel.com 2517: Not implemented. */
 }
 
 static void
@@ -275,7 +359,8 @@ attach_criterion_to_search_bar (NautilusComplexSearchBar *bar,
 				NautilusSearchBarCriterion *criterion,
 				int row)
 {
-
+	GtkWidget *hbox;
+	
 	g_return_if_fail (NAUTILUS_IS_COMPLEX_SEARCH_BAR (bar));
 
 	gtk_table_attach_defaults (bar->details->table,
@@ -286,22 +371,54 @@ attach_criterion_to_search_bar (NautilusComplexSearchBar *bar,
 				    GTK_WIDGET (criterion->details->relation_menu),
 				    1, 2, row - 1, row);
 	g_assert (criterion->details->use_value_entry + 
-		  criterion->details->use_value_menu == 1);
+		  criterion->details->use_value_menu == 1 ||
+		  criterion->details->type == NAUTILUS_DATE_MODIFIED_SEARCH_CRITERION);
 	if (criterion->details->use_value_entry) {
-		gtk_table_attach_defaults (bar->details->table,
-					   GTK_WIDGET (criterion->details->value_entry),
-					   2, 3, row - 1, row);
+		/* If there is a suffix, we want to take the space from
+		   the entry, to keep things neat.  So make a box 
+		   for the entry and the suffix together */
+		if (criterion->details->use_value_suffix) {
+			hbox = gtk_hbox_new (FALSE, 0);
+			gtk_box_pack_start (GTK_BOX (hbox),
+					    GTK_WIDGET (criterion->details->value_entry),
+					    TRUE, TRUE,
+					    GNOME_PAD_SMALL);
+			gtk_box_pack_start (GTK_BOX (hbox),
+					    GTK_WIDGET (criterion->details->value_suffix),
+					    FALSE, TRUE, GNOME_PAD_SMALL);
+			gtk_widget_show_all (hbox);
+			gtk_table_attach_defaults (bar->details->table,
+						   GTK_WIDGET (hbox),
+						   2, 3, row - 1, row);
+		}
+		else {
+			gtk_table_attach_defaults (bar->details->table,
+						   GTK_WIDGET (criterion->details->value_entry),
+						   2, 3, row - 1, row);
+		}
 		/* We want to track whether the entry text is empty or not. */
 		gtk_signal_connect_object (GTK_OBJECT (criterion->details->value_entry),
 				    	   "changed", 
 				    	   update_find_button_state, 
 				    	   GTK_OBJECT (bar));
+
+		/* We want to activate the "Find" button when any entry text is not empty */
+		g_assert (GTK_IS_BUTTON (bar->details->find_them));
+		gtk_signal_connect_object (GTK_OBJECT (criterion->details->value_entry), 
+					   "activate",
+					   nautilus_gtk_button_auto_click, 
+					   GTK_OBJECT (bar->details->find_them));
 	}
 	if (criterion->details->use_value_menu) {
 		gtk_table_attach_defaults (bar->details->table,
 					   GTK_WIDGET (criterion->details->value_menu),
 					   2, 3, row - 1, row);
 	}
+	if (criterion->details->type == NAUTILUS_DATE_MODIFIED_SEARCH_CRITERION) {
+		gtk_table_attach_defaults (bar->details->table,
+					   GTK_WIDGET (criterion->details->date),
+					   2, 3, row - 1, row);
+	}
 	gtk_table_resize (bar->details->table, row, 3);
 
 }
@@ -315,9 +432,11 @@ unattach_criterion_from_search_bar (NautilusComplexSearchBar *bar,
 
 	gtk_container_remove (GTK_CONTAINER (bar->details->table),
 			      GTK_WIDGET (criterion->details->relation_menu));
-	g_assert (criterion->details->use_value_entry + 
-		  criterion->details->use_value_menu == 1);
-	if (criterion->details->use_value_entry) {
+	g_assert ((criterion->details->use_value_entry + 
+		   criterion->details->use_value_menu == 1) ||
+		  criterion->details->type == NAUTILUS_DATE_MODIFIED_SEARCH_CRITERION);
+	if (criterion->details->use_value_entry 
+	    && !criterion->details->use_value_suffix) {
 		gtk_container_remove (GTK_CONTAINER (bar->details->table),
 				      GTK_WIDGET (criterion->details->value_entry));
 	}
@@ -325,6 +444,14 @@ unattach_criterion_from_search_bar (NautilusComplexSearchBar *bar,
 		gtk_container_remove (GTK_CONTAINER (bar->details->table),
 				      GTK_WIDGET (criterion->details->value_menu));
 	}
+	if (criterion->details->use_value_suffix) {
+		gtk_container_remove (GTK_CONTAINER (bar->details->table),
+				      GTK_WIDGET (criterion->details->value_suffix)->parent);
+	}	
+	if (criterion->details->type == NAUTILUS_DATE_MODIFIED_SEARCH_CRITERION) {
+		gtk_container_remove (GTK_CONTAINER (bar->details->table),
+				      GTK_WIDGET (criterion->details->date));
+	}
 	gtk_table_resize (bar->details->table, 
 			  g_slist_length (bar->details->search_criteria) - 1, 
 			  3);
@@ -339,8 +466,8 @@ load_find_them_pixmap_widget (void)
 	GdkBitmap *mask;
 	
 	pixbuf = gdk_pixbuf_new_from_file (NAUTILUS_PIXMAPDIR "/search.png");
-	if(pixmap != NULL) {
-		gdk_pixbuf_render_pixmap_and_mask (pixbuf, &pixmap, &mask, 128);
+	if (pixmap != NULL) {
+		gdk_pixbuf_render_pixmap_and_mask (pixbuf, &pixmap, &mask, NAUTILUS_STANDARD_ALPHA_THRESHHOLD);
 		gdk_pixbuf_unref (pixbuf);
 		return gtk_pixmap_new (pixmap, mask);
 	} else
@@ -349,9 +476,16 @@ load_find_them_pixmap_widget (void)
 				  
 
 GtkWidget *
-nautilus_complex_search_bar_new (void)
+nautilus_complex_search_bar_new (NautilusWindow *window)
 {
-	return gtk_widget_new (NAUTILUS_TYPE_COMPLEX_SEARCH_BAR, NULL);
+	GtkWidget *bar;
+
+	g_return_val_if_fail (NAUTILUS_IS_WINDOW (window), NULL);
+
+	bar = gtk_widget_new (NAUTILUS_TYPE_COMPLEX_SEARCH_BAR, NULL);
+	gtk_object_set_data (GTK_OBJECT (bar), "associated_window", window);
+
+	return bar;
 }
 
 
@@ -370,16 +504,21 @@ more_options_callback (GtkObject *object,
 
 	list = bar->details->search_criteria;
 	last_criterion = (NautilusSearchBarCriterion *)((g_slist_last (list))->data);
-	criterion = nautilus_search_bar_criterion_next_new (last_criterion->details->type);
-	nautilus_search_bar_criterion_set_callback (criterion, 
-						    criterion_callback, 
-						    (gpointer) bar);
+	criterion = nautilus_search_bar_criterion_next_new (last_criterion->details->type,
+							    bar);
+	gtk_signal_connect (GTK_OBJECT (criterion),
+			    "criterion_type_changed",
+			    search_bar_criterion_type_changed_callback,
+			    (gpointer) bar);
 	bar->details->search_criteria = g_slist_append (list, criterion);
 
 	attach_criterion_to_search_bar (bar, criterion, 
 					g_slist_length (bar->details->search_criteria));
 	nautilus_search_bar_criterion_show (criterion);
 	update_dynamic_buttons_state (bar);
+	g_slist_foreach (bar->details->search_criteria,
+			 update_criteria_choices,
+			 bar);	
 }
 
 
@@ -389,6 +528,7 @@ fewer_options_callback (GtkObject *object,
 {
 	NautilusSearchBarCriterion *criterion;
 	NautilusComplexSearchBar *bar;
+	GtkWidget *dock;
 	GSList *last;
 	int old_length, new_length;
 
@@ -412,13 +552,22 @@ fewer_options_callback (GtkObject *object,
 	new_length = g_slist_length (bar->details->search_criteria);
 	g_assert (new_length + 1 == old_length);
 
-	/* FIXME: the folowing is pretty much evil since it relies on the run-time
-	   widget hierarchy of the nautilus toolbar. 
-	   Any better fix is wellcome.
-	*/
-	gtk_widget_queue_resize (GTK_WIDGET (bar->details->bar_container)->parent->parent->parent->parent->parent->parent->parent->parent);
-
 	update_dynamic_buttons_state (bar);
+	g_slist_foreach (bar->details->search_criteria,
+			 update_criteria_choices,
+			 bar);
+
+	/* FIXME bugzilla.eazel.com 3171:
+	 * We don't know why this line is needed here, but if it's removed
+	 * then the bar sometimes won't shrink when we press the fewer options
+	 * button. Specifically, if the window is very wide, then it won't 
+	 * shrink when pressing the fewer options button, but it will if the
+	 * window is fairly narrow.
+	 */
+	dock = gtk_widget_get_ancestor (GTK_WIDGET (bar), GNOME_TYPE_DOCK);
+	if (dock != NULL) {
+		gtk_widget_queue_resize (dock);
+	}
 }
 
 static void
@@ -426,8 +575,9 @@ update_options_buttons_state (NautilusComplexSearchBar *bar)
 {
 	/* "Fewer Options" is enabled unless there's only one criterion */
 	gtk_widget_set_sensitive (GTK_WIDGET (bar->details->fewer_options), g_slist_length (bar->details->search_criteria) > 1);
+	gtk_widget_set_sensitive (GTK_WIDGET (bar->details->more_options), g_slist_length (bar->details->search_criteria) < NAUTILUS_NUMBER_OF_SEARCH_CRITERIA);
 
-	/* FIXME:
+	/* FIXME bugzilla.eazel.com 1937:
 	 * When we limit the number of criteria available for a single search, we need to
 	 * sensitize the "More Options" button correctly here.
 	 */
@@ -479,4 +629,8 @@ update_dynamic_buttons_state (NautilusComplexSearchBar *bar)
 }
 
 
-
+GSList *   
+nautilus_complex_search_bar_get_search_criteria (NautilusComplexSearchBar *bar)
+{
+	return bar->details->search_criteria;
+}
