@@ -1,26 +1,29 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 
+/*
+ * Nautilus
+ *
+ * Copyright (C) 2000 Eazel, Inc.
+ *
+ * Nautilus is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * Nautilus is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * Authors: Darin Adler <darin@eazel.com>
+ */
+
 /* nautilus-desktop-window.c
-
-   Copyright (C) 2000 Eazel, Inc.
-
-   The Gnome Library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   The Gnome Library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public
-   License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
-
-   Authors: Darin Adler <darin@eazel.com>
-*/
+ */
 
 #include <config.h>
 #include "nautilus-desktop-window.h"
@@ -34,7 +37,8 @@
 #include <libnautilus-extensions/nautilus-gtk-extensions.h>
 #include <libnautilus-extensions/nautilus-file-utilities.h>
 #include <libnautilus-extensions/nautilus-link.h>
-
+#include <X11/Xatom.h>
+#include <gdk/gdkx.h>
 
 struct NautilusDesktopWindowDetails {
 	GList *unref_list;
@@ -85,44 +89,44 @@ nautilus_desktop_window_initialize (NautilusDesktopWindow *window)
 			       FALSE, FALSE, FALSE);
 }
 
+static void
+nautilus_desktop_window_realized (NautilusDesktopWindow *window)
+{
+	/* Tuck the desktop windows xid in the root to indicate we own the desktop.
+	 */
+	Window window_xid;
+	window_xid = GDK_WINDOW_XWINDOW (GTK_WIDGET (window)->window);
+	gdk_property_change (NULL, gdk_atom_intern ("NAUTILUS_DESKTOP_WINDOW_ID", FALSE),
+			     XA_WINDOW, 32, PropModeReplace, (guchar *) &window_xid, 1);
+}
+
 NautilusDesktopWindow *
 nautilus_desktop_window_new (NautilusApplication *application)
 {
 	NautilusDesktopWindow *window;
 	char *desktop_directory_path;
 	char *desktop_directory_uri;
-	GnomeVFSURI *trash_dir_uri;
-	char *trash_dir_uri_text;
-	GnomeVFSResult result;
 
 	window = NAUTILUS_DESKTOP_WINDOW
-		(gtk_object_new (nautilus_desktop_window_get_type(),
+		(gtk_widget_new (nautilus_desktop_window_get_type(),
 				 "app", application,
 				 "app_id", "nautilus",
 				 NULL));
 
+	nautilus_window_disable_keyboard_navigation_for_menus (NAUTILUS_WINDOW (window));
+
+
+	/* Special sawmill setting*/
+	gtk_window_set_wmclass (GTK_WINDOW (window), "desktop_window", "Nautilus");
+
+	gtk_signal_connect (GTK_OBJECT (window), "realize", GTK_SIGNAL_FUNC (nautilus_desktop_window_realized), NULL);
+
 	desktop_directory_path = nautilus_get_desktop_directory ();
-
-	/* Create the trash.
-	 * We can do this at some other time if we want, but this
-	 * might be a good time.
-	 */
-	result = gnome_vfs_find_directory (NULL, GNOME_VFS_DIRECTORY_KIND_TRASH, 
-					   &trash_dir_uri, TRUE, 0777);
-	if (result == GNOME_VFS_OK) {
-		trash_dir_uri_text = gnome_vfs_uri_to_string (trash_dir_uri,
-							      GNOME_VFS_URI_HIDE_NONE);
-		gnome_vfs_uri_unref (trash_dir_uri);
-		nautilus_link_create (desktop_directory_path,
-				      "Trash", "trash-empty.png", 
-				      trash_dir_uri_text);
-		g_free (trash_dir_uri_text);
-	}
-
+	
 	/* Point window at the desktop folder.
 	 * Note that nautilus_desktop_window_initialize is too early to do this.
 	 */
-	desktop_directory_uri = nautilus_get_uri_from_local_path
+	desktop_directory_uri = gnome_vfs_get_uri_from_local_path
 		(desktop_directory_path);
 	g_free (desktop_directory_path);
 	nautilus_window_goto_uri (NAUTILUS_WINDOW (window), desktop_directory_uri);
@@ -140,6 +144,8 @@ destroy (GtkObject *object)
 
 	window = NAUTILUS_DESKTOP_WINDOW (object);
 
+	gdk_property_delete (NULL, gdk_atom_intern ("NAUTILUS_DESKTOP_WINDOW_ID", TRUE));
+
 	nautilus_gtk_object_list_free (window->details->unref_list);
 	g_free (window->details);
 
@@ -150,16 +156,19 @@ static void
 realize (GtkWidget *widget)
 {
 	NautilusDesktopWindow *window;
+#ifdef UIH
 	GtkContainer *dock_as_container;
 	GList *children, *p;
 	GtkWidget *child;
+#endif
 
 	window = NAUTILUS_DESKTOP_WINDOW (widget);
 
-	/* Hide unused pieces of the GnomeApp.
+	/* Hide unused pieces of the BonoboWin.
 	 * We don't want a menu bar, toolbars, or status bar on the desktop.
 	 * But we don't want to hide the client area!
 	 */
+#ifdef UIH
 	gtk_widget_hide (GNOME_APP (window)->menubar);
 	gtk_widget_hide (GNOME_APP (window)->statusbar);
 	dock_as_container = GTK_CONTAINER (GNOME_APP (window)->dock);
@@ -175,6 +184,7 @@ realize (GtkWidget *widget)
 		}
 	}
 	g_list_free (children);
+#endif
 
 	/* Make sure we get keyboard events */
 	gtk_widget_set_events (widget, gtk_widget_get_events (widget) 
@@ -209,8 +219,7 @@ realize (GtkWidget *widget)
 	 * skip the window.
 	 */
 	gnome_win_hints_set_hints (widget,
-				   WIN_HINTS_SKIP_FOCUS
-				   | WIN_HINTS_SKIP_WINLIST
+				   WIN_HINTS_SKIP_WINLIST
 				   | WIN_HINTS_SKIP_TASKBAR);
 
 	/* FIXME bugzilla.eazel.com 1255: 
