@@ -1,56 +1,57 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 
+/*
+ * Nautilus
+ *
+ * Copyright (C) 2000 Eazel, Inc.
+ *
+ * Nautilus is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * Nautilus is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public
+ * License along with this program; see the file COPYING.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
+ * Author: Maciej Stachowiak <mjs@eazel.com>
+ *         Ettore Perazzoli <ettore@gnu.org>
+ *         Michael Meeks <michael@nuclecu.unam.mx>
+ *
+ */
+
 /* nautilus-location-bar.c - Location bar for Nautilus
-
-   Copyright (C) 1999, 2000 Free Software Foundation
-   Copyright (C) 2000 Eazel, Inc.
-
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public
-   License along with this program; see the file COPYING.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
-
-   Author: Ettore Perazzoli <ettore@gnu.org>
-   DnD code added by Michael Meeks <michael@nuclecu.unam.mx>
-*/
+ */
 
 #include <config.h>
 #include "nautilus-location-bar.h"
 
 #include "nautilus-window.h"
-
-#include <string.h>
-
-#include <gtk/gtksignal.h>
+#include "nautilus-window-private.h"
 #include <gtk/gtkdnd.h>
 #include <gtk/gtkeventbox.h>
-#include <libgnomevfs/gnome-vfs.h>
-
+#include <gtk/gtksignal.h>
 #include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-mime.h>
 #include <libgnome/gnome-i18n.h>
-
+#include <libgnome/gnome-mime.h>
 #include <libgnomeui/gnome-stock.h>
 #include <libgnomeui/gnome-uidefs.h>
-
+#include <libgnomevfs/gnome-vfs.h>
+#include <libnautilus/nautilus-clipboard.h>
 #include <libnautilus-extensions/nautilus-entry.h>
-#include <libnautilus-extensions/nautilus-string.h>
-#include <libnautilus-extensions/nautilus-glib-extensions.h>
-#include <libnautilus-extensions/nautilus-stock-dialogs.h>
-#include <libnautilus-extensions/nautilus-gtk-macros.h>
 #include <libnautilus-extensions/nautilus-file-utilities.h>
-
+#include <libnautilus-extensions/nautilus-glib-extensions.h>
+#include <libnautilus-extensions/nautilus-gtk-macros.h>
+#include <libnautilus-extensions/nautilus-stock-dialogs.h>
+#include <libnautilus-extensions/nautilus-string.h>
 #include <stdio.h>
+#include <string.h>
 
 #define NAUTILUS_DND_URI_LIST_TYPE 	  "text/uri-list"
 #define NAUTILUS_DND_TEXT_PLAIN_TYPE 	  "text/plain"
@@ -81,13 +82,17 @@ static void  nautilus_location_bar_set_location     (NautilusNavigationBar    *n
 						     const char               *location);
 static void  nautilus_location_bar_initialize_class (NautilusLocationBarClass *class);
 static void  nautilus_location_bar_initialize       (NautilusLocationBar      *bar);
+static void  nautilus_location_bar_update_label     (NautilusLocationBar      *bar);
 
-NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusLocationBar, nautilus_location_bar, NAUTILUS_TYPE_NAVIGATION_BAR)
+NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusLocationBar,
+				   nautilus_location_bar,
+				   NAUTILUS_TYPE_NAVIGATION_BAR)
 
 static NautilusWindow *
-nautilus_location_bar_get_window (GtkWidget *widget)
+nautilus_location_bar_get_window (GtkWidget *bar)
 {
-	return NAUTILUS_WINDOW (gtk_widget_get_toplevel (widget));
+
+	return NAUTILUS_WINDOW (gtk_object_get_data (GTK_OBJECT (bar), "associated_window"));
 }
 
 static void
@@ -97,7 +102,8 @@ drag_data_received_callback (GtkWidget *widget,
 		       	     int y,
 		       	     GtkSelectionData *data,
 		             guint info,
-		             guint32 time)
+		             guint32 time,
+			     gpointer callback_data)
 {
 	GList *names, *node;
 	NautilusApplication *application;
@@ -108,6 +114,7 @@ drag_data_received_callback (GtkWidget *widget,
 
 	g_assert (NAUTILUS_IS_LOCATION_BAR (widget));
 	g_assert (data != NULL);
+	g_assert (callback_data == NULL);
 
 	names = gnome_uri_list_extract_uris (data->data);
 
@@ -129,6 +136,7 @@ drag_data_received_callback (GtkWidget *widget,
 					  name_count);
 		new_windows_for_extras = nautilus_simple_dialog 
 			(GTK_WIDGET (nautilus_location_bar_get_window (widget)),
+			 TRUE,
 			 prompt,
 			 _("View in Multiple Windows?"),
 			 GNOME_STOCK_BUTTON_OK,
@@ -165,13 +173,16 @@ drag_data_get_callback (GtkWidget *widget,
 		  	GdkDragContext *context,
 		  	GtkSelectionData *selection_data,
 		  	guint info,
-		 	guint32 time)
+		 	guint32 time,
+			gpointer callback_data)
 {
+	NautilusNavigationBar *bar;
 	char *entry_text;
 
 	g_assert (selection_data != NULL);
+	bar = NAUTILUS_NAVIGATION_BAR (callback_data);
 
-	entry_text = nautilus_navigation_bar_get_location (NAUTILUS_NAVIGATION_BAR (widget->parent));
+	entry_text = nautilus_navigation_bar_get_location (bar);
 	
 	switch (info) {
 	case NAUTILUS_DND_URI_LIST:
@@ -190,22 +201,24 @@ drag_data_get_callback (GtkWidget *widget,
 
 /* utility routine to determine the string to expand to.  If we don't have anything yet, accept
    the whole string, otherwise accept the largest part common to both */
-   
+
 static char *
 accumulate_name(char *full_name, char *candidate_name)
 {
 	char *result_name, *str1, *str2;
+
 	if (full_name == NULL)
-		result_name = g_strdup(candidate_name);
+		result_name = g_strdup (candidate_name);
 	else {
 		result_name = full_name;
-		if (!nautilus_str_has_prefix(full_name, candidate_name)) {
+		if (!nautilus_str_has_prefix (full_name, candidate_name)) {
 			str1 = full_name;
 			str2 = candidate_name;
-			while ((*str1++ == *str2++)) {};	
+			while ((*str1++ == *str2++)) { }
 			*--str1 = '\0';
 		}
 	}
+	
 	return result_name;
 }
 
@@ -214,7 +227,7 @@ accumulate_name(char *full_name, char *candidate_name)
   find something, add it to the entry */
   
 static void
-try_to_expand_path(GtkEditable *editable)
+try_to_expand_path (GtkEditable *editable)
 {
 	GnomeVFSResult result;
 	GnomeVFSFileInfo *current_file_info;
@@ -223,14 +236,21 @@ try_to_expand_path(GtkEditable *editable)
 	int base_length;
 	int current_path_length;
 	int offset;
-	const char *base_name;
+	const char *base_name_ptr;
+	char *base_name;
 	char *user_location;
 	char *current_path;
 	char *dir_name;
 	char *expand_text;
-
+	char *expand_name;
+	
 	user_location = gtk_editable_get_chars (editable, 0, -1);
- 	
+	
+	/* if it's just '~' or '~/', don't expand because slash shouldn't be appended */
+	if (nautilus_strcmp (user_location, "~") == 0
+	    || nautilus_strcmp (user_location, "~/") == 0) {
+		return;
+	}
 	current_path = nautilus_make_uri_from_input (user_location);
 
 	if (!nautilus_istr_has_prefix (current_path, "file://")) {
@@ -243,10 +263,11 @@ try_to_expand_path(GtkEditable *editable)
 
 	uri = gnome_vfs_uri_new (current_path);
 	
-	base_name = gnome_vfs_uri_get_basename (uri);
-	if (base_name)
+	base_name_ptr = gnome_vfs_uri_get_basename (uri);
+	if (base_name_ptr) {
+		base_name = gnome_vfs_unescape_string (base_name_ptr, NULL);
 		base_length = strlen (base_name);
-	else {
+	} else {
 		gnome_vfs_uri_unref (uri);
 		g_free (current_path);
 		return;	
@@ -257,11 +278,12 @@ try_to_expand_path(GtkEditable *editable)
 	/* get file info for the directory */
 
 	result = gnome_vfs_directory_list_load (&list, dir_name,
-					       GNOME_VFS_FILE_INFO_DEFAULT, NULL);
+						GNOME_VFS_FILE_INFO_DEFAULT, NULL);
 	if (result != GNOME_VFS_OK) {
-		g_free(dir_name);
-		gnome_vfs_uri_unref(uri);
-		g_free(current_path);
+		g_free (dir_name);
+		g_free (base_name);
+		gnome_vfs_uri_unref (uri);
+		g_free (current_path);
 		return;
 	}
 
@@ -272,7 +294,13 @@ try_to_expand_path(GtkEditable *editable)
 	expand_text = NULL;
 	while (current_file_info != NULL) {
 		if (nautilus_str_has_prefix (current_file_info->name, base_name)) {
-			expand_text = accumulate_name (expand_text, current_file_info->name);
+			if (current_file_info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
+				expand_name = g_strconcat (current_file_info->name, "/", NULL);
+			} else {
+				expand_name = g_strdup (current_file_info->name);
+			}
+			expand_text = accumulate_name (expand_text, expand_name);
+			g_free (expand_name);
 		}
 		current_file_info = gnome_vfs_directory_list_next (list);
 	}
@@ -285,10 +313,11 @@ try_to_expand_path(GtkEditable *editable)
 		g_free (expand_text);
 	}
 	
-	g_free(dir_name);
-	g_free(current_path);
-	g_free(user_location);
-	gnome_vfs_directory_list_destroy(list);
+	g_free (dir_name);
+	g_free (base_name);
+	g_free (current_path);
+	g_free (user_location);
+	gnome_vfs_directory_list_destroy (list);
 }
 
 /* Until we have a more elegant solution, this is how we figure out if
@@ -336,8 +365,8 @@ editable_key_press_callback (GtkObject *object,
 	GdkEventKey *event;
 	int position;
 	gboolean *return_value_location;
-
-	g_assert (data == NULL);
+	NautilusLocationBar *bar;
+	
 	g_assert (n_args == 1);
 	g_assert (args != NULL);
 
@@ -358,6 +387,20 @@ editable_key_press_callback (GtkObject *object,
 	    && entry_would_have_inserted_characters (event))  {
 		try_to_expand_path (editable);
 	}
+	
+	bar = NAUTILUS_LOCATION_BAR (data);
+	nautilus_location_bar_update_label (bar);
+}
+
+static void
+real_activate (NautilusNavigationBar *navigation_bar)
+{
+	NautilusLocationBar *bar;
+
+	bar = NAUTILUS_LOCATION_BAR (navigation_bar);
+
+	/* Put the keyboard focus in the text field when switching to this mode */
+	gtk_widget_grab_focus (GTK_WIDGET (bar->entry));
 }
 
 static void
@@ -366,6 +409,8 @@ destroy (GtkObject *object)
 	NautilusLocationBar *bar;
 
 	bar = NAUTILUS_LOCATION_BAR (object);
+	
+	g_free (bar->last_location);
 
 	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
 }
@@ -381,6 +426,7 @@ nautilus_location_bar_initialize_class (NautilusLocationBarClass *class)
 	
 	navigation_bar_class = NAUTILUS_NAVIGATION_BAR_CLASS (class);
 
+	navigation_bar_class->activate = real_activate;
 	navigation_bar_class->get_location = nautilus_location_bar_get_location;
 	navigation_bar_class->set_location = nautilus_location_bar_set_location;
 }
@@ -393,6 +439,8 @@ nautilus_location_bar_initialize (NautilusLocationBar *bar)
 	GtkWidget *event_box;
 	GtkWidget *hbox;
 
+	bar->last_location = NULL;
+	
 	hbox = gtk_hbox_new (0, FALSE);
 
 	event_box = gtk_event_box_new ();
@@ -400,12 +448,15 @@ nautilus_location_bar_initialize (NautilusLocationBar *bar)
 					GNOME_PAD_SMALL);
 	label = gtk_label_new (_("Location:"));
 	gtk_container_add   (GTK_CONTAINER (event_box), label);
+	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
 
 
 	gtk_box_pack_start  (GTK_BOX (hbox), event_box, FALSE, TRUE,
 			     GNOME_PAD_SMALL);
 
 	entry = nautilus_entry_new ();
+	NAUTILUS_ENTRY (entry)->special_tab_handling = TRUE;
+	
 	gtk_signal_connect_object (GTK_OBJECT (entry), "activate",
 				   nautilus_navigation_bar_location_changed, GTK_OBJECT (bar));
 
@@ -414,13 +465,14 @@ nautilus_location_bar_initialize (NautilusLocationBar *bar)
 	 */
 	gtk_signal_connect_full (GTK_OBJECT (entry), "key_press_event",
 				 NULL, editable_key_press_callback,
-				 NULL, NULL,
+				 bar, NULL,
 				 FALSE, TRUE);
 	
 	gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
 
 	gtk_container_add (GTK_CONTAINER (bar), hbox);
 
+
 	/* Drag source */
 	gtk_drag_source_set (GTK_WIDGET (event_box), 
 			     GDK_BUTTON1_MASK | GDK_BUTTON3_MASK,
@@ -443,13 +495,26 @@ nautilus_location_bar_initialize (NautilusLocationBar *bar)
 
 	bar->label = GTK_LABEL (label);
 	bar->entry = GTK_ENTRY (entry);	
+
+
 }
 
 
 GtkWidget *
-nautilus_location_bar_new (void)
+nautilus_location_bar_new (NautilusWindow *window)
 {
-	return gtk_widget_new (NAUTILUS_TYPE_LOCATION_BAR, NULL);
+	GtkWidget *bar;
+	NautilusLocationBar *location_bar;
+
+	bar = gtk_widget_new (NAUTILUS_TYPE_LOCATION_BAR, NULL);
+	location_bar = NAUTILUS_LOCATION_BAR (bar);
+
+	/* Clipboard */
+	nautilus_clipboard_set_up_editable
+		(GTK_EDITABLE (location_bar->entry),
+		 nautilus_window_get_ui_container (window));
+
+	return bar;
 }
 
 static void
@@ -459,6 +524,7 @@ nautilus_location_bar_set_location (NautilusNavigationBar *navigation_bar,
 	NautilusLocationBar *bar;
 	char *formatted_location;
 
+
 	g_assert (location != NULL);
 	
 	bar = NAUTILUS_LOCATION_BAR (navigation_bar);
@@ -470,6 +536,15 @@ nautilus_location_bar_set_location (NautilusNavigationBar *navigation_bar,
 	nautilus_entry_set_text (NAUTILUS_ENTRY (bar->entry),
 				 formatted_location);
 	g_free (formatted_location);
+
+	/* remember the original location for later comparison */
+	
+	if (bar->last_location) {
+		g_free (bar->last_location);
+	}
+
+	bar->last_location = g_strdup (location);
+	nautilus_location_bar_update_label (bar);
 }
 
 /**
@@ -484,7 +559,6 @@ nautilus_location_bar_set_location (NautilusNavigationBar *navigation_bar,
  * but not garunteed.
  *
  **/
-
 static char *
 nautilus_location_bar_get_location (NautilusNavigationBar *navigation_bar) 
 {
@@ -499,3 +573,24 @@ nautilus_location_bar_get_location (NautilusNavigationBar *navigation_bar)
 	return best_uri;
 }
 	       
+/**
+ * nautilus_location_bar_update_label
+ *
+ * if the text in the entry matches the uri, set the label to "location", otherwise use "goto"
+ *
+ **/
+static void
+nautilus_location_bar_update_label (NautilusLocationBar *bar)
+{
+	char *current_text, *current_location;
+	
+	current_text = gtk_entry_get_text (GTK_ENTRY (bar->entry));
+	current_location = nautilus_make_uri_from_input (current_text);
+	
+	if (nautilus_uris_match (bar->last_location, current_location)) {
+		gtk_label_set_text (GTK_LABEL (bar->label), _("Location:"));
+	} else {		 
+		gtk_label_set_text (GTK_LABEL (bar->label), _("   Go To:"));
+	}	
+	g_free (current_location);
+}
