@@ -1,26 +1,30 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 
+/*
+ * Nautilus
+ *
+ * Copyright (C) 2000 Eazel, Inc.
+ *
+ * Nautilus is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * Nautilus is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public
+ * License along with this program; see the file COPYING.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
+ * Author: Maciej Stachowiak <mjs@eazel.com>
+ */
+
 /* nautilus-navigation-bar.c - Abstract navigation bar class
-
-   Copyright (C) 2000 Eazel, Inc.
-
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public
-   License along with this program; see the file COPYING.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
-
-   Author: Maciej Stachowiak <mjs@eazel.com>
-*/
+ */
 
 #include <config.h>
 
@@ -33,6 +37,7 @@
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
 
 enum {
+	ACTIVATE,
 	LOCATION_CHANGED,
 	LAST_SIGNAL
 };
@@ -53,17 +58,28 @@ nautilus_navigation_bar_initialize_class (NautilusNavigationBarClass *klass)
 
 	object_class = GTK_OBJECT_CLASS (klass);
 	
-	signals[LOCATION_CHANGED]
-		= gtk_signal_new ("location_changed",
-				  GTK_RUN_FIRST,
-				  object_class->type,
-				  GTK_SIGNAL_OFFSET (NautilusNavigationBarClass,
-						     location_changed),
-				  gtk_marshal_NONE__STRING,
-				  GTK_TYPE_NONE, 1, GTK_TYPE_STRING);
+	signals[ACTIVATE] = gtk_signal_new
+		("activate",
+		 GTK_RUN_LAST,
+		 object_class->type,
+		 GTK_SIGNAL_OFFSET (NautilusNavigationBarClass,
+				    activate),
+		 gtk_marshal_NONE__NONE,
+		 GTK_TYPE_NONE, 0);
+
+	signals[LOCATION_CHANGED] = gtk_signal_new
+		("location_changed",
+		 GTK_RUN_LAST,
+		 object_class->type,
+		 GTK_SIGNAL_OFFSET (NautilusNavigationBarClass,
+				    location_changed),
+		 gtk_marshal_NONE__STRING,
+		 GTK_TYPE_NONE, 1, GTK_TYPE_STRING);
 
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 
+	klass->activate = NULL;
+
 	NAUTILUS_ASSIGN_MUST_OVERRIDE_SIGNAL (klass, nautilus_navigation_bar, get_location);
 	NAUTILUS_ASSIGN_MUST_OVERRIDE_SIGNAL (klass, nautilus_navigation_bar, set_location);
 }
@@ -73,6 +89,21 @@ nautilus_navigation_bar_initialize (NautilusNavigationBar *bar)
 {
 }
 
+/**
+ * nautilus_navigation_bar_activate
+ * 
+ * Change the navigation bar to an active state.
+ * 
+ * @bar: A NautilusNavigationBar.
+ */
+void
+nautilus_navigation_bar_activate (NautilusNavigationBar *bar)
+{
+	g_return_if_fail (NAUTILUS_IS_NAVIGATION_BAR (bar));
+
+	gtk_signal_emit (GTK_OBJECT (bar), signals[ACTIVATE]);
+}
+
 /**
  * nautilus_navigation_bar_get_location
  * 
@@ -86,7 +117,8 @@ nautilus_navigation_bar_get_location (NautilusNavigationBar *bar)
 {
 	g_return_val_if_fail (NAUTILUS_IS_NAVIGATION_BAR (bar), NULL);
 
-	return (* NAUTILUS_NAVIGATION_BAR_CLASS (GTK_OBJECT (bar)->klass)->get_location) (bar);
+	return NAUTILUS_CALL_VIRTUAL (NAUTILUS_NAVIGATION_BAR_CLASS, bar,
+				      get_location, (bar));
 }
 
 /**
@@ -103,7 +135,8 @@ nautilus_navigation_bar_set_location (NautilusNavigationBar *bar,
 {
 	g_return_if_fail (NAUTILUS_IS_NAVIGATION_BAR (bar));
 
-	(* NAUTILUS_NAVIGATION_BAR_CLASS (GTK_OBJECT (bar)->klass)->set_location) (bar, location);
+	NAUTILUS_CALL_VIRTUAL (NAUTILUS_NAVIGATION_BAR_CLASS, bar,
+			       set_location, (bar, location));
 }
 
 /**
