@@ -1,15 +1,16 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 
-/* Nautilus
+/*
+ * Nautilus
  *
  * Copyright (C) 2000 Eazel, Inc.
  *
- * This library is free software; you can redistribute it and/or
+ * Nautilus is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
+ * Nautilus is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
@@ -27,22 +28,27 @@
 #include <config.h>
 #include "nautilus-shell.h"
 
-/* FIXME: This is a workaround for ORBit bug where including idl files
- * in other idl files causes trouble.
- */
-#include "nautilus-shell-interface.h"
-#define nautilus_view_component_H
-
 #include "nautilus-desktop-window.h"
-#include <gtk/gtklabel.h>
+#include "nautilus-main.h"
+#include "nautilus-shell-interface.h"
 #include <gtk/gtkframe.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtklabel.h>
 #include <gtk/gtkmain.h>
+#include <gtk/gtksignal.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-stock.h>
 #include <libgnomeui/gnome-uidefs.h>
 #include <libnautilus-extensions/nautilus-file-utilities.h>
+#include <libnautilus-extensions/nautilus-glib-extensions.h>
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
+#include <libnautilus-extensions/nautilus-label.h>
+#include <libnautilus-extensions/nautilus-preferences.h>
 #include <libnautilus-extensions/nautilus-stock-dialogs.h>
+#include <libnautilus/nautilus-bonobo-workarounds.h>
+#include <stdlib.h>
+
+#define START_STATE_CONFIG	"start-state"
 
 struct NautilusShellDetails {
 	NautilusApplication *application;
@@ -62,6 +68,9 @@ static void corba_stop_desktop              (PortableServer_Servant  servant,
 					     CORBA_Environment      *ev);
 static void corba_quit	                    (PortableServer_Servant  servant,
 					     CORBA_Environment      *ev);
+static void corba_restart                    (PortableServer_Servant  servant,
+					     CORBA_Environment      *ev);
+static gboolean restore_window_states	    (NautilusShell *shell);
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusShell, nautilus_shell, BONOBO_OBJECT_TYPE)
 
@@ -80,6 +89,7 @@ nautilus_shell_get_epv (void)
 	epv.start_desktop = corba_start_desktop;
 	epv.stop_desktop = corba_stop_desktop;
 	epv.quit = corba_quit;
+	epv.restart = corba_restart;
 	return &epv;
 }
 
@@ -87,7 +97,7 @@ static POA_Nautilus_Shell__vepv *
 nautilus_shell_get_vepv (void)
 {
 	static POA_Nautilus_Shell__vepv vepv;
-	vepv.Bonobo_Unknown_epv = bonobo_object_get_epv ();
+	vepv.Bonobo_Unknown_epv = nautilus_bonobo_object_get_epv ();
 	vepv.Nautilus_Shell_epv = nautilus_shell_get_epv ();
 	return &vepv;
 }
@@ -138,7 +148,7 @@ nautilus_shell_new (NautilusApplication *application)
 {
 	NautilusShell *shell;
 
-	shell = gtk_type_new (NAUTILUS_TYPE_SHELL);
+	shell = NAUTILUS_SHELL (gtk_object_new (NAUTILUS_TYPE_SHELL, NULL));
 	shell->details->application = application;
 	return shell;
 }
@@ -154,7 +164,7 @@ display_caveat (GtkWindow *parent_window)
 	GtkWidget *text;
 	char *file_name;
 
-	dialog = gnome_dialog_new (_("Nautilus: caveat"),
+	dialog = gnome_dialog_new (_("Caveat"),
 				   GNOME_STOCK_BUTTON_OK,
 				   NULL);
   	gtk_container_set_border_width (GTK_CONTAINER (dialog), GNOME_PAD);
@@ -171,34 +181,36 @@ display_caveat (GtkWindow *parent_window)
 	gtk_widget_show (vbox);
 	gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);
 
-	frame = gtk_frame_new (NULL);
-	gtk_widget_show (frame);
-	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
-  	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
-	
 	file_name = nautilus_pixmap_file ("About_Image.png");
-	pixmap = gnome_pixmap_new_from_file (file_name);
-	g_free (file_name);
-	gtk_widget_show (pixmap);
-	gtk_container_add (GTK_CONTAINER (frame), pixmap);
+	if (file_name != NULL) {
+		pixmap = gnome_pixmap_new_from_file (file_name);
+		g_free (file_name);
 
-  	text = gtk_label_new
-		(_("Welcome to Nautilus Preview Release 1."
-		   "\n\n"
-		   "Thanks for your interest in using Nautilus.  As "
-		   "a user of the Preview Release, you try out "
-		   "and provide feedback on an early version of Eazel's powerful new desktop "
-		   "manager for GNOME."
-		   "\n\n"
-		   "As with any software under development, you should exercise caution "
-		   "when running this program.  Some features are not yet complete or still "
-		   "unstable.  You will encounter many more bugs than in the completed version.  "
-		   "Eazel can provide no guarantee that it will work properly, or assume liability "
-		   "for your use of it.  Use it at your own risk."
-		   "\n\n"
-		   "We hope that you will enjoy using Nautilus.  "
-		   "For more information, please go to http://www.eazel.com."));
-    	gtk_label_set_line_wrap (GTK_LABEL (text), TRUE);
+		if (pixmap != NULL) {
+			frame = gtk_frame_new (NULL);
+			gtk_widget_show (frame);
+			gtk_frame_set_shadow_type (GTK_FRAME (frame),
+						   GTK_SHADOW_IN);
+			gtk_box_pack_start (GTK_BOX (vbox), frame,
+					    FALSE, FALSE, 0);
+
+			gtk_widget_show (pixmap);
+			gtk_container_add (GTK_CONTAINER (frame), pixmap);
+		}
+	}
+
+  	text = nautilus_label_new
+		(_("Thank you for trying out Nautilus Preview Release 2.\n"
+		   "\n"
+		   "Nautilus is still under construction. Some operations may not "
+		   "have their intended consequences, so be careful and make "
+		   "sure any important files on your system are backed up."
+		   "\n"
+		   "Please visit http://www.eazel.com/feedback.html to provide feedback."));
+	nautilus_label_set_font_size (NAUTILUS_LABEL (text), 14);
+	nautilus_label_set_text_justification (NAUTILUS_LABEL (text), GTK_JUSTIFY_LEFT);
+	nautilus_label_set_line_wrap (NAUTILUS_LABEL (text), TRUE);
+	nautilus_label_set_line_wrap_width (NAUTILUS_LABEL (text), 300);
 	gtk_widget_show (text);
   	gtk_box_pack_start (GTK_BOX (hbox), text, FALSE, FALSE, 0);
 
@@ -266,8 +278,10 @@ corba_open_default_window (PortableServer_Servant servant,
 
 	shell = NAUTILUS_SHELL (((BonoboObjectServant *) servant)->bonobo_object);
 
-	/* Open a window pointing at the default location. */
-	open_window (shell, NULL);
+	if (! restore_window_states (shell)) {
+		/* Open a window pointing at the default location. */
+		open_window (shell, NULL);
+	}
 }
 
 static void
@@ -294,7 +308,121 @@ static void
 corba_quit (PortableServer_Servant servant,
 		      CORBA_Environment *ev)
 {
-	if (gtk_main_level () > 0) {
-		gtk_main_quit ();
+	nautilus_main_event_loop_quit ();
+}
+
+/*
+ * code for saving the state of nautilus windows across a restart
+ *
+ * for now, only the window geometry & uri is saved, into "start-state",
+ * in a list of strings like:
+ *     "90x90+1+1 uri"
+ */
+
+static void
+save_window_states (void)
+{
+	GSList *windows, *iter;
+	GSList *out = NULL;
+
+	windows = nautilus_application_windows ();
+	for (iter = windows; iter; iter = g_slist_next (iter)) {
+		NautilusWindow *window = (NautilusWindow *) (iter->data);
+		GdkWindow *gdk_window = GTK_WIDGET (window)->window;
+		char *window_info;
+		int x, y, width, height;
+
+		/* need root origin (origin of all the window dressing) */
+		gdk_window_get_root_origin (gdk_window, &x, &y);
+		width = GTK_WIDGET (window)->allocation.width;
+		height = GTK_WIDGET (window)->allocation.height;
+
+		/* FIXME bugzilla.eazel.com 4375
+		   This hardcoded subst should be parameterized
+		   at some point. This ensures that when eazel-install:nautilus
+		   restarts nautilus, it doesn't go to eazel-install:nautilus but
+		   to eazel: instead */
+		if (g_strncasecmp (window->location, "eazel-install:", 14) == 0) {
+			window_info = g_strdup_printf ("%dx%d+%d+%d %s", 
+						       width, height, 
+						       x, y, 
+						       "eazel:"); 
+		} else {
+			window_info = g_strdup_printf ("%dx%d+%d+%d %s", 
+						       width, height, 
+						       x, y, 
+						       window->location); 
+		}
+		out = g_slist_prepend (out, window_info);
 	}
+
+	nautilus_preferences_set_string_list (START_STATE_CONFIG, out);
+	g_slist_foreach (out, (GFunc)g_free, NULL);
+	g_slist_free (out);
+}
+
+/* returns TRUE if there was state info which has been used to create new windows */
+static gboolean
+restore_window_states (NautilusShell *shell)
+{
+	GSList *start_state, *iter;
+	NautilusWindow *window;
+	char *window_info, *p, *uri;
+	int x, y, width, height;
+
+	start_state = nautilus_preferences_get_string_list (START_STATE_CONFIG);
+	if (! start_state) {
+		return FALSE;
+	}
+
+	for (iter = start_state; iter; iter = g_slist_next (iter)) {
+		p = window_info = (char *) (iter->data);
+
+		width = strtol (p, &p, 10);
+		if (*p == 'x') {
+			p++;
+		}
+		height = strtol (p, &p, 10);
+		if (*p == '+') {
+			p++;
+		}
+		x = strtol (p, &p, 10);
+		if (*p == '+') {
+			p++;
+		}
+		y = strtol (p, &p, 10);
+		p = strchr (p, ' ');
+		if (p) {
+			uri = p+1;
+		} else {
+			uri = NULL;
+		}
+
+		window = nautilus_application_create_window (shell->details->application);
+		if (uri == NULL) {
+			nautilus_window_go_home (window);
+		} else {
+			nautilus_window_goto_uri (window, uri);
+		}
+
+		gtk_widget_set_uposition (GTK_WIDGET (window), x, y);
+		gtk_widget_set_usize (GTK_WIDGET (window), width, height);
+		display_caveat_first_time (shell, window);
+
+		g_free (window_info);
+	}
+	g_slist_free (start_state);
+	nautilus_preferences_set_string_list (START_STATE_CONFIG, NULL);
+	return TRUE;
+}
+
+static void
+corba_restart (PortableServer_Servant servant,
+	       CORBA_Environment *ev)
+{
+	save_window_states ();
+
+	nautilus_main_event_loop_quit ();
+	nautilus_setenv ("_NAUTILUS_RESTART", "yes", 1);
+
 }
