@@ -1,14 +1,16 @@
  /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 
-/* Nautilus
+/*
+ * Nautilus
+ *
  * Copyright (C) 1999, 2000 Eazel, Inc.
  *
- * This program is free software; you can redistribute it and/or modify
+ * Nautilus is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * Nautilus is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -19,25 +21,34 @@
  *
  * Author: Andy Hertzfeld <andy@eazel.com>
  *
- * This is the sidebar widget, which displays overview information
+ */
+
+/* This is the sidebar widget, which displays overview information
  * hosts individual panels for various views.
- *
  */
 
 #include <config.h>
 #include "nautilus-sidebar.h"
 
-#include <math.h>
+#include "nautilus-link-set-window.h"
+#include "nautilus-sidebar-tabs.h"
+#include "nautilus-sidebar-title.h"
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gnome-xml/parser.h>
+#include <gtk/gtkcheckmenuitem.h>
+#include <gtk/gtkdnd.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtknotebook.h>
+#include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-uidefs.h>
+#include <libgnomevfs/gnome-vfs-application-registry.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
-#include <liboaf/liboaf.h>
-#include <parser.h>
-
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <libnautilus-extensions/nautilus-background.h>
 #include <libnautilus-extensions/nautilus-directory.h>
+#include <libnautilus-extensions/nautilus-drag.h>
 #include <libnautilus-extensions/nautilus-file.h>
 #include <libnautilus-extensions/nautilus-file-utilities.h>
 #include <libnautilus-extensions/nautilus-glib-extensions.h>
@@ -46,16 +57,15 @@
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
 #include <libnautilus-extensions/nautilus-keep-last-vertical-box.h>
 #include <libnautilus-extensions/nautilus-metadata.h>
-#include <libnautilus-extensions/nautilus-program-choosing.h>
-#include <libnautilus-extensions/nautilus-string.h>
 #include <libnautilus-extensions/nautilus-mime-actions.h>
 #include <libnautilus-extensions/nautilus-preferences.h>
+#include <libnautilus-extensions/nautilus-program-choosing.h>
+#include <libnautilus-extensions/nautilus-stock-dialogs.h>
+#include <libnautilus-extensions/nautilus-string.h>
 #include <libnautilus-extensions/nautilus-theme.h>
 #include <libnautilus-extensions/nautilus-view-identifier.h>
-
-#include "nautilus-sidebar-tabs.h"
-#include "nautilus-sidebar-title.h"
-#include "nautilus-link-set-window.h"
+#include <liboaf/liboaf.h>
+#include <math.h>
 
 struct NautilusSidebarDetails {
 	GtkVBox *container;
@@ -67,10 +77,11 @@ struct NautilusSidebarDetails {
 	GtkVBox *button_box;
 	gboolean has_buttons;
 	char *uri;
+	NautilusFile *file;
+	guint file_changed_connection;
 	char *default_background_color;
 	char *default_background_image;
 	int selected_index;
-	NautilusDirectory *directory;
 	gboolean background_connected;
 	int old_width;
 };
@@ -104,7 +115,7 @@ static void     nautilus_sidebar_update_buttons     (NautilusSidebar  *sidebar);
 static void     add_command_buttons                 (NautilusSidebar  *sidebar,
 						     GList            *application_list);
 
-/* FIXME bug 1245: hardwired sizes */
+/* FIXME bugzilla.eazel.com 1245: hardwired sizes */
 #define DEFAULT_TAB_COLOR "rgb:9999/9999/9999"
 
 #define SIDEBAR_MINIMUM_WIDTH 1
@@ -166,14 +177,14 @@ nautilus_sidebar_initialize_class (GtkObjectClass *object_klass)
 	widget_class->size_allocate = nautilus_sidebar_size_allocate;
 
 	/* add the "location changed" signal */
-	signals[LOCATION_CHANGED]
-		= gtk_signal_new ("location_changed",
-			GTK_RUN_FIRST,
-			object_klass->type,
-			GTK_SIGNAL_OFFSET (NautilusSidebarClass,
-				location_changed),
-			gtk_marshal_NONE__STRING,
-			GTK_TYPE_NONE, 1, GTK_TYPE_STRING);
+	signals[LOCATION_CHANGED] = gtk_signal_new
+		("location_changed",
+		 GTK_RUN_LAST,
+		 object_klass->type,
+		 GTK_SIGNAL_OFFSET (NautilusSidebarClass,
+				    location_changed),
+		 gtk_marshal_NONE__STRING,
+		 GTK_TYPE_NONE, 1, GTK_TYPE_STRING);
 
 	gtk_object_class_add_signals (object_klass, signals, LAST_SIGNAL);
 }
@@ -183,7 +194,6 @@ static void
 make_button_box (NautilusSidebar *sidebar)
 {
 	sidebar->details->button_box_centerer = GTK_HBOX (gtk_hbox_new (FALSE, 0));
-	gtk_widget_show (GTK_WIDGET (sidebar->details->button_box_centerer));
 	gtk_box_pack_start_defaults (GTK_BOX (sidebar->details->container),
 			    	     GTK_WIDGET (sidebar->details->button_box_centerer));
 
@@ -214,6 +224,9 @@ nautilus_sidebar_initialize (GtkObject *object)
 
 	/* load the default background from the current theme */
 	nautilus_sidebar_read_theme(sidebar);
+
+	/* enable mouse tracking */
+	gtk_widget_add_events (GTK_WIDGET (sidebar), GDK_POINTER_MOTION_MASK);
 	  	
 	/* create the container box */
   	sidebar->details->container = GTK_VBOX (gtk_vbox_new (FALSE, 0));
@@ -273,7 +286,12 @@ nautilus_sidebar_destroy (GtkObject *object)
 
 	gtk_object_unref (GTK_OBJECT (sidebar->details->notebook));
 
-	nautilus_directory_unref (sidebar->details->directory);
+	if (sidebar->details->file != NULL) {
+		gtk_signal_disconnect (GTK_OBJECT (sidebar->details->file), 
+				       sidebar->details->file_changed_connection);
+		nautilus_file_monitor_remove (sidebar->details->file, sidebar);
+		nautilus_file_unref (sidebar->details->file);
+	}
 
 	g_free (sidebar->details->uri);
 	g_free (sidebar->details->default_background_color);
@@ -285,6 +303,7 @@ nautilus_sidebar_destroy (GtkObject *object)
 					      nautilus_sidebar_theme_changed,
 					      sidebar);
 
+
 	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
 }
 
@@ -301,35 +320,35 @@ static gboolean
 nautilus_sidebar_sidebar_panel_enabled (const char *panel_iid)
 {
 	gboolean enabled;
-        gchar	 *key;
+        char *key;
 
 	key = nautilus_sidebar_get_sidebar_panel_key (panel_iid);
         enabled = nautilus_preferences_get_boolean (key, FALSE);
-
         g_free (key);
         return enabled;
 }
 
 /* callback to handle resetting the background */
 static void
-reset_background_callback(GtkWidget *menu_item, GtkWidget *sidebar)
+reset_background_callback (GtkWidget *menu_item, GtkWidget *sidebar)
 {
 	NautilusBackground *background;
+
 	background = nautilus_get_widget_background(sidebar);
-	if (background) { 
-		nautilus_background_reset(background); 
+	if (background != NULL) { 
+		nautilus_background_reset (background); 
 	}
 }
 
 /* callback for sidebar panel menu items to toggle their visibility */
 static void
-toggle_sidebar_panel(GtkWidget *widget, char *sidebar_id)
+toggle_sidebar_panel (GtkWidget *widget, char *sidebar_id)
 {
-        gchar	 *key;
+        char *key;
 
 	key = nautilus_sidebar_get_sidebar_panel_key (sidebar_id);
-	nautilus_preferences_set_boolean(key, !nautilus_preferences_get_boolean(key, FALSE));
-	g_free(key); 
+	nautilus_preferences_set_boolean (key, !nautilus_preferences_get_boolean (key, FALSE));
+	g_free (key); 
 }
 
 /* utility routine to add a menu item for each potential sidebar panel */
@@ -348,7 +367,7 @@ nautilus_sidebar_add_panel_items(NautilusSidebar *sidebar, GtkWidget *menu)
 
 	CORBA_exception_init (&ev);
 
-	/* ask OAF for all of the sidebars panel */
+	/* ask OAF for all of the sidebars panels */
 	query = "nautilus:sidebar_panel_name.defined() AND repo_ids.has ('IDL:Bonobo/Control:1.0')";
 	oaf_result = oaf_query (query, NULL, &ev);
 	
@@ -361,12 +380,13 @@ nautilus_sidebar_add_panel_items(NautilusSidebar *sidebar, GtkWidget *menu)
 			/* check to see if we've seen this one */
 			if (g_list_find_custom (name_list, id->name, (GCompareFunc) strcmp) == NULL) {
 				name_list = g_list_append (name_list, g_strdup (id->name));
-			
+				
 				/* add a check menu item */
-				menu_item = gtk_check_menu_item_new_with_label(id->name);
-				enabled = nautilus_sidebar_sidebar_panel_enabled(id->iid);
-				gtk_check_menu_item_set_show_toggle(GTK_CHECK_MENU_ITEM(menu_item), enabled);
-				gtk_widget_show(menu_item);
+				menu_item = gtk_check_menu_item_new_with_label (id->name);
+				enabled = nautilus_sidebar_sidebar_panel_enabled (id->iid);
+				gtk_check_menu_item_set_show_toggle (GTK_CHECK_MENU_ITEM(menu_item), TRUE);
+				gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), enabled);
+				gtk_widget_show (menu_item);
 				gtk_menu_append (GTK_MENU(menu), menu_item);
 				gtk_signal_connect_full (GTK_OBJECT (menu_item), "activate",
 							 GTK_SIGNAL_FUNC (toggle_sidebar_panel),
@@ -393,12 +413,12 @@ nautilus_sidebar_background_is_default (NautilusSidebar *sidebar)
 	char *background_color, *background_image;
 	gboolean is_default;
 	
-	background_color = nautilus_directory_get_metadata (sidebar->details->directory,
-							    NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
-							    NULL);
-	background_image = nautilus_directory_get_metadata (sidebar->details->directory,
-							    NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
-								    NULL);
+	background_color = nautilus_file_get_metadata (sidebar->details->file,
+						       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
+						       NULL);
+	background_image = nautilus_file_get_metadata (sidebar->details->file,
+						       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
+						       NULL);
 	
 	is_default = background_color == NULL && background_image == NULL;
 	g_free (background_color);
@@ -430,7 +450,7 @@ nautilus_sidebar_create_context_menu (NautilusSidebar *sidebar)
 	/* add a separator */
 	menu_item = gtk_menu_item_new ();
 	gtk_widget_show (menu_item);
-	gtk_menu_append (GTK_MENU(menu), menu_item);
+	gtk_menu_append (GTK_MENU (menu), menu_item);
 	
 	/* add the sidebar panels */
 	nautilus_sidebar_add_panel_items(sidebar, menu);
@@ -441,7 +461,7 @@ nautilus_sidebar_create_context_menu (NautilusSidebar *sidebar)
 NautilusSidebar *
 nautilus_sidebar_new (void)
 {
-	return NAUTILUS_SIDEBAR (gtk_type_new (nautilus_sidebar_get_type ()));
+	return NAUTILUS_SIDEBAR (gtk_widget_new (nautilus_sidebar_get_type (), NULL));
 }
 
 /* utility routine to handle mapping local file names to a uri */
@@ -458,7 +478,7 @@ map_local_data_file (char *file_name)
 		}
 		
 		g_free (file_name);
-		file_name = nautilus_get_uri_from_local_path (temp_str);
+		file_name = gnome_vfs_get_uri_from_local_path (temp_str);
 		g_free (temp_str);
 	}
 	return file_name;
@@ -542,7 +562,11 @@ uri_is_local_image (const char *uri)
 		return FALSE;
 	}
 	
-	image_path = nautilus_get_local_path_from_uri (uri);
+	image_path = gnome_vfs_get_local_path_from_uri (uri);
+	if (image_path == NULL) {
+		return FALSE;
+	}
+
 	pixbuf = gdk_pixbuf_new_from_file (image_path);
 	g_free (image_path);
 	
@@ -560,15 +584,16 @@ receive_dropped_uri_list (NautilusSidebar *sidebar,
 {
 	char **uris;
 	gboolean exactly_one;
-	NautilusFile *file;
-
+	GtkWindow *window;
+	
 	uris = g_strsplit (selection_data->data, "\r\n", 0);
 	exactly_one = uris[0] != NULL && uris[1] == NULL;
-
+	window = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (sidebar)));
+	
 	switch (hit_test (sidebar, x, y)) {
 	case NO_PART:
 	case BACKGROUND_PART:
-		/* FIXME: Does this work for all images, or only background images?
+		/* FIXME bugzilla.eazel.com 2507: Does this work for all images, or only background images?
 		 * Other views handle background images differently from other URIs.
 		 */
 		if (exactly_one && uri_is_local_image (uris[0])) {
@@ -586,24 +611,43 @@ receive_dropped_uri_list (NautilusSidebar *sidebar,
 		break;
 	case ICON_PART:
 		/* handle images dropped on the logo specially */
-		/* FIXME bugzilla.eazel.com 605: 
-		 * Need feedback for cases where there is more than one URI 
-		 * and where the URI is not a local image.
-		 */
-		if (exactly_one && uri_is_local_image (uris[0])) {
-			file = nautilus_file_get (sidebar->details->uri);
-			if (file != NULL) {
-				nautilus_file_set_metadata (file,
+		
+		if (!exactly_one) {
+			nautilus_error_dialog (
+				_("You can't assign more than one custom icon at a time! "
+				  "Please drag just one image to set a custom icon."), 
+				_("More Than One Image"),
+				window);
+			break;
+		}
+		
+		if (uri_is_local_image (uris[0])) {
+			if (sidebar->details->file != NULL) {
+				nautilus_file_set_metadata (sidebar->details->file,
 							    NAUTILUS_METADATA_KEY_CUSTOM_ICON,
 							    NULL,
 							    uris[0]);
-				nautilus_file_set_metadata (file,
+				nautilus_file_set_metadata (sidebar->details->file,
 							    NAUTILUS_METADATA_KEY_ICON_SCALE,
 							    NULL,
 							    NULL);
-				nautilus_file_unref (file);
 			}
-		}
+		} else {	
+			if (nautilus_is_remote_uri (uris[0])) {
+				nautilus_error_dialog (
+					_("The file that you dropped is not local.  "
+					  "You can only use local images as custom icons."), 
+					_("Local Images Only"),
+					window);
+			
+			} else {
+				nautilus_error_dialog (
+					_("The file that you dropped is not an image.  "
+					  "You can only use local images as custom icons."),
+					_("Images Only"),
+					window);
+			}
+		}	
 		break;
 	}
 
@@ -636,8 +680,8 @@ receive_dropped_color (NautilusSidebar *sidebar,
 			(sidebar->details->sidebar_tabs,
 			 x, y, selection_data);
 		
-		nautilus_directory_set_metadata
-			(sidebar->details->directory,
+		nautilus_file_set_metadata
+			(sidebar->details->file,
 			 NAUTILUS_METADATA_KEY_SIDEBAR_TAB_COLOR,
 			 DEFAULT_TAB_COLOR,
 			 color_spec);
@@ -649,8 +693,8 @@ receive_dropped_color (NautilusSidebar *sidebar,
 			(sidebar->details->title_tab,
 			 x, y, selection_data);
 		
-		nautilus_directory_set_metadata
-			(sidebar->details->directory,
+		nautilus_file_set_metadata
+			(sidebar->details->file,
 			 NAUTILUS_METADATA_KEY_SIDEBAR_TITLE_TAB_COLOR,
 			 DEFAULT_TAB_COLOR,
 			 color_spec);
@@ -677,34 +721,7 @@ receive_dropped_keyword (NautilusSidebar *sidebar,
 			 int x, int y,
 			 GtkSelectionData *selection_data)
 {
-	NautilusFile *file;
-	GList *keywords, *word;
-
-	/* FIXME: This is a cut and paste copy of code that's in the icon dnd code. */
-			
-	/* OK, now we've got the keyword, so add it to the metadata */
-
-	/* FIXME bugzilla.eazel.com 866: Can't expect to read the
-	 * keywords list instantly here. We might need to read the
-	 * metafile first.
-	 */
-	file = nautilus_file_get (sidebar->details->uri);
-	if (file == NULL)
-		return;
-
-	/* Check and see if it's already there. */
-	keywords = nautilus_file_get_keywords (file);
-	word = g_list_find_custom (keywords, selection_data->data, (GCompareFunc) strcmp);
-	if (word == NULL) {
-		keywords = g_list_append (keywords, g_strdup (selection_data->data));
-	} else {
-		keywords = g_list_remove_link (keywords, word);
-		g_free (word->data);
-		g_list_free (word);
-	}
-
-	nautilus_file_set_keywords (file, keywords);
-	nautilus_file_unref (file);
+	nautilus_drag_file_receive_dropped_keyword (sidebar->details->file, selection_data->data);
 	
 	/* regenerate the display */
 	nautilus_sidebar_update_appearance (sidebar);  	
@@ -735,7 +752,7 @@ nautilus_sidebar_drag_data_received (GtkWidget *widget, GdkDragContext *context,
 			receive_dropped_uri_list (sidebar, x, y, selection_data);
 		break;	
 	case TARGET_KEYWORD:
-		receive_dropped_keyword(sidebar, x, y, selection_data);
+		receive_dropped_keyword (sidebar, x, y, selection_data);
 		break;
 	default:
 		g_warning ("unknown drop type");
@@ -870,12 +887,14 @@ nautilus_sidebar_motion_event (GtkWidget *widget, GdkEventMotion *event)
 
 	gtk_widget_get_pointer(widget, &x, &y);
 	
-	/* if the click is in the main tabs, tell them about it */
+	/* if the motion is in the main tabs, tell them about it */
 	sidebar_tabs = sidebar->details->sidebar_tabs;
 	if (y >= GTK_WIDGET (sidebar_tabs)->allocation.y) {
 		which_tab = nautilus_sidebar_tabs_hit_test (sidebar_tabs, x, y);
 		nautilus_sidebar_tabs_prelight_tab (sidebar_tabs, which_tab);
-	}
+	} else
+		nautilus_sidebar_tabs_prelight_tab (sidebar_tabs, -1);
+	
 
 	/* also handle prelighting in the title tab if necessary */
 	if (sidebar->details->selected_index >= 0) {
@@ -921,6 +940,10 @@ nautilus_sidebar_press_event (GtkWidget *widget, GdkEventButton *event)
 	int rounded_y;
 	int which_tab;
 		
+	if (widget->window != event->window) {
+		return FALSE;
+	}
+
 	sidebar = NAUTILUS_SIDEBAR (widget);
 
 	/* handle the context menu */
@@ -971,22 +994,22 @@ background_settings_changed_callback (NautilusBackground *background, NautilusSi
 	g_assert (NAUTILUS_IS_BACKGROUND (background));
 	g_assert (NAUTILUS_IS_SIDEBAR (sidebar));
 
-	if (sidebar->details->directory == NULL) {
+	if (sidebar->details->file == NULL) {
 		return;
 	}
 	
 	color_spec = nautilus_background_get_color (background);
-	nautilus_directory_set_metadata (sidebar->details->directory,
-					 NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
-					 sidebar->details->default_background_color,
-					 color_spec);
+	nautilus_file_set_metadata (sidebar->details->file,
+				    NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
+				    sidebar->details->default_background_color,
+				    color_spec);
 	g_free (color_spec);
 
-	image = nautilus_background_get_tile_image_uri (background);
-	nautilus_directory_set_metadata (sidebar->details->directory,
-					 NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
-					 sidebar->details->default_background_image,
-					 image);	
+	image = nautilus_background_get_image_uri (background);
+	nautilus_file_set_metadata (sidebar->details->file,
+				    NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
+				    sidebar->details->default_background_image,
+				    image);	
 	g_free (image);
 }
 
@@ -998,15 +1021,15 @@ background_appearance_changed_callback (NautilusBackground *background, Nautilus
 	gboolean is_default_color, is_default_image;
 	char *background_image, *background_color;
 
-	background_color = nautilus_directory_get_metadata (sidebar->details->directory,
-							    NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
-							    sidebar->details->default_background_color);
+	background_color = nautilus_file_get_metadata (sidebar->details->file,
+						       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
+						       sidebar->details->default_background_color);
 
-	background_image = nautilus_directory_get_metadata (sidebar->details->directory,
-							    NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
-							    sidebar->details->default_background_image);
+	background_image = nautilus_file_get_metadata (sidebar->details->file,
+						       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
+						       sidebar->details->default_background_image);
 	
-	nautlius_sidebar_title_select_text_color (sidebar->details->title);
+	nautilus_sidebar_title_select_text_color (sidebar->details->title);
 	
 	is_default_color = !nautilus_strcmp(background_color, sidebar->details->default_background_color);
 	is_default_image = !nautilus_strcmp(background_image, sidebar->details->default_background_image);
@@ -1025,16 +1048,16 @@ background_reset_callback (NautilusBackground *background, NautilusSidebar *side
 {
 	char *combine_mode;
 	
-	if (sidebar->details->directory == NULL) {
+	if (sidebar->details->file == NULL) {
 		return;
 	}
 	
 	/* set up the defaults, but don't write the metdata */
-	gtk_signal_handler_block_by_func (GTK_OBJECT(background),
+	gtk_signal_handler_block_by_func (GTK_OBJECT (background),
 					  background_settings_changed_callback,
 					  sidebar);
 	nautilus_background_set_color (background, sidebar->details->default_background_color);	
-	nautilus_background_set_tile_image_uri (background, sidebar->details->default_background_image);
+	nautilus_background_set_image_uri (background, sidebar->details->default_background_image);
 	
 	combine_mode = nautilus_theme_get_theme_data ("sidebar", "COMBINE");
 	nautilus_background_set_combine_mode (background, combine_mode != NULL);
@@ -1045,19 +1068,29 @@ background_reset_callback (NautilusBackground *background, NautilusSidebar *side
 					    sidebar);
 					   
 	/* reset the metadata */
-	nautilus_directory_set_metadata (sidebar->details->directory,
-					 NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
-					 sidebar->details->default_background_color,
-					 NULL);
-	nautilus_directory_set_metadata (sidebar->details->directory,
-					 NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
-					 sidebar->details->default_background_image,
-					 NULL);
+	nautilus_file_set_metadata (sidebar->details->file,
+				    NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
+				    sidebar->details->default_background_color,
+				    NULL);
+	nautilus_file_set_metadata (sidebar->details->file,
+				    NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
+				    sidebar->details->default_background_image,
+				    NULL);
 
 	gtk_signal_emit_stop_by_name (GTK_OBJECT (background),
 				      "reset");
 }
 
+static GtkWindow *
+nautilus_sidebar_get_window (NautilusSidebar *sidebar)
+{
+	GtkWidget *result;
+
+	result = gtk_widget_get_ancestor (GTK_WIDGET (sidebar), GTK_TYPE_WINDOW);
+
+	return result == NULL ? NULL : GTK_WINDOW (result);
+}
+
 static void
 command_button_callback (GtkWidget *button, char *id_str)
 {
@@ -1066,26 +1099,34 @@ command_button_callback (GtkWidget *button, char *id_str)
 	
 	sidebar = NAUTILUS_SIDEBAR (gtk_object_get_user_data (GTK_OBJECT (button)));
 
-	application = gnome_vfs_mime_application_new_from_id (id_str);
+	application = gnome_vfs_application_registry_get_mime_application (id_str);
 
-	nautilus_launch_application (application, sidebar->details->uri);	
+	if (application != NULL) {
+		nautilus_launch_application (application, sidebar->details->uri,
+					     nautilus_sidebar_get_window (sidebar));	
 
-	gnome_vfs_mime_application_free (application);
+		gnome_vfs_mime_application_free (application);
+	}
 }
 
 /* interpret commands for buttons specified by metadata. Handle some built-in ones explicitly, or fork
    a shell to handle general ones */
 /* for now, we only handle a few built in commands */
 static void
-metadata_button_callback (GtkWidget *button, char *command_str)
+metadata_button_callback (GtkWidget *button, const char *command_str)
 {
 	GtkWindow *window;
 	NautilusSidebar *sidebar;
+	char *path;
 	
 	sidebar = NAUTILUS_SIDEBAR (gtk_object_get_user_data (GTK_OBJECT (button)));
-	if (!strcmp(command_str, "#linksets")) {
+	if (strcmp (command_str, "#linksets") == 0) {
 		window = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (sidebar)));
-		nautilus_link_set_toggle_configure_window(sidebar->details->uri, window);
+		path = gnome_vfs_get_local_path_from_uri (sidebar->details->uri);
+		if (path != NULL) {
+			nautilus_link_set_toggle_configure_window (path, window);
+			g_free (path);
+		}
 	}
 }
 
@@ -1093,12 +1134,15 @@ static void
 nautilus_sidebar_chose_application_callback (GnomeVFSMimeApplication *application,
 					     gpointer callback_data)
 {
-	g_assert (NAUTILUS_IS_SIDEBAR (callback_data));
+	NautilusSidebar *sidebar;
+
+	sidebar = NAUTILUS_SIDEBAR (callback_data);
 
 	if (application != NULL) {
-		nautilus_launch_application 
+		nautilus_launch_application
 			(application, 
-			 NAUTILUS_SIDEBAR (callback_data)->details->uri);
+			 sidebar->details->uri,
+			 nautilus_sidebar_get_window (sidebar));
 	}
 }
 
@@ -1106,24 +1150,16 @@ static void
 open_with_callback (GtkWidget *button, gpointer ignored)
 {
 	NautilusSidebar *sidebar;
-	NautilusFile *file;
 	
 	sidebar = NAUTILUS_SIDEBAR (gtk_object_get_user_data (GTK_OBJECT (button)));
 	
-	/* FIXME bugzilla.eazel.com 866: Can't expect to put this
-	 * window up instantly. We might need to read the metafile
-	 * first.
-	 */
-	file = nautilus_file_get (sidebar->details->uri);
-	g_return_if_fail (file != NULL);
+	g_return_if_fail (sidebar->details->file != NULL);
 
 	nautilus_choose_application_for_file
-		(file,
+		(sidebar->details->file,
 		 GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (sidebar))),
 		 nautilus_sidebar_chose_application_callback,
 		 sidebar);
-
-	nautilus_file_unref (file);
 }
 
 /* utility routine that allocates the command buttons from the command list */
@@ -1150,7 +1186,7 @@ add_command_buttons (NautilusSidebar *sidebar, GList *application_list)
 				    FALSE, FALSE, 
 				    0);
 
-		/* FIXME: Security hole? Can't use a string from the
+		/* FIXME bugzilla.eazel.com 2510: Security hole? Can't use a string from the
 		 * MIME file as a printf format string without first
 		 * checking it over somehow.
 		 */
@@ -1183,7 +1219,7 @@ add_command_buttons (NautilusSidebar *sidebar, GList *application_list)
 /* utility to construct command buttons for the sidebar from the passed in metadata string */
 
 static void
-add_buttons_from_metadata(NautilusSidebar *sidebar, const char *button_data)
+add_buttons_from_metadata (NautilusSidebar *sidebar, const char *button_data)
 {
 	char **terms;
 	char *current_term, *temp_str;
@@ -1208,25 +1244,24 @@ add_buttons_from_metadata(NautilusSidebar *sidebar, const char *button_data)
 			        if (button_name != NULL) {
 			        	temp_button = gtk_button_new_with_label (button_name);		    
 					gtk_box_pack_start (GTK_BOX (sidebar->details->button_box), 
-						    temp_button, 
-						    FALSE, FALSE, 
-						    0);
+							    temp_button, 
+							    FALSE, FALSE, 
+							    0);
 					sidebar->details->has_buttons = TRUE;
-
-					command_string = g_strdup(temp_str + 1);		
-					g_free(button_name);
-
+					command_string = g_strdup (temp_str + 1);
+					g_free (button_name);
+					
 					nautilus_gtk_signal_connect_free_data 
 						(GTK_OBJECT (temp_button), "clicked",
-					 	GTK_SIGNAL_FUNC (metadata_button_callback), command_string);
+						 GTK_SIGNAL_FUNC (metadata_button_callback), command_string);
 		                	gtk_object_set_user_data (GTK_OBJECT (temp_button), sidebar);
-				
+					
 					gtk_widget_show (temp_button);			
 				}
 			}
 		}
 		g_free(current_term);
-	}	
+	}
 	g_strfreev (terms);
 }
 
@@ -1235,7 +1270,7 @@ add_buttons_from_metadata(NautilusSidebar *sidebar, const char *button_data)
  * 
  * Update the list of program-launching buttons based on the current uri.
  */
-void
+static void
 nautilus_sidebar_update_buttons (NautilusSidebar *sidebar)
 {
 	char *button_data;
@@ -1248,34 +1283,37 @@ nautilus_sidebar_update_buttons (NautilusSidebar *sidebar)
 		make_button_box (sidebar);
 	}
 
-	/* create buttons from directory metadata if necessary */
+	/* create buttons from file metadata if necessary */
 	
-	button_data = nautilus_directory_get_metadata (sidebar->details->directory,
-				NAUTILUS_METADATA_KEY_SIDEBAR_BUTTONS,
-				NULL);
+	button_data = nautilus_file_get_metadata (sidebar->details->file,
+						  NAUTILUS_METADATA_KEY_SIDEBAR_BUTTONS,
+						  NULL);
 	if (button_data) {
-		add_buttons_from_metadata(sidebar, button_data);
+		add_buttons_from_metadata (sidebar, button_data);
 		g_free(button_data);
 	}
-	
+
 	/* Make buttons for each item in short list + "Open with..." catchall,
 	 * unless there aren't any applications at all in complete list. 
 	 */
-	if (nautilus_mime_has_any_applications_for_uri (sidebar->details->uri)) {
+
+	if (nautilus_mime_has_any_applications_for_file (sidebar->details->file)) {
 		short_application_list = 
-			nautilus_mime_get_short_list_applications_for_uri (sidebar->details->uri);
+			nautilus_mime_get_short_list_applications_for_file (sidebar->details->file);
 		add_command_buttons (sidebar, short_application_list);
 		gnome_vfs_mime_application_list_free (short_application_list);
 
-		/* Hide button box if a sidebar panel is showing. */
+		/* Hide button box if a sidebar panel is showing. Otherwise, show it! */
 		if (sidebar->details->selected_index != -1) {
 			gtk_widget_hide (GTK_WIDGET (sidebar->details->button_box_centerer));
 			gtk_widget_hide (GTK_WIDGET (sidebar->details->title));
+		} else {
+			gtk_widget_show (GTK_WIDGET (sidebar->details->button_box_centerer));
 		}
 	}
 }
 
-void
+static void
 nautilus_sidebar_update_appearance (NautilusSidebar *sidebar)
 {
 	NautilusBackground *background;
@@ -1304,14 +1342,14 @@ nautilus_sidebar_update_appearance (NautilusSidebar *sidebar)
 	
 	/* Set up the background color and image from the metadata. */
 	background_image = NULL;
-	background_color = nautilus_directory_get_metadata (sidebar->details->directory,
-							    NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
-							    NULL);
+	background_color = nautilus_file_get_metadata (sidebar->details->file,
+						       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_COLOR,
+						       NULL);
 	if (background_color == NULL) {
 		background_color = g_strdup (sidebar->details->default_background_color);
-		background_image = nautilus_directory_get_metadata (sidebar->details->directory,
-							    NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
-							    sidebar->details->default_background_image);
+		background_image = nautilus_file_get_metadata (sidebar->details->file,
+							       NAUTILUS_METADATA_KEY_SIDEBAR_BACKGROUND_IMAGE,
+							       sidebar->details->default_background_image);
 	}	
 		
 	/* disable the settings_changed callback, so the background doesn't get
@@ -1323,7 +1361,7 @@ nautilus_sidebar_update_appearance (NautilusSidebar *sidebar)
 	nautilus_background_set_color (background, background_color);	
 	g_free (background_color);
 	
-	nautilus_background_set_tile_image_uri (background, background_image);
+	nautilus_background_set_image_uri (background, background_image);
 	g_free (background_image);
 
 	combine_mode = nautilus_theme_get_theme_data ("sidebar", "COMBINE");
@@ -1331,15 +1369,15 @@ nautilus_sidebar_update_appearance (NautilusSidebar *sidebar)
 	g_free (combine_mode);
 	
 	/* set up the color for the tabs */
-	color_spec = nautilus_directory_get_metadata (sidebar->details->directory,
-						      NAUTILUS_METADATA_KEY_SIDEBAR_TAB_COLOR,
-						      DEFAULT_TAB_COLOR);
+	color_spec = nautilus_file_get_metadata (sidebar->details->file,
+						 NAUTILUS_METADATA_KEY_SIDEBAR_TAB_COLOR,
+						 DEFAULT_TAB_COLOR);
 	nautilus_sidebar_tabs_set_color(sidebar->details->sidebar_tabs, color_spec);
 	g_free (color_spec);
 
-	color_spec = nautilus_directory_get_metadata (sidebar->details->directory,
-						      NAUTILUS_METADATA_KEY_SIDEBAR_TITLE_TAB_COLOR,
-						      DEFAULT_TAB_COLOR);
+	color_spec = nautilus_file_get_metadata (sidebar->details->file,
+						 NAUTILUS_METADATA_KEY_SIDEBAR_TITLE_TAB_COLOR,
+						 DEFAULT_TAB_COLOR);
 	nautilus_sidebar_tabs_set_color(sidebar->details->title_tab, color_spec);
 	g_free (color_spec);
 
@@ -1349,14 +1387,34 @@ nautilus_sidebar_update_appearance (NautilusSidebar *sidebar)
 					    sidebar);
 }
 
+
+static void
+nautilus_sidebar_update_all (NautilusSidebar *sidebar)
+{
+	GList *attributes;
+	gboolean ready;
+
+	attributes = nautilus_mime_actions_get_minimum_file_attributes ();
+	ready = nautilus_file_check_if_ready (sidebar->details->file, attributes);
+	g_list_free (attributes);
+
+	if (ready) {
+		nautilus_sidebar_update_appearance (sidebar);
+		
+		/* set up the command buttons */
+		nautilus_sidebar_update_buttons (sidebar);
+	}
+}
+
 /* here is the key routine that populates the sidebar with the appropriate information when the uri changes */
 
 void
 nautilus_sidebar_set_uri (NautilusSidebar *sidebar, 
-			      const char* new_uri,
-			      const char* initial_title)
+			  const char* new_uri,
+			  const char* initial_title)
 {       
-	NautilusDirectory *directory;
+	NautilusFile *file;
+	GList *attributes;
 
 	g_return_if_fail (NAUTILUS_IS_SIDEBAR (sidebar));
 	g_return_if_fail (new_uri != NULL);
@@ -1370,19 +1428,36 @@ nautilus_sidebar_set_uri (NautilusSidebar *sidebar,
 	g_free (sidebar->details->uri);
 	sidebar->details->uri = g_strdup (new_uri);
 		
-	directory = nautilus_directory_get (sidebar->details->uri);
-	nautilus_directory_unref (sidebar->details->directory);
-	sidebar->details->directory = directory;
-		
-	nautilus_sidebar_update_appearance (sidebar);
+	if (sidebar->details->file != NULL) {
+		gtk_signal_disconnect (GTK_OBJECT (sidebar->details->file), 
+				       sidebar->details->file_changed_connection);
+
+		nautilus_file_monitor_remove (sidebar->details->file, sidebar);
+	}
+
+
+	file = nautilus_file_get (sidebar->details->uri);
+
+	nautilus_file_unref (sidebar->details->file);
+
+	sidebar->details->file = file;
+	
+	sidebar->details->file_changed_connection =
+		gtk_signal_connect_object (GTK_OBJECT (sidebar->details->file),
+					   "changed",
+					   nautilus_sidebar_update_all,
+					   GTK_OBJECT (sidebar));
+
+	attributes = nautilus_mime_actions_get_minimum_file_attributes ();
+	nautilus_file_monitor_add (sidebar->details->file, sidebar, attributes);
+	g_list_free (attributes);
+
+	nautilus_sidebar_update_all (sidebar);
 
 	/* tell the title widget about it */
-	nautilus_sidebar_title_set_uri (sidebar->details->title,
-					sidebar->details->uri,
-					initial_title);
-	
-	/* set up the command buttons */
-	nautilus_sidebar_update_buttons (sidebar);
+	nautilus_sidebar_title_set_file (sidebar->details->title,
+					 sidebar->details->file,
+					 initial_title);
 }
 
 void
@@ -1401,12 +1476,11 @@ nautilus_sidebar_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 	NautilusSidebar *sidebar = NAUTILUS_SIDEBAR(widget);
 	
 	NAUTILUS_CALL_PARENT_CLASS (GTK_WIDGET_CLASS, size_allocate, (widget, allocation));
-
+	
 	/* remember the size if it changed */
 	
 	if (widget->allocation.width != sidebar->details->old_width) {
 		sidebar->details->old_width = widget->allocation.width;
  		nautilus_preferences_set_enum(NAUTILUS_PREFERENCES_SIDEBAR_WIDTH, widget->allocation.width);
-	
 	}	
 }
