@@ -1,39 +1,52 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 
+/*
+ * Nautilus
+ *
+ * Copyright (C) 2000 Eazel, Inc.
+ *
+ * Nautilus is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * Author: Rebecca Schulman <rebecka@eazel.com>
+ */
+
 /* nautilus-simple-search-bar.c - One box Search bar for Nautilus
-
-   Copyright (C) 2000 Eazel, Inc.
-
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public
-   License along with this program; see the file COPYING.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
-
-   Author: Rebecca Schulman <rebecka@eazel.com>
-*/
+ */
 
 #include <config.h>
 #include "nautilus-simple-search-bar.h"
 
+#include "nautilus-search-bar-criterion.h"
+#include "nautilus-window-private.h"
+#include <libgnome/gnome-i18n.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <libnautilus/nautilus-clipboard.h>
+#include <libnautilus-extensions/nautilus-entry.h>
+#include <libnautilus-extensions/nautilus-gtk-extensions.h>
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
+#include <libnautilus-extensions/nautilus-search-uri.h>
 #include <libnautilus-extensions/nautilus-string.h>
+#include <libnautilus-extensions/nautilus-undo-signal-handlers.h>
+#include <stdio.h>
 
 struct NautilusSimpleSearchBarDetails {
-	GtkEntry *entry;
+	NautilusEntry *entry;
 	GtkWidget *find_button;
 };
 
+static void   real_activate				     	 (NautilusNavigationBar	       *bar);
 static char * nautilus_simple_search_bar_get_location            (NautilusNavigationBar        *bar);
 static void   nautilus_simple_search_bar_set_location            (NautilusNavigationBar        *bar,
 								  const char                   *location);
@@ -54,6 +67,7 @@ nautilus_simple_search_bar_initialize_class (NautilusSimpleSearchBarClass *klass
 {
 	GTK_OBJECT_CLASS (klass)->destroy = nautilus_simple_search_bar_destroy;
 
+	NAUTILUS_NAVIGATION_BAR_CLASS (klass)->activate = real_activate;
 	NAUTILUS_NAVIGATION_BAR_CLASS (klass)->get_location = nautilus_simple_search_bar_get_location;  
 	NAUTILUS_NAVIGATION_BAR_CLASS (klass)->set_location = nautilus_simple_search_bar_set_location;  
 }
@@ -79,33 +93,56 @@ update_simple_find_button_state (NautilusSimpleSearchBar *bar)
 }
 
 static void
-activated_search_field (NautilusSimpleSearchBar *bar)
+nautilus_simple_search_bar_initialize (NautilusSimpleSearchBar *bar)
 {
-	/* Might be called when there's no text. */
-	if (!search_text_is_invalid (bar)) {
-		nautilus_navigation_bar_location_changed 
-			(NAUTILUS_NAVIGATION_BAR (bar));
-	}
+	bar->details = g_new0 (NautilusSimpleSearchBarDetails, 1);
 }
 
 static void
-nautilus_simple_search_bar_initialize (NautilusSimpleSearchBar *bar)
+nautilus_simple_search_bar_destroy (GtkObject *object)
 {
-	GtkWidget *hbox;
-	
-	bar->details = g_new0 (NautilusSimpleSearchBarDetails, 1);
+	NautilusSimpleSearchBar *bar;
 
+	bar = NAUTILUS_SIMPLE_SEARCH_BAR (object);
+	
+	nautilus_undo_editable_set_undo_key (GTK_EDITABLE (bar->details->entry), FALSE);
+	nautilus_undo_tear_down_nautilus_entry_for_undo (bar->details->entry);
+
+	g_free (bar->details);
+	
+	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
+}
+
+GtkWidget *
+nautilus_simple_search_bar_new (NautilusWindow *window)
+{
+	GtkWidget *simple_search_bar;
+	NautilusSimpleSearchBar *bar;
+	
+	GtkWidget *hbox;
+
+	simple_search_bar =  gtk_widget_new (NAUTILUS_TYPE_SIMPLE_SEARCH_BAR, NULL);
+	bar = NAUTILUS_SIMPLE_SEARCH_BAR (simple_search_bar);
 	hbox = gtk_hbox_new (0, FALSE);
 	
-	bar->details->entry = GTK_ENTRY (gtk_entry_new ());
+	/* Create button first so we can use it for auto_click */
+	bar->details->find_button = gtk_button_new_with_label (_("Find Them!"));
+
+	bar->details->entry = NAUTILUS_ENTRY (nautilus_entry_new ());
+	nautilus_undo_set_up_nautilus_entry_for_undo (bar->details->entry);
+	nautilus_undo_editable_set_undo_key (GTK_EDITABLE (bar->details->entry), TRUE);
+	nautilus_clipboard_set_up_editable
+		(GTK_EDITABLE (bar->details->entry),
+		 nautilus_window_get_ui_container (window));
+	
 	gtk_signal_connect_object (GTK_OBJECT (bar->details->entry), "activate",
-				   activated_search_field, GTK_OBJECT (bar));
+				   nautilus_gtk_button_auto_click, 
+				   GTK_OBJECT (bar->details->find_button));
 	gtk_signal_connect_object (GTK_OBJECT (bar->details->entry), "changed",
 				   update_simple_find_button_state, GTK_OBJECT (bar));
 
 	gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (bar->details->entry), TRUE, TRUE, 0);
 
-	bar->details->find_button = gtk_button_new_with_label (_("Find Them!"));
 	gtk_signal_connect_object (GTK_OBJECT (bar->details->find_button), "clicked",
 				   nautilus_navigation_bar_location_changed,
 				   GTK_OBJECT (bar));
@@ -114,20 +151,20 @@ nautilus_simple_search_bar_initialize (NautilusSimpleSearchBar *bar)
 	
 	gtk_container_add (GTK_CONTAINER (bar), hbox);
 
-	gtk_widget_show_all (hbox);
+	gtk_widget_show_all (hbox);	
+
+	return simple_search_bar;
 }
 
 static void
-nautilus_simple_search_bar_destroy (GtkObject *object)
+real_activate (NautilusNavigationBar *navigation_bar)
 {
-	g_free (NAUTILUS_SIMPLE_SEARCH_BAR (object)->details);
-	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
-}
+	NautilusSimpleSearchBar *bar;
 
-GtkWidget *
-nautilus_simple_search_bar_new (void)
-{
-	return gtk_widget_new (NAUTILUS_TYPE_SIMPLE_SEARCH_BAR, NULL);
+	bar = NAUTILUS_SIMPLE_SEARCH_BAR (navigation_bar);
+
+	/* Put the keyboard focus in the text field when switching to search mode */
+	gtk_widget_grab_focus (GTK_WIDGET (bar->details->entry));
 }
 
 static void
@@ -144,7 +181,7 @@ nautilus_simple_search_bar_set_location (NautilusNavigationBar *navigation_bar,
 
 	/* Set the words in the box to be the words originally done in the search */ 
 	criteria = nautilus_search_uri_to_simple_search_criteria (location);
-	gtk_entry_set_text (bar->details->entry, criteria);
+	nautilus_entry_set_text (bar->details->entry, criteria);
 	g_free (criteria);
 }
 
@@ -155,7 +192,7 @@ nautilus_simple_search_bar_get_location (NautilusNavigationBar *navigation_bar)
 	char *search_entry_text;
 
 	bar = NAUTILUS_SIMPLE_SEARCH_BAR (navigation_bar);
-	search_entry_text = gtk_entry_get_text (bar->details->entry);
+	search_entry_text = gtk_entry_get_text (GTK_ENTRY (bar->details->entry));
 	return nautilus_simple_search_criteria_to_search_uri (search_entry_text);
 }
 
@@ -164,7 +201,7 @@ nautilus_simple_search_bar_get_location (NautilusNavigationBar *navigation_bar)
 char *
 nautilus_search_uri_to_simple_search_criteria (const char *uri)
 {
-	/* FIXME: Not yet implemented. */
+	/* FIXME bugzilla.eazel.com 2511: Not yet implemented. */
 	return g_strdup ("");
 }
 
@@ -177,26 +214,34 @@ nautilus_simple_search_criteria_to_search_uri (const char *search_criteria)
 	char *escaped_fragment;
 	int length, i; 
 
+	/* FIXME bugzilla.eazel.com 2512: 
+	 * The logic here should be exactly the same as the logic for
+	 * a complex search-by-file-name. Currently the complex search doesn't
+	 * do the multi-word handling that this function does. They should use
+	 * the same code.
+	 */
 	g_return_val_if_fail (search_criteria != NULL, NULL);
 
 	words = g_strsplit (search_criteria, " ", strlen (search_criteria));
-	/* FIXME: this should eventually be: length = strlen ("[file%3A%2F%2F%2F]"); */
+	/* FIXME bugzilla.eazel.com 2513: this should eventually be: length = strlen ("[file%3A%2F%2F%2F]"); */
 	length = strlen ("[file:///]");
 	/* Count total length */
 	for (i = 0; words[i] != NULL; i++) {
-		length += strlen (words[i]) + strlen ("file_name contains & ");
+		length += strlen (words[i]) + strlen (NAUTILUS_SEARCH_URI_TEXT_NAME) + strlen (" contains & ");
 	}
 	fragment = g_new0 (char, length + 1);
-	/* FIXME: this should eventually be: sprintf (fragment, "[file%%3A%%2F%%2F%%2F]"); */
+	/* FIXME bugzilla.eazel.com 2513: this should eventually be: sprintf (fragment, "[file%%3A%%2F%%2F%%2F]"); */
 	sprintf (fragment, "[file:///]");
 	if (words[0] != NULL) {
 		for (i = 0; words[i+1] != NULL; i++) {
-			strcat (fragment, "file_name contains ");
+			strcat (fragment, NAUTILUS_SEARCH_URI_TEXT_NAME);
+			strcat (fragment, " contains ");
 			strcat (fragment, words[i]);
 			strcat (fragment, " & ");
 		}
-		strcat (fragment, "file_name contains ");
-			strcat (fragment, words[i]);
+		strcat (fragment, NAUTILUS_SEARCH_URI_TEXT_NAME);
+		strcat (fragment, " contains ");
+		strcat (fragment, words[i]);
 	}
 	g_strfreev (words);
 	escaped_fragment = gnome_vfs_escape_string (fragment);
