@@ -1,26 +1,30 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 
+/*
+ * Nautilus
+ *
+ * Copyright (C) 2000 Eazel, Inc.
+ *
+ * Nautilus is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * Nautilus is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public
+ * License along with this program; see the file COPYING.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ *
+ * Author: Rebecca Schulman <rebecka@eazel.com> 
+ */
+
 /* nautilus-switchable-search-bar.c - multimodal search bar
-
-   Copyright (C) 2000 Eazel, Inc.
-
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public
-   License along with this program; see the file COPYING.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
-
-   Author: Rebecca Schulman <rebecka@eazel.com>
-*/
+ */
 
 #include <config.h>
 #include "nautilus-switchable-search-bar.h"
@@ -29,13 +33,16 @@
 #include "nautilus-simple-search-bar.h"
 #include <gtk/gtkeventbox.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkvbox.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-dock.h>
 #include <libgnomeui/gnome-uidefs.h>
 #include <libnautilus-extensions/nautilus-directory.h>
 #include <libnautilus-extensions/nautilus-global-preferences.h>
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
 
+static void		     real_activate				     (NautilusNavigationBar	       *bar);
 static void                  nautilus_switchable_search_bar_set_location     (NautilusNavigationBar            *bar,
 									      const char                       *location);
 static char *                nautilus_switchable_search_bar_get_location     (NautilusNavigationBar            *bar);
@@ -55,6 +62,7 @@ NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusSwitchableSearchBar,
 static void
 nautilus_switchable_search_bar_initialize_class (NautilusSwitchableSearchBarClass *klass)
 {
+	NAUTILUS_NAVIGATION_BAR_CLASS (klass)->activate = real_activate;
 	NAUTILUS_NAVIGATION_BAR_CLASS (klass)->get_location = nautilus_switchable_search_bar_get_location;
 	NAUTILUS_NAVIGATION_BAR_CLASS (klass)->set_location = nautilus_switchable_search_bar_set_location;
 
@@ -64,27 +72,56 @@ nautilus_switchable_search_bar_initialize_class (NautilusSwitchableSearchBarClas
 static void
 search_bar_preference_changed_callback (gpointer user_data)
 {
+	char *location;
+
 	g_assert (NAUTILUS_IS_SWITCHABLE_SEARCH_BAR (user_data));
 
 	/* Switch immediately as long as the current search_uri doesn't veto the switch.
-	 * FIXME: Perhaps switch immediately anyway and blow away partially-formed
+	 * FIXME bugzilla.eazel.com 2515: 
+	 * Perhaps switch immediately anyway and blow away partially-formed
 	 * search criteria?
 	 */
+	location = nautilus_switchable_search_bar_get_location 
+		(NAUTILUS_NAVIGATION_BAR (user_data));
 	nautilus_switchable_search_bar_set_mode 
 		(NAUTILUS_SWITCHABLE_SEARCH_BAR (user_data), 
-		 nautilus_search_uri_to_search_bar_mode 
-			(nautilus_switchable_search_bar_get_location 
-				(NAUTILUS_NAVIGATION_BAR (user_data))));
+		 nautilus_search_uri_to_search_bar_mode (location));
+	g_free (location);
 }
 
 static void
 nautilus_switchable_search_bar_initialize (NautilusSwitchableSearchBar *bar)
 {
-	
+
+}	
+
+
+static void
+nautilus_switchable_search_bar_destroy (GtkObject *object)
+{
+	NautilusSwitchableSearchBar *bar;
+
+	bar = NAUTILUS_SWITCHABLE_SEARCH_BAR (object);
+
+	nautilus_preferences_remove_callback (NAUTILUS_PREFERENCES_SEARCH_BAR_TYPE,
+					      search_bar_preference_changed_callback,
+					      bar);
+
+	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
+}
+
+GtkWidget *
+nautilus_switchable_search_bar_new (NautilusWindow *window)
+{
 	GtkWidget *label;
 	GtkWidget *event_box;
 	GtkWidget *vbox;
 	GtkWidget *hbox;
+	GtkWidget *switchable_search_bar;
+	NautilusSwitchableSearchBar *bar;
+
+	switchable_search_bar = gtk_widget_new (nautilus_switchable_search_bar_get_type (), NULL);
+	bar = NAUTILUS_SWITCHABLE_SEARCH_BAR (switchable_search_bar);
 	
 	hbox = gtk_hbox_new (0, FALSE);
 	event_box = gtk_event_box_new ();
@@ -96,8 +133,8 @@ nautilus_switchable_search_bar_initialize (NautilusSwitchableSearchBar *bar)
 	gtk_container_add (GTK_CONTAINER (event_box), label);
 	
 	gtk_box_pack_start (GTK_BOX (hbox), event_box, FALSE, TRUE, GNOME_PAD_SMALL);	
-	bar->complex_search_bar = nautilus_complex_search_bar_new ();
-	bar->simple_search_bar = nautilus_simple_search_bar_new ();
+	bar->complex_search_bar = nautilus_complex_search_bar_new (window);
+	bar->simple_search_bar = nautilus_simple_search_bar_new (window);
 
 	gtk_signal_connect_object (GTK_OBJECT (bar->complex_search_bar),
 				   "location_changed",
@@ -126,26 +163,32 @@ nautilus_switchable_search_bar_initialize (NautilusSwitchableSearchBar *bar)
 	nautilus_preferences_add_callback (NAUTILUS_PREFERENCES_SEARCH_BAR_TYPE,
 					   search_bar_preference_changed_callback,
 					   bar);
+
+	return switchable_search_bar;
+
 }
 
 static void
-nautilus_switchable_search_bar_destroy (GtkObject *object)
+real_activate (NautilusNavigationBar *navigation_bar)
 {
 	NautilusSwitchableSearchBar *bar;
+	NautilusNavigationBar *bar_to_activate;
 
-	bar = NAUTILUS_SWITCHABLE_SEARCH_BAR (object);
+	bar = NAUTILUS_SWITCHABLE_SEARCH_BAR (navigation_bar);
 
-	nautilus_preferences_remove_callback (NAUTILUS_PREFERENCES_SEARCH_BAR_TYPE,
-					      search_bar_preference_changed_callback,
-					      bar);
+	switch (bar->mode) {
+	default:
+		g_assert_not_reached();
+	case NAUTILUS_SIMPLE_SEARCH_BAR:
+		bar_to_activate = NAUTILUS_NAVIGATION_BAR (bar->simple_search_bar);
+		break;
+	case NAUTILUS_COMPLEX_SEARCH_BAR:
+		bar_to_activate = NAUTILUS_NAVIGATION_BAR (bar->complex_search_bar);
+		break;
+	}
 
-	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
-}
-
-GtkWidget *
-nautilus_switchable_search_bar_new (void)
-{
-	return gtk_widget_new (nautilus_switchable_search_bar_get_type (), NULL);
+	g_assert (bar_to_activate != NULL);
+	nautilus_navigation_bar_activate (bar_to_activate);
 }
 
 void
@@ -153,6 +196,7 @@ nautilus_switchable_search_bar_set_mode (NautilusSwitchableSearchBar *bar,
 					 NautilusSearchBarMode mode)
 {
 	char *location;
+	GtkWidget *dock;
 
 	g_return_if_fail (NAUTILUS_IS_SWITCHABLE_SEARCH_BAR (bar));
 	g_return_if_fail (mode == NAUTILUS_SIMPLE_SEARCH_BAR
@@ -160,7 +204,7 @@ nautilus_switchable_search_bar_set_mode (NautilusSwitchableSearchBar *bar,
 
 	/* Ignore requests for impossible modes for now */
 	location = nautilus_navigation_bar_get_location (NAUTILUS_NAVIGATION_BAR (bar));
-	if (nautilus_search_uri_is_displayable_by_mode (location, mode) == FALSE) {
+	if (!nautilus_search_uri_is_displayable_by_mode (location, mode)) {
 		g_free (location);
 		return;
 	}
@@ -186,6 +230,16 @@ nautilus_switchable_search_bar_set_mode (NautilusSwitchableSearchBar *bar,
 		g_assert_not_reached();
 		break;
 	}
+
+	/* FIXME bugzilla.eazel.com 3171:
+	 * We don't know why this line is needed here, but if it's removed
+	 * then the bar won't shrink when we switch to the simple search bar
+	 * (though it does grow when switching to the complex one).
+	 */
+	dock = gtk_widget_get_ancestor (GTK_WIDGET (bar), GNOME_TYPE_DOCK);
+	if (dock != NULL) {
+		gtk_widget_queue_resize (dock);
+	}
 }
 
 static char *
@@ -218,7 +272,7 @@ nautilus_switchable_search_bar_set_location (NautilusNavigationBar *navigation_b
 	/* Set the mode of the search bar,
 	   in case preferences have changed 
 	*/
-	/* FIXME:  This doesn't work yet. */
+	/* FIXME bugzilla.eazel.com 2514:  This doesn't work yet. */
 	mode = nautilus_search_uri_to_search_bar_mode (location);
 	nautilus_switchable_search_bar_set_mode (bar, mode);
 						 
@@ -251,7 +305,7 @@ gboolean
 nautilus_search_uri_is_displayable_by_mode (const char *uri,
 					    NautilusSearchBarMode mode)
 {
-	/* FIXME */
+	/* FIXME bugzilla.eazel.com 2514 */
 	return TRUE;
 }
 
