@@ -6,18 +6,18 @@
  *  Copyright (C) 1999, 2000 Red Hat, Inc.
  *  Copyright (C) 1999, 2000 Eazel, Inc.
  *
- *  This library is free software; you can redistribute it and/or
+ *  Nautilus is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License as
  *  published by the Free Software Foundation; either version 2 of the
  *  License, or (at your option) any later version.
  *
- *  This library is distributed in the hope that it will be useful,
+ *  Nautilus is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this library; if not, write to the Free Software
+ *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  *  Author: Elliot Lee <sopwith@redhat.com>
@@ -50,7 +50,8 @@ BonoboObject *impl_Nautilus_ZoomableFrame__create             (NautilusViewFrame
 void          nautilus_view_frame_open_location               (NautilusViewFrame *view,
                                                                const char        *location);
 void          nautilus_view_frame_open_location_in_new_window (NautilusViewFrame *view,
-                                                               const char        *location);
+                                                               const char        *location,
+                                                               GList             *selection);
 void          nautilus_view_frame_report_location_change      (NautilusViewFrame *view,
                                                                const char        *location);
 void          nautilus_view_frame_report_selection_change     (NautilusViewFrame *view,
@@ -75,14 +76,4 @@ void          nautilus_view_frame_zoom_parameters_changed     (NautilusViewFrame
                                                                double             min_zoom_level,
                                                                double             max_zoom_level);
 
-struct NautilusViewComponentType {
-        const char *primary_repoid;
-        gboolean (* try_load) (NautilusViewFrame *view, CORBA_Object obj, CORBA_Environment *ev);
-        void (* destroy) (NautilusViewFrame *view, CORBA_Environment *ev);
-        void (* load_location) (NautilusViewFrame *view, Nautilus_URI location, CORBA_Environment *ev);
-        void (* stop_loading) (NautilusViewFrame *view, CORBA_Environment *ev);
-        void (* selection_changed) (NautilusViewFrame *view, const Nautilus_URIList *selection, CORBA_Environment *ev);
-};
-
 #endif /* NAUTILUS_VIEW_FRAME_PRIVATE_H */
-
