@@ -1,14 +1,16 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 
-/* Nautilus
+/* 
+ * Nautilus
+ *
  * Copyright (C) 2000 Eazel, Inc.
  *
- * This program is free software; you can redistribute it and/or modify
+ * Nautilus is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * Nautilus is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -37,6 +39,7 @@
 #include <gtk/gtkmenuitem.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-pixmap.h>
+#include <libnautilus-extensions/nautilus-graphic-effects.h>
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
 #include <libnautilus-extensions/nautilus-gtk-extensions.h>
 #include <libnautilus-extensions/nautilus-glib-extensions.h>
@@ -53,6 +56,13 @@ enum {
 	LAST_SIGNAL
 };
 
+typedef enum {
+	PRELIGHT_NONE,
+	PRELIGHT_MINUS,
+	PRELIGHT_CENTER,
+	PRELIGHT_PLUS
+} PrelightMode;
+
 #define	GAP_WIDTH 2
 
 struct NautilusZoomControlDetails {
@@ -66,8 +76,12 @@ struct NautilusZoomControlDetails {
 	GdkPixbuf *zoom_decrement_image;
 	GdkPixbuf *zoom_increment_image;
 	GdkPixbuf *number_strip;
+	PrelightMode prelight_mode;
+
+	gboolean marking_menu_items;
 };
 
+
 static guint signals[LAST_SIGNAL];
 
 static void     nautilus_zoom_control_initialize_class 	 (NautilusZoomControlClass *klass);
@@ -75,14 +89,20 @@ static void     nautilus_zoom_control_initialize       	 (NautilusZoomControl *z
 static void	nautilus_zoom_control_destroy		 (GtkObject *object);
 static void     nautilus_zoom_control_draw 	       	 (GtkWidget *widget, 
 							  GdkRectangle *box);
-static int     nautilus_zoom_control_expose 		 (GtkWidget *widget, 
+static int	nautilus_zoom_control_expose 		 (GtkWidget *widget, 
 							  GdkEventExpose *event);
-static gboolean nautilus_zoom_control_button_press_event (GtkWidget *widget, 
+static 		gboolean nautilus_zoom_control_button_press_event (GtkWidget *widget, 
 							  GdkEventButton *event);
+static int	nautilus_zoom_control_leave_notify	 (GtkWidget        *widget,
+			 				  GdkEventCrossing *event);
+static gboolean nautilus_zoom_control_motion_notify       (GtkWidget        *widget,
+						     	  GdkEventMotion   *event);
+
 static void	nautilus_zoom_control_load_images	 (NautilusZoomControl *zoom_control);
 static void	nautilus_zoom_control_unload_images	 (NautilusZoomControl *zoom_control);
 static void	nautilus_zoom_control_theme_changed 	 (gpointer user_data);
 static void	nautilus_zoom_control_size_allocate	 (GtkWidget *widget, GtkAllocation *allocation);
+static void	nautilus_zoom_control_set_prelight_mode	 (NautilusZoomControl *zoom_control, PrelightMode mode);
 
 
 void            draw_number		                 (GtkWidget *widget, 
@@ -104,6 +124,9 @@ nautilus_zoom_control_initialize_class (NautilusZoomControlClass *zoom_control_c
 	widget_class->draw = nautilus_zoom_control_draw;
 	widget_class->expose_event = nautilus_zoom_control_expose;
 	widget_class->button_press_event = nautilus_zoom_control_button_press_event;
+  	widget_class->leave_notify_event = nautilus_zoom_control_leave_notify;
+	widget_class->motion_notify_event = nautilus_zoom_control_motion_notify;
+
 	widget_class->size_allocate = nautilus_zoom_control_size_allocate;
 	
 	signals[ZOOM_IN] =
@@ -206,15 +229,16 @@ nautilus_zoom_control_initialize (NautilusZoomControl *zoom_control)
 	nautilus_preferences_add_callback (NAUTILUS_PREFERENCES_THEME,
 					  nautilus_zoom_control_theme_changed,
 					  zoom_control);	
-
+	
+	/* enable mouse motion events */
+	gtk_widget_add_events (GTK_WIDGET (zoom_control), GDK_POINTER_MOTION_MASK);
 }
 
 /* allocate a new zoom control */
-GtkWidget*
-nautilus_zoom_control_new ()
+GtkWidget *
+nautilus_zoom_control_new (void)
 {
-	NautilusZoomControl *zoom_control = gtk_type_new (nautilus_zoom_control_get_type ());
-	return GTK_WIDGET (zoom_control);
+	return gtk_widget_new (nautilus_zoom_control_get_type (), NULL);
 }
 
 /* handler for handling theme changes */
@@ -258,8 +282,10 @@ void draw_number (GtkWidget *widget, GdkRectangle *box)
 	int char_height, char_width, char_offset;
 	int num_v_offset, num_h_offset;
 	NautilusZoomControl *zoom_control;
+	GdkPixbuf *number_pixbuf;
 	
 	zoom_control = NAUTILUS_ZOOM_CONTROL (widget);
+	number_pixbuf = NULL;
 	
 	num_v_offset = get_zoom_offset ("NUMBER_V_OFFSET");
 	num_h_offset = get_zoom_offset ("NUMBER_H_OFFSET");
@@ -270,19 +296,25 @@ void draw_number (GtkWidget *widget, GdkRectangle *box)
 	percent = floor((100.0 * zoom_control->details->zoom_level) + .5);
 	g_snprintf(buffer, 8, "%d", percent);
 
-	y = 1 + num_v_offset + (box->height >> 1);
-	x = num_h_offset + ((box->width - char_width * strlen(buffer)) >> 1);  
+	y = 1 + num_v_offset + (widget->allocation.height >> 1);
+	x = num_h_offset + ((widget->allocation.width - char_width * strlen(buffer)) >> 1);  
 	
 	temp_gc = gdk_gc_new(widget->window);
 	
 	if (zoom_control->details->number_strip) {
 		cur_char = &buffer[0];
 		char_height = gdk_pixbuf_get_height (zoom_control->details->number_strip);
+		
+		number_pixbuf = zoom_control->details->number_strip;
+		if (zoom_control->details->prelight_mode == PRELIGHT_CENTER) {
+			number_pixbuf = nautilus_create_spotlight_pixbuf (number_pixbuf);
+		}
+		
 		while (*cur_char) {
 			/* draw the character */
 			
 			char_offset = (*cur_char++ - '0') * char_width;
-			gdk_pixbuf_render_to_drawable_alpha (zoom_control->details->number_strip,
+			gdk_pixbuf_render_to_drawable_alpha (number_pixbuf,
 					     widget->window, 
 					     char_offset, 0, x, y,
 					     char_width,
@@ -293,18 +325,22 @@ void draw_number (GtkWidget *widget, GdkRectangle *box)
 			x += char_width;
 		}
 	} else {
-		label_font = gdk_font_load("-bitstream-courier-medium-r-normal-*-9-*-*-*-*-*-*-*");
+              label_font = gdk_fontset_load (_("-bitstream-courier-medium-r-normal-*-9-*-*-*-*-*-*-*"));
 				
-		x = num_h_offset + ((box->width - gdk_string_width(label_font, buffer)) >> 1);  
+		x = num_h_offset + ((widget->allocation.width - gdk_string_width(label_font, buffer)) >> 1);  
 		
 		gdk_draw_string (widget->window, label_font, temp_gc, x, y, &buffer[0]);
 		gdk_font_unref(label_font);
 	}
 	
+	if (number_pixbuf != zoom_control->details->number_strip) {
+		gdk_pixbuf_unref (number_pixbuf);
+	}
+	
 	gdk_gc_unref(temp_gc);
 }
 
-/* utility to simplify drawing */
+/* utilities to simplify drawing */
 static void
 draw_pixbuf (GdkPixbuf *pixbuf, GdkDrawable *drawable, int x, int y)
 {
@@ -315,6 +351,22 @@ draw_pixbuf (GdkPixbuf *pixbuf, GdkDrawable *drawable, int x, int y)
 					     0, 0);
 }
 
+static void
+draw_pixbuf_with_prelight (NautilusZoomControl *zoom_control, GdkPixbuf *pixbuf, int x_pos, int y_pos, PrelightMode mode)
+{
+	GdkPixbuf *temp_pixbuf;
+	
+	temp_pixbuf = pixbuf;
+	if (zoom_control->details->prelight_mode == mode) {
+		temp_pixbuf = nautilus_create_spotlight_pixbuf (temp_pixbuf);
+	}
+	draw_pixbuf (temp_pixbuf, GTK_WIDGET (zoom_control)->window, x_pos, y_pos + zoom_control->details->y_offset);
+	if (pixbuf != temp_pixbuf) {
+		gdk_pixbuf_unref (temp_pixbuf);
+	}
+
+}
+
 /* draw the zoom control image into the passed-in rectangle */
 
 static void
@@ -325,9 +377,10 @@ draw_zoom_control_image (GtkWidget *widget, GdkRectangle *box)
 		
 	zoom_control = NAUTILUS_ZOOM_CONTROL (widget);
 
-	/* draw the decrement symbol if necessary */
+	/* draw the decrement symbol if necessary, complete with prelighting */
 	if (zoom_control->details->zoom_level > zoom_control->details->min_zoom_level) {
-		draw_pixbuf (zoom_control->details->zoom_decrement_image, widget->window, box->x, box->y + zoom_control->details->y_offset);
+		draw_pixbuf_with_prelight (zoom_control, zoom_control->details->zoom_decrement_image,
+						box->x, box->y, PRELIGHT_MINUS);
 	} else {
 		width  = gdk_pixbuf_get_width (zoom_control->details->zoom_decrement_image);
 		height = gdk_pixbuf_get_height (zoom_control->details->zoom_decrement_image);		
@@ -341,13 +394,16 @@ draw_zoom_control_image (GtkWidget *widget, GdkRectangle *box)
 	}
 	
 	offset = gdk_pixbuf_get_width (zoom_control->details->zoom_decrement_image) + GAP_WIDTH;
-	/* draw the body image */
-	draw_pixbuf (zoom_control->details->zoom_body_image, widget->window, box->x + offset, box->y + zoom_control->details->y_offset);
+	/* draw the body image, prelighting if necessary */
+	draw_pixbuf_with_prelight (zoom_control, zoom_control->details->zoom_body_image,
+					box->x + offset, box->y, PRELIGHT_CENTER);
+	
 	offset += gdk_pixbuf_get_width (zoom_control->details->zoom_body_image) + GAP_WIDTH;
 	
-	/* draw the increment symbol if necessary */
+	/* draw the increment symbol if necessary, complete with prelighting */
 	if (zoom_control->details->zoom_level < zoom_control->details->max_zoom_level) {
-		draw_pixbuf (zoom_control->details->zoom_increment_image, widget->window, box->x + offset, box->y + zoom_control->details->y_offset);
+		draw_pixbuf_with_prelight (zoom_control, zoom_control->details->zoom_increment_image,
+					   box->x + offset, box->y, PRELIGHT_PLUS);
 	} else {
 		width  = gdk_pixbuf_get_width (zoom_control->details->zoom_increment_image);
 		height = gdk_pixbuf_get_height (zoom_control->details->zoom_increment_image);		
@@ -397,6 +453,16 @@ nautilus_zoom_control_expose (GtkWidget *widget, GdkEventExpose *event)
 	return FALSE;
 }
 
+/* set the prelight mode and redraw as necessary */
+static void
+nautilus_zoom_control_set_prelight_mode (NautilusZoomControl *zoom_control, PrelightMode mode)
+{
+	if (mode != zoom_control->details->prelight_mode) {
+		zoom_control->details->prelight_mode = mode;		
+		gtk_widget_queue_draw (GTK_WIDGET (zoom_control));	
+	}
+	
+}
 
 /* routines to load the images used to draw the control */
 
@@ -461,8 +527,14 @@ zoom_menu_callback (GtkMenuItem *item, gpointer callback_data)
 	double zoom_level;
 	NautilusZoomControl *zoom_control;
 		
-	zoom_level = *(double*) gtk_object_get_data (GTK_OBJECT (item), "zoom_level");
 	zoom_control = NAUTILUS_ZOOM_CONTROL (callback_data);
+
+	/* Don't do anything if we're just setting the toggle state of menu items. */
+	if (zoom_control->details->marking_menu_items) {
+		return;
+	}
+
+	zoom_level = * (double *) gtk_object_get_data (GTK_OBJECT (item), "zoom_level");
 	
 	/* Check to see if we can zoom out */
 	if ((zoom_control->details->min_zoom_level <= zoom_level && zoom_level <  zoom_control->details->zoom_level) ||
@@ -471,20 +543,38 @@ zoom_menu_callback (GtkMenuItem *item, gpointer callback_data)
 	}
 }
 
-static void
-create_zoom_menu_item (GtkMenu *menu, GtkWidget *zoom_control, double zoom_level)
+static GtkRadioMenuItem *
+create_zoom_menu_item (GtkMenu *menu, GtkWidget *widget, double zoom_level, GtkRadioMenuItem *previous_radio_item)
 {
 	GtkWidget *menu_item;
 	double	  *zoom_level_ptr;
 	char	  item_text[8];
-	
-	g_snprintf(item_text, sizeof (item_text), _("%.0f%%"), 100.0 * zoom_level);
+	NautilusZoomControl *zoom_control;
+	GSList	  *radio_item_group;
 
-	menu_item = gtk_menu_item_new_with_label (item_text);
+	zoom_control = NAUTILUS_ZOOM_CONTROL (widget);
+
+	/* Set flag so that callback isn't activated when set_active called
+	 * to set toggle state of other radio items.
+	 */
+	zoom_control->details->marking_menu_items = TRUE;
+	
+	/* This is marked for localization in case the % sign is not
+	 * appropriate in some locale. I guess that's unlikely.
+	 */
+	g_snprintf (item_text, sizeof (item_text), _("%.0f%%"), 100.0 * zoom_level);
+	radio_item_group = previous_radio_item == NULL
+		? NULL
+		: gtk_radio_menu_item_group (previous_radio_item);
+	menu_item = gtk_radio_menu_item_new_with_label (radio_item_group, item_text);
 
 	zoom_level_ptr = g_new (double, 1);
 	*zoom_level_ptr = zoom_level;
 
+	gtk_check_menu_item_set_show_toggle (GTK_CHECK_MENU_ITEM (menu_item), TRUE);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), 
+					zoom_level == zoom_control->details->zoom_level);
+	
 	gtk_object_set_data_full (GTK_OBJECT (menu_item), "zoom_level", zoom_level_ptr, g_free);
 	gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
 			    GTK_SIGNAL_FUNC (zoom_menu_callback),
@@ -492,6 +582,10 @@ create_zoom_menu_item (GtkMenu *menu, GtkWidget *zoom_control, double zoom_level
 
   	gtk_widget_show (menu_item);
 	gtk_menu_append (menu, menu_item);
+
+	zoom_control->details->marking_menu_items = FALSE;
+
+	return GTK_RADIO_MENU_ITEM (menu_item);
 }
 
 static GtkMenu*
@@ -499,13 +593,15 @@ create_zoom_menu(GtkWidget *zoom_control)
 {
 	GList *p;
 	GtkMenu *menu;
+	GtkRadioMenuItem *menu_item;
 
 	menu = GTK_MENU (gtk_menu_new ());
 
 	p = NAUTILUS_ZOOM_CONTROL (zoom_control)->details->preferred_zoom_levels;
-	
+
+	menu_item = NULL;
 	while (p != NULL) {
-		create_zoom_menu_item(menu, zoom_control,  *(double*)p->data);
+		menu_item = create_zoom_menu_item (menu, zoom_control, * (double *) p->data, menu_item);
 		p = g_list_next (p);
 	}
 	
@@ -523,13 +619,10 @@ nautilus_zoom_control_button_press_event (GtkWidget *widget, GdkEventButton *eve
 	
 	/* check for the context menu button and handle by creating and showing the menu */  
 	if (event->button == CONTEXTUAL_MENU_BUTTON) {
-		GtkMenu *zoom_menu = create_zoom_menu (widget);
-		
-		gtk_object_ref (GTK_OBJECT (zoom_menu));
-		gtk_object_sink (GTK_OBJECT (zoom_menu));
-		
-		gtk_menu_popup (zoom_menu, NULL, NULL, NULL, NULL, 3, GDK_CURRENT_TIME);
-		gtk_object_unref (GTK_OBJECT (zoom_menu));
+		nautilus_pop_up_context_menu (create_zoom_menu (widget), 
+					      NAUTILUS_DEFAULT_POPUP_MENU_DISPLACEMENT, 
+					      NAUTILUS_DEFAULT_POPUP_MENU_DISPLACEMENT, 
+					      CONTEXTUAL_MENU_BUTTON);
 		return TRUE;	  
  	}
 	
@@ -552,6 +645,48 @@ nautilus_zoom_control_button_press_event (GtkWidget *widget, GdkEventButton *eve
 	return TRUE;
 }
 
+/* handle setting the prelight mode */
+
+/* handle enter, leave and motion events to maintain the prelight state */
+
+static gint
+nautilus_zoom_control_leave_notify (GtkWidget        *widget,
+			 	    GdkEventCrossing *event)
+{
+	nautilus_zoom_control_set_prelight_mode (NAUTILUS_ZOOM_CONTROL (widget), PRELIGHT_NONE);
+  	return FALSE;
+}
+
+static gboolean nautilus_zoom_control_motion_notify (GtkWidget *widget, GdkEventMotion   *event)
+{
+	int x, y, x_offset;
+	int width, center;
+	NautilusZoomControl *zoom_control;
+	PrelightMode mode;
+	
+	zoom_control = NAUTILUS_ZOOM_CONTROL (widget);
+	
+	gtk_widget_get_pointer(widget, &x, &y);
+	
+	x_offset = x ;
+	width = widget->allocation.width;
+	center = width >> 1;
+
+	mode = PRELIGHT_NONE;
+	if (x_offset < (width / 3) && (zoom_control->details->zoom_level > zoom_control->details->min_zoom_level)) {
+		mode = PRELIGHT_MINUS;		
+	} else if ((x_offset > ((2 * width) / 3)) && (zoom_control->details->zoom_level < zoom_control->details->max_zoom_level)) {
+		mode = PRELIGHT_PLUS;
+	} else if ((x_offset >= (center - (width >> 3))) && (x_offset <= (center + (width >> 3)))) {
+		mode = PRELIGHT_CENTER;
+	}
+
+	nautilus_zoom_control_set_prelight_mode (zoom_control, mode);
+	
+	return TRUE;
+}
+
+
 /* handle setting the size */
 static void
 nautilus_zoom_control_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
